package com.bea.lsd.avitek.ordermanagement;

/**
 * FedexSessionBean
 *
 * This bean is the (loose) facade to the the FedexShippingPartner entity bean. It returns
 * multiple Fedex shiments and single shipments based on primary key and tracking id.
 *
 * @Author Jason Lee
 * $Id: $
 */

import java.rmi.RemoteException;
import javax.ejb.*;
import java.util.*;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import com.bea.lsd.avitek.ordermanagement.entities.FedexShippingPartner;
import com.bea.lsd.avitek.ordermanagement.entities.FedexShippingPartnerBean;
import com.bea.lsd.avitek.ordermanagement.entities.FedexShippingPartnerHome
;
public class FedexSessionBean implements SessionBean {
    
    private SessionContext ctx;

    /*
     * Returns a Collection ofFedexShipment entities based on a tracking id.
     *
     * @param String aTracking
     * @return Collection fedexShipments
     */
    public Collection getFedexShipments(String aTrackingId) throws RemoteException{
	
	try {

	    InitialContext ctx = new InitialContext();
	    
	    FedexShippingPartnerHome home = (FedexShippingPartnerHome)ctx.lookup("FedexShippingPartnerBean");
	    Collection fedexShipments = (Collection)home.findByTrackingId(aTrackingId);
	    
	    return fedexShipments;
	    
	} catch(Exception e) {
	    System.out.println("Exception in getFedexShipments: " + e.getMessage());
	}
	
	return null;
	
    } // getFedexShipments

    /*
     * Returns a FedexShipment entitiy based on a tracking id.
     *
     * @param String aTracking
     * @return Collection fedexShipments
     */
    public FedexShippingPartner getFedexShipment(String aTrackingId) throws RemoteException{
	
	try {

	    InitialContext ctx = new InitialContext();
	    
	    FedexShippingPartnerHome home = (FedexShippingPartnerHome)ctx.lookup("FedexShippingPartnerBean");
	    FedexShippingPartner fedexShipment = (FedexShippingPartner)home.findByTrackingId(aTrackingId);
	    
	    return fedexShipment;
	    
	} catch(Exception e) {
	    System.out.println("Exception in getFedexShipment tracking id: " + e.getMessage());
	}
	
	return null;
	
    } // getFedexShipment


    /*
     * Returns a FedexShipment entitiy based on a tracking id.
     *
     * @param Long aPrimaryKeyId
     * @return FedexShippingPartner aFedexShipments
     */
    public FedexShippingPartner getFedexShipmentById(Long aPKId) throws RemoteException{

	try {

	    InitialContext ctx = new InitialContext();
	    
	    FedexShippingPartnerHome home = (FedexShippingPartnerHome)ctx.lookup("FedexShippingPartnerBean");
	    FedexShippingPartner fedexShipment = (FedexShippingPartner)home.findByPrimaryKey(aPKId);
	    
	    return fedexShipment;
	    
	} catch(Exception e) {
	    System.out.println("Exception in getFedexShipment pk: " + e.getMessage());
	}
	
	return null;

    } // getFedexShipment
    
    public void ejbActivate() {}
    
    public void ejbRemove() {}
    
    public void ejbPassivate() {}
    
    public void setSessionContext(SessionContext ctx) {this.ctx = ctx;}
    
    public void ejbCreate () throws CreateException { }

} // FedexSessionBean








