package com.bea.lsd.avitek.ordermanagement;

/**
 * OrderSessionBean
 *
 * This bean is the (loose) facade to the the Order entity bean. It returns
 * multiple Order items and single items based on primary key.
 *
 * @Author Jason Lee
 * $Id: $
 */

import java.rmi.RemoteException;
import javax.ejb.*;
import java.util.*;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import com.bea.lsd.avitek.ordermanagement.entities.Order;
import com.bea.lsd.avitek.ordermanagement.entities.OrderBean;
import com.bea.lsd.avitek.ordermanagement.entities.OrderHome;

public class OrderSessionBean implements SessionBean {
    
    private SessionContext ctx;

    /*
     * Returns a Collection of Order entities.
     *
     * @return Collection of Order entities
     */
    public Collection getOrderAll() throws RemoteException {
	
	try {

	    InitialContext ctx = new InitialContext();
	    
	    OrderHome home = (OrderHome)ctx.lookup("OrderBean");
	    Collection orders = (Collection)home.findAll();
	    
	    return orders;
	    
	} catch(Exception e) {
	    System.out.println("Exception in getOrderAll: " + e.getMessage());
	}
	
	return null;
	
    } // getOrderAll


    /*
     * Returns a Order entitiy based on a primary key id.
     *
     * @param Long aPrimaryKeyId
     * @return Order aOrder
     */
    public Order getOrderByPKId(Long aPKId) throws RemoteException {

	try {

	    InitialContext ctx = new InitialContext();
	    
	    OrderHome home = (OrderHome)ctx.lookup("OrderBean");
	    Order order = (Order)home.findByPrimaryKey(aPKId);
	    
	    return order;
	    
	} catch(Exception e) {
	    System.out.println("Exception in getOrderByPKId: " + e.getMessage());
	}
	
	return null;
	
    } // getOrderShipment

    /*
     * Returns a Order entitiy based on a primary key id.
     *
     * @param String aSku
     * @return Order aOrder
     */
    public Order getOrderBySku(String aSku) throws RemoteException {

	try {

	    InitialContext ctx = new InitialContext();
	    
	    OrderHome home = (OrderHome)ctx.lookup("OrderBean");
	    Order order = (Order)home.findBySku(aSku);
	    
	    return order;
	    
	} catch(Exception e) {
	    System.out.println("Exception in getOrderBySku: " + e.getMessage());
	}
	
	return null;
	
    } // getOrderBySku

    /*
     * Returns a Order entity based on a carrier id.
     *
     * @param String aCarrier (APL, DHL, FEDEX, UPS, USPS)
     * @return Order aOrder
     */
    public Order getOrderByCarrier(String aCarrierId) throws RemoteException {

	try {

	    InitialContext ctx = new InitialContext();
	    
	    OrderHome home = (OrderHome)ctx.lookup("OrderBean");
	    Order order = (Order)home.findByCarrierId(aCarrierId);
	    
	    return order;
	    
	} catch(Exception e) {
	    System.out.println("Exception in getOrderByCarrier: " + e.getMessage());
	}
	
	return null;
	
    } // getOrderByCarrier

    /*
     * Returns a Order entity based on a tracking id.
     *
     * @param String aTrackingId
     * @return Order aOrder
     */
    public Order getOrderByTrackingId(String aTrackingId) throws RemoteException {

	try {

	    InitialContext ctx = new InitialContext();
	    
	    OrderHome home = (OrderHome)ctx.lookup("OrderBean");
	    Order order = (Order)home.findByTrackingId(aTrackingId);
	    
	    return order;
	    
	} catch(Exception e) {
	    System.out.println("Exception in getOrderByTrackingId: " + e.getMessage());
	}
	
	return null;
	
    } // getOrderByTrackingId

    public void ejbActivate() {}
    
    public void ejbRemove() {}
    
    public void ejbPassivate() {}
    
    public void setSessionContext(SessionContext ctx) {this.ctx = ctx;}
    
    public void ejbCreate () throws CreateException { }

} // OrderSessionBean








