package com.bea.lsd.avitek.ordermanagement.entities;

/*
 * OrderHome
 *
 * Home interface of the OrderBean
 *
 * @Author Jason Lee
 * $id:$
 */

import java.util.Collection;
import javax.ejb.EJBHome; 
import java.rmi.RemoteException; 
import javax.ejb.CreateException; 
import javax.ejb.FinderException; 

public interface OrderHome extends EJBHome {

    /* 
     * Creates a bean with the corresponding db table passed in values 
     *
     * @param String aLotNumber
     * @param String aSku
     * @param String aCarrierId
     * @param String aTrackingId
     * @param String aCreateDate
     * @param int    aQuantity
     * @param String aStatus
     * @return Order aOrder
     */
    public Order create(String aLotNumber, String aSku, String aCarrierId, String aTrackingId,
			String aCreateDate, int aQuantityOrdered, String aStatus)
	throws RemoteException, CreateException;

    /*
     * Returns a Order object based on a the primary key supplied.
     *
     * @param Long aPrimaryKey
     * @return Order aOrder
     *
     */
    public Order findByPrimaryKey(Long id) throws FinderException, RemoteException;

    /*
     * Returns a Order object based on a the matching lot id.
     *
     * @param Long aLotId
     * @return Order aOrder
     *
     */
    public Order findByLotId(String aLotId) throws FinderException, RemoteException;

    /*
     * Returns a Order object based on a the matching sku.
     *
     * @param String aSku
     * @return Order aOrder
     *
     */
    public Order findBySku(String aSku) throws FinderException, RemoteException;

    /*
     * Returns a Order object based on a the matching tracking id.
     *
     * @param String aTrackingId
     * @return Order aOrder
     *
     */
    public Order findByTrackingId(String aTrackingId) throws FinderException, RemoteException;

    /*
     * Returns a Order object based on a the matching carrier id.
     *
     * @param String aCarrierId
     * @return Order aOrder
     *
     */
    public Order findByCarrierId(String aCarrierId) throws FinderException, RemoteException;
    
    /*
     * Returns a Collection of all Order objects.
     *
     * @return Collection of all Order objects
     *
     */
    public Collection findAll() throws FinderException, RemoteException;


} // OrderHome
