package com.bea.lsd.avitek.ordermanagement.entities;

/*
 * ShippingHistoryHome
 *
 * Home interface of the ShippingHistoryBean
 *
 * @Author Jason Lee
 * $id:$
 */

import java.util.Collection;
import javax.ejb.EJBHome; 
import java.rmi.RemoteException; 
import javax.ejb.CreateException; 
import javax.ejb.FinderException; 

public interface ShippingHistoryHome extends EJBHome {

    /* 
     * Creates a bean with the corresponding db table passed in values 
     *
     * @param String aCarrierId
     * @param String aShipmentId
     * @param Long   aLocationId
     * @return ShippingHistory aShippingHistory
     */
    public ShippingHistory create(String aCarrierId, String aShipmentId, Long aLocationId)
	throws RemoteException, CreateException;

    /*
     * Returns a ShippingHistory object based on a the primary key supplied.
     *
     * @param Long aPrimaryKey
     * @return ShippingHistory aShippingHistory
     *
     */
    public ShippingHistory findByPrimaryKey(Long id) throws FinderException, RemoteException;

    /*
     * Returns a Collection of ShippingHistory objects based on a the carrier id.
     *
     * @param String aCarrierId
     * @return Collection of ShippingHistory objects.
     *
     */
    public Collection findByCarrierId(String aCarrierId) throws FinderException, RemoteException;

    /*
     * Returns a Collection object based on a the shipment id field.
     *
     * @param Long aShipmentId
     * @return Collection of ShippingHistory objects.
     *
     */
    public Collection findByShipmentId(Long aShipmentId) throws FinderException, RemoteException;
        
    /*
     * Returns a Collection of all ShippingHistory objects.
     *
     * @return Collection of all ShippingHistory objects
     *
     */
    public Collection findAll() throws FinderException, RemoteException;


} // ShippingHistoryHome
