/*
 * B E A   S Y S T E M S
 *
 * P O R T A L
 *
 * Copyright (c) 2000-2002  BEA Systems, Inc.
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws
 * of the United States. The software contained on this media is proprietary
 * to and embodies the confidential technology of BEA Systems, Inc. The
 * possession or receipt of this information does not convey any right to disclose
 * its contents, reproduce it, or use,  or license the use, for manufacture or
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/b2bPortal/WEB-INF/src/examples/e2e/b2b/B2BTourGuide.java,v 1.2 2002/05/02 00:22:09 toneill Exp $
 */

package examples.e2e.b2b;

import examples.e2e.common.tourguide.TourGuide;
import examples.e2e.common.tourguide.TourGuideData;

import com.bea.portal.appflow.PortalRequest;
import com.bea.portal.appflow.PortalSession;
import com.bea.portal.render.servlets.jsp.PortalRenderHelper;
import com.bea.portal.appflow.PortalAppflowFactory;
import com.bea.portal.manager.PortalFactory;
import com.bea.portal.model.PageIdentifier;
import com.bea.portal.model.PortletIdentifier;
import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.exception.ProcessingException;
import com.bea.p13n.appflow.webflow.WebflowResponse;
import com.bea.p13n.util.debug.Debug;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;


/**
 * This class decides what content to display in the Tour Guide
 * portlet based on the state of the b2bPortal. It also mantains
 * information about each instance of the Tour Guide. There is 
 * one Tour Guide portlet instance per portal page.
 */
public class B2BTourGuide implements TourGuide, JspConstants {

    private static Debug debug =
            Debug.getInstance( B2BTourGuide.class );

    /** Stores instance data for each Tour Guide portlet in the app */
    protected Map instanceData;


    public B2BTourGuide() {

        /* Initialize the instance data for each portal page */
        instanceData = new HashMap();

        instanceData.put( B2BPortalPage.INVENTORY, new TourGuideData() );
        instanceData.put( B2BPortalPage.PURCHASING, new TourGuideData() );
        instanceData.put( B2BPortalPage.ORDER_HISTORY, new TourGuideData() );
    }


    public TourGuideData determineState( HttpServletRequest request )
    throws JspException {

        PipelineSession pSession = Helper.getPipelineSession( request );
        Object context = Helper.getWebflowContext( request );

        RequestState rState = null;
        SessionState sState = null;

        try {
            rState = Helper.getRequestState( pSession, context );
            sState = Helper.getSessionState( pSession );
        }
        catch ( ProcessingException ex ) {

            throw new JspException( ex.getMessage() );
        }

        String currentPage =
                PortalRenderHelper.getPageIdentifier( request ).getName();

        B2BPortalPage page = B2BPortalPage.getObject( currentPage );

        if ( page == null ) {

            throw new JspException( "Unknown portal page( " + currentPage + " )" );
        }

        TourGuideData data = getInstanceData( page );

        switch ( page.getInt() ) {
            
            case B2BPortalPage.I_INVENTORY: {

                    determineInventoryPageState( request, sState, page );
                    break;

                }

            case B2BPortalPage.I_PURCHASING: {

                    determinePurchasingPageState( request, sState, page );
                    break;
                }

            case B2BPortalPage.I_ORDER_HISTORY: {

                    determinePOPageState( request, sState, page );
                    break;
                }

            default: {

                    data.setState( TG_STATE_DEFAULT );
                    data.setCodeFile( TG_FILE_DEFAULT );
                    break;
                }
        }

        /* Return the Tour Guide instance data to the caller */
        return data;
    }

    public TourGuideData getInstanceData( String pageName ) {

        TourGuideData rtnVal = null;
        B2BPortalPage page = B2BPortalPage.getObject( pageName );

        if ( page == null ) {

            rtnVal = null;
        }
        else {

            rtnVal = getInstanceData( page ); 
        }

        return rtnVal;
    }

    public TourGuideData getInstanceData( B2BPortalPage page ) {

        return(TourGuideData)instanceData.get( page );
    }


    private void determineInventoryPageState( HttpServletRequest request,
            SessionState sState, B2BPortalPage page ) throws JspException {

        StateValue state = sState.getState();
        TourGuideData data = getInstanceData( page );

        switch ( state.getInt() ) {
            
            case StateValue.I_START: {

                    data.setState( TG_STATE_PRODUCT_INVENTORY );
                    data.setCodeFile( TG_FILE_PRODUCT_INVENTORY );
                    break;
                }

            case StateValue.I_PRODUCT_SELECTED: {

                    data.setState( TG_STATE_PART_INVENTORY );
                    data.setCodeFile( TG_FILE_PART_INVENTORY );
                    break;
                }

            default: {

                    data.setState( TG_STATE_DEFAULT );
                    data.setCodeFile( TG_FILE_DEFAULT );
                    break;
                }

        }

    }

    private void determinePurchasingPageState( HttpServletRequest request,
            SessionState sState, B2BPortalPage page ) throws JspException {

        StateValue state = sState.getState();
        TourGuideData data = getInstanceData( page );

        switch ( state.getInt() ) {
            
            case StateValue.I_START: {

                    data.setState( TG_STATE_PURCHASING );
                    data.setCodeFile( TG_FILE_SUMMARY );
                    break;
                }

            case StateValue.I_PART_SELECTED: {

                    data.setState( TG_STATE_QPA );
                    data.setCodeFile( TG_FILE_QPA );
                    break;
                }

            case StateValue.I_QPA_SENT: {

                    data.setState( TG_STATE_QPA );

                    // content varies depending on the availability of
                    // quotes.  Obtain the page being rendered for this
                    // portlet by the portal framework
                    data.setCodeFile( getPortletContent( request,
                            page, QUOTE_PORTLET_ID ) );
                    break;
                }

            default: {

                    data.setState( TG_STATE_DEFAULT );
                    data.setCodeFile( TG_FILE_DEFAULT );
                    break;
                }

        }

    }

    private void determinePOPageState( HttpServletRequest request,
            SessionState sState, B2BPortalPage page ) throws JspException {

        StateValue state = sState.getState();
        TourGuideData data = getInstanceData( page );

        switch ( state.getInt() ) {
            
            case StateValue.I_START: {

                    data.setState( TG_STATE_ORDER_HISTORY );
                    data.setCodeFile( TG_FILE_PO_STATUS_NORMAL );
                    break;
                }

            default: {

                    data.setState( TG_STATE_DEFAULT );
                    data.setCodeFile( TG_FILE_DEFAULT );
                    break;
                }

        }

    }

    private String getPortletContent( HttpServletRequest request,
            B2BPortalPage page, String portletName ) throws JspException {

        PortalRequest pRequest =
                PortalAppflowFactory.getPortalRequest( request );
        PortalSession pSession = pRequest.getPortalSession();

        // may throw JspException
        PageIdentifier pageId = PortalRenderHelper.getPageIdentifier( request );

        PortletIdentifier portletId = PortalFactory.createPortletIdentifier(
                pRequest.getPortalContext().getPortalIdentifier(),  portletName );

        WebflowResponse response = pSession.getContentForPortlet( pageId, portletId );
        String rtnVal = null;

        if ( response == null ) {

            rtnVal = null;
        }
        else {

            rtnVal = response.url();
        }

        if ( debug.ON ) {
            debug.out( "rtnVal(" + rtnVal + ")" );
        }

        return rtnVal;
    }

}
