
/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/b2bPortal/WEB-INF/src/examples/e2e/b2b/Helper.java,v 1.2 2002/05/02 00:22:09 toneill Exp $
 */

package examples.e2e.b2b;

import examples.e2e.b2b.RequestState;
import examples.e2e.b2b.SessionState;
import examples.e2e.b2b.StateFactory;

import com.bea.portal.appflow.PortalAppflowConstants;

import com.bea.p13n.util.debug.Debug;

import com.bea.p13n.appflow.common.internal.AppflowConstants;
import com.bea.p13n.appflow.webflow.InputProcessorSupport;
import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.exception.ProcessingException;

import java.text.NumberFormat;
import java.util.ResourceBundle;
import java.util.MissingResourceException;
import java.util.Locale;
import javax.servlet.http.HttpServletRequest;


public class Helper implements JspConstants {

    private static Debug debug =
            Debug.getInstance( Helper.class );

    public static final String STATE_KEY = "state";
    public static final String STATE_NAMESPACE = "e2e.b2b";

    public static final String CURRENCY_KEY = "currency";

    /**
     * <code>ResourceBundle</code> property for designating
     * the language for a <code>Locale</code>.  The <code>Locale</code>
     * is used for formatting text in the web application.
     *
     * The base file containing the resources is
     * <web app directory>/portlets/b2b.properties.
     */
    public static final String ISO_639 = "locale.iso-639";

    /**
     * <code>ResourceBundle</code> property for designating
     * the language for a <code>Locale</code>.  The <code>Locale</code>
     * is used for formatting text in the web application.
     *
     * The base file containing the resources is
     * <web app directory>/portlets/b2b.properties.
     */
    public static final String ISO_3166 = "locale.iso-3166";

    private static ResourceBundle bundle;
    private static Locale locale;


    private Helper() {
    } 


    /**
     * Obtains the <code>ResourceBundle</code> for this web application.
     *
     * @return the <code>ResourceBundle</code> for this web application.
     *
     * @throws <code>MissingResourceException</code> if the resource cannot
     * be obtained.
     *
     */
    public static ResourceBundle getBundle() throws MissingResourceException {

        // not synchronized, last one wins, OK because read-only.
        if ( bundle == null ) {

            // may throw runtime exception MissingResourceException
            bundle = ResourceBundle.getBundle( localizedPropertiesFile );
        }

        return bundle;
    }

    /**
     * Provides the currency string associatied with this portal (web application).
     * The value is obtained through a resource bunlde and is therefore localizable.
     * The base file containing the resources is
     * <web app directory>/portlets/b2b.properties.
     *
     * @return a string representing the currency for this portal.
     *
     * @throws <code>MissingResourceException</code> if the resource cannot
     * be obtained.
     *
     */
    public static String getCurrency() throws MissingResourceException {

        // may throw runtime exception MissingResourceException
        return getBundle().getString( CURRENCY_KEY );
    }

    /**
     * Provides the <code>Locale</code> object associatied with this
     * portal (web application).
     * The value is obtained through a resource bundle and is
     * therefore localizable. The base file containing the resources is
     * <web app directory>/portlets/b2b.properties.  Set the property keys
     * defined in this class by constants ISO_639 and ISO_3166 to
     * configure the <code>Locale</code> as necessary.
     *
     * @return <code>Locale</code> object for this portal.  If the properties
     * are not defined the resource file the default <code>Locale</code> for
     * the JVM will be returned.
     *
     * @throws <code>MissingResourceException</code> if the resource cannot
     * be obtained.
     *
     */
    public static Locale getLocale() {

        // not synchronized, last one wins, OK because read-only.
        if ( locale == null ) {

            String language = null;
            String country = null;

            try {

                // iso-639 code, language
                // may throw runtime exception MissingResourceException
                language = getBundle().getString( ISO_639 );

                // iso-3166 code, country
                // may throw runtime exception MissingResourceException
                country = getBundle().getString( ISO_3166 );
            }
            catch ( MissingResourceException ex ) {

                // ignore, logic below will handle producing a Locale
            }

            if ( language == null || language == EMPTY_STRING ) {

                locale = Locale.getDefault();
            }
            else {

                if ( country == null ) {

                    country = "";
                }

                locale = new Locale( language, country );
            }

            if ( debug.ON ) {
                debug.out( "language property( " + language + " )" );
                debug.out( "country property( " + country + " )" );
                debug.out( "locale( " + locale + " )" );
            }
        }

        return locale;
    }

    public static NumberFormat getCurrencyNumberFormat() {

        return NumberFormat.getCurrencyInstance( getLocale() );
    }

    public static PipelineSession getPipelineSession(
            HttpServletRequest request ) {

        return InputProcessorSupport.getPipelineSession( request );
    }

    public static Object getWebflowContext(
            HttpServletRequest request ) {

        // Get the request context from the request
        return request.getAttribute(
                AppflowConstants.REQUEST_CONTEXT_ATTRIBUTE_NAME );
    }

    public static RequestState getRequestState( PipelineSession session,
            Object requestContext ) throws ProcessingException {

        RequestState obj = null;

        try {
            obj = (RequestState) InputProcessorSupport.getRequestAttribute(
                    STATE_KEY, STATE_NAMESPACE,
                    requestContext, session );
        }
        catch ( ProcessingException pe ) {
            // TODO implement
        }

        if ( obj == null ) {
            obj = StateFactory.getRequestStateInstance();

            InputProcessorSupport.setRequestAttribute(
                    STATE_KEY, obj, STATE_NAMESPACE,
                    requestContext, session );
        }

        return obj;
    }

    public static SessionState getSessionState( PipelineSession session )
    throws ProcessingException {

        SessionState obj = null;

        try {
            obj = (SessionState) InputProcessorSupport.getSessionAttribute(
                    STATE_KEY, STATE_NAMESPACE, session );
        }
        catch ( ProcessingException pe ) {
            // TODO implement
        }

        if ( obj == null ) {
            obj = StateFactory.getSessionStateInstance();

            InputProcessorSupport.setSessionAttribute(
                    STATE_KEY, obj, STATE_NAMESPACE, session );
        }

        return obj;
    }

    public static String getPortletEvent( HttpServletRequest request ) {

        return request.getParameter(
                PortalAppflowConstants.PORTLET_WEBFLOW_EVENT_PARAMETER );
    }

    public static String getProductParameter( HttpServletRequest request ) {

        return request.getParameter( PRODUCT_PARAMETER );
    }

    public static String getPartParameter( HttpServletRequest request ) {

        return request.getParameter( PART_PARAMETER );
    }

    public static String getQuantityParameter( HttpServletRequest request ) {

        return request.getParameter( QTY_PARAMETER );
    }

    public static String getUnitPriceParameter( HttpServletRequest request ) {

        return request.getParameter( UNIT_PRICE_PARAMETER );
    }

    public static String getMonthParameter( HttpServletRequest request ) {

        return request.getParameter( MONTH_PARAM );
    }

    public static String getDayParameter( HttpServletRequest request ) {

        return request.getParameter( DAY_PARAM );
    }

    public static String getQuoteParameter( HttpServletRequest request ) {

        return request.getParameter( QUOTE_PARAM );
    }

}
