
/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/b2bPortal/WEB-INF/src/examples/e2e/b2b/LongValueValidator.java,v 1.2 2002/05/02 00:22:09 toneill Exp $
 */

package examples.e2e.b2b;

import com.bea.p13n.appflow.webflow.forms.Validator;
import com.bea.p13n.appflow.webflow.forms.ValidatedValues;
import com.bea.p13n.i18n.ValidationExceptionTextFormatter;

import com.bea.p13n.util.debug.Debug;


/**
 * Validate a numerical value from an HTML Form.  The
 * <code>String</code> value must represent a valid <code>long</code>
 * number.  An expression object restricts the numerical values that the
 * <code>long</code> value may be by the following expression:
 * expression.min() <
 * 
 * be a 
 */


/**
 The number must contain all 
 * digits [0-9], and be at least <b>minLength</b> digits but cannot 
 * be exceed <b>maxLength</b>, as set in the expression object.<br>
 * @param name name of the field in the request object
 * @param expression <code>MinMaxExpression</code> object with min max set
 * @return value of the form field, with the status, value, and message also
 * updated.
 */
public class LongValueValidator implements Validator {

    private static Debug debug =
            Debug.getInstance( LongValueValidator.class );

    public static final String REQUIRED =
            "a value is required for the quantity field";
    public static final String NOT_A_NUMBER =
            "the value supplied for the quantity field is not a number";
    public static final String BELOW_MIN =
            "the value supplied for the quantity field is below the minimum allowed of ";
    public static final String ABOVE_MAX =
            "the value supplied for the quantity field is above the maximum allowed of ";


    public LongValueValidator() {
    } 


    public String validate( ValidatedValues validatedValues, 
            String key, Object expressionObject, String message ) {

        long longVal = 0L;
        boolean done = false;
        String rtnVal = validatedValues.getValueAsString( key );
        rtnVal = rtnVal.trim();

        if ( debug.ON ) {
            debug.out( "key(" + key + ")" );
            debug.out( "value(" + rtnVal + ")" );
            debug.out( "message(" + message + ")" );
            debug.out( "expressionObject class(" +
                    expressionObject.getClass().getName() + ")" );
        }

        validatedValues.setValue( key, (Object) rtnVal );
        validatedValues.setStatus( key, STATUS_INVALID );

        // insure the rtnVal is not empty
        if ( rtnVal.length() <= 0 ) {

            // BAD ENTRY
            String msg = ( message == null ) ? 
                    REQUIRED : message;
            validatedValues.setMessage( key, msg );

            done = true;
        }

        if ( ! done ) {
            // Make sure there are only numbers
            try {

                longVal = Long.parseLong( rtnVal );
            }
            catch ( NumberFormatException e ) {

                // BAD ENTRY
                String msg = (message == null) ? 
                        NOT_A_NUMBER : message;
                validatedValues.setMessage(key, msg);

                done = true;
            }
        }

        if ( ! done ) {

            if ( expressionObject == null ) {

                // no min/max, must be a long and it is
                validatedValues.setStatus( key, STATUS_VALID );
            }
            else if ( expressionObject instanceof LongMinMaxExpression ) {

                // validate

                LongMinMaxExpression expression =
                        (LongMinMaxExpression) expressionObject;

                if ( debug.ON ) {
                    debug.out( "expression.min()(" + expression.min() + ")" );
                    debug.out( "expression.max()(" + expression.max() + ")" );
                }

                String msg = null;

                if ( longVal < expression.min() ) {

                    // BAD ENTRY
                    msg = ( message == null ) ?
                            ( BELOW_MIN + expression.min() ) : message;
                    done = true;
                }
                else if ( longVal > expression.max() ) {

                    // BAD ENTRY
                    msg = ( message == null ) ?
                            ( ABOVE_MAX + expression.max() ) : message;
                    done = true;
                }
                else {

                    validatedValues.setStatus( key, STATUS_VALID );
                }

                if ( done ) {

                    validatedValues.setMessage( key, msg );
                }
            }
            else {
                // PROGRAMMER ERROR
                // expression not an instance of LongMinMaxExpression

                throw new RuntimeException(
                        "expression not an instance of LongMinMaxExpression" );
            }

        }

        return rtnVal;
    }

}





