
/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 */

package examples.e2e.b2b;

import examples.e2e.b2b.CalendarFormData;

import java.io.Serializable;
import java.util.List;
import java.util.Calendar;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;


/**
 *
 * @author Copyright (c) 2000-2002  BEA Systems, Inc. All Rights Reserved.
 */
public interface RequestState extends Serializable {

    /**
     * Returns a <code>Calendar</code> for use during this request.  By using this
     * <code>Calendar</code> all processors and JSPs can use the same effective time
     * when evaluating logic or display output.
     */
    public Calendar calendar();

    /**
     * Returns a URL for obtaining the titlebar icon for the requesting portlet.
     * This method must be called during portlet rendering for the results to
     * be meaningful.
     */
    public String getIconUrl( HttpServletRequest request, SessionState sState )
    throws JspException;

    /**
     * Returns the product inventory to be displayed.
     *
     * @return a <code>List</code> of <code>Inventory</code> objects.
     */
    public List getProducts();

    /**
     * Sets the product inventory to be displayed.
     *
     * @param a <code>List</code> of <code>Inventory</code> objects.
     */
    public void setProducts( List products );

    /**
     * Returns the parts inventory to be displayed.
     *
     * @return a <code>List</code> of <code>Inventory</code> objects.
     */
    public List getParts();

    /**
     * Sets this parts inventory to be displayed.
     *
     * @param a <code>List</code> of <code>Inventory</code> objects.
     */
    public void setParts( List parts );

    /**
     * Returns the default quantity value to be displayed in the QPA form.
     */
    public long getDefaultQpaQty();

    /**
     * Sets the default quantity value to be displayed in the QPA form.
     */
    public void setDefaultQpaQty( long qty );

    /**
     * Returns the quotes to be displayed.
     *
     * @return a <code>List</code> of <code>PurchaseQuote</code> objects.
     */
    public List getQuotes();

    /**
     * Sets the quotes to be displayed.
     *
     * @param a <code>List</code> of <code>PurchaseQuote</code> objects.
     */
    public void setQuotes( List quotes );

    /**
     * Returns the purchase orders to be displayed
     *
     * @return a <code>List</code> of <code>PO</code> objects.
     */
    public List getPOs();

    /**
     * Sets the purchase orders to be displayed.
     *
     * @param a <code>List</code> of <code>PO</code> objects.
     */
    public void setPOs( List pos );

    /**
     * Returns the image URL for the process portlet.
     */
    public String getProcessImageURL();

    /**
     * Sets the image URL for the process portlet.
     *
     * @param the new state.
     */
    public void setProcessImageURL( String url );

    /**
     * Returns true if an error occured when validating form data.
     * Messages corresponding to the error(s) can be retrieved with
     * the <code>qpaFormDataErrorMsg()</code> method.
     *
     * @returns true if an error message has been set, false otherwise.
     * @see #qpaFormDataErrorMsg
     * @see #qpaFormDataErrorCount
     * @see #setQpaFormDataError
     */
    public boolean qpaFormDataError();

    /**
     * Returns a count of the number of errors that have
     * been recorded for the QPA form.
     *
     * @return zero if <code>qpaFormDataError()</code> returns false, otherwise the
     * the number of errors will be returned and will be equal to the size
     * of the <code>List</code> returned by <code>qpaFormDataErrorMsg()</code>.
     * @see #qpaFormDataError
     * @see #setQpaFormDataError
     * @see #qpaFormDataErrorMsg
     */
    public int qpaFormDataErrorCount();

    /**
     * If qpaFormDataError() returns true this method will return
     * a <code>List</code> of <code>String</code> messages associated
     * with the error(s).
     *
     * @return a <code>List</code> of <code>String</code> objects or null
     * if no errors have occured.
     * @see #qpaFormDataError
     * @see #qpaFormDataErrorCount
     * @see #setQpaFormDataError
     */
    public List qpaFormDataErrorMsg();

    /**
     * Use to indicate an error has occured.  The message will be added
     * to a list of messages and can be retrieved with the
     * <code>qpaFormDataErrorMsg()</code> call.  Use of this method will
     * cause a call to <code>qpaFormDataError()</code> to return true.
     *
     * @param a message indicating the error that occured.
     * @see #qpaFormDataErrorMsg
     * @see #qpaFormDataError
     * @see #qpaFormDataErrorCount
     */
    public void setQpaFormDataError( String message );

    public CalendarFormData qpaCalendarFormData();
    public CalendarFormData setQpaCalendarFormData( CalendarFormData data );

    /**
     * Returns true if an error occured when validating form data.
     * A message corresponding to the error(s) can be retrieved with
     * the <code>quoteFormDataErrorMsg()</code> method.
     *
     * @returns true if an error message has been set, false otherwise.
     * @see #setQuoteFormDataError
     * @see #quoteFormDataErrorMsg
     */
    public boolean quoteFormDataError();

    /**
     * If quoteFormDataError() returns true this method will return
     * a message associated with the error.
     *
     * @return a <code>String</code> corresponding to the error that occurred.
     * @see #quoteFormDataError
     * @see #setQuoteFormDataError
     */
    public String quoteFormDataErrorMsg();

    /**
     * Use to indicate an error has occured.  The message 
     * can be retrieved with a call to <code>quoteFormDataErrorMsg()</code>.
     * Use of this method will cause a call to
     * <code>quoteFormDataError()</code> to return true.
     *
     * @param a message indicating the error that occured.
     * @see #quoteFormDataErrorMsg
     * @see #quoteFormDataError
     */
    public void setQuoteFormDataError( String msg );

}
