
/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 */

package examples.e2e.b2b;

import examples.e2e.b2b.CalendarFormData;

import com.bea.portal.model.Portlet;
import com.bea.portal.model.PortletState;
import com.bea.portal.render.servlets.jsp.PortletRenderHelper;

import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.util.debug.Debug;

import java.text.SimpleDateFormat;
import java.util.List;
import java.util.LinkedList;
import java.util.Calendar;
import java.util.Collections;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;


/**
 * Represents a query for price and availability that is to be communicated
 * to supplier(s).
 *
 * @author Copyright (c) 2000-2002  BEA Systems, Inc. All Rights Reserved.
 */
public class RequestStateImpl implements RequestState {

    private static Debug debug =
            Debug.getInstance( RequestStateImpl.class );

    private static final SimpleDateFormat format = 
            new SimpleDateFormat( "hh:mm:ss:SSSaa dd MMM zzz yyyy" );

    public static final String NULL = "null";
    public static final String TITLE_PREFIX = "titlebar_";
    public static final String ON_SUFFIX = ".gif";
    public static final String OFF_SUFFIX = "_off.gif";

    private Calendar calendar;
    private CalendarFormData qpaCalendarFormData;
    private List products;
    private List parts;
    private long defaultQPAQty = -1;
    private List quotes;
    private List pos;
    private String processImage;
    private List qpaFormDataErrorMsg;
    private String quoteFormDataErrorMsg;


    public RequestStateImpl() {
    }


    /**
     * Returns a <code>Calendar</code> for use during this request.  By using this
     * <code>Calendar</code> all processors and JSPs can use the same effective time
     * when evaluating logic or display output.
     */
    public Calendar calendar() {

        if ( calendar == null ) {

            calendar = calendar.getInstance();
        }

        return calendar;
    }

    /**
     * Returns a URL for obtaining the titlebar icon for the requesting portlet.
     * This method must be called during portlet rendering for the results to
     * be meaningful.
     */
    public String getIconUrl( HttpServletRequest request, SessionState sState )
    throws JspException {

        PortletState pState = PortletRenderHelper.getPortletState( request );
        pState.getUrl( Portlet.URL_ICON );
        String portletName = pState.getPortletIdentifier().getName();
        String icon = pState.getUrl( Portlet.URL_ICON );
        
        boolean on = false;
        StateValue state = sState.getState();

        if ( debug.ON ) {
            debug.out( "state(" + state + ")" );
            debug.out( "portletName(" + portletName + ")" );
            debug.out( "icon(" + icon + ")" );
        }

        switch ( state.getInt() ) {
            
            // step 1
            case StateValue.I_START: {                                     

                    if ( JspConstants.PRODUCT_PORTLET_ID.equals( portletName ) ) {
                        on = true;
                    }
                    else {
                        on = false;
                    }

                    break;
                }

            // step 2
            case StateValue.I_PRODUCT_SELECTED: {                                     

                    if ( JspConstants.PART_PORTLET_ID.equals( portletName ) ) {
                        on = true;
                    }
                    else {
                        on = false;
                    }

                    break;
                }

            // step 3
            case StateValue.I_PART_SELECTED: {                                     

                    if ( JspConstants.QPA_PORTLET_ID.equals( portletName ) ) {
                        on = true;
                    }
                    else {
                        on = false;
                    }

                    break;
                }

            // step 4
            case StateValue.I_QPA_SENT: {                                     

                    if ( JspConstants.QUOTE_PORTLET_ID.equals( portletName ) ) {
                        on = true;
                    }
                    else {
                        on = false;
                    }

                    break;
                }

            // step 5
            case StateValue.I_CREATE_PO: {                                     

                    if ( JspConstants.PO_REVIEW_PORTLET_ID.equals( portletName ) ) {
                        on = true;
                    }
                    else {
                        on = false;
                    }

                    break;
                }
            
            default: {

                on = false;
                break;
            }

        }

        String rtnVal = TITLE_PREFIX + icon +
                ( on == true ? ON_SUFFIX : OFF_SUFFIX );

        if ( debug.ON ) {
            debug.out( "rtnVal(" + rtnVal + ")" );
        }
        
        return rtnVal;
    }

    /**
     * Returns the product inventory to be displayed.
     *
     * @return a <code>List</code> of <code>Inventory</code> objects.
     */
    public List getProducts() {

        return products;
    }

    /**
     * Sets the product inventory to be displayed.
     *
     * @param a <code>List</code> of <code>Inventory</code> objects.
     */
    public void setProducts( List products ) {

        this.products = products;
    }

    /**
     * Returns the parts inventory to be displayed.
     *
     * @return a <code>List</code> of <code>Inventory</code> objects.
     */
    public List getParts() {

        return parts;
    }

    /**
     * Sets this parts inventory to be displayed.
     *
     * @param a <code>List</code> of <code>Inventory</code> objects.
     */
    public void setParts( List parts ) {

        this.parts = parts;
    }

    /**
     * Returns the default quantity value to be displayed in the QPA form.
     */
    public long getDefaultQpaQty() {

        return defaultQPAQty;
    }

    /**
     * Sets the default quantity value to be displayed in the QPA form.
     */
    public void setDefaultQpaQty( long qty ) {

        this.defaultQPAQty = qty;
    }

    /**
     * Returns the quotes to be displayed.
     *
     * @return a <code>List</code> of <code>PurchaseQuote</code> objects.
     */
    public List getQuotes() {

        return quotes;
    }

    /**
     * Sets the quotes to be displayed.
     *
     * @param a <code>List</code> of <code>PurchaseQuote</code> objects.
     */
    public void setQuotes( List quotes ) {

        this.quotes = quotes;
    }

    /**
     * Returns the purchase orders to be displayed
     *
     * @return a <code>List</code> of <code>PO</code> objects.
     */
    public List getPOs() {

        return pos;
    }

    /**
     * Sets the purchase orders to be displayed.
     *
     * @param a <code>List</code> of <code>PO</code> objects.
     */
    public void setPOs( List pos ) {

        this.pos = pos;
    }

    /**
     * Returns the image URL for the process portlet.
     */
    public String getProcessImageURL() {

        return processImage;
    }

    /**
     * Sets the image URL for the process portlet.
     *
     * @param the new state.
     */
    public void setProcessImageURL( String url ) {

        processImage = url;
    }

    /**
     * Returns true if an error occured when validating form data.
     * Messages corresponding to the error(s) can be retrieved with
     * the <code>qpaFormDataErrorMsg()</code> method.
     *
     * @returns true if an error message has been set, false otherwise.
     * @see #qpaFormDataErrorMsg
     * @see #qpaFormDataErrorCount
     * @see #setQpaFormDataError
     */
    public boolean qpaFormDataError() {

        if ( qpaFormDataErrorMsg == null ) {

            return false;
        }

        return true;
    }

    /**
     * Returns a count of the number of errors that have
     * been recorded for the QPA form.
     *
     * @return zero if <code>qpaFormDataError()</code> returns false, otherwise the
     * the number of errors will be returned and will be equal to the size
     * of the <code>List</code> returned by <code>qpaFormDataErrorMsg()</code>.
     * @see #qpaFormDataError
     * @see #setQpaFormDataError
     * @see #qpaFormDataErrorMsg
     */
    public int qpaFormDataErrorCount() {

        int rtnVal = -1;

        if ( qpaFormDataErrorMsg == null ) {

            rtnVal = 0;
        }
        else {

            rtnVal = qpaFormDataErrorMsg.size();
        }

        return rtnVal;
    }

    /**
     * If qpaFormDataError() returns true this method will return
     * a <code>List</code> of <code>String</code> messages associated
     * with the error(s).
     *
     * @return a <code>List</code> of <code>String</code> objects or null
     * if no errors have occured.
     * @see #qpaFormDataError
     * @see #qpaFormDataErrorCount
     * @see #setQpaFormDataError
     */
    public List qpaFormDataErrorMsg() {

        return Collections.unmodifiableList( qpaFormDataErrorMsg );
    }

    /**
     * Use to indicate an error has occured.  The message will be added
     * to a list of messages and can be retrieved with the
     * <code>qpaFormDataErrorMsg()</code> call.  Use of this method will
     * cause a call to <code>qpaFormDataError()</code> to return true.
     *
     * @param a message indicating the error that occured.
     * @see #qpaFormDataErrorMsg
     * @see #qpaFormDataError
     * @see #qpaFormDataErrorCount
     */
    public void setQpaFormDataError( String message ) {

        if ( qpaFormDataErrorMsg == null ) {

            qpaFormDataErrorMsg = new LinkedList();
        }

        qpaFormDataErrorMsg.add( message );
    }

    public CalendarFormData qpaCalendarFormData() {

        return qpaCalendarFormData;
    }

    public CalendarFormData setQpaCalendarFormData( CalendarFormData data ) {

        qpaCalendarFormData = data;
        return qpaCalendarFormData;
    }

    /**
     * Returns true if an error occured when validating form data.
     * A message corresponding to the error(s) can be retrieved with
     * the <code>quoteFormDataErrorMsg()</code> method.
     *
     * @returns true if an error message has been set, false otherwise.
     * @see #setQuoteFormDataError
     * @see #quoteFormDataErrorMsg
     */
    public boolean quoteFormDataError() {

        if ( quoteFormDataErrorMsg == null ) {

            return false;
        }

        return true;
    }

    /**
     * If quoteFormDataError() returns true this method will return
     * a message associated with the error.
     *
     * @return a <code>String</code> corresponding to the error that occurred.
     * @see #quoteFormDataError
     * @see #setQuoteFormDataError
     */
    public String quoteFormDataErrorMsg() {

        return quoteFormDataErrorMsg;
    }
    
    /**
     * Use to indicate an error has occured.  The message 
     * can be retrieved with a call to <code>quoteFormDataErrorMsg()</code>.
     * Use of this method will cause a call to
     * <code>quoteFormDataError()</code> to return true.
     *
     * @param a message indicating the error that occured.
     * @see #quoteFormDataErrorMsg
     * @see #quoteFormDataError
     */
    public void setQuoteFormDataError( String msg ) {

        quoteFormDataErrorMsg = msg;
    }


    public String toString() {

        StringBuffer sb = new StringBuffer( 256 );

        sb.append( this.getClass().getName() + "[ " );
        sb.append( "products( " + Util.getString( products ) + " )" );
        sb.append( ", parts( " + Util.getString( parts ) + " )" );
        sb.append( ", defaultQPAQty( " + defaultQPAQty + " )" );
        sb.append( ", quotes( " + Util.getString( quotes ) + " )" );
        sb.append( ", POs( " + Util.getString( pos ) + " )" );
        sb.append( ", processImage( " + processImage + " )" );
        sb.append( ", qpaFormDataErrorMsg( " + Util.getString( qpaFormDataErrorMsg ) + " )" );

        sb.append( ", calendar( " +
                (( calendar == null ) ? NULL :
                ( format.format( calendar.getTime() ) )) + " )" );

        sb.append( " ]" );

        return sb.toString();
    }

}
