
/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 */

package examples.e2e.b2b;

import examples.e2e.common.Inventory;
import examples.e2e.common.QPA;
import examples.e2e.common.PurchaseQuote;

import java.io.Serializable;
import java.util.Calendar;


/**
 * Represents a query for price and availability that is to be communicated
 * to supplier(s).
 *
 * @author Copyright (c) 2000-2002  BEA Systems, Inc. All Rights Reserved.
 */
public interface SessionState extends Serializable {
        
    /**
     * Clears all values from the object
     */
    public void reset();

    /**
     * Returns the product selected.
     */
    public Inventory getProduct();

    /**
     * Sets this product as the selected product.
     *
     * @param the selected product.
     */
    public void setProduct( Inventory product );
    
    /**
     * Returns the part selected.
     */
    public Inventory getPart();
    
    /**
     * Sets this part as the selected part.
     *
     * @param the selected part.
     */
    public void setPart( Inventory part );
    
    /**
     * Returns the QPA.
     */
    public QPA getQpa();
    
    /**
     * Sets the QPA.
     *
     * @param the QPA for this session.
     */
    public void setQpa( QPA qpa );
    
    /**
     * Returns the PurchaseQuote.
     */
    public PurchaseQuote getQuote();
    
    /**
     * Returns the PurchaseQuote.
     *
     * @param the selected PurchaseQuote.
     */
    public void setQuote( PurchaseQuote quote );
    
    /**
     * Returns the current state for this session.
     */
    public StateValue getState();
    
    /**
     * Sets the current state for this session.
     *
     * @param the new state.
     */
    public void setState( StateValue state );
    
    public boolean quoteTimedOut( Calendar currentDate );

    public boolean poTimedOut( Calendar currentDate );

    public Calendar setQuoteTimeout( Calendar timeoutDate );
    
    public Calendar setPOTimeout( Calendar timeoutDate );

    /**
     * Sets the time at which the customer selected the part in
     * the part inventory portlet.  This value is used for display
     * purposes.
     *
     * @param a calendar representing the time the part was selected.
     */
    public Calendar setPartSelectedTime( Calendar calendar );
    
    /**
     * Retrieves the time at which the customer selected the part in
     * the part inventory portlet.  This value is used for display
     * purposes.
     */
    public Calendar partSelectedTime();

    /**
     * Sets the time at which the customer sent the qpa in
     * the price query portlet.  This value is used for display
     * purposes.
     *
     * @param a calendar.
     */
    public Calendar setQPASentTime( Calendar calendar );
    
    /**
     * Retrieves the time at which the customer sent the QPA in
     * the price query portlet.  This value is used for display
     * purposes.
     */
    public Calendar qpaSentTime();
    
    /**
     * Sets the time at which the customer selected a quote in
     * the price quote portlet.  This value is used for display
     * purposes.
     *
     * @param a calendar.
     */
    public Calendar setQuoteSelectedTime( Calendar calendar );
    
    /**
     * Retrieves the time at which the customer selected the quote in
     * the price quote portlet.  This value is used for display
     * purposes.
     */
    public Calendar quoteSelectedTime();

    /**
     * Sets the time at which the customer submitted the PO in
     * the PO review portlet.  This value is used for display
     * purposes.
     *
     * @param a calendar.
     */
    public Calendar setPOSubmittedTime( Calendar calendar );
    
    /**
     * Retrieves the time at which the customer submitted the PO in
     * the PO review portlet.  This value is used for display
     * purposes.
     */
    public Calendar poSubmittedTime();

}
