
/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 */

package examples.e2e.b2b;

import examples.e2e.common.Inventory;
import examples.e2e.common.QPA;
import examples.e2e.common.PurchaseQuote;

import com.bea.p13n.util.debug.Debug;

import java.text.SimpleDateFormat;
import java.util.Calendar;


/**
 * Represents a query for price and availability that is to be communicated
 * to supplier(s).
 *
 * @author Copyright (c) 2000-2002  BEA Systems, Inc. All Rights Reserved.
 */
public class SessionStateImpl implements SessionState {

    private static Debug debug =
            Debug.getInstance( SessionStateImpl.class );

    private static final SimpleDateFormat format = 
            new SimpleDateFormat( "hh:mm:ss:SSSaa dd MMM zzz yyyy" );

    public static final String NULL = "null";


    private Inventory product;
    private Inventory part;
    private QPA qpa;
    private PurchaseQuote quote;
    private StateValue state;
    private Calendar quoteTimeout;
    private Calendar poTimeout;
    private Calendar partSelectedTime;
    private Calendar qpaSentTime;
    private Calendar quoteSelectedTime;
    private Calendar poSubmittedTime;


    public SessionStateImpl( StateValue state ) {

        this.state = state;
    }     

    /**
     * Clears all values from the object
     */
    public void reset() {

        product = null;
        part = null;
        qpa = null;
        quote = null;
        state = null;
        quoteTimeout = null;
        poTimeout = null;
        partSelectedTime = null;
        qpaSentTime = null;
        quoteSelectedTime = null;
        poSubmittedTime = null;
    }


    /**
     * Returns the product selected.
     */
    public Inventory getProduct() {

        return product;
    }

    /**
     * Sets this product as the selected product.
     *
     * @param the selected product.
     */
    public void setProduct( Inventory product ) {

        this.product = product;
    }

    /**
     * Returns the part selected.
     */
    public Inventory getPart() {

        return part;
    }

    /**
     * Sets this part as the selected part.
     *
     * @param the selected part.
     */
    public void setPart( Inventory part ) {

        this.part = part;
    }

    /**
     * Returns the QPA.
     */
    public QPA getQpa() {

        return qpa;
    }

    /**
     * Sets the QPA.
     *
     * @param the QPA for this session.
     */
    public void setQpa( QPA qpa ) {

        this.qpa = qpa;
    }

    /**
     * Returns the PurchaseQuote.
     */
    public PurchaseQuote getQuote() {

        return quote;
    }

    /**
     * Returns the PurchaseQuote.
     *
     * @param the selected PurchaseQuote.
     */
    public void setQuote( PurchaseQuote quote ) {

        this.quote = quote;
    }

    /**
     * Returns the current state for this session.
     */
    public StateValue getState() {

        return state;
    }

    /**
     * Sets the current state for this session.
     *
     * @param the new state.
     */
    public void setState( StateValue state ) {

        this.state = state;
    }

    public boolean quoteTimedOut( Calendar currentDate ) {

        if ( quoteTimeout == null ) {

            return false;
        }
        return currentDate.after( quoteTimeout );
    }

    public boolean poTimedOut( Calendar currentDate ) {

        if ( poTimeout == null ) {

            return false;
        }
        return currentDate.after( poTimeout );
    }

    public Calendar setQuoteTimeout( Calendar timeoutDate ) {

        this.quoteTimeout = timeoutDate;
        return quoteTimeout;
    }

    public Calendar setPOTimeout( Calendar timeoutDate ) {

        this.poTimeout = timeoutDate;
        return poTimeout;
    }

    /**
     * Sets the time at which the customer selected the part in
     * the part inventory portlet.  This value is used for display
     * purposes.
     *
     * @param a calendar representing the time the part was selected.
     */
    public Calendar setPartSelectedTime( Calendar calendar ) {

        partSelectedTime = calendar;
        return partSelectedTime;
    }
    
    /**
     * Retrieves the time at which the customer selected the part in
     * the part inventory portlet.  This value is used for display
     * purposes.
     */
    public Calendar partSelectedTime() {

        return partSelectedTime;
    }

    /**
     * Sets the time at which the customer sent the qpa in
     * the price query portlet.  This value is used for display
     * purposes.
     *
     * @param a calendar.
     */
    public Calendar setQPASentTime( Calendar calendar ) {

        qpaSentTime = calendar;
        return qpaSentTime;
    }
    
    /**
     * Retrieves the time at which the customer sent the QPA in
     * the price query portlet.  This value is used for display
     * purposes.
     */
    public Calendar qpaSentTime() {

        return qpaSentTime;
    }
    
    /**
     * Sets the time at which the customer selected a quote in
     * the price quote portlet.  This value is used for display
     * purposes.
     *
     * @param a calendar.
     */
    public Calendar setQuoteSelectedTime( Calendar calendar ) {

        quoteSelectedTime = calendar;
        return quoteSelectedTime;
    }
    
    /**
     * Retrieves the time at which the customer selected the quote in
     * the price quote portlet.  This value is used for display
     * purposes.
     */
    public Calendar quoteSelectedTime() {

        return quoteSelectedTime;
    }

    /**
     * Sets the time at which the customer submitted the PO in
     * the PO review portlet.  This value is used for display
     * purposes.
     *
     * @param a calendar.
     */
    public Calendar setPOSubmittedTime( Calendar calendar ) {

        poSubmittedTime = calendar;
        return poSubmittedTime;
    }
    
    /**
     * Retrieves the time at which the customer submitted the PO in
     * the PO review portlet.  This value is used for display
     * purposes.
     */
    public Calendar poSubmittedTime() {

        return poSubmittedTime;
    }


    public String toString() {

        StringBuffer sb = new StringBuffer( 256 );

        sb.append( this.getClass().getName() + "[ " );
        sb.append( "product( " + product + " )" );
        sb.append( ", part( " + part + " )" );
        sb.append( ", QPA( " + qpa + " )" );
        sb.append( ", quote( " + quote + " )" );
        sb.append( ", state( " + state + " )" );
        sb.append( ", quoteTimeout( " +
                (( quoteTimeout == null ) ? NULL :
                ( format.format( quoteTimeout.getTime() ) )) + " )" );
        sb.append( ", poTimeout( " + 
                (( poTimeout == null ) ? NULL :
                ( format.format( poTimeout.getTime() ) )) + " )" );
        sb.append( " ]" );

        return sb.toString();
    }


}
