
/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 */

package examples.e2e.b2b;

import java.io.Serializable;


public final class StateValue implements Serializable {

    private int intValue;

    public static final int I_START = 1;
    public static final int I_PRODUCT_SELECTED = 2;
    public static final int I_PART_SELECTED = 3;
    public static final int I_QPA_SENT = 5;
    public static final int I_QUOTE_QUERY = 6;
    public static final int I_CREATE_PO = 7;
    public static final int I_PO_SUBMITTED = 9;

    public static final StateValue START =
            new StateValue( I_START );

    public static final StateValue PRODUCT_SELECTED =
            new StateValue( I_PRODUCT_SELECTED );

    public static final StateValue PART_SELECTED =
            new StateValue( I_PART_SELECTED );

    public static final StateValue QPA_SENT =
            new StateValue( I_QPA_SENT );

    public static final StateValue QUOTE_QUERY =
            new StateValue( I_QUOTE_QUERY );

    public static final StateValue CREATE_PO =
            new StateValue( I_CREATE_PO );

    public static final StateValue PO_SUBMITTED =
            new StateValue( I_PO_SUBMITTED );


    public static final String S_START = "START";
    public static final String S_PRODUCT_SELECTED = "PRODUCT_SELECTED";
    public static final String S_PART_SELECTED = "PART_SELECTED";
    public static final String S_QPA_SENT = "QPA_SENT";
    public static final String S_QUOTE_QUERY = "QUOTE_QUERY";
    public static final String S_CREATE_PO = "CREATE_PO";
    public static final String S_PO_SUBMITTED = "PO_SUBMITTED";


    /**
     */
    private StateValue( int intValue ) {
        init( intValue );
    }

    public String toString() {
        return getString();
    }

    public String getString() {
        switch ( intValue ) {
            case I_START: 
                { return S_START;}
            case I_PRODUCT_SELECTED: 
                { return S_PRODUCT_SELECTED;}
            case I_PART_SELECTED: 
                { return S_PART_SELECTED;}
            case I_QPA_SENT: 
                { return S_QPA_SENT;}
            case I_QUOTE_QUERY: 
                { return S_QUOTE_QUERY;}
            case I_CREATE_PO: 
                { return S_CREATE_PO;}
            case I_PO_SUBMITTED: 
                { return S_PO_SUBMITTED;}
        }

        // should never get here (as long as the constructor is private)
        throw new RuntimeException(
                "PROGRAMMER ERROR"  );
    }

    public int getInt() {
        return intValue;
    }

    public static StateValue getObject( String token ) {
        if ( token.equals( StateValue.S_START ) ) {
            return START;
        }
        if ( token.equals( StateValue.S_PRODUCT_SELECTED ) ) {
            return PRODUCT_SELECTED;
        }
        if ( token.equals( StateValue.S_PART_SELECTED ) ) {
            return PART_SELECTED;
        }
        if ( token.equals( StateValue.S_QPA_SENT ) ) {
            return QPA_SENT;
        }
        if ( token.equals( StateValue.S_QUOTE_QUERY ) ) {
            return QUOTE_QUERY;
        }
        if ( token.equals( StateValue.S_CREATE_PO ) ) {
            return CREATE_PO;
        }
        if ( token.equals( StateValue.S_PO_SUBMITTED ) ) {
            return PO_SUBMITTED;
        }
        else {
            throw new RuntimeException( "getObject(): invalid type string: '" +
                    token + "'"  );
        }
    }

    private void init( int intValue ) {
        this.intValue = intValue;
    }

    public boolean equals( Object object ) {
        if ( this == object ) {
            return true;
        }
        else if ( object == null || getClass() != object.getClass() ) {
            return false;
        }

        StateValue other = (StateValue) object;
        return this.intValue == other.intValue;
    }

    private int hashCode;
    private boolean hashCodeValid = false;
    private static final int HASHCODE_SEED = 1000003;

    public int hashCode() {
        
        if ( hashCodeValid == true ) {
            return hashCode;
        }

        computeHashCode();
        return hashCode;
    }
    private void computeHashCode() {
        
        hashCode = intValue * HASHCODE_SEED;
        hashCodeValid = true;
    }

}
