
/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/b2bPortal/WEB-INF/src/examples/e2e/b2b/processors/AbstractProcessor.java,v 1.2 2002/05/02 00:22:09 toneill Exp $
 */

package examples.e2e.b2b.processors;

import examples.e2e.b2b.JspConstants;
import examples.e2e.b2b.Helper;
import examples.e2e.b2b.RequestState;
import examples.e2e.b2b.SessionState;
import examples.e2e.b2b.StateValue;

import examples.e2e.common.QPA;
import examples.e2e.common.PurchaseQuote;
import examples.e2e.common.Inventory;
import examples.e2e.common.EjbHelper;
import examples.e2e.common.E2EProcessingException;
import examples.e2e.common.inventory.spi.InventoryProvider;
import examples.e2e.common.purchase.spi.PurchaseManager;

import com.bea.p13n.util.debug.Debug;
import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.exception.ProcessingException;

import com.bea.portal.appflow.PortalRequest;
import com.bea.portal.appflow.PortalSession;
import com.bea.portal.model.PortalState;
import com.bea.portal.model.PageState;
import com.bea.portal.appflow.PortalAppflowEvent;
import com.bea.portal.appflow.PortalAppflowConstants;
import com.bea.portal.appflow.processor.AbstractPortalProcessor;

import java.util.List;
import java.util.Calendar;
import java.util.Date;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import java.rmi.RemoteException;
import javax.ejb.CreateException;
import javax.naming.NamingException;


public abstract class AbstractProcessor extends AbstractPortalProcessor
        implements JspConstants {

    private static Debug debug =
            Debug.getInstance( AbstractProcessor.class );


    public AbstractProcessor() {
    } 


    /**
     * Processors extending this class must provide an implementation. This method
     * will be called if the processor is session insensitive or the session has not
     * timed-out.
     */
    public Object process(HttpServletRequest request,
            HttpServletResponse response,
            PortalRequest portalRequest,
            Object requestContext)
    throws ProcessingException {

        if ( debug.ON ) {
            debug.out( EMPTY_STRING );
        }

        if ( debug.ON ) {
            debug.out( "appflow event( " + getAppflowEvent( portalRequest ) + " )" );
            debug.out( "portlet event( " + getPortletEvent( request ) + " )" );
        }

        PipelineSession session = getPipelineSession( request );
        SessionState sState = getSessionState( session );
        RequestState rState = getRequestState( session, requestContext );

        StateValue state = sState.getState();

        if ( debug.ON ) {
            debug.out( "pre-execution state(" + state + ")" );
            //debug.out( "rState(" + rState + ")" );
            //debug.out( "sSstate(" + sState + ")" );
        }

        Object rtnVal = null;
        try {

            rtnVal = process( request, response, portalRequest, requestContext,
                    session, rState, sState );
        }
        catch ( ProcessingException e ) {

            if ( debug.ON ) {
                debug.out( "exception( " + e + " )" );
            }

            throw e;
        }
        catch ( RuntimeException re ) {

            if ( debug.ON ) {
                debug.out( "runtime exception( " + re + " )" );
            }

            throw re;
        }
        finally {

            if ( debug.ON ) {
                debug.out( "post-execution state(" + state + ")" );
                //debug.out( "rState(" + rState + ")" );
                //debug.out( "sSstate(" + sState + ")" );
            }
        }

        return rtnVal;
    }

    /**
     * Processors extending this class must provide an implementation. This method
     * will be called if the processor is session insensitive or the session has not
     * timed-out.
     */
    public abstract Object process(HttpServletRequest request,
            HttpServletResponse response,
            PortalRequest portalRequest,
            Object requestContext,
            PipelineSession session,
            RequestState requestState, 
            SessionState sessionState )
    throws ProcessingException;


    protected static RequestState getRequestState( PipelineSession session,
            Object requestContext ) throws ProcessingException {

        return Helper.getRequestState( session, requestContext );
    }

    protected static SessionState getSessionState( PipelineSession session )
    throws ProcessingException {

        return Helper.getSessionState( session );
    }


    protected static PipelineSession getPipelineSession(
            HttpServletRequest request ) {

        return Helper.getPipelineSession( request );
    }

    protected static PortalAppflowEvent getAppflowEvent( PortalRequest pRequest ) {

        PortalAppflowEvent event = pRequest.getPortalAppflowEvent();
        return event;

    }

    protected static String getPortalEvent( PortalRequest pRequest ) {

        return pRequest.getPortalAppflowEvent().getEvent();
    }

    protected static String getPortletEvent( HttpServletRequest request ) {

        return Helper.getPortletEvent( request );
    }

    protected static boolean isPortletMaximized( PortalRequest portalRequest ) {

        PortalSession portalSession = portalRequest.getPortalSession();
        PortalState portalState = portalRequest.getPortalState();
        PageState pageState = portalState.getCurrentPageState();

        boolean rtnVal = portalSession.isPortletMaximized(
                pageState.getPageIdentifier() );

        if ( debug.ON ) {
            debug.out( "rtnVal(" + rtnVal + ")" );
        }

        return rtnVal;
    }

    public static InventoryProvider getInventoryProvider()
    throws RemoteException, CreateException, NamingException {

        return EjbHelper.getInventoryProvider();
    }

    public static PurchaseManager getPurchaseManager()
    throws RemoteException, CreateException, NamingException {

        return EjbHelper.getPurchaseManager();
    }


    protected static List getProducts( List idList ) throws E2EProcessingException {

        List products = null;
        InventoryProvider invBean = null;

        try {
            invBean = EjbHelper.getInventoryProvider();
        }
        catch ( Exception ex ) {

            if ( debug.ON ) {
                debug.out( "ex(" + ex + ")" );
            }

            throw new E2EProcessingException(
                    "failed getting inventory bean", ex );
        }

        try {
            products = invBean.getProductInventory( idList );
        }
        catch ( Exception ex ) {

            if ( debug.ON ) {
                debug.out( "ex(" + ex + ")" );
            }

            throw new E2EProcessingException(
                    "failed calling inventory bean", ex );
        }

        return products;
    }

    protected static List getParts( Inventory product )
    throws E2EProcessingException {

        List parts = null;
        InventoryProvider invBean = null;

        try {
            invBean = EjbHelper.getInventoryProvider();
        }
        catch ( Exception ex ) {

            if ( debug.ON ) {
                debug.out( "ex(" + ex + ")" );
            }

            throw new E2EProcessingException(
                    "failed getting inventory bean", ex );
        }

        try {
            parts = invBean.getProductPartInventory( product.id() );
        }
        catch ( Exception ex ) {

            if ( debug.ON ) {
                debug.out( "ex(" + ex + ")" );
            }

            throw new E2EProcessingException(
                    "failed calling inventory bean", ex );
        }

        return parts;
    }

    protected static void submitQPA( QPA qpa )
    throws E2EProcessingException {

        PurchaseManager bean = null;

        try {
            bean = EjbHelper.getPurchaseManager();
        }
        catch ( Exception ex ) {

            if ( debug.ON ) {
                debug.out( "ex(" + ex + ")" );
            }

            throw new E2EProcessingException(
                    "failed getting purchase bean", ex );
        }

        try {
            bean.queueQPA( qpa );
        }
        catch ( Exception ex ) {

            if ( debug.ON ) {
                debug.out( "ex(" + ex + ")" );
            }

            throw new E2EProcessingException(
                    "failed calling purchase bean", ex );
        }

        return;
    }

    protected static void submitPO( PurchaseQuote quote )
    throws E2EProcessingException {

        PurchaseManager bean = null;

        try {
            bean = EjbHelper.getPurchaseManager();
        }
        catch ( Exception ex ) {

            if ( debug.ON ) {
                debug.out( "ex(" + ex + ")" );
            }

            throw new E2EProcessingException(
                    "failed getting purchase bean", ex );
        }

        try {
            bean.queuePORequest( quote );
        }
        catch ( Exception ex ) {

            if ( debug.ON ) {
                debug.out( "ex(" + ex + ")" );
            }

            throw new E2EProcessingException(
                    "failed calling purchase bean", ex );
        }

        return;
    }


    protected Calendar getTimeoutCalendar( Calendar currentTime, long timeoutSeconds ) {

        Calendar calendar = (Calendar) currentTime.clone();

        Date date = calendar.getTime();
        long time = date.getTime();

        time += (timeoutSeconds * 1000);

        date.setTime( time );
        calendar.setTime( date );
        calendar.getTime();

        return calendar;
    }

}

