
/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/b2bPortal/WEB-INF/src/examples/e2e/b2b/processors/POStatusProcessor.java,v 1.2 2002/05/02 00:22:09 toneill Exp $
 */

package examples.e2e.b2b.processors;

import examples.e2e.b2b.Constants;
import examples.e2e.b2b.RequestState;
import examples.e2e.b2b.SessionState;
import examples.e2e.b2b.StateValue;

import examples.e2e.common.PurchaseQuote;
import examples.e2e.common.PO;
import examples.e2e.common.BuyerFactory;
import examples.e2e.common.ParsingException;

import com.bea.p13n.util.debug.Debug;
import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.exception.ProcessingException;

import com.bea.portal.appflow.PortalRequest;
import com.bea.portal.appflow.PortalAppflowConstants;

import java.util.List;
import java.util.Calendar;
import java.util.ArrayList;
import java.util.Iterator;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.xml.sax.InputSource;


public class POStatusProcessor extends AbstractProcessor {

    private static Debug debug =
            Debug.getInstance( POStatusProcessor.class );

    public static final long TIMEOUT_SECONDS = 300;

    // override the timeout by setting the system property
    private static final String TIMEOUT_PROP =
            "samples.e2e.b2b.po.timeout";
    private static long timeoutSeconds = TIMEOUT_SECONDS;

    // if property is true canned data will be provided by test factory
    private static final String TEST_PROP =
            "samples.e2e.b2b.po.submit.useTestImpl";
    private static boolean useTestImpl;
    // if property is true canned data will be provided by test factory
    private static final String XML_TEST_PROP =
            "samples.e2e.b2b.po.xml.useTestImpl";
    private static boolean useXMLTestImpl;

    // Testing only
    private static final String DELETE_PROP =
            "samples.e2e.b2b.doNotDelete";
    private static boolean doNotDelete;

    static {
        try {
            doNotDelete = Boolean.getBoolean( DELETE_PROP );

        }
        catch ( Exception e ) {
            debug.out( "unable to get system property( " +
                    DELETE_PROP + " )" );
        }

        try {
            timeoutSeconds = Long.getLong( TIMEOUT_PROP,
                    TIMEOUT_SECONDS ).longValue();

            if ( debug.ON ) {
                debug.out( "timeoutSeconds(" + timeoutSeconds + ")" );
            }
        }
        catch ( Exception e ) {
            debug.out( "timeoutSeconds(" + timeoutSeconds + ")", e );
        }

        try {
            useTestImpl = Boolean.getBoolean( TEST_PROP );
            if ( debug.ON ) {
                debug.out( "useTestImpl(" + useTestImpl + ")" );
            }
        }
        catch ( Exception e ) {
            debug.out( "useTestImpl(" + useTestImpl + ")", e );
        }

        try {
            useXMLTestImpl = Boolean.getBoolean( XML_TEST_PROP );
            if ( debug.ON ) {
                debug.out( "useXMLTestImpl(" + useXMLTestImpl + ")" );
            }
        }
        catch ( Exception e ) {
            debug.out( "useXMLTestImpl(" + useXMLTestImpl + ")", e );
        }
    }


    public POStatusProcessor() {
    } 


    /**
     * Processors extending this class must provide an implementation. This method
     * will be called if the processor is session insensitive or the session has not
     * timed-out.
     */
    public Object process(HttpServletRequest request,
            HttpServletResponse response,
            PortalRequest portalRequest,
            Object requestContext,
            PipelineSession session,
            RequestState rState,
            SessionState sState )
    throws ProcessingException {

        String rtnVal = null;

        if ( debug.ON ) {
            debug.out( EMPTY_STRING );
            debug.out( "timeoutSeconds(" + timeoutSeconds + ")" );
            debug.out( "useTestImpl(" + useTestImpl + ")" );
            debug.out( "useXMLTestImpl(" + useXMLTestImpl + ")" );
        }

        // actual event for this portlet
        // one of "submitPO", "bea.portal.framework.internal.refresh" or "null"
        String event = getPortletEvent( request );

        String portalEvent = getPortalEvent( portalRequest );

        // obtain POs and add to request state
        getPOs( rState );

        StateValue state = sState.getState();

        // submitPO event
        if ( EVENT_PO.equals( event ) ) {

            PurchaseQuote quote = sState.getQuote();

            if ( quote == null ) {

                throw new IllegalStateException(
                        "quote( " + quote + " )" );
            }

            // submit PO
            if ( useTestImpl ) {

                // do nothing
                sState.setState( StateValue.PO_SUBMITTED );
            }
            else {

                // remove XML
                if ( ! doNotDelete ) {

                    if ( debug.ON ) {
                        debug.out( "deleting XML files" );
                    }

                    // may throw a RuntimeException
                    BuyerFactory.deletePoXML();
                }

                // may throw E2EProcessingException
                submitPO( quote );
                sState.setState( StateValue.PO_SUBMITTED );
            }

            // set PO timeout time
            sState.setPOTimeout(
                    getTimeoutCalendar( rState.calendar(), timeoutSeconds ) );
            
            sState.setPOSubmittedTime( rState.calendar() );

            if ( isPortletMaximized( portalRequest ) ) {

                rtnVal = MAXIMIZED;
            }
            else {

                rtnVal = DISPLAY;
            }
        }

        // refresh
        else {

            if ( isPortletMaximized( portalRequest ) ) {

                rtnVal = MAXIMIZED;
            }
            else {

                rtnVal = DISPLAY;
            }
        }

        if ( debug.ON ) {
            debug.out( "rtnVal(" + rtnVal + ")" );
        }

        return rtnVal;
    }


    public static void getPOs( RequestState rState ) throws ParsingException {

        // get POs to display
        InputSource source = null;
        List staticSrcs = null;

        if ( useXMLTestImpl ) {

            // do nothing, use null soruce
        }
        else {

            source = BuyerFactory.getPOSource();

            if ( source == null ) {

                if ( debug.ON ) {
                    debug.out( "source(" + source + ")" );
                }
            }

            staticSrcs = BuyerFactory.getStaticPOSources();

            if ( staticSrcs == null ) {
                if ( debug.ON ) {
                    debug.out( "staticSrcs(" + staticSrcs + ")" );
                }

            }
        }

        // may throw ParsingException
        List pos = aggregatePOs( BuyerFactory.parsePO( source ),
                BuyerFactory.parsePOs( staticSrcs ) );

        if ( pos == null || pos.size() == 0 ) {

            if ( debug.ON ) {
                debug.out( "pos(" + pos + ")" );
            }
        }

        rState.setPOs( pos );
    }

    public static List aggregatePOs( PO po, List b ) {

        List rtnVal = new ArrayList( ( po == null ? 0 : 1 ) +
                ( b == null ? 0 : b.size() ) );

        if ( po != null ) {

            rtnVal.add( po );
        }

        if ( b != null ) {

            Iterator it = b.iterator();

            while ( it.hasNext() ) {

                rtnVal.add( it.next() );
            }

        }

        return rtnVal;
    }
}

