
/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/b2bPortal/WEB-INF/src/examples/e2e/b2b/processors/PartInventory.java,v 1.2 2002/05/02 00:22:09 toneill Exp $
 */

package examples.e2e.b2b.processors;

import examples.e2e.b2b.RequestState;
import examples.e2e.b2b.SessionState;
import examples.e2e.b2b.StateValue;
import examples.e2e.b2b.Util;

import examples.e2e.common.Inventory;
import examples.e2e.common.TestInventoryFactory;
import examples.e2e.common.E2EProcessingException;
import examples.e2e.common.IllegalStateException;

import com.bea.p13n.util.debug.Debug;
import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.exception.ProcessingException;

import com.bea.portal.appflow.PortalRequest;

import java.util.List;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


// external imports

public class PartInventory extends AbstractProcessor {

    private static Debug debug =
            Debug.getInstance( PartInventory.class );

    // if property is true canned data will be provided by test factory
    private static final String TEST_PROP =
            "samples.e2e.b2b.inventory.part.useTestImpl";
    private static boolean useTestImpl = false;

    static {
        try {
            useTestImpl = Boolean.getBoolean( TEST_PROP );
            if ( debug.ON ) {
                debug.out( "useTestImpl(" + useTestImpl + ")" );
            }
        }
        catch ( Exception e ) {
            debug.out( "useTestImpl(" + useTestImpl + ")", e );
        }
    }


    public PartInventory() {
    } 


    /**
     * Processors extending this class must provide an implementation. This method
     * will be called if the processor is session insensitive or the session has not
     * timed-out.
     */
    public Object process(HttpServletRequest request,
            HttpServletResponse response,
            PortalRequest portalRequest,
            Object requestContext,
            PipelineSession session,
            RequestState rState,
            SessionState sState )
    throws ProcessingException {

        String rtnVal = null;

        if ( debug.ON ) {
            debug.out( EMPTY_STRING );
            debug.out( "useTestImpl(" + useTestImpl + ")" );
        }

        StateValue state = sState.getState();

        // actual event for this portlet
        // one "bea.portal.framework.internal.refresh" or "null"
        String event = getPortletEvent( request );


        if ( state.equals( StateValue.START ) ) {

            // no product selected
            rtnVal = NOT_SELECTED;
        }
        else {

            // obtain the parts for the selected product
            List parts = null;

            // list of product ids
            Inventory product = sState.getProduct();

            if ( product == null ) {

                throw new IllegalStateException( "product( " + product + " )" );
            }

            // test implementation
            if ( useTestImpl ) {

                parts = TestInventoryFactory.createPart( product.id() );
            }
            else {

                // may throw E2EProcessingException
                parts = getParts( product );
            }

            if ( parts == null || parts.size() == 0 ) {

                // LOG

                // warning
                if ( debug.ON ) {
                    debug.out( "parts(" + Util.getString( parts ) + ")" );
                }
            }

            rState.setParts( parts );
            rtnVal = DISPLAY;

        }

        if ( debug.ON ) {
            debug.out( "rtnVal(" + rtnVal + ")" );
        }

        return rtnVal;
    }

}
