
/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/b2bPortal/WEB-INF/src/examples/e2e/b2b/processors/ProductInventory.java,v 1.2 2002/05/02 00:22:09 toneill Exp $
 */

package examples.e2e.b2b.processors;

import examples.e2e.b2b.Helper;
import examples.e2e.b2b.Util;
import examples.e2e.b2b.RequestState;
import examples.e2e.b2b.SessionState;
import examples.e2e.b2b.StateValue;

import examples.e2e.common.Inventory;
import examples.e2e.common.TestInventoryFactory;
import examples.e2e.common.BuyerFactory;
import examples.e2e.common.E2EProcessingException;
import examples.e2e.common.IllegalStateException;

import com.bea.p13n.util.debug.Debug;
import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.exception.ProcessingException;

import com.bea.portal.appflow.PortalRequest;
import com.bea.portal.appflow.PortalAppflowEvent;

import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


// external imports

public class ProductInventory extends AbstractProcessor {

    private static Debug debug =
            Debug.getInstance( ProductInventory.class );

    // if property is true canned data will be provided by test factory
    private static final String TEST_PROP =
            "samples.e2e.b2b.inventory.product.useTestImpl";
    private static boolean useTestImpl;

    private static List idList = new ArrayList();
    static {
        idList.add( "pix1000" );
        idList.add( "pix3000" );
        idList.add( "pix5000" );
    }

    static {
        try {
            useTestImpl = Boolean.getBoolean( TEST_PROP );
            if ( debug.ON ) {
                debug.out( "useTestImpl(" + useTestImpl + ")" );
            }
        }
        catch ( Exception e ) {
            debug.out( "useTestImpl(" + useTestImpl + ")", e );
        }
    }

    // Testing only
    private static final String DELETE_PROP =
            "samples.e2e.b2b.doNotDelete";
    private static boolean doNotDelete;

    static {
        try {
            doNotDelete = Boolean.getBoolean( DELETE_PROP );

        }
        catch ( Exception e ) {
            debug.out( "unable to get system property( " +
                    DELETE_PROP + " )" );
        }
    }


    public ProductInventory() {
    } 


    /**
     * Processors extending this class must provide an implementation. This method
     * will be called if the processor is session insensitive or the session has not
     * timed-out.
     */
    public Object process(HttpServletRequest request,
            HttpServletResponse response,
            PortalRequest portalRequest,
            Object requestContext,
            PipelineSession session,
            RequestState rState,
            SessionState sState )
    throws ProcessingException {

        String rtnVal = null;

        if ( debug.ON ) {
            debug.out( EMPTY_STRING );
            debug.out( "useTestImpl(" + useTestImpl + ")" );
            debug.out( "doNotDelete( " + doNotDelete + " )" );
        }


        // actual event for this portlet
        // one of "selectProduct" or "bea.portal.framework.internal.refresh"
        // "reset" or "null"
        String event = getPortletEvent( request );


        // always get a list of products
        List products = null;

        // test implementation
        if ( useTestImpl ) {

            products = TestInventoryFactory.createProduct( idList );
        }
        else {

            // may throw E2EProcessingException
            products = getProducts( idList );
        }

        if ( products == null || products.size() == 0 ) {

            // inventory problem
            // warning
            if ( debug.ON ) {
                debug.out( "products(" + Util.getString( products ) + ")" );
            }
        }

        if ( debug.ON ) {
            debug.out( "products(" + Util.getString( products ) + ")" );
        }

        rState.setProducts( products );


        // product selection event
        if ( EVENT_PRODUCT.equals( event ) ) {

            // a product has been selected
            String productID = Helper.getProductParameter( request );

            if ( productID == null || productID.length() == 0 ) {

                // error, no product ID specified in the http request
                if ( debug.ON ) {
                    debug.out( "productID(" + productID + ")" );
                }

                throw new IllegalStateException( "productID(" + productID + ")" );
            }

            // find the product selected
            Iterator it = products.iterator();
            Inventory product = null;
            while ( it.hasNext() ) {

                product = (Inventory) it.next();
                if ( productID.equals( product.id() ) ) {

                    // found it
                    break;
                }

                product = null;
            }

            if ( product == null ) {

                // bogus href link ??
                if ( debug.ON ) {
                    debug.out( "product(" + product + ")" );
                }

                throw new IllegalStateException(
                        "product not found for productID(" + productID + ")" );
            }

            // place the Inventory object in the session state
            sState.setProduct( product );
            // change the current state of the system
            sState.setState( StateValue.PRODUCT_SELECTED );

            rtnVal = DISPLAY;
        }

        // bea.portal.framework.internal.refresh, reset or null event
        else {

            // reset the state machine if a reset event is encountered
            if ( EVENT_RESET.equals( event ) ) {

                sState.reset();
                sState.setState( StateValue.START );

                // remove the XML for another run thru the purchasing process
                if ( ! doNotDelete ) {

                    if ( debug.ON ) {
                        debug.out( "deleting XML files" );
                    }

                    // may throw RuntimeException
                    BuyerFactory.deleteXML();
                }
            }

            rtnVal = DISPLAY;
        }


        if ( debug.ON ) {
            debug.out( "rtnVal(" + rtnVal + ")" );
        }

        return rtnVal;
    }

}
