
/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/b2bPortal/WEB-INF/src/examples/e2e/b2b/processors/QpaProcessor.java,v 1.2 2002/05/02 00:22:09 toneill Exp $
 */

package examples.e2e.b2b.processors;

import examples.e2e.b2b.Helper;
import examples.e2e.b2b.Util;
import examples.e2e.b2b.Constants;
import examples.e2e.b2b.LongMinMaxExpression;
import examples.e2e.b2b.LongValueValidator;
import examples.e2e.b2b.DoubleMinMaxExpression;
import examples.e2e.b2b.DoubleValueValidator;
import examples.e2e.b2b.RequestState;
import examples.e2e.b2b.SessionState;
import examples.e2e.b2b.StateValue;
import examples.e2e.b2b.CalendarFormData;

import examples.e2e.common.TestInventoryFactory;
import examples.e2e.common.Inventory;
import examples.e2e.common.QPA;
import examples.e2e.common.BuyerFactory;
import examples.e2e.common.E2EProcessingException;

import com.bea.p13n.util.debug.Debug;
import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.exception.ProcessingException;
import com.bea.commerce.ebusiness.price.quote.ImmutableMoney;
import com.bea.commerce.ebusiness.price.quote.Money;

import com.bea.p13n.appflow.webflow.forms.WebflowFormConstants;
import com.bea.p13n.appflow.webflow.forms.ValidatedValues;
import com.bea.p13n.appflow.webflow.forms.ValidatedValuesFactory;
import com.bea.p13n.appflow.webflow.forms.ValidatedForm;
import com.bea.p13n.appflow.webflow.forms.ValidatedFormFactory;
import com.bea.p13n.appflow.webflow.forms.MissingFormFieldException;
import com.bea.p13n.appflow.webflow.forms.InvalidFormDataException;
import com.bea.p13n.appflow.webflow.forms.MinMaxExpression;

import com.bea.portal.appflow.PortalRequest;
import com.bea.portal.appflow.PortalAppflowEvent;

import java.util.List;
import java.util.Calendar;
import java.util.Date;
import java.util.Iterator;
import java.rmi.server.UID;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


public class QpaProcessor extends AbstractProcessor {

    private static Debug debug =
            Debug.getInstance( QpaProcessor.class );

    public static final long TIMEOUT_SECONDS = 300;

    // increment to generate QPA IDs
    private static short qpaId = 0;

    // override the timeout by setting the system property
    private static final String TIMEOUT_PROP =
            "samples.e2e.b2b.qpa.timeout";
    private static long timeoutSeconds = TIMEOUT_SECONDS;

    // if property is true canned data will be provided by test factory
    private static final String INV_TEST_PROP =
            "samples.e2e.b2b.qpa.inventory.useTestImpl";
    private static boolean useInvTestImpl;
    // if property is true canned data will be provided by test factory
    private static final String EJB_TEST_PROP =
            "samples.e2e.b2b.qpa.ejb.useTestImpl";
    private static boolean useEjbTestImpl;

    // Testing only
    private static final String DELETE_PROP =
            "samples.e2e.b2b.doNotDelete";
    private static boolean doNotDelete;

    static {
        try {
            doNotDelete = Boolean.getBoolean( DELETE_PROP );

        }
        catch ( Exception e ) {
            debug.out( "unable to get system property( " +
                    DELETE_PROP + " )" );
        }

        try {
            timeoutSeconds = Long.getLong( TIMEOUT_PROP,
                    TIMEOUT_SECONDS ).longValue();

            if ( debug.ON ) {
                debug.out( "timeoutSeconds(" + timeoutSeconds + ")" );
            }
        }
        catch ( Exception e ) {
            debug.out( "timeoutSeconds(" + timeoutSeconds + ")", e );
        }

        try {
            useInvTestImpl = Boolean.getBoolean( INV_TEST_PROP );
            if ( debug.ON ) {
                debug.out( "useInvTestImpl(" + useInvTestImpl + ")" );
            }
        }
        catch ( Exception e ) {
            debug.out( "useInvTestImpl(" + useInvTestImpl + ")", e );
        }

        try {
            useEjbTestImpl = Boolean.getBoolean( EJB_TEST_PROP );
            if ( debug.ON ) {
                debug.out( "useEjbTestImpl(" + useEjbTestImpl + ")" );
            }
        }
        catch ( Exception e ) {
            debug.out( "useEjbTestImpl(" + useEjbTestImpl + ")", e );
        }
    }


    public QpaProcessor() {
    } 


    /**
     * Processors extending this class must provide an implementation. This method
     * will be called if the processor is session insensitive or the session has not
     * timed-out.
     */
    public Object process(HttpServletRequest request,
            HttpServletResponse response,
            PortalRequest portalRequest,
            Object requestContext,
            PipelineSession session,
            RequestState rState,
            SessionState sState )
    throws ProcessingException, InvalidFormDataException {

        String rtnVal = null;

        if ( debug.ON ) {
            debug.out( EMPTY_STRING );
            debug.out( "timeoutSeconds(" + timeoutSeconds + ")" );
            debug.out( "useInvTestImpl(" + useInvTestImpl + ")" );
            debug.out( "useEjbTestImpl(" + useEjbTestImpl + ")" );
        }

        List parts = null;
        // actual event for this portlet
        // one of "selectPart", "submitQPA",
        // or "bea.portal.framework.internal.refresh" or "null"
        String event = getPortletEvent( request );

        // part selected event
        if ( EVENT_PART.equals( event ) ) {

            // a product has been selected
            String partID = Helper.getPartParameter( request );
            Inventory product = sState.getProduct();

            // insure required info exists
            if ( product == null || partID == null || partID.length() == 0 ) {

                // error, no product  set in the session or no
                // part specified in the request
                if ( debug.ON ) {
                    debug.out( "product(" + product + ")" );
                    debug.out( "partID(" + partID + ")" );
                }

                throw new IllegalStateException( "product(" + product +
                        "), partID(" + partID + ")" );
            }

            // test implementation
            if ( useInvTestImpl ) {

                parts = TestInventoryFactory.createPart( product.id() );
            }
            else {

                // may throw E2EProcessingExcpetion
                parts = getParts( product );
            }

            if ( parts == null || parts.size() == 0 ) {

                // inventory problem
                // warning
                if ( debug.ON ) {
                    debug.out( "parts(" + Util.getString( parts ) + ")" );
                }
            }

            // find the part selected
            Iterator it = parts.iterator();
            Inventory part = null;

            while ( it.hasNext() ) {

                part = (Inventory) it.next();
                if ( partID.equals( part.id() ) ) {

                    // found it
                    break;
                }

                part = null;
            }

            if ( part == null ) {

                // bogus href link ??
                if ( debug.ON ) {
                    debug.out( "part(" + part + ")" );
                }

                throw new IllegalStateException(
                        "part not found for partID(" + partID + ")" );
            }

            // place the Inventory object in the session state
            sState.setPart( part );
            // change the current state of the system
            sState.setState( StateValue.PART_SELECTED );
            // capture the current time to indicate when the part was selected
            sState.setPartSelectedTime( rState.calendar() );

            rtnVal = DISPLAY;
        }

        // submit QPA event
        else if ( EVENT_QPA.equals( event ) ) {

            boolean invalidData = false;
            Inventory part = sState.getPart();

            // insure required info exists
            if ( part == null ) {

                // error, no part set in the session
                if ( debug.ON ) {
                    debug.out( "part(" + part + ")" );
                }

                throw new IllegalStateException(
                        "part not found in session" );
            }

            ValidatedValues vValues = 
                    ValidatedValuesFactory.getValidatedValues( request );
            ValidatedForm vForm = ValidatedFormFactory.getValidatedForm();

            // Validate the html form, let any missing fields bubble as a
            // MissingFormFieldException
            LongMinMaxExpression minMax = new LongMinMaxExpression();

            String qtyString = vForm.validate( vValues, 
                    LongValueValidator.class.getName(), QTY_PARAMETER,
                    minMax.set( 1L, true ) );

            DoubleMinMaxExpression doubleminMax = new DoubleMinMaxExpression();

            String priceString = vForm.validate( vValues, 
                    DoubleValueValidator.class.getName(), UNIT_PRICE_PARAMETER,
                    doubleminMax.set( 0.0d, true ) );

            // Did all fields pass the validation test ?
            if ( vValues.getInvalidFieldCount() > 0 ) {

                if ( debug.ON ) {
                    debug.out( "invalid form field count(" + 
                            vValues.getInvalidFieldCount() + ")" );
                }

                String status = vValues.getStatus( QTY_PARAMETER );
                if ( status == WebflowFormConstants.STATUS_INVALID ) {

                    // add error message to request state for display in browser
                    rState.setQpaFormDataError( vValues.getMessage( QTY_PARAMETER ) );
                }

                status = vValues.getStatus( UNIT_PRICE_PARAMETER );
                if ( status == WebflowFormConstants.STATUS_INVALID ) {

                    // add error message to request state for display in browser
                    rState.setQpaFormDataError(
                            vValues.getMessage( UNIT_PRICE_PARAMETER ) );
                }

                // InvalidFormFieldException will be thrown after we validate
                // the rest of the fields.
                invalidData = true;
            }

            // may throw InvalidFormDataException
            Calendar deliveryDate = getCalendar( request, rState.calendar(), rState );

            if ( invalidData ) {

                // throw the InvalidFormFieldException and let Webflow
                // redirect the user back to the origin JSP, preserving all our input
                throw new InvalidFormDataException(
                        "Please fix the fields in red and resubmit" );
            }

            // we've got good data if we made it this far.

            long qty = 0L;
            double price = 0.0d;

            try {

                qty = Long.parseLong( qtyString );
                price = Double.parseDouble( priceString );
            }
            catch ( NumberFormatException e ) {
                // shouldn't happen, validator should have already checked

                if ( debug.ON ) {
                    debug.out( "exception(" + e + ")", e );
                }

                throw new RuntimeException( "Invalid quantity or price" );
            }

            // may throw runtime exception java.util.MissingResourceException
            QPA qpa = BuyerFactory.createQPA( part.id(), Short.toString( getQpaId() ),
                    part.description(),
                    (Money) new ImmutableMoney( price, Helper.getCurrency() ),
                    qty, rState.calendar(),
                    deliveryDate );

            if ( useEjbTestImpl ) {

                // do nothing
                rtnVal = READ_ONLY;
            }
            else {

                // submit QPA
                if ( debug.ON ) {
                    debug.out( "useEjbTestImpl(" + useEjbTestImpl + ")" );
                }

                // remove the XML for another run thru the purchasing process
                if ( ! doNotDelete ) {

                    if ( debug.ON ) {
                        debug.out( "deleting XML files" );
                    }

                    // logs errors
                    // may throw RuntimeException of some variety
                    BuyerFactory.deleteXML();
                }

                // may throw E2EProcessingException
                submitQPA( qpa );
                rtnVal = READ_ONLY;
            }

            // QPA successfully submitted
            sState.setQpa( qpa );

            // set Quote timeout time
            sState.setQuoteTimeout(
                    getTimeoutCalendar( rState.calendar(), timeoutSeconds ) );

            // set the time at which the QPA was sent
            sState.setQPASentTime( rState.calendar() );

            sState.setState( StateValue.QPA_SENT );
        }

        // bea.portal.framework.internal.refresh or null event
        else {

            StateValue state = sState.getState();

            if ( StateValue.START.equals( state ) ||
                    StateValue.PRODUCT_SELECTED.equals( state ) ) {

                rtnVal = NOT_SELECTED;
            }
            else if ( StateValue.PART_SELECTED.equals( state ) ) {

                rtnVal = DISPLAY;
            }
            else {

                rtnVal = READ_ONLY;
            }
        }


        if ( debug.ON ) {
            debug.out( "rtnVal(" + rtnVal + ")" );
        }

        return rtnVal;
    }

    protected Calendar getCalendar( HttpServletRequest request,
            Calendar currentDate,
            RequestState rState ) 
    throws InvalidFormDataException {

        // month is an integer that represents the month offset from
        // the current date
        String monthString = Helper.getMonthParameter( request );
        String dayString = Helper.getDayParameter( request );
        Calendar calendar = null;
        int month;
        int day;

        // this should only happen if client edits html
        if ( monthString == null || dayString == null ) {

            if ( debug.ON ) {
                debug.out( "Invalid Client HTML, empty parameter(s)" );
            }

            throw new InvalidFormDataException(
                    "Please fix the fields in red and resubmit" );
        }

        try {

            month = Integer.parseInt( monthString );
            day = Integer.parseInt( dayString );
        }
        catch ( NumberFormatException e ) {

            // this should only happen if client edits html
            if ( debug.ON ) {
                debug.out( "Invalid Client HTML" );
                debug.out( "exception(" + e + ")" );
            }

            throw new InvalidFormDataException(
                    "Please fix the fields in red and resubmit" );
        }

        if ( month == 0 && ( day < currentDate.get( calendar.DAY_OF_MONTH ) ) ) {

            // value submitted is prior to current date
            rState.setQpaFormDataError( "the date chosen is prior to the current date" );

            throw new InvalidFormDataException(
                    "Please fix the fields in red and resubmit" );
        }

        // Calendar will handle invalid day of month
        // by rolling forward to the next month
        calendar = (Calendar) currentDate.clone();

        // increment the month of the year
        calendar.add( Calendar.MONTH, month );
        calendar.set( Calendar.DAY_OF_MONTH, day );

        // forces calendar to recalculate
        calendar.getTime();

        rState.setQpaCalendarFormData( new CalendarFormData(
                monthFormat.format( calendar.getTime() ), month, day ) );

        return calendar;
    }


    private static synchronized short getQpaId() {

        qpaId++;

        return qpaId;
    }
}

