
/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/b2bPortal/WEB-INF/src/examples/e2e/b2b/processors/QuoteProcessor.java,v 1.2 2002/05/02 00:22:09 toneill Exp $
 */

package examples.e2e.b2b.processors;

import examples.e2e.b2b.Helper;
import examples.e2e.b2b.RequestState;
import examples.e2e.b2b.SessionState;
import examples.e2e.b2b.StateValue;

import examples.e2e.common.PurchaseQuote;
import examples.e2e.common.BuyerFactory;
import examples.e2e.common.IllegalStateException;

import com.bea.p13n.util.debug.Debug;
import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.exception.ProcessingException;
import com.bea.p13n.appflow.webflow.forms.InvalidFormDataException;

import com.bea.portal.appflow.PortalRequest;

import java.util.List;
import java.util.Iterator;
import java.util.Calendar;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.xml.sax.InputSource;


public class QuoteProcessor extends AbstractProcessor {

    private static Debug debug =
            Debug.getInstance( QuoteProcessor.class );

    // if property is true canned data will be provided by test factory
    private static final String TEST_PROP =
            "samples.e2e.b2b.quote.refresh.useTestImpl";
    private static boolean useTestImpl;

    static {
        try {
            useTestImpl = Boolean.getBoolean( TEST_PROP );
            if ( debug.ON ) {
                debug.out( "useTestImpl(" + useTestImpl + ")" );
            }
        }
        catch ( Exception e ) {
            debug.out( "useTestImpl(" + useTestImpl + ")", e );
        }
    }


    public QuoteProcessor() {
    } 


    /**
     * Processors extending this class must provide an implementation. This method
     * will be called if the processor is session insensitive or the session has not
     * timed-out.
     */
    public Object process(HttpServletRequest request,
            HttpServletResponse response,
            PortalRequest portalRequest,
            Object requestContext,
            PipelineSession session,
            RequestState rState,
            SessionState sState )
    throws ProcessingException {

        String rtnVal = null;

        if ( debug.ON ) {
            debug.out( EMPTY_STRING );
            debug.out( "useTestImpl(" + useTestImpl + ")" );
        }

        // actual event for this portlet
        // one of "selectQuote",
        // "bea.portal.framework.internal.refresh" or "null"
        String event = getPortletEvent( request );

        // current system state
        StateValue state = sState.getState();

        Calendar currentDate = Calendar.getInstance();

        // select quote event
        if ( EVENT_QUOTE.equals( event ) ) {

            if ( ! StateValue.QPA_SENT.equals( state ) ) {

                throw new IllegalStateException( "current state( " +
                        state.getString() +
                        " ), with event( " +
                        event + " )" );
            }

            // get the parameter identifying the selected quote
            String quoteId = Helper.getQuoteParameter( request );

            // did a quote get selected?
            if ( quoteId == null ) {

                // error, a quote must be chosen
                rState.setQuoteFormDataError( "please choose a quote" );
            }
            else {

                quoteId = quoteId.trim();

                // error, empty parameter
                //
                // this shouldn't happen unless the client (HTML) code
                // was modified or programmer error
                if ( quoteId.length() == 0 ) {

                    throw new IllegalStateException( "quoteId( " + quoteId + " )" );
                }
            }

            // get source XML file
            InputSource source = getSource();

            // get List of PurchaseQuote objects
            // may throw ParsingException
            List quotes = BuyerFactory.parseQuote( source );

            if ( quotes == null || quotes.size() == 0 ) {

                throw new IllegalStateException( "no quotes" );
            }

            // put the quotes in the request for re-display
            rState.setQuotes( quotes );

            // was a quote selected in the form?
            if ( rState.quoteFormDataError() ) {

                // quoteId == null
                // re-display quotes
                throw new InvalidFormDataException(
                        rState.quoteFormDataErrorMsg() );
            }

            // get the quote and add to session
            // may throw IllegalStateException if quote cannot be found
            sState.setQuote( getQuote( quotes, quoteId ) );

            sState.setQuoteSelectedTime( rState.calendar() );

            sState.setState( StateValue.CREATE_PO );

            rtnVal = READ_ONLY;
        }

        // bea.portal.framework.internal.refresh or null event
        else {

            if ( StateValue.START.equals( state ) ||
                    StateValue.PRODUCT_SELECTED.equals( state ) ||
                    StateValue.PART_SELECTED.equals( state ) ) {

                rtnVal = NOT_SELECTED;
            }
            else if ( StateValue.QPA_SENT.equals( state ) ) {

                // get source XML file
                InputSource source = getSource();

                // may throw ParsingException
                // a null source will be handled properly
                List quotes = BuyerFactory.parseQuote( source );

                if ( quotes == null || quotes.size() == 0 ) {

                    if ( sState.quoteTimedOut( currentDate ) ) {

                        rtnVal = TIMEOUT;
                    }
                    else {

                        rtnVal = NO_DATA;
                    }
                }
                else {

                    rState.setQuotes( quotes );
                    rtnVal = DISPLAY;
                }

            }
            else {

                rtnVal = READ_ONLY;
            }
        }


        if ( debug.ON ) {
            debug.out( "rtnVal(" + rtnVal + ")" );
        }

        return rtnVal;
    }


    private static InputSource getSource() {

        InputSource source = null;

        // get the XML source for the data
        if ( useTestImpl ) {

            // test implementation
            source = null;
        }
        else {

            source = BuyerFactory.getQuoteSource();

            if ( source == null ) {

                if ( debug.ON ) {
                    debug.out( "source(" + source + ")" );
                }

                // cannot locate file
                // this error will be caught later
            }

        }

        return source;
    }


    private static PurchaseQuote getQuote( List quotes, String quoteId ) 
    throws IllegalStateException {

        Iterator it = quotes.iterator();
        PurchaseQuote quote = null;
        boolean done = false;

        // find the quote
        while ( it.hasNext() ) {

            quote = (PurchaseQuote) it.next();

            if ( quoteId.equals( quote.quoteId() ) ) {

                // found it
                done = true;
                break;
            }
        }

        if ( ! done ) {

            // couldn't find the quote
            throw new IllegalStateException( "unable to find quote" );
        }

        return quote;
    }
}

