/*
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/b2cPortal/WEB-INF/src/examples/e2e/b2c/catalog/webflow/CatalogIP.java,v 1.2 2002/05/02 00:22:10 toneill Exp $
 */
package examples.e2e.b2c.catalog.webflow;

import javax.servlet.http.HttpServletRequest;

import com.beasys.commerce.util.Logger;
import com.beasys.commerce.webflow.PipelineSessionConstants;
import com.beasys.commerce.webflow.HttpRequestConstants;

import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.exception.*;
import com.bea.p13n.appflow.webflow.SessionManagerFactory;
import com.bea.p13n.appflow.webflow.InputProcessorSupport;

/**
 * This <code>InputProcessor</code> is the base class for all Catalog
 * related <code>InputProcessor</code> components.  The
 * <code>process</code> method extracts a view size String from the
 * HTTP requests, validates the String, and then adds the integer
 * value of the view index String to the <code>PipelineSession</code>.
 *
 * @see com.bea.p13n.appflow.webflow.InputProcessor
 * @see com.bea.p13n.appflow.webflow.HttpRequestConstants
 * @see com.beasys.commerce.webflow.PipelineSessionConstants
 * @see com.beasys.commerce.foundation.pipeline.PipelineSession
 * @see com.beasys.commerce.foundation.pipeline.PipelineConstants
 */
public abstract class CatalogIP extends InputProcessorSupport
{
    static final boolean DEBUG = false;

    public Object process(HttpServletRequest request, Object requestContext) 
		throws ProcessingException
    {
        // Extract a view size
        Integer viewSize = null;
        String viewSizeString = (String)request.getParameter(HttpRequestConstants.CATALOG_VIEW_SIZE);
        
        if(viewSizeString != null) {
            try {
                viewSize = Integer.valueOf(viewSizeString);
            }
            catch(NumberFormatException e) {
                throw new ProcessingException("user.message.invalid.view.size", e);
            }

            if(DEBUG) {
                Logger.getInstance().debug("In CatalogIP : viewSize = " + viewSize);
            }
        }

        //
        // Add any global parameters to the PipelineSession in the session scope
        //
	PipelineSession pSession = SessionManagerFactory.getSessionManager().getPipelineSession(request);
	String namespace = (String) pSession.getDefaultNamespace();

        // Add a view size
        if(viewSize != null) {
	    setSessionAttribute(PipelineSessionConstants.CATALOG_VIEW_SIZE, 
				viewSize, namespace, pSession);
        }
        	
        // Operation succeeded
        return "success";
    }
}
