/*
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/b2cPortal/WEB-INF/src/examples/e2e/b2c/catalog/webflow/GetCategoryIP.java,v 1.2 2002/05/02 00:22:10 toneill Exp $
 */
package examples.e2e.b2c.catalog.webflow;

import javax.servlet.http.HttpServletRequest;

import com.beasys.commerce.ebusiness.catalog.*;
import com.beasys.commerce.util.*;
import com.beasys.commerce.foundation.pipeline.PipelineConstants;
import com.beasys.commerce.webflow.PipelineSessionConstants;
import com.beasys.commerce.webflow.HttpRequestConstants;

import com.bea.p13n.appflow.exception.*;
import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.webflow.SessionManagerFactory;

/**
 * This <code>InputProcessor</code> takes a Category ID String from
 * the HTTP request, validates the ID String, creates a
 * <code>CategoryKey</code> based on the ID String, and adds it to the
 * <code>PipelineSession</code> in the session scope.  In the event
 * that the Category ID String is not specified, the root category key
 * will be added to the <code>PipelineSession</code>.
 *
 * @see com.bea.p13n.appflow.webflow.InputProcessor
 * @see com.beasys.commerce.ebusiness.catalog.webflow.CatalogIP
 */
public class GetCategoryIP extends CatalogIP
{
    public Object process(HttpServletRequest request, Object requestContext) 
	throws ProcessingException
    {
        super.process(request, requestContext);

        // Extract the ID from the HTTP request
        String id = (String)request.getParameter(HttpRequestConstants.CATALOG_CATEGORY_ID);

        if(DEBUG) {
            Logger.getInstance().debug("In GetCategoryIP : id = " + id);
        }
        	
        // Create a CategoryKey based on the ID and add it to the PipelineSession
        CategoryKey key = null;
        if(id == null || id.length() == 0) {
            key = CategoryKey.ROOT;
        }
        else {
            key = new CategoryKey(id);
        }
        	
        if(DEBUG) {
            Logger.getInstance().debug("In GetCategoryIP : created CategoryKey " + key);
        }
	
	PipelineSession pSession = SessionManagerFactory.getSessionManager().getPipelineSession(request);
	String namespace = (String) pSession.getDefaultNamespace();
	setSessionAttribute(PipelineSessionConstants.CATALOG_CATEGORY_KEY, key, namespace, pSession);
	
        // Operation succeeded
        return "success";
    }
}
