/*
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 1997-2002 BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/b2cPortal/WEB-INF/src/examples/e2e/b2c/catalog/webflow/GetProductItemIP.java,v 1.2 2002/05/02 00:22:10 toneill Exp $
 */
package examples.e2e.b2c.catalog.webflow;

import javax.servlet.http.HttpServletRequest;

import com.beasys.commerce.ebusiness.catalog.*;
import com.beasys.commerce.util.*;
import com.beasys.commerce.foundation.pipeline.PipelineConstants;
import com.beasys.commerce.webflow.PipelineSessionConstants;
import com.beasys.commerce.webflow.HttpRequestConstants;

import com.bea.p13n.appflow.exception.*;
import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.webflow.SessionManagerFactory;

/**
 * This <code>InputProcessor</code> takes a SKU String from the HTTP
 * request, validates the SKU String, creates a
 * <code>ProductItemKey</code> based on the SKU String, and adds it to
 * the <code>PipelineSession</code> in the session scope.
 *
 * @see com.bea.p13n.appflow.webflow.InputProcessor
 * @see com.beasys.commerce.ebusiness.catalog.webflow.CatalogIP
 */
public class GetProductItemIP extends CatalogIP
{
    public Object process(HttpServletRequest request, Object requestContext) 
	throws ProcessingException
    {
        super.process(request, requestContext);

        // Extract the SKU from the HTTP request and validate
        String sku = (String)request.getParameter(HttpRequestConstants.CATALOG_ITEM_SKU);
        
        if(sku == null || sku.length() == 0) {
            throw new ProcessingException("user.message.item.missing.sku");
        }

        if(DEBUG) {
            Logger.getInstance().debug("In GetProductItemIP : sku = " + sku);
        }
        	
        // Create a ProductItemKey based on the SKU and add it to the PipelineSession
        ProductItemKey key = new ProductItemKey(sku);
	
	PipelineSession pipelineSession = SessionManagerFactory.getSessionManager().getPipelineSession(request);
	String namespace = (String) pipelineSession.getDefaultNamespace();
        	
	setSessionAttribute(PipelineSessionConstants.CATALOG_ITEM_KEY, key, 
			    namespace, pipelineSession);

        // Operation succeeded
        return "success";
    }
}



