/*
 * B E A   S Y S T E M S
 *
 * C O M M E R C E   C O M P O N E N T S
 *
 * Copyright (c) 2000-2002  BEA Systems, Inc.
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws
 * of the United States. The software contained on this media is proprietary
 * to and embodies the confidential technology of BEA Systems, Inc. The
 * possession or receipt of this information does not convey any right to disclose
 * its contents, reproduce it, or use,  or license the use, for manufacture or
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/b2cPortal/WEB-INF/src/examples/e2e/b2c/catalog/webflow/KeywordSearchIP.java,v 1.2 2002/05/02 00:22:10 toneill Exp $
 */
package examples.e2e.b2c.catalog.webflow;

import java.util.StringTokenizer;
import java.util.LinkedList;
import javax.servlet.http.HttpServletRequest;

import com.beasys.commerce.ebusiness.catalog.*;
import com.beasys.commerce.ebusiness.catalog.service.query.*;
import com.beasys.commerce.util.Logger;
import com.beasys.commerce.foundation.pipeline.PipelineConstants;
import com.beasys.commerce.webflow.PipelineSessionConstants;
import com.beasys.commerce.webflow.HttpRequestConstants;

import com.bea.p13n.appflow.exception.*;
import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.webflow.SessionManagerFactory;
import com.bea.p13n.util.debug.Debug;

/**
 * This <code>InputProcessor</code> takes a keyword search String from
 * the HTTP request, validates the search String, creates a
 * <code>KeywordQuery</code> based on the search String, adds it to
 * the <code>PipelineSession</code> in the session scope, and clears
 * the <code>PipelineSession</code> of any previous search results.
 * If a keyword search String is not supplied, the
 * <code>PipelineSession</code> will be examined for previous cached
 * search results.  If previous results exist and are valid, the
 * results are left in the <code>PipelineSession</code>.
 *
 * @see com.bea.p13n.appflow.webflow.InputProcessor
 * @see com.beasys.commerce.ebusiness.catalog.webflow.CatalogIP
 */
public class KeywordSearchIP extends CatalogIP
{
    public Object process(HttpServletRequest request, Object requestContext)
		throws ProcessingException
    {
        super.process(request, requestContext);
	
        // Get the PipelineSession
	PipelineSession pipelineSession = SessionManagerFactory.getSessionManager().getPipelineSession(request);
	
	String namespace = (String) pipelineSession.getDefaultNamespace();
	
        // Extract the search String from the HTTP request
        String searchString = (String)request.getParameter(HttpRequestConstants.CATALOG_SEARCH_STRING);
	
        if(searchString == null) {
	    CatalogQuery previousQuery = (CatalogQuery) getSessionAttribute(PipelineSessionConstants.CATALOG_QUERY, namespace, pipelineSession);

            if(previousQuery == null || !(previousQuery instanceof KeywordQuery)) {
                // Session has expired and along with it previous
                // search results; therefore, throw an
                // InvalidSessionStateException
		throw new ProcessingException("wlcs-catalog:user.message.search.results.expired");
            }

            return "success";
        }

        // Validate parameters
        if(searchString.length() == 0) {
            throw new ProcessingException("user.message.search.missing.keywords");
        }

        // Create a KeywordQuery based on the search String and add it
        // to the PipelineSession
        KeywordQuery query = CatalogFactory.createKeywordQuery();
        StringTokenizer tokenizer = new StringTokenizer(searchString);

        // Create an array of keywords
        LinkedList list = new LinkedList();
        while (tokenizer.hasMoreTokens()) {
            list.add(tokenizer.nextToken().toLowerCase());
        }
        String[] keywords = new String[list.size()];
        keywords = (String[])list.toArray(keywords);
	
        // Set the query keywords
        query.setKeywords(keywords);
	
        // Add the query to the PipelineSession and clear the
        // PipelineSession of any previous results
	setSessionAttribute(PipelineSessionConstants.CATALOG_QUERY, query, 
			    namespace, pipelineSession);
	try {
	    pipelineSession.removeSessionScopedAttribute(PipelineSessionConstants.CATALOG_SEARCH_RESULTS, namespace);
	} catch (InvalidArgumentException e) {
	    throw new ProcessingException("invalidArgumentException:removeAttribute");
	}
	
        // Operation succeeded
        return "success";
    }
}
