/*
 * B E A   S Y S T E M S
 *
 * P O R T A L
 *
 * Copyright (c) 2000-2002  BEA Systems, Inc.
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws
 * of the United States. The software contained on this media is proprietary
 * to and embodies the confidential technology of BEA Systems, Inc. The
 * possession or receipt of this information does not convey any right to disclose
 * its contents, reproduce it, or use,  or license the use, for manufacture or
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/b2cPortal/WEB-INF/src/examples/e2e/b2c/catalog/webflow/StoreCatalogEventIP.java,v 1.2 2002/05/02 00:22:10 toneill Exp $
 */

package examples.e2e.b2c.catalog.webflow;

import javax.servlet.http.HttpServletRequest;

import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.exception.*;
import com.bea.p13n.appflow.webflow.InputProcessorSupport;
import com.bea.p13n.appflow.webflow.SessionManagerFactory;

import com.bea.p13n.util.debug.Debug;

import com.bea.portal.appflow.PortalAppflowConstants;

import examples.e2e.b2c.util.B2CPortalConstants;

/**
 * This input processor is used to store the current portlet event as a
 * session scoped pipeline attribute. The catalog browse portlet uses this
 * information to decide which view of the catalog to display. It must be
 * a session scoped pipeline attribute in case the portlet is refreshed for
 * any reason.
 *
 * @see com.bea.p13n.appflow.webflow.InputProcessor
 * @see com.beasys.commerce.webflow.PipelineSessionConstants
 * @see com.beasys.commerce.foundation.pipeline.PipelineSession
 * @see com.beasys.commerce.foundation.pipeline.PipelineSessionConstants
 */
public class StoreCatalogEventIP extends InputProcessorSupport
{
    private static Debug debug =
        Debug.getInstance( StoreCatalogEventIP.class );

    public Object process( HttpServletRequest request, Object requestContext )
        throws ProcessingException
    {
        if ( debug.ON )
        {
            debug.here();
        }

        PipelineSession pSession = 
            SessionManagerFactory.getSessionManager().getPipelineSession( request );
        String namespace = (String) pSession.getDefaultNamespace();

        /* Get the portlet webflow event from the HTTP request */

        String event =
            (String) request.getParameter( PortalAppflowConstants.PORTLET_WEBFLOW_EVENT_PARAMETER );

        String returnEvent = "init";

        if ( event != null )
        {
            /* Store the event in the pipeline session */

            setSessionAttribute( B2CPortalConstants.LAST_CATALOG_EVENT_ATTRIB,
                                 event,
                                 namespace,
                                 pSession );

            returnEvent = new String( event );
        }

        /* 
         * Return the event so webflow can continue processing for that event.
         * If no portlet event was part of the HTTP request we will return
         * a "fake" event called "init".
         */

        return returnEvent;
    }
}
