/*
 * B E A   S Y S T E M S
 *
 * C O M M E R C E   C O M P O N E N T S
 *
 * Copyright (c) 2000-2002  BEA Systems, Inc.
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws
 * of the United States. The software contained on this media is proprietary
 * to and embodies the confidential technology of BEA Systems, Inc. The
 * possession or receipt of this information does not convey any right to disclose
 * its contents, reproduce it, or use,  or license the use, for manufacture or
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/b2cPortal/WEB-INF/src/examples/e2e/b2c/payment/webflow/PaymentAuthorizationIP.java,v 1.2 2002/05/02 00:22:10 toneill Exp $
 */
package examples.e2e.b2c.payment.webflow;

import java.util.Map;
import javax.servlet.http.HttpServletRequest;

import com.beasys.commerce.ebusiness.shoppingcart.*;
import com.beasys.commerce.axiom.contact.CreditCard;
import com.beasys.commerce.axiom.units.Money;
import com.beasys.commerce.axiom.units.Price;
import com.beasys.commerce.axiom.units.PriceHome;
import com.beasys.commerce.util.Logger;

import com.beasys.commerce.webflow.PipelineSessionConstants;
import com.beasys.commerce.webflow.HttpRequestConstants;

import com.bea.p13n.appflow.exception.*;
import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.webflow.SessionManagerFactory;

/**
 * This <code>InputProcessor</code> retrieves a shopping cart from the
 * pipeline session, determines the total price of the order
 * represented by the shopping cart contents, and adds this total to
 * the <code>PipelineSession</code> as the
 * <code>PipelineSessionConstants.PAYMENT_AUTHORIZATION_AMOUNT</code>
 * attribute.  Additionally, a customer credit card, as specified by
 * the <code>HttpRequestConstants.CREDITCARD_KEY</code> HTTP request
 * parameter, is added to the <code>PipelineSession</code> as the
 * <code>PipelineSessionConstants.PAYMENT_CREDIT_CARD</code>
 * attribute.
 *
 * @see com.bea.p13n.appflow.webflow.InputProcessor
 * @see com.bea.p13n.appflow.webflow.HttpRequestConstants
 * @see com.beasys.commerce.webflow.PipelineSessionConstants
 * @see com.beasys.commerce.foundation.pipeline.PipelineSession
 * @see com.beasys.commerce.ebusiness.payment.webflow.PaymentIP
 */
public class PaymentAuthorizationIP extends PaymentIP
{
    /**
     * Process the query parameters present in the HttpSession.
     *
     * @param The HttpServletRequest object
     * @return A status string
     */
    public Object process(HttpServletRequest request, Object requestContext) throws ProcessingException
    {
        // Get PipelineSession
	PipelineSession pipelineSession = SessionManagerFactory.getSessionManager().getPipelineSession(request);
	String namespace = (String) pipelineSession.getDefaultNamespace();
	
        // Get the ShoppingCart from the PipelineSession session scope
        ShoppingCart shoppingCart = (ShoppingCart) getSessionAttribute(PipelineSessionConstants.SHOPPING_CART,
								       namespace, pipelineSession);
        if(shoppingCart == null) {
            throw new ProcessingException("ebusiness-payment:shopping.cart.null");
        }

        Money shoppingCartTotal = shoppingCart.getTotalPrice();
        double orderAmount = shoppingCartTotal.getValue();

        // Extract the credit card key from the from the HttpRequest
        // and validate
        String creditCardKey = (String)request.getParameter(HttpRequestConstants.CREDITCARD_KEY);
        if(creditCardKey == null || creditCardKey.length() == 0) {
            throw new ProcessingException("ebusiness-payment:invalid.credit.card.key");
        }

        // Create a Price object based upon the ShoppingCart total
        Price authorizationAmount = PriceHome.create();
        try {
            authorizationAmount.setValue(orderAmount);
            authorizationAmount.setCurrency(shoppingCartTotal.getCurrency());
        }
        catch(com.beasys.commerce.foundation.exception.InvalidArgumentException e) {
            throw new ProcessingException("ebusiness-payment:invalid.argument", e);
        }
	
        // Get the credit card from the customer using the supplied
        // credit card key
        Map creditCardsMap = (Map)getCustomerProperty(request, "creditCardsMap");
        CreditCard creditCard = (CreditCard)creditCardsMap.get(creditCardKey);
        if(creditCard == null) {
            throw new ProcessingException("ebusiness-payment:invalid.credit.card");
        }

        // Add the CreditCard and authorization amount to the
        // PipelineSession
        setSessionAttribute(PipelineSessionConstants.PAYMENT_CREDIT_CARD, creditCard,
			    namespace, pipelineSession);
        setSessionAttribute(PipelineSessionConstants.PAYMENT_AUTHORIZATION_AMOUNT, authorizationAmount, 
			    namespace, pipelineSession);
	
        if(DEBUG) {
            Logger.getInstance().debug("In PaymentAuthorizationIP : added CreditCard <" + creditCard +
                                       "> and an authorization amount <" + authorizationAmount + 
				       "> to the PipelineSession.");
        }
	
        return "success";
    }
 }



