/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 */

package examples.e2e.b2c.shipping.webflow;

import examples.e2e.b2c.util.B2CPortalConstants;
import com.bea.portal.appflow.PortalAppflowConstants;

import java.util.List;
import java.rmi.RemoteException;
import javax.ejb.CreateException;
import javax.naming.NamingException;
import javax.servlet.http.HttpServletRequest;

import com.beasys.commerce.ebusiness.shipping.*;
import com.beasys.commerce.ebusiness.shoppingcart.ShoppingCart;

import com.beasys.commerce.webflow.PipelineSessionConstants;

import com.bea.p13n.appflow.exception.*;
import com.bea.p13n.appflow.webflow.SessionManagerFactory;
import com.bea.p13n.appflow.webflow.InputProcessorSupport;
import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.management.ApplicationHelper;
import com.bea.p13n.util.JndiHelper;

/**
 * <code>InitShippingMethodListIP</code> gets the list of shipping
 * methods from the database and stores them into the pipeline
 * session.
 */

public class InitShippingMethodListIP extends InputProcessorSupport
{
   /**
    * See the class documentation above.
    *
    * @param request the <code>HttpServletRequest</code> object
    * associated with current HTTP request.
    * @return upon method completion always returns
    * <code>String</code> "success".
    * @throws
    * com.beasys.commerce.foundation.exception.ProcessingException
    * <UL> <LI> if JNDI lookup for <code>ShippingHelper</code>
    * failed.</LI> <LI> if create failed on
    * <code>ShippingHelperHome</code>. </LI> <LI> if RemoteExecption
    * is thrown while calling create on
    * <code>ShippingHelperHome</code>. </LI> </UL>
    * @throws com.beasys.commerce.foundation.exception.InvalidSessionStateException if HttpSession times out.
    */
    public Object process(HttpServletRequest request, Object requestContext) throws ProcessingException
    {
	try {
	    // Get the pipeline session object from the http session
	    PipelineSession pipelineSession = SessionManagerFactory.getSessionManager().getPipelineSession(request);
	    String namespace = (String) pipelineSession.getDefaultNamespace();
	    
	    // Verify the shopping cart is ready for the shipping to even start!
	    ShoppingCart shoppingCart = (ShoppingCart) getSessionAttribute(PipelineSessionConstants.SHOPPING_CART,
									   namespace, pipelineSession);
	    if(shoppingCart == null || shoppingCart.isEmpty()) {
		throw new ProcessingException("ebusiness-shoppingcart:null.shoppingcart:" + 
					      PipelineSessionConstants.SHOPPING_CART);
	    }
	    
	    // JNDI lookup on shipping helper
	    ShippingHelperHome shippingHelperHome;
	    try {
		String shHome = ApplicationHelper.getApplicationName() + ".BEA_commerce.ShippingHelper";
		shippingHelperHome = (ShippingHelperHome) JndiHelper.lookupNarrow(shHome, ShippingHelperHome.class);
	    }
	    catch(NamingException e) {
		throw new ProcessingException("ebusiness-shipping:exception.toolkit.finder:ShippingHelper", e);
	    }
	    ShippingHelper shippingHelper = shippingHelperHome.create();

	    // Get the shipping methods and store into pipeline session
	    List shippingMethodList = shippingHelper.getShippingMethodList();
	    setSessionAttribute(PipelineSessionConstants.SHIPPING_METHOD_LIST, shippingMethodList,
				namespace, pipelineSession);

            // Get the portlet webflow event from the HTTP request 
            setSessionAttribute( B2CPortalConstants.DOC_PORTLET_LOCATION, "step2", namespace, pipelineSession );

	    return "success";
	}
	catch(CreateException e) {
	    throw new ProcessingException("ebusiness-shipping:shippingmethod.unavailable", e);
	}
	catch(RemoteException e) {
	    throw new ProcessingException("ebusiness-shipping:shippingmethod.unavailable", e);
	}
    }
}
