/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 */
package examples.e2e.b2c.shipping.webflow;

import examples.e2e.b2c.util.B2CPortalConstants;

import com.bea.portal.appflow.PortalAppflowConstants;
import javax.servlet.http.HttpServletRequest;

import com.beasys.commerce.axiom.contact.Address;
import com.beasys.commerce.axiom.contact.AddressHome;

import com.beasys.commerce.util.MessageCatalog;
import com.beasys.commerce.webflow.PipelineSessionConstants;
import com.beasys.commerce.webflow.HttpRequestConstants;

import com.bea.p13n.appflow.exception.*;
import com.bea.p13n.appflow.webflow.SessionManagerFactory;
import com.bea.p13n.appflow.webflow.InputProcessorSupport;
import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.webflow.forms.ValidatedValues;
import com.bea.p13n.appflow.webflow.forms.ValidatedValuesFactory;
import com.bea.p13n.appflow.webflow.forms.ValidateParameter;
import com.bea.p13n.appflow.webflow.forms.ValidationStatus;


/**
 * <code>ValidateAddressIP</code> retrieves shipping address
 * parameters from the HttpServletRequest, and validates if all the
 * required parameters are present.  It saves the address into the
 * <code>PipelineSession</code> as the
 * <code>PipelineSessionConstants.ADDRESS</code> attribute, where it
 * can be later accessed by the corresponding pipeline component.
 */
public class ValidateAddressIP extends InputProcessorSupport
{
    /**
     * Processes address parameters present in the HttpServletRequest.
     *
     * @param request HttpServletRequest object which holds address
     * data
     * @return upon method completion always returns String "success"
     * @throws
     * com.beasys.commerce.foundation.exception.ProcessingException
     * <UL> <LI> If any of the required fields were empty </LI> <LI>
     * If HttpSession times out. </LI> </UL>
     */
    public Object process(HttpServletRequest request, Object requestContext) throws ProcessingException
    {
        // Get the PipelineSession from the HttpSession
	PipelineSession pipelineSession = SessionManagerFactory.getSessionManager().getPipelineSession(request);
	String namespace = (String) pipelineSession.getDefaultNamespace();
	
	// Validate the Address field
	Address address = AddressHome.create();
        ValidationStatus invalidFieldIsPresent = new ValidationStatus();
        ValidatedValues validatedValues = ValidatedValuesFactory.getValidatedValues(request);
	getAddress(request, address, validatedValues, invalidFieldIsPresent);
	
        if(invalidFieldIsPresent.getStatus() == true) {
            throw new ProcessingException(MessageCatalog.getUserMessage("ebusiness-shipping", "invalid.attributes"));
        }
	
        //Save address into PipelineSession
        setSessionAttribute(PipelineSessionConstants.SHIPPING_ADDRESS, address, namespace, pipelineSession);
	
        // Get the portlet webflow event from the HTTP request 
        setSessionAttribute( B2CPortalConstants.DOC_PORTLET_LOCATION, "step3", namespace, pipelineSession );

        return "success";
    }
    
    /**
     * Get all the field values from the HTTPSession associated with customer
     * shipping address and validate them.
     *
     * @param request the <code>HttpServletRequest</code> object
     * associated with current HTTP request.
     * @param address empty Address object
     * @param validatedValues <code>ValidatedValues</code> object to
     * keep the status of all HttpServletRequest parameters.
     * @param invalidFieldIsPresent validation status of the field
     * values
     */
    protected void getAddress(HttpServletRequest request, 
			      Address address,
			      ValidatedValues validatedValues,
			      ValidationStatus invalidFieldIsPresent)
    {
	// Get first line of address
	String street1 = request.getParameter(HttpRequestConstants.CUSTOMER_SHIPPING_ADDRESS1);
	ValidateParameter.validateTextParameter(HttpRequestConstants.CUSTOMER_SHIPPING_ADDRESS1, 
						street1, validatedValues, invalidFieldIsPresent);
	address.setStreet1(street1);
	
	// Get second line of address ( optional )
	String street2 = request.getParameter(HttpRequestConstants.CUSTOMER_SHIPPING_ADDRESS2);
	ValidateParameter.validateOptionalTextParameter(HttpRequestConstants.CUSTOMER_SHIPPING_ADDRESS2, 
							street2, validatedValues, invalidFieldIsPresent);
	address.setStreet2(street2);
	
	// Get city
	String city = request.getParameter(HttpRequestConstants.CUSTOMER_SHIPPING_CITY);
	ValidateParameter.validateTextParameter(HttpRequestConstants.CUSTOMER_SHIPPING_CITY, city, 
						validatedValues, invalidFieldIsPresent);
	address.setCity(city);
	
	// Get zip code
	String zipCode = request.getParameter(HttpRequestConstants.CUSTOMER_SHIPPING_ZIPCODE);
	ValidateParameter.validateTextParameter(HttpRequestConstants.CUSTOMER_SHIPPING_ZIPCODE, 
						zipCode, validatedValues, invalidFieldIsPresent);
	address.setPostalCode(zipCode);
	
	// Get country
	String country = request.getParameter(HttpRequestConstants.CUSTOMER_SHIPPING_COUNTRY);
	ValidateParameter.validateListBoxParameter(HttpRequestConstants.CUSTOMER_SHIPPING_COUNTRY,
						   country, validatedValues, invalidFieldIsPresent);
	address.setCountry(country);
	
	//
	// You can't select a state for non us or canada addresses.
	// but you have to select an address for us or canada
	// addresses. Additional address verification follows:
	//
	String state = request.getParameter(HttpRequestConstants.CUSTOMER_SHIPPING_STATE);
	
	if(country.equalsIgnoreCase("united states") || country.equalsIgnoreCase("canada")) {
	    ValidateParameter.validateListBoxParameter(HttpRequestConstants.CUSTOMER_SHIPPING_STATE, state,
						       validatedValues, invalidFieldIsPresent);
	}
	else {
	    ValidateParameter.validateOptionalListBoxParameter(HttpRequestConstants.CUSTOMER_SHIPPING_STATE, state,
							       validatedValues, invalidFieldIsPresent);
	    //
	    // Unset the state.
	    //
	    state = "";
	}
	
	address.setState(state);
	
    }
}
