/*
 * B E A   S Y S T E M S
 *
 * C O M M E R C E   C O M P O N E N T S
 *
 * Copyright (c) 2000-2002  BEA Systems, Inc.
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws
 * of the United States. The software contained on this media is proprietary
 * to and embodies the confidential technology of BEA Systems, Inc. The
 * possession or receipt of this information does not convey any right to disclose
 * its contents, reproduce it, or use,  or license the use, for manufacture or
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 */
package examples.e2e.b2c.shipping.webflow;

import java.util.Collection;
import java.util.Iterator;
import javax.servlet.http.HttpServletRequest;

import com.beasys.commerce.ebusiness.shipping.*;
import com.beasys.commerce.webflow.HttpRequestConstants;
import com.beasys.commerce.webflow.PipelineSessionConstants;

import com.bea.p13n.appflow.exception.*;
import com.bea.p13n.appflow.webflow.SessionManagerFactory;
import com.bea.p13n.appflow.webflow.InputProcessorSupport;
import com.bea.p13n.appflow.common.PipelineSession;

/**
 * <code>ValidateShippingInfoIP</code> sets the shipping preference,
 * special instruction and shipping method.  It retrieves the
 * <code>HttpRequestConstants.SPLITTING_PREFERENCE<code>,
 * <code>HttpRequestConstants.SPECIAL_INSTRUCTIONS<code> and
 * <code>HttpRequestConstants.SHIPPING_METHOD<code> parameters from
 * the <code>HttpServletRequest</code> and sets them into
 * PipelineSession as
 * <code>PipelineSessionConstants.SPLITTING_PREFERENCE</code>,
 * <code>PipelineSessionConstants.SPLITTING_PREFERENCE</code> and
 * <code>PipelineSessionConstants.SPLITTING_PREFERENCE</code>
 * attributes respectively.
 */
public class ValidateShippingInfoIP extends InputProcessorSupport
{
  /**
   * See the class documentation above.
   *
   * @param request HttpServletRequest object which holds address data
   * @return upon method completion always returns String "success"
   * @throws
   * com.beasys.commerce.foundation.exception.ProcessingException if
   * HttpSession times out.
   */
    public Object process(HttpServletRequest request, Object requestContext) throws ProcessingException
    {
	// Get the pipeline session object from the http session
	PipelineSession pipelineSession = SessionManagerFactory.getSessionManager().getPipelineSession(request);
	String namespace = (String) pipelineSession.getDefaultNamespace();
	
	// Get the request parameters
	String splittingPreferenceCode = request.getParameter(HttpRequestConstants.SPLITTING_PREFERENCE_CODE);
	String splitLocal = request.getParameter(HttpRequestConstants.SPLITTING_PREFERENCE_SPLIT_LOCAL);
	String noSplitLocal = request.getParameter(HttpRequestConstants.SPLITTING_PREFERENCE_NO_SPLIT_LOCAL);
	String specialInstructions = request.getParameter(HttpRequestConstants.SPECIAL_INSTRUCTIONS);
	String shippingMethodKey = request.getParameter(HttpRequestConstants.SHIPPING_METHOD);

        // Basic error handling
        if(splittingPreferenceCode == null) {
            throw new ProcessingException("ebusiness-shipping:null.preferencecode.identifier:" +
					  HttpRequestConstants.SPLITTING_PREFERENCE_CODE);
        }
        if(splitLocal == null) {
            throw new ProcessingException("ebusiness-shipping:null.splitlocal.identifier:" + 
					  HttpRequestConstants.SPLITTING_PREFERENCE_SPLIT_LOCAL);
        }
        if(noSplitLocal == null) {
            throw new ProcessingException("ebusiness-shipping:null.nosplitlocal.identifier:" + 
					  HttpRequestConstants.SPLITTING_PREFERENCE_NO_SPLIT_LOCAL);
        }
        if(shippingMethodKey == null) {
            throw new ProcessingException("ebusiness-shipping:null.shippingmethodkey.identifier:" + 
					  HttpRequestConstants.SHIPPING_METHOD);
        }
	
        // Set which of the local strings to use
	String splittingPreferenceLocal;
        if(splittingPreferenceCode.equals(HttpRequestConstants.SPLITTING_PREFERENCE_NO_SPLIT_CODE)) {
            splittingPreferenceLocal = noSplitLocal;
        } 
	else {
            splittingPreferenceLocal = splitLocal;
        }
	
	// Set the pipeline session attributes
	setSessionAttribute(PipelineSessionConstants.SPLITTING_PREFERENCE,splittingPreferenceLocal, 
			    namespace, pipelineSession);
	setSessionAttribute(PipelineSessionConstants.SPLITTING_PREFERENCE_CODE,splittingPreferenceCode, 
			    namespace, pipelineSession);
	setSessionAttribute(PipelineSessionConstants.SPECIAL_INSTRUCTIONS,specialInstructions, 
			    namespace, pipelineSession);

	// Iterate through the shipping methods and find the value of the one the user picked
	Collection shippingMethods = (Collection) getSessionAttribute(PipelineSessionConstants.SHIPPING_METHOD_LIST, 
								      namespace, pipelineSession);
	Iterator iterator = shippingMethods.iterator();
	while (iterator.hasNext()) {
	    ShippingMethodValue shippingMethodValue = (ShippingMethodValue)iterator.next();
	    if (shippingMethodValue.identifier.equals(shippingMethodKey)) {
		setSessionAttribute(PipelineSessionConstants.SHIPPING_METHOD, shippingMethodValue, namespace, pipelineSession);
	    }
	}
	
	return "success";
    }
}
