/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 */
package examples.e2e.b2c.shoppingcart.webflow;

import examples.e2e.b2c.util.B2CPortalConstants;
import com.bea.portal.appflow.PortalAppflowConstants;

import javax.servlet.http.HttpServletRequest;

import com.beasys.commerce.ebusiness.catalog.ProductItem;
import com.beasys.commerce.foundation.exception.InvalidArgumentException;
import com.beasys.commerce.ebusiness.shoppingcart.ShoppingCart;

import com.bea.p13n.appflow.exception.ProcessingException;
import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.webflow.SessionManagerFactory;
import com.bea.p13n.appflow.webflow.InputProcessorSupport;

import com.beasys.commerce.foundation.pipeline.PipelineConstants;
import com.beasys.commerce.webflow.PipelineSessionConstants;
import com.beasys.commerce.webflow.HttpRequestConstants;

/**
 * DeleteProductItemFromShoppingCartIP deletes a specified item from
 * the shopping cart.  The sku of the item to be deleted is specified
 * as the <code>HttpRequestConstants.CATALOG_ITEM_SKU</code> parameter
 * in the request object.  The shopping cart is retrieved from the
 * <code>PipelineSessionConstants.SHOPPING_CART</code> attribute in
 * the pipeline session.  The item is deleted and the pipeline session
 * is updated with the modified shopping cart.
 *
 * @see com.beasys.commerce.ebusiness.shoppingcart.ShoppingCart
 * @see com.beasys.commerce.webflow.CommerceInputProcessor
 */

public class DeleteProductItemFromShoppingCartIP extends InputProcessorSupport
{
    /**
     * See class documentation above.
     *
     * @return String containing "success" if the operation was successful
     * @param request <code>HttpServletRequest</code> object for the current request
     * @throws ProcessingException If the
     * <code>HttpRequestConstants.CATALOG_ITEM_SKU</code> parameter is
     * not available in the request or if the
     * <code>PipelineSessionConstants.SHOPPING_CART</code> attribute
     * is not found in the pipeline session
     */
    public Object process(HttpServletRequest request, Object requestContext) throws ProcessingException
    {
        // Get the pipeline session
	PipelineSession pipelineSession = SessionManagerFactory.getSessionManager().getPipelineSession(request);
	String namespace = (String) pipelineSession.getDefaultNamespace();
	
        // Get the shopping cart from the pipeline session
        ShoppingCart shoppingCart = (ShoppingCart) getSessionAttribute(PipelineSessionConstants.SHOPPING_CART,
								       namespace, pipelineSession);
        if(shoppingCart == null || shoppingCart.isEmpty()) {
            throw new ProcessingException("ebusiness-shoppingcart:null.shoppingcart:" +
					  PipelineSessionConstants.SHOPPING_CART);
        }
	
        // Get the sku from the http request
        String itemSku = request.getParameter(HttpRequestConstants.CATALOG_ITEM_SKU);
        if(itemSku == null) {
            throw new ProcessingException("ebusiness-shoppingcart:sku.parameter.was.null:" + 
					  HttpRequestConstants.CATALOG_ITEM_SKU);
        }
	
        try {
            // Remove the item from the shopping cart
            ProductItem pi = shoppingCart.removeProductItem(itemSku);
            setRequestAttribute( PipelineSessionConstants.CATALOG_ITEM, pi, namespace, requestContext, 
				 pipelineSession );
        }
        catch(InvalidArgumentException iae) {
            throw new ProcessingException("ebusiness-shoppingcart:invalid.argument.exception.in.delete.product.item.from.shoppingcart",iae);
        }
	

        // Everything OK.
        // Get the portlet webflow event from the HTTP request 
        String event =
            (String) request.getParameter( PortalAppflowConstants.PORTLET_WEBFLOW_EVENT_PARAMETER );

        // Depending on the type of event, we store a specific doc portlet location attribute
        setSessionAttribute( B2CPortalConstants.DOC_PORTLET_LOCATION, "step1." + event, namespace, pipelineSession );

        return "success";
    }
}
