/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 */
package examples.e2e.b2c.shoppingcart.webflow;

import javax.servlet.http.HttpServletRequest;

import com.bea.p13n.appflow.exception.*;
import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.webflow.SessionManagerFactory;
import com.bea.p13n.appflow.webflow.InputProcessorSupport;

import com.beasys.commerce.foundation.pipeline.PipelineConstants;
import com.beasys.commerce.webflow.PipelineSessionConstants;
import com.beasys.commerce.webflow.HttpRequestConstants; 

import com.beasys.commerce.ebusiness.shoppingcart.ShoppingCart;
import com.beasys.commerce.ebusiness.shoppingcart.ShoppingCartHome;

import com.beasys.commerce.ebusiness.catalog.CatalogRequest;
import com.beasys.commerce.ebusiness.catalog.CatalogManager;
import com.beasys.commerce.ebusiness.catalog.ProductItem;
import com.beasys.commerce.ebusiness.catalog.ProductItemKey;
import com.beasys.commerce.ebusiness.catalog.service.item.ProductItemManager;
import com.beasys.commerce.ebusiness.catalog.CatalogException;
import com.beasys.commerce.ebusiness.catalog.CatalogFinderException;

import java.rmi.RemoteException;
import javax.ejb.CreateException;
import javax.ejb.Handle;
import javax.naming.NamingException;

import com.beasys.commerce.ebusiness.catalog.CatalogRequest;
import com.beasys.commerce.ebusiness.catalog.CatalogManager;
import com.beasys.commerce.ebusiness.catalog.CatalogManagerHome;
import com.beasys.commerce.ebusiness.catalog.util.CatalogResourceManager;
import com.beasys.commerce.util.Logger;
import com.beasys.commerce.webflow.PipelineSessionConstants;

import com.bea.p13n.appflow.exception.PipelineException;
import com.bea.p13n.appflow.exception.InvalidArgumentException;
import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.pipeline.PipelineComponent;
import com.bea.p13n.appflow.pipeline.PipelineComponentSupport;
import com.bea.p13n.appflow.common.internal.AppflowConstants;
import com.bea.p13n.management.ApplicationHelper;
import com.bea.p13n.util.JndiHelper;

/**
 * DummyCheckoutIP checks to see if the
 * <code>PipelineSessionConstants.SHOPPING_CART</code> attribute in
 * the pipeline session has been set.  If it is <code>null</code>,
 * then it is updated to hold a new <code>ShoppingCart</code> object.
 * This is used to initialize the shopping cart for the first time.
 * Simply logs in a user, always the same, and adds
 * a couple of items to the shopping cart.
 *
 * @see com.beasys.commerce.ebusiness.shoppingcart.ShoppingCart
 * @see com.beasys.commerce.webflow.CommerceInputProcessor
 */
public class DummyCheckoutIP extends InputProcessorSupport
{
    /**
     * See class documentation above.
     *
     * @return String containing "success" if the operation was
     * successful
     * @param request <code>HttpServletRequest</code> object for the
     * current request
     */
    public Object process(HttpServletRequest request, Object requestContext ) throws ProcessingException
    {
        // Get pipeline session
	PipelineSession pipelineSession = SessionManagerFactory.getSessionManager().getPipelineSession(request);
	String namespace = (String) pipelineSession.getDefaultNamespace();
        System.out.println("in DummyCheckoutIP: namespace = " + namespace);
	
        // Get the shopping cart from the pipeline session
        ShoppingCart shoppingCart = (ShoppingCart) getSessionAttribute(PipelineSessionConstants.SHOPPING_CART, 
								       namespace, pipelineSession);
	
        // If shopping cart is not in the pipeline session, create one
        if(shoppingCart == null) {
            shoppingCart = ShoppingCartHome.create();
//            setSessionAttribute(PipelineSessionConstants.USER_NAME, "democustomer", namespace, pipelineSession);
        }

        // SET THE DEMOCUSTOMER as the logged in customer


        // Putting in a FAKE product, set it in the session for consumption
        // by 'getProductItemPC'
        ProductItemKey key = new ProductItemKey("pix1000");
	setRequestAttribute(PipelineSessionConstants.CATALOG_ITEM_KEY, key, 
			    namespace, requestContext, pipelineSession);


        System.out.println("Added a product item key to the request: " + key);

        ProductItem item = null;
        try {
            // Retrieve the ProductItem with the given key and place it into the 
            // PipelineSession in the request scope
            CatalogManager manager = getCatalogManager(pipelineSession);
            CatalogRequest catrequest = getCatalogRequest(pipelineSession);
            ProductItemManager itemManager = manager.getProductItemManager(catrequest);
            item = itemManager.getItem(catrequest, key);
	    setRequestAttribute(PipelineSessionConstants.CATALOG_ITEM, item, namespace, requestContext, 
				pipelineSession);
        } catch (Exception ignore) {}

        // Add the item to the shopping cart
        try {
            shoppingCart.addProductItem(item, 2);
        } catch (Exception iae) {
            throw new ProcessingException("ebusiness-shoppingcart:invalid.argument.exception.in.add.product.item.to.shoppingcart",iae);
        }

        System.out.println("in DummyCheckoutIP: returning success");
        System.out.println("\n\n---------------------------------");

       // Operation succeeded
        return "success";
    }
	
    // Catalog Manager JNDI
    private static final String CATALOG_MANAGER_JNDI_NAME = "." + 
	CatalogResourceManager.getString( "catalog.pipeline.catalog.manager" );

    static final boolean DEBUG = false;

    // Default ViewIterator view size, loaded from the Catalog resource file
    static final Integer DEFAULT_VIEW_SIZE = CatalogResourceManager.getInteger("catalog.jsp.default.iterator.size");

    /**
     * Retrieves a cached <code>CatalogRequest</code> from the
     * <code>PipelineSession</code>.  If a <code>CatalogRequest</code>
     * is not currently cached, an instance is created and added to
     * the <code>PipelineSession</code> in the session scope.
     *
     * @param pipelineSession The PipelineSession
     * @throws CreateException on CalalogManager creation error
     * @throws RemoteException on remote errror
     */
    public static CatalogRequest getCatalogRequest(PipelineSession pipelineSession) 
	 throws CreateException, RemoteException, PipelineException, NamingException
    {
 
	String namespace = (String) pipelineSession.getDefaultNamespace();
        
        CatalogRequest request = null;
        
        try {
            request = (CatalogRequest) pipelineSession.getSessionScopedAttribute(PipelineSessionConstants.CATALOG_REQUEST, 
										 namespace);
        }
        catch (com.bea.p13n.appflow.exception.InvalidArgumentException e) {
            throw new PipelineException("Invalid argument", e);
        }
        
        if(request == null) {
            if(DEBUG) {
                Logger.getInstance().debug("In CatalogPC.getCatalogRequest(): Creating a new CatalogRequest.");
            }

            // Create the CatalogRequest and add it to the PipelineSession in the session scope.
            request = getCatalogManager(pipelineSession).createCatalogRequest();

	    try {
            	pipelineSession.setSessionScopedAttribute(PipelineSessionConstants.CATALOG_REQUEST, request, namespace);
	    }
	    catch (com.bea.p13n.appflow.exception.InvalidArgumentException e) {
            	throw new PipelineException("set.invalid.argument.exception", e);
	    }
        }
        return request;
    }

    /**
     * Retrieves a cached <code>CatalogManager</code> from the <code>PipelineSession</code>.  
     * If a <code>CatalogManager</code> is not currently cached, an instance of the
     * <code>CatalogManager</code> is added to the <code>PipelineSession</code> in the
     * session scope.
     *
     * @param pipelineSession The PipelineSession
     * @throws CreateException on CalalogManager creation error
     * @throws RemoteException on remote error
     */
    public static CatalogManager getCatalogManager(PipelineSession pipelineSession) 
	throws NamingException, CreateException, RemoteException, PipelineException
    {
        CatalogManager catalogManager = null;
	Handle managerHandle = null;
        
        String namespace = (String)pipelineSession.getContext().getCurrentNamespace();
    
        try {
            // Attempt to get a cached Handle out of the session
	    try {
		managerHandle = (Handle) pipelineSession.getSessionScopedAttribute(PipelineSessionConstants.CATALOG_MANAGER, namespace);
	    }
	    catch (com.bea.p13n.appflow.exception.InvalidArgumentException e) {
		throw new PipelineException("get.invalid.argument.exception", e);
    	    }
	    
            // Attempt to dereference the Handle to an EJBObject
            if(managerHandle != null) {
		catalogManager = (CatalogManager) managerHandle.getEJBObject();            
	    }
        }
        catch( Exception e ) {
	}

        // If we fail to dereference the handle, we try to recreate it
        // below.  this is a workaround as Handles are not
        // clusterable, so the machine with the original bean may have
        // failed, but if we recreate the mananger we can potentially
        // get an EJBObject from another machine in the cluster
        
        if(catalogManager == null) {
            if(DEBUG) {
                Logger.getInstance().debug("In CatalogPC.getCatalogManager() : creating new CatalogManager.");
	    }

            // Create the CatalogManager and add it to the PipelineSession in the session scope.
            String ref = ApplicationHelper.getApplicationName() +
              CATALOG_MANAGER_JNDI_NAME;
	    CatalogManagerHome home = (CatalogManagerHome) JndiHelper.lookupNarrow( ref,
										   CatalogManagerHome.class);
            catalogManager = home.create();
            
            // Save the new handle back into the session
	    try {
            	pipelineSession.setSessionScopedAttribute(PipelineSessionConstants.CATALOG_MANAGER, 
							  catalogManager, namespace);
	    }
	    catch (com.bea.p13n.appflow.exception.InvalidArgumentException e) {
            	throw new PipelineException("set.invalid.argument.exception", e);
	    }
        }
        
        // Return the manager
        return catalogManager;
    }
}
