/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 */
package examples.e2e.b2c.shoppingcart.webflow;

import java.util.Iterator;
import javax.servlet.http.HttpServletRequest;

import com.beasys.commerce.ebusiness.shoppingcart.*;
import com.beasys.commerce.ebusiness.catalog.ProductItem;
import com.beasys.commerce.ebusiness.shoppingcart.ShoppingCart;
import com.beasys.commerce.ebusiness.shoppingcart.ShoppingCartHome;
import com.beasys.commerce.ebusiness.shoppingcart.ShoppingCartLine;
import com.beasys.commerce.ebusiness.catalog.ProductItem;

import com.bea.p13n.appflow.exception.*;
import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.webflow.SessionManagerFactory;
import com.bea.p13n.appflow.webflow.InputProcessorSupport;

import com.beasys.commerce.foundation.pipeline.PipelineConstants;
import com.beasys.commerce.webflow.PipelineSessionConstants;
import com.beasys.commerce.webflow.HttpRequestConstants;

/**
 * EmptyShoppingCartIP sets the
 * <code>PipelineSessionConstants.SHOPPING_CART</code> attribute in
 * the pipeline session to hold a new <code>ShoppingCart</code>
 * object.  This has the effect of emptying the shopping cart.
 *
 * @see com.beasys.commerce.ebusiness.shoppingcart.ShoppingCart
 * @see com.beasys.commerce.webflow.CommerceInputProcessor
 */
public class EmptyShoppingCartIP extends InputProcessorSupport
{
    /**
     * See class documentation above.
     *
     * @return String containing "success" if the operation was
     * successful
     * @param request <code>HttpServletRequest</code> object for the current request
     */
    public Object process(HttpServletRequest request, Object requestContext) throws ProcessingException
    {

	PipelineSession pipelineSession = SessionManagerFactory.getSessionManager().getPipelineSession(request);
	String namespace = (String) pipelineSession.getDefaultNamespace();
	
        // Gget the shopping cart from the pipeline session
	ShoppingCart shoppingCart = (ShoppingCart) getSessionAttribute(PipelineSessionConstants.SHOPPING_CART,
								       namespace, pipelineSession);
	
        // If no shopping cart, no event to fire, nothing to do
        if(shoppingCart == null) {
            return "success";
        }

        // Fire events for each of the 'updated' SKUs
        ProductItem[] updatedItems = new ProductItem[shoppingCart.getShoppingCartLineCollection().size()];
        double[] deltas = new double[shoppingCart.getShoppingCartLineCollection().size()];
        int i = 0;
	
        // Loop through all the shopping cart lines and set the new quantities
        Iterator iterator = shoppingCart.getShoppingCartLineCollection().iterator();
        while(iterator.hasNext()) {
            ShoppingCartLine shoppingCartLine = (ShoppingCartLine) iterator.next();
            ProductItem productItem = shoppingCartLine.getProductItem();
            updatedItems[i] = productItem;

            String sku = productItem.getKey().getIdentifier();

            // If this fails, the delta will be 0
            try {
               deltas[i] = - shoppingCart.findShoppingCartLine(sku).getQuantity();
            }
	    catch(Exception ignore)
            {
            }

            i++;
        }

        // Get ready for the Pipeline
        setSessionAttribute(PipelineSessionConstants.UPDATED_QUANTITY_DELTAS, deltas, 
			    namespace, pipelineSession );   
        setSessionAttribute(PipelineSessionConstants.UPDATED_PRODUCT_ITEMS, updatedItems,
			    namespace, pipelineSession );   

        // Set the shopping cart attribute with a newly created empty shopping cart
        setSessionAttribute(PipelineSessionConstants.SHOPPING_CART, ShoppingCartHome.create(), 
			    namespace, pipelineSession);

        // Operation succeeded
        return "success";
    }
}
