/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 */
package examples.e2e.b2c.shoppingcart.webflow;

import javax.servlet.http.HttpServletRequest;

import com.beasys.commerce.ebusiness.shoppingcart.ShoppingCart;
import com.beasys.commerce.ebusiness.shoppingcart.ShoppingCartHome;

import com.bea.p13n.appflow.exception.*;
import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.webflow.SessionManagerFactory;
import com.bea.p13n.appflow.webflow.InputProcessorSupport;

import com.beasys.commerce.foundation.pipeline.PipelineConstants;
import com.beasys.commerce.webflow.PipelineSessionConstants;
import com.beasys.commerce.webflow.HttpRequestConstants;

/**
 * InitShoppingCartIP checks to see if the
 * <code>PipelineSessionConstants.SHOPPING_CART</code> attribute in
 * the pipeline session has been set.  If it is <code>null</code>,
 * then it is updated to hold a new <code>ShoppingCart</code> object.
 * This is used to initialize the shopping cart for the first time.
 *
 * @see com.beasys.commerce.ebusiness.shoppingcart.ShoppingCart
 * @see com.beasys.commerce.webflow.CommerceInputProcessor
 */
public class InitShoppingCartIP extends InputProcessorSupport
{
    /**
     * See class documentation above.
     *
     * @return String containing "success" if the operation was
     * successful
     * @param request <code>HttpServletRequest</code> object for the
     * current request
     */
    public Object process(HttpServletRequest request, Object requestContext ) throws ProcessingException
    {
        // Get pipeline session
	PipelineSession pipelineSession = SessionManagerFactory.getSessionManager().getPipelineSession(request);
	String namespace = (String) pipelineSession.getDefaultNamespace();
	
        // Get the shopping cart from the pipeline session
        ShoppingCart shoppingCart = (ShoppingCart) getSessionAttribute(PipelineSessionConstants.SHOPPING_CART, 
								       namespace, pipelineSession);
	
        // If shopping cart is not in the pipeline session, create one
        if(shoppingCart == null) {
            shoppingCart = ShoppingCartHome.create();
            setSessionAttribute(PipelineSessionConstants.SHOPPING_CART, shoppingCart, namespace, pipelineSession);
        }

        // Operation succeeded
        return "success";
    }
}
