/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 */

package examples.e2e.b2c.shoppingcart.webflow;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.exception.ProcessingException;
import com.bea.p13n.appflow.webflow.SessionManagerFactory;
import com.bea.p13n.appflow.webflow.InputProcessorSupport;
import com.beasys.commerce.foundation.pipeline.PipelineConstants;
import com.beasys.commerce.webflow.PipelineSessionConstants;

/**
 * <code>SessionInitializationIP</code> stores the sessionid, user
 * name and the store front name (aka the current web-app name) in the
 * <code>PipelineSession</code>.
 */
public class SessionInitializationIP extends InputProcessorSupport
{
   /**
    * See the class documentation above.
    *
    * @throws
    * com.beasys.commerce.foundation.exception.InvalidSessionStateException
    * If the session is new.
    */
    public Object process(HttpServletRequest req, Object requestContext) 
		throws ProcessingException
    {

	String namespace = null;
	
    	try {
	    PipelineSession pSession = SessionManagerFactory.getSessionManager().getPipelineSession(req);
	    HttpSession session = req.getSession(true);
	    
	    namespace = (String) pSession.getDefaultNamespace();
	    
	    // Insert session ID in pipeline session (needed for Behavior Tracking)
	    setSessionAttribute(PipelineSessionConstants.HTTP_SESSION_ID, session.getId(), namespace, pSession);
	    
	    // Insert the web-app name    
	    String webAppName = req.getContextPath();
	    
	    // The web-app name starts with a "/" (ex: /wlcs)
	    // So remove the "/"
	    webAppName = webAppName.substring(1);
	    setSessionAttribute(PipelineSessionConstants.STOREFRONT, webAppName, namespace, pSession);
	    
	    // Insert the user name: Here at this point we know that
	    // the user name usually will be null. When the user logs
	    // on we can insert the real user name
	    String sessionUserName = null;
	    String realUserName = null;
	    
	    sessionUserName = (String) getSessionAttribute(PipelineSessionConstants.USER_NAME, namespace, pSession);
	    
	    realUserName = req.getRemoteUser();
	    
	    if(sessionUserName == null || !sessionUserName.equals(realUserName)) {
		if(realUserName != null) {
		    setSessionAttribute(PipelineSessionConstants.USER_NAME, realUserName, namespace, pSession);
		}
	    }
	    
	    session.setAttribute(PipelineConstants.PIPELINE_ATTRIBUTE_NAME, pSession);
	    
    	}
        catch(Exception e) {
	    // Return "success" to let the application continue
	    // execution.
        }
        
        return "success";     
    }
 }
