/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 */
package examples.e2e.b2c.shoppingcart.webflow;
import examples.e2e.b2c.util.B2CPortalConstants;
import javax.servlet.http.HttpServletRequest;

import com.bea.p13n.appflow.exception.ProcessingException;
import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.webflow.SessionManagerFactory;
import com.bea.p13n.appflow.webflow.InputProcessorSupport;
import com.bea.portal.appflow.PortalAppflowConstants;

import com.beasys.commerce.foundation.pipeline.PipelineConstants;
import com.beasys.commerce.webflow.PipelineSessionConstants;
import com.beasys.commerce.webflow.HttpRequestConstants;

/**
 * UpdateSkuIP takes a
 * <code>HttpRequestConstants.CATALOG_ITEM_SKU</code> parameter from
 * the <code>HttpServletRequest</code> and copies it into the
 * <code>PipelineSessionConstants.CATALOG_ITEM_SKU</code> attribute in
 * the pipeline session.
 *
 * @see com.beasys.commerce.ebusiness.shoppingcart.ShoppingCart
 * @see com.beasys.commerce.webflow.CommerceInputProcessor
 */
public class UpdateSkuIP extends InputProcessorSupport
{
    /**
     * See class documentation above.
     *
     * @param request <code>HttpServletRequest</code> object for the
     * current request
     * @return String containing "success" if the operation was
     * successful
     * @throws ProcessingException If the
     * <code>HttpRequestConstants.CATALOG_ITEM_SKU</code> parameter is
     * not available in the request
     */
    public Object process(HttpServletRequest request, Object requestContext) throws ProcessingException
    {
        // Extract the sku from the http session
	PipelineSession pipelineSession = SessionManagerFactory.getSessionManager().getPipelineSession(request);
	String namespace = (String) pipelineSession.getDefaultNamespace();
	
        String sku = (String)request.getParameter(HttpRequestConstants.CATALOG_ITEM_SKU);
        if(sku == null) {
            throw new ProcessingException("ebusiness-shoppingcart:sku.was.null");
        }
	
        // Add the sku attribute to the pipeline session
        setSessionAttribute(PipelineSessionConstants.CATALOG_ITEM_SKU, sku, namespace, pipelineSession);
	
        /* Get the portlet webflow event from the HTTP request */
        String event =
            (String) request.getParameter( PortalAppflowConstants.PORTLET_WEBFLOW_EVENT_PARAMETER );

        // Depending on the type of event, we store a specific doc portlet location attribute
        setSessionAttribute( B2CPortalConstants.DOC_PORTLET_LOCATION, "step1." + event, namespace, pipelineSession );

        // Operation succeeded
        return "success";
    }
}
