/*
 * B E A   S Y S T E M S
 *
 * P O R T A L
 *
 * Copyright (c) 2000-2002  BEA Systems, Inc.
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws
 * of the United States. The software contained on this media is proprietary
 * to and embodies the confidential technology of BEA Systems, Inc. The
 * possession or receipt of this information does not convey any right to disclose
 * its contents, reproduce it, or use,  or license the use, for manufacture or
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/b2cPortal/WEB-INF/src/examples/e2e/b2c/tourguide/B2CTourGuide.java,v 1.2 2002/05/02 00:22:10 toneill Exp $
 */

package examples.e2e.b2c.tourguide;

import java.io.Serializable;
import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.servlet.jsp.JspException;

import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.webflow.SessionManagerFactory;
import com.bea.portal.render.servlets.jsp.PortalRenderHelper;
import com.bea.portal.render.servlets.jsp.PortletRenderHelper;

import examples.e2e.common.tourguide.TourGuide;
import examples.e2e.common.tourguide.TourGuideData;
import examples.e2e.b2c.util.B2CPortalConstants;

/**
 * This class decides what content to display in the Tour Guide
 * portlet based on the state of the b2cPortal. It also mantains
 * information about each instance of the Tour Guide. There is 
 * one Tour Guide portlet instance per portal page.
 */
public class B2CTourGuide implements TourGuide 
{
    /* Protected fields */

    /** Stores instance data for each Tour Guide portlet in the app */
    protected HashMap instanceData;


    /* Constructors */

    public B2CTourGuide()
    {
        /* Initialize the instance data for each portal page */

        instanceData = new HashMap();

        instanceData.put( B2CPortalConstants.MY_AVITEK_PAGE_NAME,
                          new TourGuideData() );
        instanceData.put( B2CPortalConstants.PRODUCTS_PAGE_NAME,
                          new TourGuideData() );
        instanceData.put( B2CPortalConstants.SHOPPING_CART_PAGE_NAME,
                          new TourGuideData() );
    }


    /* Public methods */

    public TourGuideData determineState( HttpServletRequest request )
        throws JspException
    {
        String state = null;
        String codeFile = null;

        /* Retrieve appropriate state information about the b2cPortal */

        HttpSession httpSession = request.getSession();

        PipelineSession pSession = 
            SessionManagerFactory.getSessionManager().getPipelineSession( request) ;

        String currentPage = 
            PortalRenderHelper.getPageIdentifier( request ).getName();

        String catalogEvent = 
            (String) pSession.getSessionScopedAttribute( B2CPortalConstants.LAST_CATALOG_EVENT_ATTRIB, 
                                                         "portal" );

        String checkoutEvent = null;

        if ( ! PortletRenderHelper.isMaximized( request ) )
        {
            checkoutEvent = (String) pSession.getSessionScopedAttribute( B2CPortalConstants.DOC_PORTLET_LOCATION, "portal" );
        }
        else
        {
            checkoutEvent = (String) pSession.getSessionScopedAttribute( B2CPortalConstants.DOC_PORTLET_MAX_LOCATION, "portal" );
        }

        /* Try to determine what content to display */

        if ( currentPage.equals( B2CPortalConstants.MY_AVITEK_PAGE_NAME ) )
        {
            state = "myavitek";
            codeFile = "/portlets/orderhistory/content.jsp";
        }
        else if ( currentPage.equals( B2CPortalConstants.PRODUCTS_PAGE_NAME ) )
        {
            if ( catalogEvent == null )
            {
                state = "products";
                codeFile = "/portlets/catalog/catalog_index.jsp";
            }
            else if ( catalogEvent.equals( "link.category" ) )
            {
                state = "category";
                codeFile = "/portlets/catalog/category.jsp";
            }
            else if ( catalogEvent.equals( "link.item" ) )
            {
                state = "item-details";
                codeFile = "/portlets/catalog/item.jsp";
            }
            else if ( catalogEvent.equals( "button.search" ) )
            {
                state = "search-results";
                codeFile = "/portlets/catalog/search_results.jsp";
            }
            else if ( ( catalogEvent.equals( "button.item.buy" )  ) ||
                      ( catalogEvent.equals( "button.category.buy" ) ) ||
                      ( catalogEvent.equals( "button.search.buy" ) ) )
            {
                state = "buynow";
                codeFile = "/portlets/summarycart/step1.jsp";
            }
            else if ( ( catalogEvent.equals( "button.item.save" ) ) ||
                      ( catalogEvent.equals( "button.category.save" ) ) ||
                      ( catalogEvent.equals( "button.search.save" ) ) )
            {
                state = "savelater";
                codeFile = "/portlets/summarycart/step1.jsp";
            }
            else
            {
                state = "products";
                codeFile = "/portlets/catalog/catalog_index.jsp";
            }
        }
        else if ( currentPage.equals( B2CPortalConstants.SHOPPING_CART_PAGE_NAME ) )
        {
            if ( checkoutEvent == null )
            {
                state = "cart";
                codeFile = "/portlets/checkout/step1.jsp";
            }
            else if ( checkoutEvent.equals( "step1.button.deleteItemFromShoppingCart" ) )
            {
                state = "removed-from-current";
                codeFile = "/portlets/checkout/step1.jsp";
            }
            else if ( checkoutEvent.equals( "step1.button.moveItemToSavedList" ) )
            {
                state = "savelater-current";
                codeFile = "/portlets/checkout/step1.jsp";
            }
            else if ( checkoutEvent.equals( "step1.button.deleteItemFromSavedList" ) )
            {
                state = "removed-from-saved";
                codeFile = "/portlets/checkout/step1.jsp";
            }
            else if ( checkoutEvent.equals( "step1.button.moveItemToShoppingCart" ) )
            {
                state = "added-from-saved";
                codeFile = "/portlets/checkout/step1.jsp";
            }
            else if ( checkoutEvent.equals( "step1.inventory" ) )
            {
                state = "low-inventory";
                codeFile = "/portlets/checkout/step1.jsp";
            }
            else if ( checkoutEvent.equals( "step1.link.cancel" ) )
            {
                state = "cancel";
                codeFile = "/portlets/checkout/step1.jsp";
            }
            else if ( checkoutEvent.equals( "step2" ) )
            {
                state = "checkout";
                codeFile = "/portlets/checkout/step2.jsp";
            }
            else if ( checkoutEvent.equals( "step3" ) )
            {
                state = "order-submission";
                codeFile = "/portlets/checkout/step3.jsp";
            }
            else if ( checkoutEvent.equals( "step4" ) )
            {
                state = "order-confirmation";
                codeFile = "/portlets/checkout/step4.jsp";
            }
            else
            {
                state = "cart";
                codeFile = "/portlets/checkout/step1.jsp";
            }
        }
        else
        {
            /* If nothing else matches display this content */

            state = "default";
            codeFile = "/portlets/tourguide/content.jsp";
        }

        /* Set the Tour Guide instance data for the appropriate portlet */

        TourGuideData data = (TourGuideData) instanceData.get( currentPage );

        data.setState( state );
        data.setCodeFile( codeFile );

        /* Return the Tour Guide instance data to the caller */

        return data;
    }

    public TourGuideData getInstanceData( String pageName )
    {
        return (TourGuideData) instanceData.get( pageName );
    }
}
