<!-- Content from td-products.jsp -->

<p><b>Contents</b>

<ul type="square">
<li><a href="#intro">Introduction</a>
<li><a href="#webflow">Location in Default Webflow</a>
<li><a href="#events">Events</a>
<li><a href="#dynamic">Dynamic Data Display</a>
<li><a href="#admin">Administration Tasks</a>
</ul>

<p><a name="intro">&nbsp;</a>
<b>Introduction</b>

<p>The product cagegories shown in this portion of the sample application demonstrates the integration of a portal user interface (UI) and a product catalog. The application is running in an instance of the WebLogic Server, a Web application server that implements standard J2EE APIs. Subsequent pages in this portion of the sample application take your through a business-to-consumer (B2C) shopping cart and checkout process. Behind the scenes, many technologies are employed. One is a BEA technology known as Webflow and Pipeline that dynamically controls the
flow of processing through the application, in response to events that occurred on the page.


<p><a name="webflow">&nbsp;</a>
<b>Location in Default Webflow</b>

<p>The Webflow for this application displayed this catalog page when you clicked the Products tab. From this page, a logged-in customer can browse the store catalog by clicking on a link to a particular category (which loads the browse.jsp portlet). Customers can also enter keywords and click the Go button to perform a search for a particular product item (which loads the searchresults.jsp portlet).


<p><a name="events">&nbsp;</a>
<b>Events</b>

<p>Every time a customer clicks a link or a button on the catalog.jsp portlet (or those displayed by the included banner file), it is considered an event. Each event triggers a particular response in the default Webflow that allows the customer to continue. While this response can be to load another JSP, it is usually the case that an Input Processor and/or Pipeline is invoked first.

<p>For example, when you click one of the images for the AviPix categories, this triggers an event called link.browse. This event has three types of Webflow responses: browseCategory and moveSiblingResults (called "input processors") and a data pipeline called "getBrowseDetails." The getBrowseDetails pipeline contains a number of pipeline components: GetCategoryPC, GetParentPC, GetSubcategoriesPC, MoveAttributePC, GetCategoryPC2, GetAncestorsPC, GetSubcategoriesPC2, and GetProductItemsPC. 

<p>You do not need to understand the role of all these pipeline components (PCs) just yet. Details about the <a href="http://edocs.bea.com/wlp/docs70/wflopipe/index.htm" target="new">webflow and pipeline technology</a> are provided on the BEA "E-docs" Web site. For now, just know that these particular PCs are used to figure out which top-level categories are in the catalog, which subcategories are in each category, and which product items are in each subcategory. In the case of the sample product catalog, below the catalog's top-level "root" category, we only have one level of subcategories: the six categories that you see on this page.</p> 

<p><b>Getting Data from the Pipeline Session</b>

<p>The primary purpose of the catalog.jsp portlet is to dynamically display Product Catalog data by category. This is accomplished using a combination of Webflow and Catalog JSP tags. WebLogic Portal provides these (and many other) JSP tags in libraries that you can import into your application. 

<p>First, the getProperty Webflow JSP tag obtains the CATALOG_CATEGORY and CATALOG_CATEGORIES properties from the Pipeline Session. 

<p>The following code fragment from the [name].jsp portlet file illustrates how these properties are obtained from the Pipeline Session using the getProperty Webflow JSP tag.</p>

<p><b>Obtaining the CATALOG_CATEGORY and CATALOG_CATEGORIES Properties</b>

&lt;%-- Get the top category from the PipelineSession. --%&gt;
&lt;webflow:getProperty id="topCategory"  property="&lt;%=PipelineSessionConstants.CATALOG_CATEGORY%&gt;"  type="com.beasys.commerce.ebusiness.catalog.Category"  scope="request" namespace="sampleapp_main" /&gt;
&lt;%-- Get the subcategories of the top category from the PipelineSession. --%&gt;
&lt;webflow:getProperty id="subcategories"  property="&lt;%=PipelineSessionConstants.CATALOG_CATEGORIES%&gt;"  type="com.beasys.commerce.ebusiness.catalog.ViewIterator"  scope="request" namespace="sampleapp_main" /&gt;

<p>Next, an iterateViewIterator Catalog JSP tag is used to iterate through all the categories (one at a time). The context for the page is captured by appending values to the previously established browse parameters, and the getProperty Catalog JSP tag is used to list the name of each category on the [name].jsp template.

<p><a name="dynamic">&nbsp;</a>
<b>Dynamic Data Display</b>

<p>&lt;catalog:iterateViewIterator iterator="&lt;%= subcategories %&gt;"  id="currentCategory"  returnType="com.beasys.commerce.ebusiness.catalog.Category"&gt;
  &lt;% String browseParameters = commonParameters +       HttpRequestConstants.CATALOG_CATEGORY_ID + "=" +       java.net.URLEncoder.encode(currentCategory.getKey().      getIdentifier()); %&gt;

<p>[show how the category images are loaded into each table cell in the portlet]


<p><a name="admin">&nbsp;</a>
<b>Administration Tasks</b>

<p>Content to-be-supplied.

<p>&nbsp;</p>

<!-- End content from td-products.jsp -->
