<!-- Content from td-search-results.jsp -->

<p><b>Contents</b>

<ul type="square">
<li><a href="#intro">Introduction</a>
<li><a href="#webflow">Location in Default Webflow</a>
<li><a href="#events">Events</a>
<li><a href="#dynamic">Dynamic Data Display</a>
<li><a href="#admin">Administration Tasks</a>
</ul>

<p><a name="intro">&nbsp;</a>
<b>Introduction</b>

<p>The search_results.jsp portlet presents information about product items that resulted from the keyword-based search. The customer can then browse through the results. Assembling the search results was accomplished using a combination of portlet, webflow, catalog, and "i18n" (internationalization) JSP tags.  

<p>For a match to occur, the customer must have entered a keyword that had already been associated with one or more product items. The assignment of keywords for product items was done by an administrator in the Catalog Management area of the Weblogic Portal Administration Tools. Information about <a name="admin">keywords and the Administration Tools</a> appears later in this discussion.

<blockquote>
<p><b>Note:</b> WebLogic Portal commerce services allow you to use query-based searches, in addition to keyword searches. To simplify the scope of this sample application, only a keyword-based search has been implemented here. For details about query-based searches, see the WebLogic Portal <a href="http://edocs.bea.com/wlp/docs70/catalog/template.htm" target="new">Product Catalog JSP Templates</a> chapter on the BEA Edocs site. Although that chapter shows a different set of sample data, the information about query-based searches is useful.
</blockquote> 

<p>By default, the search_results.jsp portlet file resides in:

<p>BEA_HOME\weblogic700\samples\platform\e2eDomain\beaApps<br>
&nbsp;&nbsp;\e2eApp\b2cPortal\portlets\catalog

<p>The searchform.jsp portlet, which provides the search input box, the Go button's graphic, and processing, resides by default in:

<p>BEA_HOME\weblogic700\samples\platform\e2eDomain\beaApps<br>
&nbsp;&nbsp;\e2eApp\b2cPortal\portlets\search

<p>As you run this sample application, it is important to understand that as a developer you are primarily interested in the JSP code before it was rendered  by the browser. That's why the code fragments in this section, and the View Code link above, describe and show you the pre-rendered JSP file for a particular portlet.

<p><a name="webflow">&nbsp;</a>
<p><b>Location in the Default Webflow</b>

<p>Customers see the search_results.jsp portlet on the Products page after they enter a keyword in the searchform.jsp portlet and click the Go button. From the search_results.jsp portlet, customers can:

<ul type=square>
<li>View more details about a particular item shown in the results list (which loads the items.jsp portlet, which includes content from the itemdetails.jsp portlet)
<li>Add a product item shown in the search results list to their shopping cart by clicking the Buy Now or Save for Later buttons.
</ul>

<p><a name="events">&nbsp;</a>
<p><b>Events</b>

<p>Every time a customer clicks a link or button on a JSP, it is considered an event. Events trigger particular responses in the default Webflow that allow customers to continue. While this response can be to load another JSP, it is usually the case that an Input Processor and/or Pipeline is invoked first. 

<p>On a page that preceded the display of the search results, the user entered a keyword in the search input box and then clicked the Go button. This took place in the searchform.jsp portlet. The form defined in the portlet includes the following:

<p>A table cell containing the search input box.
<p>&lt;input name="&lt;%=HttpRequestConstants.CATALOG_SEARCH_STRING%&gt;" type="text" class="searchPortlet" size="27"&gt;

<p>A scriptlet that defines additional Webflow parameters to display the results on the "Products" page (because the search can be submitted from other portal pages) and executes the search portlet's Webflow:

<p><pre>&lt;%
   String formParams =
       PortalAppflowConstants.PORTLET_WEBFLOW_EVENT_PARAMETER + "=" + "button.search" + "&" +
       PortalAppflowConstants.PAGE_PARAMETER + "=" + B2CPortalConstants.PRODUCTS_PAGE_NAME + "&" +
       PortalAppflowConstants.PORTLET_PARAMETER + "=" + "search" ;
%&gt;</pre>

<p>And the form definition itself:

<p>&lt;form method="POST" action="&lt;webflow:createWebflowURL event="bea.portal.framework.internal.portlet.event" origin="searchform.jsp" extraParams="&lt;%= formParams %&gt;"/&gt;" onsubmit="return ValidateSearchForm(this)" &gt; 

<p>The event <b>button.search</b> is defined separately as using keywordSearchIP (an input processor) in the searchportlet namespace. 

<p>This definition is in the searchportlet.wf (Webflow) file that resides in:

<p>BEA_HOME\weblogic700\samples\platform\e2eDomain\beaApps<br>
&nbsp;&nbsp;\e2eApp-project\application-sync\webapps\b2cPortal

<p>KeywordSearchIP.java takes a keyword search String from the HTTP request, validates the search String, creates a KeywordQuery based on the search String, adds it to the PipelineSession in the session scope, and clears
the PipelineSession of any previous search results. If a keyword search String is not supplied, the PipelineSession will be examined for previous cached
search results. If previous results exist and are valid, the results are left in the PipelineSession.

<p>Once the search results are available, the search_results.jsp portlet page is displayed (see the next section for more on the <a name="#dynamic">dynamic data display</a>).


<p>Now let's look at events defined within the search_results.jsp, for subsequent processing:

<ul type="square">
<li>The <b>link.item</b> event is triggered when the customer either clicks the product item's image (for example, an image of an AviPix 3000 camera) or the item's Details button image.
<li>The <b>button.search.buy</b> event is triggered when the customer clicks the Buy Now button next to a product item in the search results.
<li>The <b>button.search.save</b> event is triggered when the customer clicks the Save for Later button next to a product item in the search results.
</ul>

<p>The following line from search_results.jsp (pre-rendering by the browser) shows how the coding was set up for the Details button on the portlet:

<p>&lt;a href="&lt;portlet:createWebflowURL namespace="catalogportlet" event="link.item" extraParams="&lt;%=linkParams%&lt;" /&gt;"&gt;&lt;img src="&lt;webflow:createResourceURL resource='&lt;%=imagesPath + "prod_step2_details.gif"%&gt;'/&gt;

<p>In the example above, the href link is constructed by using a webflow:createResourceURL JSP tag. Notice that the Webflow namespace is catalogportlet. Let's refer to the catalogportlet.wf (Webflow) file in:

<p>BEA_HOME\weblogic700\samples\platform\e2eDomain\beaApps<br>
&nbsp;&nbsp;\e2eApp-project\application-sync\webapps\b2cPortal

<p>In catalogportlet.wf, the event named link.item identifies the following input processor: storeCatalogEventIP. This input processor is used to store the current portlet event as a session-scoped pipeline attribute. The portlet uses this information to decide which view of the catalog to display. 

<p>For example, if on the search results portlet you clicked the Details button next to an AviPix 1000 camera, notice how the value for the webflow:createResourceURL parameter results in the following. (The URL shown here spans several lines to improve the formatting.)

<p>http://localhost:7501/b2cPortal/application?origin=search_results.jsp<br>
&event=bea.portal.framework.internal.portlet.event<br>
&pageid=Products&portletid=catalog<br>
&wfevent=link.item&wlcs_catalog_item_sku=pix1000

<p><a name="dynamic">&nbsp;</a>
<p><b>Dynamic Data Display</b>

<p>In search_results.jsp, the matched results (if any) will be retrieved from the pipeline session. For example:

<pre>
&lt;webflow:getProperty
    id="searchResults"
    type="com.beasys.commerce.ebusiness.catalog.ViewIterator"
    property="&lt;%= PipelineSessionConstants.CATALOG_SEARCH_RESULTS%&gt;"
    scope="session"
    namespace="portal" /&gt;
</pre>

<p>We check to see if there are search results:

<pre>
&lt;% 
   if ( ( searchResults != null ) && ( searchResults.size() &gt; 0 ) )
   {
%&gt;
</pre>

<p>If there are search results, we iterate through the catalog, retrieving information about each product item that matched the
keyword. For example:

<pre>
&lt;catalog:iterateViewIterator
    id="item"
    returnType="com.beasys.commerce.ebusiness.catalog.ProductItem"
    iterator="&lt;%= searchResults %&gt;"&gt;
  
    &lt;catalog:getProperty
        id="itemImage"
        returnType="com.beasys.commerce.ebusiness.catalog.ImageInfo"
        object="&lt;%= item %&gt;"
        propertyName="image"
        getterArgument="&lt;%= new Integer( CatalogItem.SMALL_IMAGE_INDEX ) %&gt;" /&gt;
</pre>

<p>Other data from the catalog is similarly retrieved for each product item: currentPrice, (short) description, and productItemKey (SKU). 

<p>We then set up the HTTP request parameters for this item's detail link. 

<p>Data for each matched product item is cycled through the view iterator until there are no more items. We also reset the search results view iterator in case the portlet is refreshed.

<p>If there are no matches found for the keyword entered on searchform.jsp, we use in search_results.jsp the i18n:getMessage JSP tag provided by WebLogic Portal. I18N is an abbreviation for Internationalization, meaning the process of setting up application code so that language-specific files can be
added or modified, to customize the message text for non-English
customers. For example, in search_results.jsp, we have:

<p>&lt;i18n:getMessage messageName="noResults" bundleName="search_results" /&gt;

<p>The search_results.properties file, which is in the same directory as search_results.jsp, contains:

<p>noResults=No matches found.

<a name="admin">&nbsp;</a>
<p><b>Administration Tasks</b>

<p>The search function for this sample application uses keywords that were previously defined in the Catalog Management area of the WebLogic Portal Administration Tools. This section of the Technical Details on search_results.jsp gives you background information about this prior administration step.  

<p>In a new browser window, point to the following URL, which starts the WebLogic Portal Administration Tools:

<p>http://&lt;host&gt;:&lt;port&gt;/e2eAppTools/index.jsp

<p>These steps assume that the server for the e2eDomain is still running. You can run the e2eAppTools web application and the sample application at the same time. Both "web applications" are part of the same e2eApp enterprise application and run in the same domain.

<p>Substitute the name of your local machine (localhost) or the remote host name, and substitute the port number on which WebLogic Server is listening. For WebLogic Portal applications, the default port number is 7501. The "e2eAppTools" portion of the URL is case sensitive.

<p>You are prompted for a username and password. These values may be different for your site, but by default they are:

<p>Username: administrator<br>
Password: password

<p>The login values are case sensitive. If these credentials do not work, please check with the administrator, who may have changed these default values for the administrator account after the sample was installed.

<p>On the main Administration Tools page, click the icon in the Catalog Management banner.

<p>On the Catalog Management page, click the underlined word Items.

<p>In the Keywords input box, enter the following keyword: camera

<p>Click one of the underlined search results. For example, click AviPix 5000. (Warning: do not click the <font color="Red">red X icon</font>, which deletes the item from the catalog. Keep in mind that you are using the administration pages for the catalog data.)

<p>On the resulting page, click the Edit icon that is on the same line with "Item Core Attributes." 

<p>Some of the values on the Edit Item Information page will be familiar if you have browsed through the sample application's catalog. For example, notice the short description and long descriptions values. Also notice how the small and large graphics' location was specified (near the bottom of the page). Initially the data records were loaded from scripts or by using the DBloader program provided with WebLogic Portal. These catalog administration pages provide a view into the values and the ability to subsequently change values, if desired. (Most organizations will run catalog data scripts again, instead of using the Administration Tools to modify values.) 

<p>The search keywords can be viewed or modified on this Edit Item Information page. In the sample application, we defined a set of keywords for each product item in the catalog. Scroll down the page to find the Keywords input box and the set of existing keywords. The following graphic shows a portion of the page:

<p><img src="portlets/tourguide/docs/images/keywords.gif" width=581 height=422>

<p>Existing keywords are: avipix, camera, cameras, consumer, digital, pix.

<p>The administrator can work with the development team to determine additional keywords to help site visitors find product items. 

<p>For more information, see the <a href="http://edocs.bea.com/wlp/docs70/catalog/admin.htm" target="new">Catalog Administration Tasks</a> chapter in the WebLogic Portal documentation on the BEA Edocs site. To exit the Administration Tools, close the browser window.

<!-- End content from td-search-results.jsp -->