/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/src/examples/e2e/b2c/catalog/pipeline/GetCategoryPC.java,v 1.2 2002/05/02 00:22:11 toneill Exp $
 */

package examples.e2e.b2c.catalog.pipeline;

import java.rmi.RemoteException;
import javax.ejb.CreateException;
import javax.naming.NamingException;

import com.beasys.commerce.ebusiness.catalog.CatalogManager;
import com.beasys.commerce.ebusiness.catalog.CatalogRequest;
import com.beasys.commerce.ebusiness.catalog.Category;
import com.beasys.commerce.ebusiness.catalog.service.category.CategoryManager;
import com.beasys.commerce.ebusiness.catalog.CategoryKey;
import com.beasys.commerce.ebusiness.catalog.CatalogException;
import com.beasys.commerce.ebusiness.catalog.CatalogFinderException;
import com.beasys.commerce.webflow.PipelineSessionConstants;

import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.pipeline.PipelineComponent;
import com.bea.p13n.appflow.exception.PipelineException;

import com.beasys.commerce.util.Logger;

/**
 * This <code>PipelineComponent</code> extracts a
 * <code>CategoryKey</code> from the <code>PipelineSession</code>,
 * retrieves the <code>Category</code> with the associated
 * <code>CategoryKey</code> from the Catalog, and places the item into
 * the <code>PipelineSession</code> in the session scope.
 *
 * @see com.bea.p13n.appflow.pipeline.PipelineComponent
 * @see com.bea.p13n.appflow.pipeline.PipelineSession
 * @see com.beasys.commerce.ebusiness.catalog.CategoryKey
 * @see com.beasys.commerce.ebusiness.catalog.Category
 * @see com.beasys.commerce.ebusiness.catalog.pipeline.CatalogPC
 * @see com.beasys.commerce.ebusiness.catalog.service.category.CategoryManager
 */
public class GetCategoryPC extends CatalogPC
{
    
    public PipelineSession process( PipelineSession pipelineSession, Object requestContext )
        throws PipelineException, RemoteException
    {
        // Get the CategoryKey from the PipelineSession
    	String namespace = (String) pipelineSession.getDefaultNamespace();
	
	CategoryKey key = (CategoryKey) getSessionAttribute(PipelineSessionConstants.CATALOG_CATEGORY_KEY, 
							    namespace, pipelineSession);

	if(DEBUG) {
	    Logger.getInstance().debug("In GetCategoryPC : got CategoryKey " + key);
	}
	
        try {
            // Retrieve the Category with the given key and place it into the 
            // PipelineSession in the session scope
            CatalogManager manager = getCatalogManager(pipelineSession);
            CatalogRequest request = getCatalogRequest(pipelineSession);
            CategoryManager categoryManager = manager.getCategoryManager(request);
            Category category = categoryManager.getCategory(request, key);
            setSessionAttribute(PipelineSessionConstants.CATALOG_CATEGORY, category, namespace, pipelineSession);
        }
        catch(CatalogFinderException e) {
            throw new PipelineException("wlcs-catalog:catalog.pipeline.fatal.exception:catalogfinder:sampleapp/catalog", e);
        }
        catch(CatalogException e) {
            throw new PipelineException("wlcs-catalog:catalog.pipeline.fatal.exception:catalog:sampleapp/catalog", e);
        }
        catch(NamingException e) {
            throw new PipelineException("wlcs-catalog:catalog.pipeline.fatal.exception:toolkit:sampleapp/catalog", e);
        }
        catch(CreateException e) {
            throw new PipelineException("wlcs-catalog:catalog.pipeline.fatal.exception:create:sampleapp/catalog", e);
        }
        return pipelineSession;
    }
}
