/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/src/examples/e2e/b2c/catalog/pipeline/GetProductItemPC.java,v 1.2 2002/05/02 00:22:11 toneill Exp $
 */

package examples.e2e.b2c.catalog.pipeline;

import java.rmi.RemoteException;
import javax.ejb.CreateException;
import javax.naming.NamingException;

import com.beasys.commerce.ebusiness.catalog.CatalogRequest;
import com.beasys.commerce.ebusiness.catalog.CatalogManager;
import com.beasys.commerce.ebusiness.catalog.ProductItem;
import com.beasys.commerce.ebusiness.catalog.ProductItemKey;
import com.beasys.commerce.ebusiness.catalog.service.item.ProductItemManager;
import com.beasys.commerce.ebusiness.catalog.CatalogException;
import com.beasys.commerce.ebusiness.catalog.CatalogFinderException;
import com.beasys.commerce.webflow.PipelineSessionConstants;

import com.beasys.commerce.util.Logger;

import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.pipeline.PipelineComponent;
import com.bea.p13n.appflow.exception.PipelineException;

/**
 * This <code>PipelineComponent</code> extracts a
 * <code>ProductKey</code> from the <code>PipelineSession</code>,
 * retrieves the <code>ProductItem</code> with the associated
 * <code>ProductKey</code> from the Catalog, and places the item into
 * the <code>PipelineSession</code> in the session scope.
 *
 * @see com.bea.p13n.appflow.pipeline.PipelineComponent
 * @see com.bea.p13n.appflow.pipeline.PipelineSession
 * @see com.beasys.commerce.ebusiness.catalog.ProductItemKey
 * @see com.beasys.commerce.ebusiness.catalog.ProductItem
 * @see com.beasys.commerce.ebusiness.catalog.pipeline.CatalogPC
 * @see com.beasys.commerce.ebusiness.catalog.service.item.ProductItemManager;
 */
public class GetProductItemPC extends CatalogPC
{

    public PipelineSession process( PipelineSession pipelineSession, Object requestContext )
        throws PipelineException, RemoteException
    {
	String namespace = (String) pipelineSession.getDefaultNamespace();
	
        // Get the ProductItemKey from the PipelineSession
	ProductItemKey key = (ProductItemKey) getSessionAttribute(PipelineSessionConstants.CATALOG_ITEM_KEY, 
								  namespace, pipelineSession);

	if(DEBUG) {
	    Logger.getInstance().debug("In GetProductItemPC : got ProductItemKey " + key);
	}

        try {
            // Retrieve the ProductItem with the given key and place it into the 
            // PipelineSession in the session scope
            CatalogManager manager = getCatalogManager(pipelineSession);
            CatalogRequest request = getCatalogRequest(pipelineSession);
            ProductItemManager itemManager = manager.getProductItemManager(request);
            ProductItem item = itemManager.getItem(request, key);
	    setSessionAttribute(PipelineSessionConstants.CATALOG_ITEM, item, namespace, 
				pipelineSession);
        }
        catch(CatalogFinderException e) {
            throw new PipelineException("wlcs-catalog:catalog.pipeline.fatal.exception:sampleapp/catalog", e);
        }
        catch(CatalogException e) {
            throw new PipelineException("wlcs-catalog:catalog.pipeline.fatal.exception:sampleapp/catalog", e);
        }
        catch(NamingException e) {
            throw new PipelineException("wlcs-catalog:catalog.pipeline.fatal.exception:sampleapp/catalog", e);
        }
        catch(CreateException e) {
            throw new PipelineException("wlcs-catalog:catalog.pipeline.fatal.exception:sampleapp/catalog", e);
        }
        
        return pipelineSession;
    }
}
