/*
 * B E A   S Y S T E M S
 *
 * C O M M E R C E   C O M P O N E N T S
 *
 * Copyright (c) 2000-2002  BEA Systems, Inc.
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws
 * of the United States. The software contained on this media is proprietary
 * to and embodies the confidential technology of BEA Systems, Inc. The
 * possession or receipt of this information does not convey any right to disclose
 * its contents, reproduce it, or use,  or license the use, for manufacture or
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/src/examples/e2e/b2c/catalog/pipeline/SearchPC.java,v 1.2 2002/05/02 00:22:11 toneill Exp $
 */

package examples.e2e.b2c.catalog.pipeline;

import java.rmi.RemoteException;
import javax.ejb.CreateException;
import javax.naming.NamingException;

import com.beasys.commerce.util.Logger;
import com.beasys.commerce.util.Log;
import com.beasys.commerce.ebusiness.catalog.ViewIterator;
import com.beasys.commerce.ebusiness.catalog.Category;
import com.beasys.commerce.ebusiness.catalog.CategoryKey;
import com.beasys.commerce.ebusiness.catalog.CatalogFinderException;
import com.beasys.commerce.ebusiness.catalog.CatalogException;
import com.beasys.commerce.ebusiness.catalog.CatalogManager;
import com.beasys.commerce.ebusiness.catalog.CatalogRequest;
import com.beasys.commerce.ebusiness.catalog.service.query.CatalogQuery;
import com.beasys.commerce.ebusiness.catalog.service.query.CatalogQueryManager;

import com.beasys.commerce.webflow.PipelineSessionConstants;

import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.pipeline.PipelineComponent;
import com.bea.p13n.appflow.exception.PipelineException;
import com.bea.p13n.util.debug.Debug;

/**
 * This <code>PipelineComponent</code> extracts a
 * <code>CatalogQuery</code> from the <code>PipelineSession</code>,
 * performs a catalog search based upon the <code>CatalogQuery</code>,
 * and places the resultant <code>ViewIterator</code> into the
 * <code>PipelineSession</code> session scope.  The view size of the
 * resultant <code>ViewIterator</code> may be specified using an
 * optional <code>PipelineSession</code> request attribute named
 * <code>PipelineSessionConstants.CATALOG_VIEW_SIZE</code>.  If search
 * results already exist in the <code>PipelineSession</code>, the
 * previous results are returned.
 *
 * @see com.bea.p13n.appflow.pipeline.PipelineComponent
 * @see com.bea.p13n.appflow.pipeline.PipelineSession
 * @see com.beasys.commerce.ebusiness.catalog.ViewIterator
 * @see com.beasys.commerce.ebusiness.catalog.pipeline.CatalogPC
 * @see com.beasys.commerce.ebusiness.catalog.service.query.CatalogQueryManager
 */
public class SearchPC extends CatalogPC
{
    
    public PipelineSession process( PipelineSession pipelineSession, Object requestContext )
        throws PipelineException, RemoteException
    {
        // If a previous search result exists in the PipelineSession, simply return
	String namespace = (String) pipelineSession.getDefaultNamespace();
	if(getSessionAttribute(PipelineSessionConstants.CATALOG_SEARCH_RESULTS, namespace, 
			       pipelineSession) != null) {
            return pipelineSession;
        }

        // Get the view size from the PipelineSession
	Integer viewSize = (Integer) getRequestAttribute(PipelineSessionConstants.CATALOG_VIEW_SIZE, 
							 namespace, requestContext, pipelineSession);
        if(viewSize == null) {
            viewSize = DEFAULT_VIEW_SIZE;
        }

        // Get the CatalogQuery from the PipelineSession
	CatalogQuery query = (CatalogQuery) getSessionAttribute(PipelineSessionConstants.CATALOG_QUERY,
								namespace, pipelineSession);

        try {
            // Perform the query using the CatalogQuery and place the resultant ViewIterator into
            // the PipelineSession
            CatalogManager manager = getCatalogManager(pipelineSession);
            CatalogRequest request = getCatalogRequest(pipelineSession);
            CatalogQueryManager catalogQueryManager = manager.getCatalogQueryManager(request);
            ViewIterator results = catalogQueryManager.search(request, query, viewSize.intValue());
	    setSessionAttribute(PipelineSessionConstants.CATALOG_SEARCH_RESULTS, results, 
				namespace, pipelineSession);

        }
        catch(CatalogFinderException e) {
            Logger.getInstance().log(Log.LOG_WARNING, "Exception was thrown in SearchPC ",  e);
            throw new PipelineException("catalog.pipeline.fatal.exception", e);
        }
        catch(CatalogException e) {
            Logger.getInstance().log(Log.LOG_WARNING, "Exception was thrown in SearchPC ",  e);
            throw new PipelineException("sampleapp/catalog", e);
        }
        catch(NamingException e) {
            Logger.getInstance().log(Log.LOG_WARNING, "Exception was thrown in SearchPC ",  e);
            throw new PipelineException("sampleapp/catalog", e);
        }
        catch(CreateException e) {
            Logger.getInstance().log(Log.LOG_WARNING, "Exception was thrown in SearchPC ",  e);
            throw new PipelineException("sampleapp/catalog", e);
        }

        return pipelineSession;
    }
}
