/*
 * B E A   S Y S T E M S
 *
 * C O M M E R C E   C O M P O N E N T S
 *
 * Copyright (c) 2000-2002  BEA Systems, Inc.
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws
 * of the United States. The software contained on this media is proprietary
 * to and embodies the confidential technology of BEA Systems, Inc. The
 * possession or receipt of this information does not convey any right to disclose
 * its contents, reproduce it, or use,  or license the use, for manufacture or
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 */

package examples.e2e.b2c.order.pipeline;

import java.rmi.RemoteException;
import java.sql.SQLException;
import java.util.Set;
import java.util.ArrayList;
import java.util.Iterator;
import javax.ejb.Handle;
import javax.ejb.FinderException;
import javax.naming.NamingException;

import com.beasys.commerce.ebusiness.order.Order;
import com.beasys.commerce.ebusiness.order.OrderPk;
import com.beasys.commerce.ebusiness.order.OrderHome;
import com.beasys.commerce.ebusiness.order.OrderValue;
import com.beasys.commerce.ebusiness.order.Order;
import com.beasys.commerce.ebusiness.order.OrderManager;
import com.beasys.commerce.ebusiness.order.OrderManagerHome;
import com.beasys.commerce.ebusiness.order.OrderLine;
import com.beasys.commerce.ebusiness.order.OrderLineHome;

import com.beasys.commerce.axiom.util.weblogic.helper.WebLogicHelper;
import java.sql.*;


import com.beasys.commerce.webflow.PipelineSessionConstants;
import examples.e2e.b2c.webflow.CommercePipelineComponentSupport;
import examples.e2e.b2c.util.B2CPortalConstants;

import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.pipeline.PipelineComponent;
import com.bea.p13n.appflow.exception.PipelineException;

import com.bea.p13n.management.ApplicationHelper;
import com.bea.p13n.util.JndiHelper;

/**
 * This pipeline component is responsible for persisting the order.
 * It expects the following artifacts in the PipelineSession:
 *
 * <ul>
 * <li>ShoppingCart</li>
 * <li>User name</li>
 * <li>ShippingAddress</li>
 * <li>ShippingMethod</li>
 * <li>SplittingPreference</li>
 * </ul>
 *
 * After validating that these artifacts exist in the PipelineSession,
 * it creates an Order entity. Subsequent
 * PipelineComponent objects can access the Order
 * entity using the Order Handle from the PipelineSession.
 */
public class CommitPaymentTxPC extends CommercePipelineComponentSupport
{
    /**
     * See class documentation above.
     *
     * @param pipelineSession the <code>PipelineSession</code> objects that the component should use
     * @return The modified PipelineSession object
     * @throws com.beasys.commerce.foundation.exception.PipelineFatalException if any of the expected
     *          attributes are not present in the pipeline session, or if the shopping cart was empty
     */
    public PipelineSession process(PipelineSession pipelineSession, Object requestContext) 
	throws PipelineException
    {

	String namespace = (String) pipelineSession.getDefaultNamespace();
	
        // Get OrderValue
        OrderValue orderValue = (OrderValue)getRequestAttribute( PipelineSessionConstants.ORDER_VALUE,
                                                              namespace, requestContext, pipelineSession);

        // In this particular example, we simply persist the 
        // specific transaction ID, as opposed to the overall
        // Payment Transaction, which is managed by the 'Cajun'
        // Web Service. In other words, we do not create
        // a 'PaymentTransaction' object, we simply persist
        // the ID returned from the Cajun world.
        // Get special instructions from the pipeline session

        // 1. retrieve the ID from the Cajun world
	Long txId = (Long) getSessionAttribute(B2CPortalConstants.PAYMENT_AUTH_CAPTURE_SETTLE_RESULT, 
								  namespace, pipelineSession);
        PreparedStatement ps = null;

        // Build the SQL update statement
        String sqlStmt = "UPDATE WLCS_ORDER SET WLCS_ORDER.TRANSACTION_ID ='" + txId.toString()
        + "' WHERE WLCS_ORDER.ORDER_ID ='" + orderValue.identifier + "'";


        // Execute the SQL statement
        Connection connection = null;
        try
        {
            connection = WebLogicHelper.getInstance().getConnection();
            ps = connection.prepareStatement(sqlStmt);
            ps.executeUpdate();
        } catch ( Exception e )
        {
            // If an exception occurred, the tx column will keep the 'null'
            // which is ok
        } finally
        {
            // Make sure that whatever happens, the connection is closed
            try
            {
                ps.close ();
            } catch (Exception e)
            {
            }

            try
            {
                connection.close ();
            } catch ( Exception e )
            {
            }
        }

        // Return modified pipeline session
        return pipelineSession;
    }
}
