/*
 * B E A   S Y S T E M S
 *
 * C O M M E R C E   C O M P O N E N T S
 *
 * Copyright (c) 2000-2002  BEA Systems, Inc.
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws
 * of the United States. The software contained on this media is proprietary
 * to and embodies the confidential technology of BEA Systems, Inc. The
 * possession or receipt of this information does not convey any right to disclose
 * its contents, reproduce it, or use,  or license the use, for manufacture or
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 */

package examples.e2e.b2c.order.pipeline;

import com.bea.p13n.appflow.pipeline.PipelineComponentSupport;
import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.exception.*;
import com.beasys.commerce.ebusiness.order.OrderValue;
import com.beasys.commerce.webflow.PipelineSessionConstants;
import com.bea.p13n.exceptions.*;
import java.rmi.RemoteException;
import javax.ejb.CreateException;
import javax.naming.NamingException;

import examples.e2e.common.purchase.spi.*;
import com.bea.p13n.util.JndiHelper;
import com.bea.p13n.management.ApplicationHelper;

/**
  * 
  * Note: The order representation does not include order/orderline
  * "adjustments".
  * 
  **/

public class ConvertOrderRepPC extends PipelineComponentSupport 
{

    private static final String PURCHASE_MGR_JNDI = ".BEA_commerce.PurchaseManager";


    public PipelineSession process(
				   PipelineSession pipelineSession,
				   Object requestContext)
	throws PipelineException {

	OrderValue ov = null;
    PurchaseManagerHome purchaseMgrHome= null;
    
	//get the namespace
	String namespace = (String) pipelineSession.getDefaultNamespace();

	try {
       // Get OrderValue
       ov = (OrderValue) getRequestAttribute( PipelineSessionConstants.ORDER_VALUE, namespace, 
				   requestContext, pipelineSession );

       ov.customerHandle = ov.customer.getHandle();  

	} catch (InvalidArgumentException iae) {
	    throw new PipelineException("ebusiness-order:invalid.argument.exception", iae);
	} catch (RemoteException re) {
	    throw new PipelineException("ebusiness-order:remote.exception", re);
    }

	if (ov == null) {
	    throw new PipelineException("ebusiness-order:order.missing.exception");
	}

    try {
        String ref = ApplicationHelper.getApplicationName() + PURCHASE_MGR_JNDI;
        purchaseMgrHome = (PurchaseManagerHome) JndiHelper.lookupNarrow(ref, PurchaseManagerHome.class);
        PurchaseManager purchaseMgr = purchaseMgrHome.create(); 
        purchaseMgr.queueOrder(ov);
    } catch (NamingException ne) {
	    throw new PipelineException("ebusiness-order:naming.exception", ne);
    } catch (CreateException ce) {
	    throw new PipelineException("ebusiness-order:create.exception", ce);
    } catch (IllegalArgumentException ie) {
	    throw new PipelineException("ebusiness-order:illegal.argument.exception", ie);
    } catch (ApplicationException ae) {
	    throw new PipelineException("ebusiness-order:application.exception", ae);
    } catch (RemoteException re) {
	    throw new PipelineException("ebusiness-order:remote.exception", re);
    }
	
	return pipelineSession;
    }
    
}
