/*
 * B E A   S Y S T E M S

 * C O M M E R C E   C O M P O N E N T S
 *
 * Copyright (c) 2000-2002  BEA Systems, Inc.
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws
 * of the United States. The software contained on this media is proprietary
 * to and embodies the confidential technology of BEA Systems, Inc. The
 * possession or receipt of this information does not convey any right to disclose
 * its contents, reproduce it, or use,  or license the use, for manufacture or
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/src/examples/e2e/b2c/order/pipeline/FillOrderAdjustmentsPC.java,v 1.2 2002/05/02 00:22:11 toneill Exp $
 */
package examples.e2e.b2c.order.pipeline;

import java.rmi.RemoteException;
import javax.ejb.CreateException;
import javax.naming.NamingException;

import com.beasys.commerce.util.Logger;

import com.beasys.commerce.ebusiness.order.OrderValue;
import com.beasys.commerce.ebusiness.order.OrderManager;
import com.beasys.commerce.ebusiness.order.OrderPk;
import com.beasys.commerce.ebusiness.order.OrderManagerHome;
import com.beasys.commerce.ebusiness.customer.CustomerPk;

import com.beasys.commerce.webflow.PipelineSessionConstants;

import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.pipeline.PipelineComponent;
import com.bea.p13n.appflow.exception.PipelineException;
import com.bea.p13n.appflow.pipeline.PipelineComponentSupport;
import com.bea.p13n.management.ApplicationHelper;
import com.bea.p13n.util.JndiHelper;


/**
 * This pipeline component fills an Order Value's adjustments based on
 * the OrderPK stored in the
 * <code>PipelineSessionConstants.ORDER_ADJSUTMENTS</code> and the
 * OrderValue stored in the
 * <code>PipelineSessionConstants.SELECTED_ORDER</code> attribute is
 * present in the pipeline session.
 *
 * @see com.bea.p13n.appflow.pipeline.PipelineSession
 * @see com.bea.p13n.appflow.webflow.PipelineSessionConstants
 * @see com.beasys.commerce.ebusiness.order.OrderValue
 */
public class FillOrderAdjustmentsPC extends PipelineComponentSupport
{

    /**
     * See class documentation above.
     *
     * @param pipelineSession the <code>PipelineSession</code> object
     * that the component should use
     * @return The modified PipelineSession object
     *
     * @throws
     * com.beasys.commerce.foundation.exception.PipelineFatalException
     * if the user name attribute is missing from the pipeline
     * session, or there are remote exceptions while creating the list
     * of orders.
     */
    public PipelineSession process(PipelineSession pSession, Object requestContext) 
	throws PipelineException
    {
	
	String namespace = (String) pSession.getDefaultNamespace();
	
        // Validate the PipelineSession
	
        // Check for the order pk object.
        String orderIdentifier = (String) getSessionAttribute(PipelineSessionConstants.ORDER_ADJUSTMENTS, 
							      namespace, pSession);
        if(orderIdentifier == null) {
	    Logger.getInstance().debug("order identifier does not exist in the PipelineSession.");
            throw new PipelineException("webflow-targetprocessor:invalid.session");
        }

        // Check for the order value object
        OrderValue orderValue = (OrderValue) getSessionAttribute(PipelineSessionConstants.SELECTED_ORDER, 
								 namespace, pSession);
        if(orderValue == null) {
	    Logger.getInstance().debug("order value does not exist in the PipelineSession.");
            throw new PipelineException("webflow-targetprocessor:invalid.session");
        }
	
        // Use it to get all orders
        OrderManagerHome orderManagerHome = null;
        OrderManager orderManager= null;

        // Get the order home
        try {
	    String omHome = ApplicationHelper.getApplicationName() + ".BEA_commerce.OrderManager";
            orderManagerHome = (OrderManagerHome) JndiHelper.lookupNarrow(omHome, OrderManagerHome.class);
        }
        catch(NamingException ne) {
	    Logger.getInstance().debug("OrderManagerHome lookup failed.");
            throw new PipelineException("ebusiness-order:order.lookup.exception:com.beasys.commerce.ebusiness.order.OrderManagerHome", ne);
        }

        // Fill the order adjustments for presentations purposes.
        try {
            orderManager = orderManagerHome.create();
            orderManager.queryOrderAdjusments(new OrderPk(orderIdentifier), orderValue);
        }
        catch(CreateException fe) {
	    Logger.getInstance().debug("Order(s) not found for Customer-ID " + orderIdentifier);
            throw new PipelineException("ebusiness-order:orderManager.findByCustomer.exception:" + orderIdentifier, fe);
        }
        catch(RemoteException re) {
	    Logger.getInstance().debug("Order retrieval failed");
            throw new PipelineException("ebusiness-order:orderManager.findByCustomer.remoteexception" + orderIdentifier, re);
        }
	
        setSessionAttribute(PipelineSessionConstants.SELECTED_ORDER, orderValue, namespace, pSession);
	
        // Return the modified pipeline session
        return pSession;
    }
}
