/*
 * B E A   S Y S T E M S
 *
 * C O M M E R C E   C O M P O N E N T S
 *
 * Copyright (c) 2000-2002  BEA Systems, Inc.
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws
 * of the United States. The software contained on this media is proprietary
 * to and embodies the confidential technology of BEA Systems, Inc. The
 * possession or receipt of this information does not convey any right to disclose
 * its contents, reproduce it, or use,  or license the use, for manufacture or
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 */

package examples.e2e.b2c.order.pipeline;

import java.util.Iterator;
import java.util.ArrayList;
import java.util.Collection;
import java.rmi.RemoteException;
import javax.ejb.CreateException;
import javax.naming.NamingException;

import com.bea.commerce.ebusiness.price.service.Line;
import com.bea.commerce.ebusiness.price.service.PricingReply;
import com.bea.commerce.ebusiness.price.service.PriceService;
import com.bea.commerce.ebusiness.price.service.PriceServiceHome;
import com.bea.commerce.ebusiness.price.service.PricingRequestImpl;
import com.bea.commerce.ebusiness.price.service.PricingException;
import com.beasys.commerce.ebusiness.customer.CustomerPk;
import com.beasys.commerce.ebusiness.shoppingcart.ShoppingCart;
import com.beasys.commerce.ebusiness.shoppingcart.ShoppingCartLine;
import com.beasys.commerce.ebusiness.shoppingcart.ShoppingCartConstants;
import com.beasys.commerce.ebusiness.order.OrderConstants;
import com.beasys.commerce.ebusiness.catalog.ProductItem;
import com.beasys.commerce.axiom.units.Money;  

import com.beasys.commerce.webflow.PipelineSessionConstants;

import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.pipeline.PipelineComponent;
import com.bea.p13n.appflow.exception.PipelineException;
import com.bea.p13n.appflow.pipeline.PipelineComponentSupport;
import com.bea.p13n.management.ApplicationHelper;
import com.bea.p13n.util.JndiHelper;

/**
 * This pipeline component invokes the Pricing Service to price each of
 * the shopping cart lines and compute the subtotal.
 *
 * @see com.beasys.commerce.ebusiness.shoppingcart.pipeline.DeleteProductItemFromSavedListPCHome
 * @see com.beasys.commerce.ebusiness.shoppingcart.pipeline.DeleteProductItemFromSavedListPCImpl
 * @see com.bea.p13n.appflow.pipeline.PipelineComponent
 * @see com.bea.p13n.appflow.pipeline.PipelineSession
 * @see com.beasys.commerce.webflow.PipelineSessionConstants
 * @see com.beasys.commerce.ebusiness.shoppingcart.ShoppingCart
 */
public class PriceOrderPC extends PipelineComponentSupport 
{

    private final String PRICE_SERVICE_HOME = ".BEA_commerce.PriceService";

    public PipelineSession process(PipelineSession pipelineSession, Object requestContext) throws PipelineException
    {
	String namespace = (String) pipelineSession.getDefaultNamespace();
	
        // Get the shopping cart from the pipeline session
        ShoppingCart shoppingCart = (ShoppingCart) getSessionAttribute(PipelineSessionConstants.SHOPPING_CART,
								       namespace, pipelineSession);

        // Nothing to do if no shopping cart or an empty on.
        if(shoppingCart == null || shoppingCart.isEmpty()) {
            return pipelineSession; 
        }

        // Prepare the argument to the pricing engine
        PricingRequestImpl pricingRequest = new PricingRequestImpl();

        // Get the customerId, sku, and saved list attributes from the pipeline session
        String customerId = (String) getSessionAttribute(PipelineSessionConstants.USER_NAME, 
							 namespace, pipelineSession);
	
        // If the customerId is null, the PricingRequest's customerId is null
        if(customerId != null) {
            pricingRequest.setCustomerPK(new CustomerPk(customerId) );
        }

        // Set the lines on the request.
        Iterator lineIterator = shoppingCart.shoppingCartLineCollectionIterator();
        Line[] lines = new Line[shoppingCart.shoppingCartLineCollectionSize( )];
        int i = 0;

        while(lineIterator.hasNext()) {
            ShoppingCartLine scl = (ShoppingCartLine) lineIterator.next();
            ProductItem pi = scl.getProductItem();
            double q = scl.getQuantity();
            lines[i] = new Line(pi, q);
            i++;
        }
        pricingRequest.setLines(lines);  

        // Fill the shipping info. OrderConstants.ORDER_SHIPPING)
        try {
            shoppingCart.setShipping( shoppingCart.getTotal(OrderConstants.LINE_SHIPPING + 
							    OrderConstants.ORDER_SHIPPING) );
        }
        catch(Exception e) {
            // An empty shipping price is passed over. Takea appropriate action.
        }

        pricingRequest.setBaseShipping(shoppingCart.getShipping());

        // Make the call to the Pricing Service
        PricingReply pricingReply = null;
        String ref = ApplicationHelper.getApplicationName() +
          PRICE_SERVICE_HOME;
        try {
            PriceServiceHome psHome = (PriceServiceHome) JndiHelper.lookupNarrow(ref,
										 PriceServiceHome.class);
            PriceService ps = psHome.create();
            pricingReply = ps.computeTotal(pricingRequest);
        }
        catch(PricingException pe) {
            throw new PipelineException("ebusiness-order:create.priceservice.remote.exception", pe);
        }
        catch(javax.ejb.CreateException ce) {
            throw new PipelineException("ebusiness-order:order.lookup.exception" + ref, ce);
        }
        catch(NamingException ne) {
            throw new PipelineException("ebusiness-order:order.lookup.exception" + ref, ne);
        }
        catch(RemoteException re) {
            throw new PipelineException("ebusiness-order:create.priceservice.remote.exception", re);
        }

        // Use the returned values to set the attributes.
        Line[] returnLines = pricingReply.getLines();
        lineIterator = shoppingCart.shoppingCartLineCollectionIterator();
	
        // Use the returned value from the Pricing Service to
        // fill the discounted price
        i = 0;
        while(lineIterator.hasNext()) {
            ShoppingCartLine scl = (ShoppingCartLine) lineIterator.next();
            scl.setDiscountPresentations( (ArrayList) returnLines[i].getDiscountDescriptions() );
            scl.setDiscountedPrice(returnLines[i].getTotal());
            i++;
        }

        // Set the discounted shipping cost on the shopping cart.
        Money actualShipping = pricingReply.getActualShipping();
        shoppingCart.setShipping( actualShipping );

        // add the shipping presentation objects to the shopping cart
        // CR060165.
        shoppingCart.setShippingDiscountPresentations(
            new ArrayList( pricingReply.getShippingDiscounts() ) );

        // Set the shopping cart subtotal.
        shoppingCart.setSubtotal( pricingReply.getSubtotal() );

        // set the total from the pricing service
        Money total = pricingReply.getTotal();
        shoppingCart.setTotal(total);    

        // Set the adjustments in the session for use by next PC
        setSessionAttribute(PipelineSessionConstants.ORDER_ADJUSTMENTS, pricingReply.getQuote(), 
			    namespace, pipelineSession );

        // Set the discountId and global discount id sets in the session for use by next PC
        setSessionAttribute(PipelineSessionConstants.DISCOUNT_IDS, pricingReply.getUserDiscounts(), 
			    namespace, pipelineSession );

        // Set the adjustments in the session for use by next PC
        setSessionAttribute(PipelineSessionConstants.GLOBAL_DISCOUNT_IDS, pricingReply.getGlobalDiscounts(), 
			    namespace, pipelineSession );

        return pipelineSession;
    }

}
