/*
 * B E A   S Y S T E M S

 * C O M M E R C E   C O M P O N E N T S
 *
 * Copyright (c) 2000-2002  BEA Systems, Inc.
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws
 * of the United States. The software contained on this media is proprietary
 * to and embodies the confidential technology of BEA Systems, Inc. The
 * possession or receipt of this information does not convey any right to disclose
 * its contents, reproduce it, or use,  or license the use, for manufacture or
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/src/examples/e2e/b2c/order/pipeline/RefreshOrderHistoryPC.java,v 1.2 2002/05/02 00:22:11 toneill Exp $
 */
package examples.e2e.b2c.order.pipeline;

import java.rmi.RemoteException;
import javax.ejb.CreateException;
import javax.naming.NamingException;

import com.beasys.commerce.util.Logger;
import com.beasys.commerce.axiom.util.ToolkitException;
import com.beasys.commerce.ebusiness.order.OrderManager;
import com.beasys.commerce.ebusiness.order.OrderManagerHome;
import com.beasys.commerce.ebusiness.customer.CustomerPk;
import com.beasys.commerce.webflow.PipelineSessionConstants;
import com.beasys.commerce.ebusiness.util.ScrollableModel;

import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.pipeline.PipelineComponent;
import com.bea.p13n.appflow.exception.PipelineException;
import com.bea.p13n.appflow.pipeline.PipelineComponentSupport;
import com.bea.p13n.management.ApplicationHelper;
import com.bea.p13n.util.JndiHelper;

/**
 * This pipeline component finds all orders made by a given customer and stores
 * them in the <code>PipelineSessionConstants.ORDER_HISTORY</code> pipeline session attribute.
 * It requires that the <code>PipelineSessionConstants.USER_NAME</code> attribute is present
 * in the pipeline session.
 *
 * @see com.beasys.commerce.foundation.pipeline.PipelineSession
 * @see com.beasys.commerce.webflow.PipelineSessionConstants
 * @see com.beasys.commerce.ebusiness.order.OrderValue
 */
public class RefreshOrderHistoryPC extends PipelineComponentSupport
{
    /**
     * See class documentation above.
     *
     * @param pipelineSession the <code>PipelineSession</code> object
     * that the component should use
     * @return The modified PipelineSession object
     *
     * @throws
     * com.beasys.commerce.foundation.exception.PipelineFatalException
     * if the user name attribute is missing from the pipeline
     * session, or there are remote exceptions while creating the list
     * of orders.
     */
    public PipelineSession process(PipelineSession pSession, Object requestContext) throws
        PipelineException
    {

	String namespace = (String) pSession.getDefaultNamespace();
	
        // Validate the PipelineSession
	
        // Check for the user name object.
        String customerIdentifier = (String) getSessionAttribute(PipelineSessionConstants.USER_NAME,
								 namespace, pSession);

        if(customerIdentifier == null) {
            log("User name does not exist in the PipelineSession.");
            throw new PipelineException("webflow-targetprocessor:invalid.session");
        }

        // Use it to get all orders
        OrderManagerHome orderManagerHome = null;
        OrderManager orderManager= null;

        // Get the order home
        try {
	    String omHome = ApplicationHelper.getApplicationName() + ".BEA_commerce.OrderManager";
            orderManagerHome = (OrderManagerHome) JndiHelper.lookupNarrow(omHome, OrderManagerHome.class);
        }
        catch(NamingException ne) {
            throw new PipelineException("ebusiness-order:order.lookup.exception:com.beasys.commerce.ebusiness.order.OrderManagerHome", ne);
        }

        // Find orders for customer as a scrollable model, for a page size of
        // 10
        ScrollableModel sm;
        try {
            // This will never be null as an empty vector is returned
            // for no matches
            orderManager = orderManagerHome.create();
            sm = orderManager.findByCustomer(new CustomerPk(customerIdentifier), 10);
        }
        catch(CreateException fe) {
            throw new PipelineException("ebusiness-order:orderManager.findByCustomer.exception:" + customerIdentifier, fe);
        }
        catch(RemoteException re) {
            throw new PipelineException("ebusiness-order:orderManager.findByCustomer.remoteexception:" + customerIdentifier, re);
        }
	
        setSessionAttribute(PipelineSessionConstants.SCROLLABLE_MODEL, sm, namespace, pSession);

        // Return the modified pipeline session
        return pSession;
    }

    /**
     * A utility method to log messages
     * @param String the message to be printed
     */
    private void log(String message)
    {
    	Logger.getInstance().debug(message);
    }

}
