/*
 * B E A   S Y S T E M S

 * C O M M E R C E   C O M P O N E N T S
 *
 * Copyright (c) 2000-2002  BEA Systems, Inc.
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws
 * of the United States. The software contained on this media is proprietary
 * to and embodies the confidential technology of BEA Systems, Inc. The
 * possession or receipt of this information does not convey any right to disclose
 * its contents, reproduce it, or use,  or license the use, for manufacture or
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/src/examples/e2e/b2c/order/pipeline/RefreshPaymentHistoryPC.java,v 1.2 2002/05/02 00:22:11 toneill Exp $
 */


package examples.e2e.b2c.order.pipeline;

import java.util.LinkedList;
import java.util.Iterator;
import java.util.Collection;
import java.rmi.RemoteException;
import javax.ejb.FinderException;
import javax.naming.NamingException;

import com.beasys.commerce.util.Logger;
import com.beasys.commerce.ebusiness.order.Order;
import com.beasys.commerce.ebusiness.order.OrderHome;
import com.beasys.commerce.ebusiness.util.ScrollableModel;

import com.beasys.commerce.webflow.PipelineSessionConstants;

import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.pipeline.PipelineComponent;
import com.bea.p13n.appflow.exception.PipelineException;
import com.bea.p13n.appflow.pipeline.PipelineComponentSupport;
import com.bea.p13n.management.ApplicationHelper;
import com.bea.p13n.util.JndiHelper;

/**
 * This pipeline component finds payments related to a given customer
 * and stores them in the
 * <code>PipelineSessionConstants.PAYMENT_HISTORY</code> attribute in
 * the pipeline session as a Collection of PaymentTransactionValue
 * objects.  The <code>PipelineSessionConstants.USER_NAME</code>
 * attribute is required to be in the pipeline session.
 *
 * @see com.beasys.commerce.ebusiness.payment.PaymentTransactionValue
 * @see com.beasys.commerce.foundation.pipeline.PipelineSession
 * @see com.beasys.commerce.webflow.PipelineSessionConstants
 */
public class RefreshPaymentHistoryPC extends PipelineComponentSupport
{
    /**
     * See class documentation above.
     *
     * @param pipelineSession The <code>PipelineSession</code> object
     * that the component should use
     * @return the updated <code>PipelineSession</code> object with a list of <code>PaymentTransactionValue</code> objects in it
     * @throws
     * com.beasys.commerce.foundation.exception.PipelineFatalException
     * if the user name attribute is missing from the pipeline
     * session, or there are remote exceptions while creating the list
     * of payments.
     */
    public PipelineSession process(PipelineSession pSession, Object requestContext) throws
        PipelineException
    {
	String namespace = (String) pSession.getDefaultNamespace();
	
        // Validate the PipelineSession
	
        // Check for the customer value object.
        String customerIdentifier = (String) getSessionAttribute(PipelineSessionConstants.USER_NAME, 
								 namespace, pSession);
        if(customerIdentifier == null) {
            throw new PipelineException("webflow-targetprocessor:invalid.session");
        }

        // From the customer value object extract the customer
        // identifier and use it to get all orders
        OrderHome orderHome = null;

        // Get the order home
        try {
	    String omHome = ApplicationHelper.getApplicationName() + ".BEA_commerce.Order";
            orderHome = (OrderHome) JndiHelper.lookupNarrow(omHome, OrderHome.class);
        }
        catch(NamingException ne)
        {
            throw new PipelineException("ebusiness-order:order.lookup:com.beasys.commerce.ebusiness.order.OrderHome", ne);
        }

        // Find orders for customer
        Collection orderCollection = null;
        try {
            // This will never be null as an empty vector is returned
            // for no matches
            orderCollection = orderHome.findByCustomer(customerIdentifier);
        }
        catch(FinderException fe) {
            throw new PipelineException("ebusiness-order:order.findByCustomer.exception:" + 
					customerIdentifier, fe);
        }
        catch(RemoteException re) {
            throw new PipelineException("ebusiness-order:order.findByCustomer.remoteexception:" + 
					customerIdentifier, re);
        }

        // Put the collection into a list and stuff the latter
        // into the Pipeline session
        Iterator iterator = orderCollection.iterator();
        LinkedList paymentList = new LinkedList();
        while(iterator.hasNext()) {
            try {
                paymentList.add(((Order) iterator.next()).getPaymentTransaction().getPaymentTransactionByValue());
            }
            catch(RemoteException re) {
                throw new PipelineException("ebusiness-order:value.object.exception", re);
            }
        }

        setRequestAttribute(PipelineSessionConstants.PAYMENT_HISTORY, paymentList, namespace, 
			    requestContext, pSession);

        return pSession;
    }

    /**
     * A utility method to print out messages
     * @param String the message to be printed
     */
    private void log(String message)
    {
    	Logger.getInstance().debug(message);
    }

}
