/*
 * B E A   S Y S T E M S

 * C O M M E R C E   C O M P O N E N T S
 *
 * Copyright (c) 2000-2002  BEA Systems, Inc.
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws
 * of the United States. The software contained on this media is proprietary
 * to and embodies the confidential technology of BEA Systems, Inc. The
 * possession or receipt of this information does not convey any right to disclose
 * its contents, reproduce it, or use,  or license the use, for manufacture or
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/src/examples/e2e/b2c/order/pipeline/ResetCheckoutPC.java,v 1.2 2002/05/02 00:22:11 toneill Exp $
 */


package examples.e2e.b2c.order.pipeline;

import com.beasys.commerce.axiom.contact.CreditCard;
import com.beasys.commerce.webflow.PipelineSessionConstants;

import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.pipeline.PipelineComponent;
import com.bea.p13n.appflow.exception.*;
import com.bea.p13n.appflow.pipeline.PipelineComponentSupport;

/**
 * Resets the pipeline session attributes that relate to a customer's
 * checkout.
 *
 * <p> This component removes the following attributes from the pipeline session :
 *
 * <ul>
 * <li><code>PipelineSessionConstants.SHOPPING_CART</code></li>
 * <li><code>PipelineSessionConstants.SHIPPING_ADDRESS</code></li>
 * <li><code>PipelineSessionConstants.SPLITTING_PREFERENCE</code></li>
 * <li><code>PipelineSessionConstants.SHIPPING_METHOD</code></li>
 * <li><code>PipelineSessionConstants.SPECIAL_INSTRUCTIONS</code></li>
 * <li><code>PipelineSessionConstants.ORDER_HANDLE</code></li>
 * <li><code>PipelineSessionConstants.PAYMENT_AUTHORIZATION_AMOUNT</code></li>
 * <li><code>PipelineSessionConstants.AVS_SHIPPING_ADDRESS</code></li>
 * <li><code>PipelineSessionConstants.PAYMENT_CREDIT_CARD</code></li>
 * </ul>
 */
public class ResetCheckoutPC extends PipelineComponentSupport
{
    /**
     * See class documentation above.
     *
     * @param pipelineSession <code>PipelineSession</code> object that
     * the pipeline component will use
     *
     * @return The modified <code>PipelineSession</code> object
     */
    public PipelineSession process(PipelineSession pipelineSession, Object requestContext) throws PipelineException
    {
	String namespace = (String) pipelineSession.getDefaultNamespace();
	
	try {
	    // Remove shopping cart
    	    pipelineSession.removeSessionScopedAttribute(PipelineSessionConstants.SHOPPING_CART, 
							 namespace);
	    
	    // Remove shipping address
	    pipelineSession.removeSessionScopedAttribute(PipelineSessionConstants.SHIPPING_ADDRESS, 
							 namespace);
	    
	    // Remove splitting preference
	    pipelineSession.removeSessionScopedAttribute(PipelineSessionConstants.SPLITTING_PREFERENCE, 
							 namespace);
	    
	    // Remove shipping method
	    pipelineSession.removeSessionScopedAttribute(PipelineSessionConstants.SHIPPING_METHOD, 
							 namespace);
	    
	    // Remove special instructions
	    pipelineSession.removeSessionScopedAttribute(PipelineSessionConstants.SPECIAL_INSTRUCTIONS, 
							 namespace);
	    
	    // Remove order handle
	    pipelineSession.removeRequestScopedAttribute(PipelineSessionConstants.ORDER_HANDLE, 
							 namespace, requestContext);
	    
	    // Remove authorization amount
	    pipelineSession.removeSessionScopedAttribute(PipelineSessionConstants.PAYMENT_AUTHORIZATION_AMOUNT, 
							 namespace);
	    
	    // Remove avs shipping address
	    pipelineSession.removeSessionScopedAttribute(PipelineSessionConstants.AVS_SHIPPING_ADDRESSES,
							 namespace);
	    
	    // Get the credit card number and store it in the session
	    // before we remove the credit card from the pipeline
	    // session
	    CreditCard cc = (CreditCard) getSessionAttribute(PipelineSessionConstants.PAYMENT_CREDIT_CARD, 
							     namespace, pipelineSession);
	    setSessionAttribute(PipelineSessionConstants.CREDITCARD_KEY, cc.getDisplayNumber(), namespace, 
				pipelineSession);
	    
	    // Remove payment creditcard
	    pipelineSession.removeSessionScopedAttribute(PipelineSessionConstants.PAYMENT_CREDIT_CARD, 
							 namespace);
	} catch (InvalidArgumentException e) {
	    throw new PipelineException("error in ResetCheckoutPC");
	}
	
        // return modified pipeline session
        return pipelineSession;
    }
}
