/*
 * B E A   S Y S T E M S
 *
 * C O M M E R C E   C O M P O N E N T S
 *
 * Copyright (c) 2000-2002  BEA Systems, Inc.
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws
 * of the United States. The software contained on this media is proprietary
 * to and embodies the confidential technology of BEA Systems, Inc. The
 * possession or receipt of this information does not convey any right to disclose
 * its contents, reproduce it, or use,  or license the use, for manufacture or
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/src/examples/e2e/b2c/payment/CreditCardServiceImpl.java,v 1.2 2002/05/02 00:22:11 toneill Exp $
 */


package examples.e2e.b2c.payment;

import java.rmi.RemoteException;
import java.sql.SQLException;
import java.sql.Connection;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Vector;
import javax.sql.DataSource;
import javax.ejb.CreateException;
import javax.ejb.SessionContext;
import javax.ejb.EJBException;
import javax.ejb.FinderException;
import javax.naming.NamingException;


import com.beasys.commerce.foundation.SessionImpl;
import com.beasys.commerce.util.Log;
import com.beasys.commerce.util.Logger;

import com.beasys.commerce.foundation.IllegalWorkflowTransitionException;
import com.beasys.commerce.foundation.exception.SystemException;
import com.beasys.commerce.foundation.exception.RemoteSystemException;
import com.beasys.commerce.foundation.exception.CreateSystemException;
import com.beasys.commerce.foundation.exception.FinderSystemException;
import com.beasys.commerce.ebusiness.security.Decryptor;
import com.beasys.commerce.ebusiness.security.DecryptorHome;
import com.beasys.commerce.ebusiness.security.InconsistentStateException;
import com.beasys.commerce.ebusiness.payment.PaymentTransaction;
import com.beasys.commerce.ebusiness.payment.PaymentTransactionPk;
import com.beasys.commerce.ebusiness.payment.PaymentTransactionHome;
import com.beasys.commerce.ebusiness.payment.TransactionEntry;
import com.beasys.commerce.ebusiness.payment.TransactionEntryHome;
import com.beasys.commerce.ebusiness.payment.TransactionConstants;
import com.beasys.commerce.ebusiness.payment.TransactionResponse;
import com.beasys.commerce.ebusiness.payment.TransactionResponse;
import com.beasys.commerce.ebusiness.payment.AuthorizationException;
import com.beasys.commerce.ebusiness.payment.InvalidCreditCardException;
import com.beasys.commerce.ebusiness.payment.GeneralServiceException;
import com.beasys.commerce.ebusiness.payment.ServiceUnavailableException;
import com.beasys.commerce.ebusiness.payment.ServiceConfigurationException;
import com.beasys.commerce.ebusiness.payment.CaptureException;
import com.beasys.commerce.ebusiness.payment.PaymentException;
import com.beasys.commerce.ebusiness.payment.SettleException;
import com.beasys.commerce.ebusiness.payment.CurrencyConvertor;
import com.beasys.commerce.axiom.units.Price;
import com.beasys.commerce.axiom.contact.CreditCard;
import com.beasys.commerce.axiom.contact.CreditCardTypes;
import com.beasys.commerce.axiom.util.helper.CommercePropertiesHelper;
import com.beasys.commerce.util.Sequencer;
import com.beasys.commerce.util.SequencerFactory;

import examples.wlcs.sampleapp.payment.PaymentWebServiceHelper;
import examples.wlcs.sampleapp.payment.PSRequest;
import examples.wlcs.sampleapp.payment.PSResponse;
import examples.wlcs.sampleapp.payment.PSResponseElement;
import examples.wlcs.sampleapp.payment.PaymentServicePort;


/**
 * This stateless session bean is the gateway/wrapper to Payment services.
 *
 * @see com.beasys.commerce.ebusiness.payment.CreditCardService
 * @see com.beasys.commerce.ebusiness.payment.CreditCardServiceHome
 */
public class CreditCardServiceImpl extends SessionImpl implements TransactionConstants
{
    protected static String configFileName = null;
    protected PaymentServicePort ccWebService = null;
    protected PaymentWebServiceHelper pwsHelper = null;

    protected static String PAYMENT_TRANSACTION_HOME = "/ejb/PaymentTransaction";
    protected static String DECRYPTOR_HOME = "/ejb/Decryptor";

    protected static String PAYMENT_MODEL = null;

    protected static String ENCRYPTION_PROPERTY = null;
    protected static String ENCRYPTION_PROPERTY_NAME = "is.encryption.enabled";

    protected static String DEFER_AUTHORIZATION_PROPERTY = null;
    protected static boolean DEFER_AUTHORIZATION = true;

    protected static boolean IS_INITIALIZED = false;

    // Date and currency formats
    protected final static DecimalFormat CURRENCY_FORMAT = new DecimalFormat("#0.00");
    protected final static SimpleDateFormat DATE_FORMAT = new SimpleDateFormat("MM/yy");

    // Debug flag
    private final static boolean DEBUG = false;

    private static final String MERRCODE = "MErrCode";
    private static final String BAD_LUHN_CHECK = "CR-0008";

    public CreditCardServiceImpl()
    {
        super();
    }

    public void ejbCreate() throws CreateException
    {
        super.ejbCreate();

        initialize();
    }

    public void ejbPostCreate() throws CreateException
    {
        super.ejbPostCreate();
    }

    public void ejbActivate() throws EJBException
    {
        super.ejbActivate();
    }

    public void ejbPassivate() throws EJBException
    {
        super.ejbPassivate();
    }

    public void ejbRemove() throws EJBException
    {
        super.ejbRemove();
    }

    public void setSessionContext(SessionContext ctx) throws EJBException
    {
        super.setSessionContext(ctx);
    }

    /**
     * Initializes this bean for commission.
     */
    public void initialize() throws CreateException
    {
        if(!IS_INITIALIZED) {
            pwsHelper = new PaymentWebServiceHelper();

            // Get the value of the encryption property from the WebLogic Commerce Server property file
            if(ENCRYPTION_PROPERTY == null) {
                ENCRYPTION_PROPERTY = CommercePropertiesHelper.getInstance().getProperty(ENCRYPTION_PROPERTY_NAME);
            }

            // Should we defer authorization to the administration tool?
            if(DEFER_AUTHORIZATION_PROPERTY == null) {
                DEFER_AUTHORIZATION_PROPERTY = pwsHelper.getPaymentServiceClientMBean().getDeferAuthorization();

                if(DEFER_AUTHORIZATION_PROPERTY != null && DEFER_AUTHORIZATION_PROPERTY.trim().length() > 0) {
                    DEFER_AUTHORIZATION = DEFER_AUTHORIZATION_PROPERTY.trim().equalsIgnoreCase("true");
                }
            }

            IS_INITIALIZED = true;
        }
    }

    /**
     * Reserve credit on the supplied card.  In this case, the caller
     * must follow up with a second call to complete the transaction.
     * The two possible calls are capture, which bills the customer,
     * and void, which aborts the transaction.  Typically, this
     * procedure is used if the caller cannot guarantee shipment
     * within 24 hours of the purchase.  Authorizes a given payment
     * transaction for the amount associated with the transaction.
     * The payment transaction is updated with the current date and a
     * transaction entry is added to the payment transaction as the
     * result of the authorization attempt.  The modified payment
     * transaction and any service-specific results are then returned.
     *
     * @param card The credit card to be used for the transaction
     * @param amount Amount of the transaction
     * @param useAvs Flag indicating whether to use address verfication
     * @throws InvalidCreditCardException If the associated credit card is invalid
     * @throws AuthorizationException If the associated credit card could not be authorized for the specified amount
     * @throws ServiceUnavailableException If the authorization service is unavailable
     * @throws ServiceConfigurationException If the authorization service is misconfigured
     * @throws GeneralServiceException On general service error
     * @throws RemoteException On remote error
     */
    public TransactionResponse authorize(CreditCard card,
					 Price amount,
					 boolean useAvs)
	throws InvalidCreditCardException, AuthorizationException, ServiceUnavailableException,
	ServiceConfigurationException, PaymentException, GeneralServiceException, RemoteException
    {
        // First create a record of the transaction - if this fails we can avoid going to the processor
        PaymentTransactionHome paymentTransactionHome = getPaymentTransactionHome();
        PaymentTransaction paymentTransaction = null;
        try {
            PaymentTransactionPk pk = new PaymentTransactionPk(generateTransactionId());
            paymentTransaction = paymentTransactionHome.create(pk);
        }
        catch(CreateException ce) {
            // Since a payment transaction could not be created - abort.
            // TransactionResponse is not returned as the pipeline component will
            // throw a PipelineFatalException in this case
            throw new CreateSystemException("foundation", "create.exception", ce);
        }

        // This sets an encrypted credit card on the payment transaction
        paymentTransaction.setCreditCard(card);
        paymentTransaction.setTransactionAmount(amount);

        // Set the date of the paymentTransaction to that of the current date
        paymentTransaction.setTransactionDate(new java.sql.Date(System.currentTimeMillis()));

        // Create argument list for Payment Web Service
        PSRequest psRequest = buildArgsForAuthorize( paymentTransaction, MAUTHONLY, useAvs );

        // Invoke the Payment Web Service then process results
        PSResponse psResponse = getCCWebService().authorize( psRequest );
        TransactionResponse transactionResponse = processAuthorizeResponse( psResponse, paymentTransaction, true );

        return transactionResponse;
    }

    /**
     * This method does the authorization and settle at one go. It
     * represents the business model where soft goods are sold.
     *
     * @param card The credit card to be used for the transaction
     * @param amount Amount of the transaction
     * @param useAvs Flag indicating whether to use address verfication
     * @throws InvalidCreditCardException If the associated credit card is invalid
     * @throws AuthorizationException If the associated credit card could not be authorized for the specified amount
     * @throws ServiceUnavailableException If the authorization service is unavailable
     * @throws ServiceConfigurationException If the authorization service is misconfigured
     * @throws GeneralServiceException On general service error
     * @throws RemoteException On remote error
     */
    public TransactionResponse authorizeAndCapture(CreditCard card,
						   Price amount,
						   boolean useAvs)
	throws AuthorizationException, InvalidCreditCardException, ServiceUnavailableException,
	ServiceConfigurationException, PaymentException, GeneralServiceException, RemoteException
    {
        // First create a record of the transaction - if this fails we can avoid going to the processor
        PaymentTransactionHome paymentTransactionHome = getPaymentTransactionHome();
        PaymentTransaction paymentTransaction = null;
        try {
            PaymentTransactionPk pk = new PaymentTransactionPk(generateTransactionId());
            paymentTransaction = paymentTransactionHome.create(pk);
        }
        catch(CreateException ce) {
            // Since a payment transaction could not be created - abort.
            // TransactionResponse is not returned as the pipeline component will
            // throw a PipelineFatalException in this case
            throw new CreateSystemException("foundation", "create.exception", ce);
        }

        // This sets an encrypted credit card on the payment transaction and the transaction amount
        paymentTransaction.setCreditCard(card);
        paymentTransaction.setTransactionAmount(amount);

        // Set the date of the paymentTransaction to that of the current date
        paymentTransaction.setTransactionDate(new java.sql.Date(System.currentTimeMillis()));

        // For the HOST_AUTH_CAPTURE model, use the MAUTHCAPTURE call.
        // For the HOST_POST_AUTH_CAPTURE mode, use the MAUTHONLY call.
        String authorizationMode = MAUTHONLY;
        if (getPaymentModel().equalsIgnoreCase(HOST_AUTH_CAPTURE)) {
            authorizationMode = MAUTHCAPTURE;
        }

        // Create argument list for Payment Web Service
        PSRequest psRequest = buildArgsForAuthorize( paymentTransaction, authorizationMode, useAvs );

        // Invoke the Payment Web Service then process results
        PSResponse psResponse = getCCWebService().authorizeAndCapture( psRequest );
        TransactionResponse transactionResponse = processAuthorizeResponse( psResponse, paymentTransaction, true );

        return transactionResponse;
    }

    /**
     * Reauthorizes a given payment transaction for the amount
     * associated with the transaction. The payment transaction is
     * updated with the current date and a transaction entry is added
     * to the payment transaction as the result of the authorization
     * attempt. The modified payment transaction and any
     * service-specific results are then returned.
     *
     * @param paymentTransactionId The ID of the payment transaction
     * to reauthorize
     * @param useAvs Flag indicating whether to use address
     * verfication
     * @throws FinderException If the payment transaction with the
     * specified ID could not be found
     * @throws InvalidCreditCardException If the associated credit
     * card is invalid
     * @throws AuthorizationException If the associated credit card
     * could not be authorized for the specified amount
     * @throws ServiceUnavailableException If the authorization
     * service is unavailable
     * @throws ServiceConfigurationException If the authorization
     * service is misconfigured
     * @throws GeneralServiceException On general service error
     * @throws RemoteException On remote error
     */
    public TransactionResponse reauthorize(String paymentTransactionId,
					   boolean useAvs)
	throws FinderException, InvalidCreditCardException, AuthorizationException,
	ServiceUnavailableException, ServiceConfigurationException,
	PaymentException, GeneralServiceException, RemoteException
    {
        // Find the payment transaction using the supplied ID
        PaymentTransactionHome home = getPaymentTransactionHome();
        PaymentTransaction paymentTransaction = home.findByPrimaryKey(new PaymentTransactionPk(paymentTransactionId));

        // Create argument list for Payment Web Service
        PSRequest psRequest = buildArgsForAuthorize( paymentTransaction, MAUTHONLY, useAvs );

        // Invoke the Payment Web Service then process results
        PSResponse psResponse = getCCWebService().reAuthorize( psRequest );
        TransactionResponse transactionResponse = processAuthorizeResponse( psResponse, paymentTransaction, false );

        // Simply call the general authorize method with the payment
        // transaction, the MAUTHONLY authorization method, and the
        // address verification flag.
        return transactionResponse;
    }

    /**
     * Reauthorizes and captures a given payment transaction for the
     * amount associated with the transaction. The payment transaction
     * is updated with the current date and a transaction entry is
     * added to the payment transaction as the result of the
     * authorization attempt. The modified payment transaction and any
     * service-specific results are then returned.
     *
     * @param paymentTransactionId The ID of the payment transaction
     * to reauthorize
     * @param useAvs Flag indicating whether to use address
     * verfication
     * @throws FinderException If the payment transaction with the
     * specified ID could not be found
     * @throws InvalidCreditCardException If the associated credit
     * card is invalid
     * @throws AuthorizationException If the associated credit card
     * could not be authorized for the specified amount
     * @throws ServiceUnavailableException If the authorization
     * service is unavailable
     * @throws ServiceConfigurationException If the authorization
     * service is misconfigured
     * @throws GeneralServiceException On general service error
     * @throws RemoteException On remote error
     */
    public TransactionResponse reauthorizeAndCapture(String paymentTransactionId,
						     boolean useAvs)
	throws FinderException, InvalidCreditCardException, AuthorizationException,
	ServiceUnavailableException, ServiceConfigurationException,
	PaymentException, GeneralServiceException, RemoteException
    {
        // Find the payment transaction using the supplied ID
        PaymentTransactionHome home = getPaymentTransactionHome();
        PaymentTransaction paymentTransaction = home.findByPrimaryKey(new PaymentTransactionPk(paymentTransactionId));

        // For the HOST_AUTH_CAPTURE model, use the MAUTHCAPTURE call.
        // For the HOST_POST_AUTH_CAPTURE mode, use the MAUTHONLY call.
        String authorizationMode = MAUTHONLY;
        if(getPaymentModel().equalsIgnoreCase(HOST_AUTH_CAPTURE)) {
            authorizationMode = MAUTHCAPTURE;
        }

        // Create argument list for Payment Web Service
        PSRequest psRequest = buildArgsForAuthorize( paymentTransaction, authorizationMode, useAvs );

        // Invoke the Payment Web Service then process results
        PSResponse psResponse = getCCWebService().reAuthorizeAndCapture( psRequest );
        TransactionResponse transactionResponse = processAuthorizeResponse( psResponse, paymentTransaction, false );

        return transactionResponse;
    }

    /**
     * Marks a portion of a payment transaction for settlement such
     * that it will be processed in the next batch.  The amount can be
     * less than or equal to the authorization amount.  <br><br>
     * Before a transaction can be settled (and funds transfered into
     * a merchant's account) it must first be captured.  This marks
     * the transaction such that it will be settled as part of a batch
     * process.  <b>This method should be used for terminal-based
     * payment models with a Manual Mark processor configuration</b>.
     * <b>This method should be used for host-based, post
     * authorization and capture payment model</b>.
     *
     * @param paymentTransactionId The ID of the payment transaction
     * to reauthorize.
     * @param amount The amount of the transaction to capture.
     * @return A TransactionResponse that refers to the updated PaymentTransaction.
     * @throws CaptureException If it was not possible to capture the
     * amount from the transaction.
     * @throws ServiceUnavailableException If the underlying service
     * is unreachable.
     * @throws PaymentException On general payment exception.
     * @throws RemoteException On remote error
     */
    public TransactionResponse capture(String transactionId, Price amount)
	throws CaptureException, ServiceUnavailableException, PaymentException, RemoteException
    {
        // Lookup the PaymentTransaction by primary key
        PaymentTransaction paymentTransaction = getPaymentTransaction(transactionId);

        // Call Payment Service with the transaction identifier and capture amount.
        String captureAmount = getCurrencyType(amount) +  CURRENCY_FORMAT.format(amount.getValue());
        String [] arglist = { ORDER_ID, transactionId,
                              AMOUNT, captureAmount };
        PSRequest psRequest = new PSRequest( arglist );
        PSResponse result = getCCWebService().capture( psRequest );

        // Initialize a new TransactionResponse based upon the existing PaymentTransaction
        // and Service-specific information
        TransactionResponse transactionResponse = convertPSResponseToTransactionResponse(result);
        transactionResponse.setTransactionId(paymentTransaction.getTransactionId());
        transactionResponse.setTransactionAmount(paymentTransaction.getTransactionAmount());
        transactionResponse.setTransactionStatus(getCurrentStatus(paymentTransaction.getStatus()));
        transactionResponse.setPaymentTransactionHandle(paymentTransaction.getHandle());
        transactionResponse.setTransactionDate(paymentTransaction.getTransactionDate());

        // Get the payment model
        String paymentModel = getPaymentModel();

        // Check to see if Paymwnt service returned a response, and respond appropriately
        String transactionStatus = transactionResponse.getTransactionResult(MSTATUS);
        if(transactionStatus != null) {
            // Success
            if(transactionStatus.equals(MSTATUS_SUCCESS)||transactionStatus.equals(MSTATUS_SUCCESS_DUPLICATE)) {
                try {
                    // The transaction is marked for settlement
                    paymentTransaction.markForSettle();
                    TransactionEntry transactionEntry = createTransactionEntry(paymentTransaction, amount);
                    paymentTransaction.setTransactionAmount(amount);
		    paymentTransaction.addTransactionEntry(transactionEntry);
                    transactionResponse.setTransactionStatus(getCurrentStatus(paymentTransaction.getStatus()));

                    // In the case of auto settle or host based post authorization capture model, settle the transaction.
                    if(paymentModel.equalsIgnoreCase(MANUAL_MARK_AUTO_SETTLE) ||
                       paymentModel.equalsIgnoreCase(MANUAL_MARK_AUTO_SETTLE_AVS) ||
                       paymentModel.equalsIgnoreCase(HOST_POST_AUTH_CAPTURE) ||
                       paymentModel.equalsIgnoreCase(HOST_POST_AUTH_CAPTURE_AVS))
			{
			    paymentTransaction.settle();
			    TransactionEntry settleTransactionEntry = createTransactionEntry(paymentTransaction, amount);
			    paymentTransaction.setTransactionAmount(amount);
			    paymentTransaction.addTransactionEntry(settleTransactionEntry);
			    transactionResponse.setTransactionStatus(getCurrentStatus(paymentTransaction.getStatus()));
			}
                }
                catch(IllegalWorkflowTransitionException iwte) {
                    throw new SystemException("ebusiness-payment", "workflow.exception", iwte);
                }
            }
            // Hard failure
            else if(transactionStatus.equals(MSTATUS_FAILURE_HARD) ||
		    transactionStatus.equals(MSTATUS_FAILURE_BAD_MONEY)) {
                // The value of MSTATUS is something other than
                // success do nothing in this case - this ensures that
                // the transaction stays in an authorized state and
                // later attempts for settling can be made on it throw
                // a capture exception
                throw new CaptureException("ebusiness-payment", "capture.exception", transactionResponse);
            }
            // Communication failure
            else if(transactionStatus.equals(MSTATUS_FAILURE_Q_OR_CANCEL) ||
		    transactionStatus.equals(MSTATUS_FAILURE_Q_OR_DISCARD))
		{
		    // The value of MSTATUS is something other than
		    // success do nothing in this case - this ensures that
		    // the transaction stays in an authorized state and
		    // later attempts for settling can be made on it throw
		    // a capture exception
		    ServiceUnavailableException unavailableException = new ServiceUnavailableException("ebusiness-payment", "service.unavailable", transactionResponse);
		    Logger.getInstance().log(Log.LOG_WARNING, "CreditCardServiceImpl", unavailableException);
		    throw unavailableException;
		}
        }
        return transactionResponse;
    }

    /**
     * Finalize a transaction by transferring a portion of the funds
     * previously captured from the customer's account to the
     * merchant's account.  The amount can be less than or equal to
     * the captured amount.  <br><br> <b>This method should be used
     * for terminal-based payment models with a Manual Settle
     * processor configuration</b>.
     * @param transactionId The ID of the payment transaction to
     * settle.
     * @param amount The amount of the transaction to mark for
     * settlement.
     * @throws SettleException If the transaction cannot be settled.
     * @throws ServiceUnavailableException If the authorization service is unavailable
     * @throws PaymentException General payment error.
     * @throws RemoteException On remote error
     */
    public TransactionResponse settle(String transactionId, Price amount)
	throws SettleException, ServiceUnavailableException, PaymentException, RemoteException
    {
        // Look up the PaymentTransaction by primary key
        PaymentTransaction paymentTransaction  = getPaymentTransaction(transactionId);

        Vector info = new Vector();

        // Create a settle batch consiting of one transaction
        info.addElement(NUMBER_OF_TRANSACTIONS);
        info.addElement("1");
        info.addElement(ORDER_ID + "-" + 1);
        info.addElement(transactionId);

        // Transaction type is always going to be "marked"
        info.addElement(TRANSACTION_TYPE + "-" + 1);
        info.addElement("marked");

        // Add the amount
        info.addElement(AMOUNT + "-1");
        info.addElement(getCurrencyType(amount) + CURRENCY_FORMAT.format(amount.getValue()));

        // Send the settle command to Payment Service
        String[] args = new String[info.size()];
        for(int i= 0 ; i < info.size();i++)
	    {
		args[i] = (String)info.elementAt(i);
	    }
        PSRequest psRequest = new PSRequest( args );
        PSResponse result = getCCWebService().settle( psRequest );

        // Initialize a new TransactionResponse based upon the existing PaymentTransaction
        // and Service-specific information
        TransactionResponse transactionResponse = convertPSResponseToTransactionResponse(result);
        transactionResponse.setTransactionId(paymentTransaction.getTransactionId());
        transactionResponse.setTransactionAmount(paymentTransaction.getTransactionAmount());
        transactionResponse.setTransactionStatus(getCurrentStatus(paymentTransaction.getStatus()));
        transactionResponse.setPaymentTransactionHandle(paymentTransaction.getHandle());
        transactionResponse.setTransactionDate(paymentTransaction.getTransactionDate());

        try {
            // Set the batch id on the PaymentTransaction
            String transactionBatchIdResult = transactionResponse.getTransactionResult(BATCH_ID);
            if(transactionBatchIdResult!= null) {
                paymentTransaction.setBatchId(transactionBatchIdResult);
            }

            // Get the result for the sole payment transaction
            String transactionStatus = null;
            String transactionStatusResult = transactionResponse.getTransactionResult(RESPONSE_CODE + "-1");
            if(transactionStatusResult!= null) {
                transactionStatus = transactionStatusResult;
            }

            // Get the result for the batch
            String batchStatus = null;
            String batchStatusResult = transactionResponse.getTransactionResult(BATCH_STATUS);
            if(batchStatusResult != null) {
                batchStatus = batchStatusResult;
            }

            // If the batch was accepted, check for the transaction status
            if(batchStatus != null) {
                if(batchStatus.equals(BATCH_ACCEPTED)) {
                    paymentTransaction.settle();
                    TransactionEntry transactionEntry = createTransactionEntry(paymentTransaction, amount);
                    paymentTransaction.setTransactionAmount(amount);
		    paymentTransaction.addTransactionEntry(transactionEntry);
                    transactionResponse.setTransactionStatus(getCurrentStatus(paymentTransaction.getStatus()));
                }
                else {
                    // Keep in the same state so that it can be retried later
                    // so now it will be in a "MarkedForSettle" state
                    throw new SettleException("ebusiness-payment", "settle.exception", transactionResponse);
                }
            }

            // Get the response status from Payment Service
            String transactionMStatus = transactionResponse.getTransactionResult(MSTATUS);
            if(transactionMStatus != null && (transactionMStatus.equals(MSTATUS_FAILURE_Q_OR_CANCEL) ||
					      transactionMStatus.equals(MSTATUS_FAILURE_Q_OR_DISCARD))) {
                // Keep in the same state so that it can be retried later
                // so now it will be in a "MarkedForSettle" state
                ServiceUnavailableException unavailableException = new ServiceUnavailableException("ebusiness-payment", "service.unavailable", transactionResponse);
                Logger.getInstance().log(Log.LOG_WARNING, "CreditCardServiceImpl", unavailableException);
                throw unavailableException;
            }
        }
        catch(IllegalWorkflowTransitionException iwte) {
            throw new SystemException("ebusiness-payment", "workflow.exception", iwte);
        }
        return transactionResponse;
    }

    /**
     * The batch-query message is used to update and reconcile the
     * status of a transaction committed in a given batch.  This
     * message is used to determine if a particular transaction has
     * failed, and is essential for payment processors where the
     * status of an item cannot be determined correctly from the
     * output fields of a batch-commit message.  <br><br> <b> This
     * method is implemented as a pass through to the underlying
     * service provider.  As such all return information is service
     * specific.  Refer to the service provider manual for details on
     * return codes and results.  <\b>
     *
     * @param transactionId The ID of the payment transaction to query
     * status for.
     * @throws RemoteException On remote error.
     */
    public TransactionResponse batchQuery(String transactionId) throws RemoteException, PaymentException
    {
        // Get the PaymentTransaction by primary key
        PaymentTransaction paymentTransaction = getPaymentTransaction(transactionId);

        // Get the batch identifier associated with the PaymentTransaction
        String batchId = paymentTransaction.getBatchId();

        // Send the query to Payment Web Service
        String[] info = { batchId };
        PSRequest psRequest = new PSRequest( info );
        PSResponse result = getCCWebService().batchQuery( psRequest );

        // Initialize a new TransactionResponse based upon the existing PaymentTransaction
        // and Service-specific information
        TransactionResponse transactionResponse = convertPSResponseToTransactionResponse(result);
        transactionResponse.setTransactionId(paymentTransaction.getTransactionId());
        transactionResponse.setTransactionAmount(paymentTransaction.getTransactionAmount());
        transactionResponse.setTransactionStatus(getCurrentStatus(paymentTransaction.getStatus()));
        transactionResponse.setPaymentTransactionHandle(paymentTransaction.getHandle());
        transactionResponse.setTransactionDate(paymentTransaction.getTransactionDate());

        // Get the batch status
        String batchStatus = transactionResponse.getTransactionResult(BATCH_STATUS);

        // If batch accepted check for the transaction status
        try {
            if(batchStatus != null && batchStatus.equals(BATCH_ACCEPTED)) {
                // Get the status for the sole transaction
                String transactionStatus = transactionResponse.getTransactionResult(RESPONSE_CODE + "-1");
                if (transactionStatus != null &&
                    (transactionStatus.equals(MSTATUS_SUCCESS) ||
		     transactionStatus.equals(MSTATUS_SUCCESS_DUPLICATE))) {
                    // Success
                    paymentTransaction.querySuccess();
                    TransactionEntry transactionEntry = createTransactionEntry(paymentTransaction,
									       paymentTransaction.getTransactionAmount());
                    paymentTransaction.addTransactionEntry(transactionEntry);
                    transactionResponse.setTransactionStatus(getCurrentStatus(paymentTransaction.getStatus()));
                }
                else
		    {
			// Failure
			paymentTransaction.queryFail();
			TransactionEntry transactionEntry = createTransactionEntry(paymentTransaction, paymentTransaction.getTransactionAmount());
			paymentTransaction.addTransactionEntry(transactionEntry);
			transactionResponse.setTransactionStatus(getCurrentStatus(paymentTransaction.getStatus()));
		    }
            }
        }
        catch(IllegalWorkflowTransitionException iwte) {
            throw new SystemException("ebusiness-payment", "workflow.exception", iwte);
        }
        return transactionResponse;
    }

    /**
     * Abort a transaction which was previously submitted.  The following transactions may be voided:
     * <UL>
     * <LI>
     * HOST_AUTH_CAPTURE transactions in the pending settlement state.
     * </LI>
     * <LI>
     * HOST_POST_AUTH_CAPTURE transactions in the pending settlement state.
     * </LI>
     * <LI>
     * AUTO_MARK_AUTO_SETTLE transactions in the pending settlement state.
     * </LI>
     * <LI>
     * MANUAL_MARK_AUTO_SETTLE transactions in the pending settlement state.
     * </LI>
     * <LI>
     * AUTO_MARK_MANUAL_SETTLE transactions in the captured state.
     * </LI>
     * <LI>
     * MANUAL_MARK_MANUAL_SETTLE transactions in the captured state.
     * </LI>
     * </UL>
     *
     * @param transactionId The ID of the payment transaction to void.
     * @throws PaymentException General payment error.
     * @throws RemoteException On remote error.
     */
    public TransactionResponse voidTransaction(String transactionId) throws PaymentException, RemoteException
    {
        // Get the PaymentTransaction by primary key
        PaymentTransaction paymentTransaction = getPaymentTransaction(transactionId);

        // Verify that the given transaction can be voided and determine the transaction type
        String status = paymentTransaction.getStatus();
        String transactionType = null;
        String paymentModel = getPaymentModel();

        // If we are using a HOST_AUTH_CAPTURE payment model, only
        // transactions with a status of PENDINGSETTLE may be voided.
        if((paymentModel.equalsIgnoreCase(HOST_AUTH_CAPTURE) ||
            paymentModel.equalsIgnoreCase(HOST_AUTH_CAPTURE_AVS)) &&
           status.equalsIgnoreCase(PENDINGSETTLE))
	    {
		transactionType = "capture";
	    }

        // If we are using a HOST_POST_AUTH_CAPTURE payment model, only
        // transactions with a status of PENDINGSETTLE may be voided.
        else if ((paymentModel.equalsIgnoreCase(HOST_POST_AUTH_CAPTURE) ||
                  paymentModel.equalsIgnoreCase(HOST_POST_AUTH_CAPTURE_AVS)) &&
                 status.equalsIgnoreCase(PENDINGSETTLE))
	    {
		transactionType = "postauth";
	    }

        // If we are using a AUTO_MARK_AUTO_SETTLE payment model, only
        // transactions with a status of PENDINGSETTLE may be voided.
        else if ((paymentModel.equalsIgnoreCase(AUTO_MARK_AUTO_SETTLE) ||
                  paymentModel.equalsIgnoreCase(AUTO_MARK_AUTO_SETTLE_AVS)) &&
                 status.equalsIgnoreCase(PENDINGSETTLE))
	    {
		transactionType = "marked";
	    }

        // If we are using a MANUAL_MARK_AUTO_SETTLE payment model, only
        // transactions with a status of PENDINGSETTLE may be voided.
        else if ((paymentModel.equalsIgnoreCase(MANUAL_MARK_AUTO_SETTLE) ||
                  paymentModel.equalsIgnoreCase(MANUAL_MARK_AUTO_SETTLE_AVS)) &&
                 status.equalsIgnoreCase(PENDINGSETTLE))
	    {
		transactionType = "marked";
	    }

        // If we are using a AUTO_MARK_MANUAL_SETTLE or MANUAL_MARK_MANUAL_SETTLE
        // payment model, only transactions with a status of MARKEDFORSETTLE may be voided.
        else if ((paymentModel.equalsIgnoreCase(AUTO_MARK_MANUAL_SETTLE) ||
                  paymentModel.equalsIgnoreCase(AUTO_MARK_MANUAL_SETTLE_AVS) ||
                  paymentModel.equalsIgnoreCase(MANUAL_MARK_MANUAL_SETTLE) ||
                  paymentModel.equalsIgnoreCase(MANUAL_MARK_MANUAL_SETTLE_AVS)) &&
                 status.equalsIgnoreCase(MARKEDFORSETTLE))
	    {
		transactionType = "marked";
	    }

        // Otherwise, throw an exception...
        else
	    {
		throw new PaymentException("ebusiness-payment", "invalid.status.to.void", status);
	    }

        // Send the void command to Payment Web Service
        String [] arglist = { ORDER_ID, transactionId,
                              TRANSACTION_TYPE, transactionType };
        PSRequest psRequest = new PSRequest( arglist );
        PSResponse result = getCCWebService().voidTransaction( psRequest );

        // Initialize a new TransactionResponse based upon the existing PaymentTransaction
        // and Service-specific information
        TransactionResponse transactionResponse = convertPSResponseToTransactionResponse(result);
        transactionResponse.setTransactionId(paymentTransaction.getTransactionId());
        transactionResponse.setTransactionAmount(paymentTransaction.getTransactionAmount());
        transactionResponse.setTransactionStatus(getCurrentStatus(paymentTransaction.getStatus()));
        transactionResponse.setPaymentTransactionHandle(paymentTransaction.getHandle());
        transactionResponse.setTransactionDate(paymentTransaction.getTransactionDate());

        return transactionResponse;
    }

    /**
     * Query the Payment server for transactions which match the
     * supplied parameters.  The caller need only supply non-null
     * values for those parameters that they want to query against,
     * but must supply at least one non-null parameter.  <br><br> <b>
     * This method is implemented as a pass through to the underlying
     * service provider.  As such all return information is service
     * specific.  Refer to the service provider manual for details on
     * return codes and results.  </b>
     * @param transactionId The ID of a specific transaction.
     * @throws RemoteException On remote error
     */
    public TransactionResponse queryTransactions(String transactionId,
						 String transactionType,
						 String transactionStatus,
						 String transactionOrigin,
						 String cardType,
						 java.sql.Date startTime,
						 java.sql.Date endTime,
						 Price lowAmount,
						 Price highAmount,
						 Integer maxNumberOfTransactions)
	    throws RemoteException, PaymentException
    {

        Vector info = new Vector();

        // Query by ID
        if (transactionId != null)
	    {
		info.addElement(ORDER_ID);
		info.addElement(transactionId);
	    }

        // Query by type
        if (transactionType != null)
	    {
		info.addElement(TRANSACTION_TYPE);
		info.addElement(transactionType);
	    }

        // Query by status
        if (transactionStatus != null)
	    {
		info.addElement(TRANSACTION_STATUS);
		info.addElement(transactionStatus);

	    }

        // Query by time
        if (startTime != null)
	    {
		info.addElement(TRANSACTION_START_TIME);
		info.addElement(startTime.toString().substring(2,8).replace('-','/'));
	    }
        if (endTime != null)
	    {
		info.addElement(TRANSACTION_END_TIME);
		info.addElement(endTime.toString().substring(2,8).replace('-','/'));
	    }

        // Query by amount
        if (lowAmount != null)
	    {
		info.addElement(TRANSACTION_QUERY_LOW_AMOUNT);
		info.addElement(getCurrencyType(lowAmount) +  CURRENCY_FORMAT.format(lowAmount.getValue()));
	    }
        if (highAmount != null)
	    {
		info.addElement(TRANSACTION_QUERY_HIGH_AMOUNT);
		info.addElement(getCurrencyType(highAmount) +  CURRENCY_FORMAT.format(highAmount.getValue()));
	    }

        // Set the maximum number of transactions that are returned
        if (maxNumberOfTransactions != null)
	    {
		info.addElement(TRANSACTION_QUERY_MAX_COUNT);
		info.addElement(maxNumberOfTransactions.toString());
	    }

        // Send the query to Payment Web Service
        String[] args = new String[info.size()];
        for(int i= 0 ; i < info.size();i++)
	    {
		args[i] = (String)info.elementAt(i);
	    }
        PSRequest psRequest = new PSRequest( args );
        PSResponse result = getCCWebService().queryTransactions( psRequest );

        // Return a new TransactionResponse based upon Service-specific information
        return convertPSResponseToTransactionResponse(result);

    }

    private PSRequest buildArgsForAuthorize(PaymentTransaction paymentTransaction,
					    String authorizationMode,
					    boolean verifyAddress)
	throws InvalidCreditCardException, RemoteException
    {

        //
        // Create a vector of transaction information to send to the payment service
        //
        Vector info = new Vector();

        // Payment ID
        info.addElement(ORDER_ID);
        info.addElement(paymentTransaction.getTransactionId());

        // Amount of payment
        com.beasys.commerce.axiom.units.Price amount = paymentTransaction.getTransactionAmount();
        info.addElement(AMOUNT);
        info.addElement(getCurrencyType(amount) + CURRENCY_FORMAT.format(amount.getValue()));

        if (DEBUG)
	    {
		Logger.getInstance().debug("In CreditCardServiceImpl.authorize() :: authorizing for amount <" +
					   getCurrencyType(amount) + CURRENCY_FORMAT.format(amount.getValue()) + ">.");
	    }

        // Credit card expiration date
        CreditCard card = paymentTransaction.getCreditCard();
        info.addElement(CREDIT_CARD_EXPIRATION_DATE);
        info.addElement(DATE_FORMAT.format(card.getExpirationDate()));

        if (DEBUG)
	    {
		Logger.getInstance().debug("In CreditCardServiceImpl.authorize() :: authorizing using credit card with expiration date <" +
					   DATE_FORMAT.format(card.getExpirationDate()) + ">.");
	    }

        // In the case of a Discover card, verify that the card holder address is specified
        if (card.getType().equals(CreditCardTypes.DISCOVER))
	    {
		info.addElement(CREDIT_CARD_ADDRESS_STREET);
		if (card.getBillingAddress().getStreet1() != null)
		    {
			info.addElement(card.getBillingAddress().getStreet1());
		    }
		else throw new InvalidCreditCardException("ebusiness-payment", "discover.address.null");

		info.addElement(CREDIT_CARD_ADDRESS_CITY);
		if (card.getBillingAddress().getCity() != null)
		    {
			info.addElement(card.getBillingAddress().getCity());
		    }
		else throw new InvalidCreditCardException("ebusiness-payment", "discover.address.null");

		info.addElement(CREDIT_CARD_ADDRESS_ZIP);
		if (card.getBillingAddress().getPostalCode() != null)
		    {
			info.addElement(card.getBillingAddress().getPostalCode());
		    }
		else throw new InvalidCreditCardException("ebusiness-payment", "discover.address.null");

		info.addElement(CREDIT_CARD_ADDRESS_STATE);
		if (card.getBillingAddress().getState() != null)
		    {
			info.addElement(card.getBillingAddress().getState());
		    }
		else throw new InvalidCreditCardException("ebusiness-payment", "discover.address.null");
	    }

        // For address verification card address and zip code is required
        else if (verifyAddress)
	    {
		info.addElement(CREDIT_CARD_ADDRESS);
		if (card.getBillingAddress().getStreet1() != null)
		    {
			info.addElement(card.getBillingAddress().getStreet1());
		    }
		else throw new InvalidCreditCardException("ebusiness-payment", "avs.address.null");

		info.addElement(CREDIT_CARD_ADDRESS_ZIP);
		if (card.getBillingAddress().getPostalCode() != null)
		    {
			info.addElement(card.getBillingAddress().getPostalCode());
		    }
		else throw new InvalidCreditCardException("ebusiness-payment", "avs.address.null");
	    }

        // Lastly, add the decrypted credit card to the payment information.  Of course,
        // we must first decrypt the number before sending it to the payment service.
        // Finally, call the payment service with all payment information.
        info.addElement(CREDIT_CARD_NUMBER);

        //try
	  //  {
		//info.addElement(getDecryptedNumber(card.getNumber()));
                info.addElement(card.getNumber());
	    //} catch (InconsistentStateException e)
	      //  {
		//    Logger.getInstance().log( Log.LOG_WARNING, "CreditCardServiceImpl", e );
//		}

        String[] args = new String[info.size()];
        for (int i=0; i < info.size(); i++) {
            args[i] = (String)info.elementAt(i);
	}
        PSRequest psRequest = new PSRequest( args );
        psRequest.setAuthorizationMode( authorizationMode );

        return psRequest;
    }


    private TransactionResponse processAuthorizeResponse( PSResponse psResponse,
							  PaymentTransaction paymentTransaction,
							  boolean deferAuthorization )
	throws ServiceUnavailableException, SystemException, InvalidCreditCardException,
	AuthorizationException, ServiceConfigurationException, GeneralServiceException, RemoteException
    {
        Price amount = paymentTransaction.getTransactionAmount();

        // Set up a transaction response from available data.
        // Other attributes of the transaction response will be set
        // depending on the results of the authorizatio service call.
        TransactionResponse transactionResponse = convertPSResponseToTransactionResponse(psResponse);
        transactionResponse.setTransactionId(paymentTransaction.getTransactionId());
        transactionResponse.setTransactionAmount(amount);
        transactionResponse.setTransactionStatus(getCurrentStatus(paymentTransaction.getStatus()));
        transactionResponse.setPaymentTransactionHandle(paymentTransaction.getHandle());
        transactionResponse.setTransactionDate(paymentTransaction.getTransactionDate());

        // Get the result of the authorization service call
        String transactionStatus = transactionResponse.getTransactionResult(MSTATUS);

        // Extract addition result status
        String transactionActionCode = transactionResponse.getTransactionResult(ACTION_CODE);
        String transactionErrorCode = transactionResponse.getTransactionResult(MERRCODE);

        // Get the payment model from the configuration file
        String paymentModel = getPaymentModel();

        // A null transaction status indicates that a service is unavailable
        if (transactionStatus == null)
	    {
		// Mark the transaction for retry and throw a SystemUnavailable Exception, which
		// being an applicaton-level exception will NOT rollback transaction, and the
		// transaction will be persisted
		try
		    {
			paymentTransaction.systemFailure();
		    }
		catch (IllegalWorkflowTransitionException e)
		    {
			// This indicates that the transaction was already in the RETRY state
		    }

		TransactionEntry transactionEntry = createTransactionEntry(paymentTransaction, amount);
		paymentTransaction.addTransactionEntry(transactionEntry);
		transactionResponse.setTransactionStatus(getCurrentStatus(paymentTransaction.getStatus()));

		// Only throw the exception if payment is not deferred
		if (!deferAuthorization)
		    {
			ServiceUnavailableException unavailableException =
			    new ServiceUnavailableException("ebusiness-payment", "service.unavailable", transactionResponse);
			Logger.getInstance().log(Log.LOG_WARNING, "CreditCardServiceImpl", unavailableException);
			throw unavailableException;
		    }
	    }
        // Success
        else if (transactionStatus.equals(MSTATUS_SUCCESS) || transactionStatus.equals(MSTATUS_SUCCESS_DUPLICATE))
	    {
		try
		    {
			// For all other payment models move the payment transaction to an "authorized" state.
			paymentTransaction.authorize();
			paymentTransaction.addTransactionEntry(createTransactionEntry(paymentTransaction, amount));
			transactionResponse.setTransactionStatus(getCurrentStatus(paymentTransaction.getStatus()));

			// For AUTO_MARK_MANUAL_SETTLE move the transaction to the "marked" stage
			if (paymentModel.equalsIgnoreCase(AUTO_MARK_MANUAL_SETTLE)|| paymentModel.equalsIgnoreCase(AUTO_MARK_MANUAL_SETTLE_AVS))
			    {
				// A transactionEntry was created above even though this whole process happens at the same time
				// for logging purposes.
				paymentTransaction.markForSettle();
				paymentTransaction.addTransactionEntry(createTransactionEntry(paymentTransaction, amount));
				transactionResponse.setTransactionStatus(getCurrentStatus(paymentTransaction.getStatus()));
			    }

			// For AUTO_MARK_AUTO_SETTLE move the transaction all the way to the "settled" stage
			else if (paymentModel.equalsIgnoreCase(AUTO_MARK_AUTO_SETTLE)|| paymentModel.equalsIgnoreCase(AUTO_MARK_AUTO_SETTLE_AVS))
			    {
				// A transactionEntry was created above even though this whole process happens at the same time
				// for logging purposes.
				paymentTransaction.settle();
				paymentTransaction.addTransactionEntry(createTransactionEntry(paymentTransaction, amount));
				transactionResponse.setTransactionStatus(getCurrentStatus(paymentTransaction.getStatus()));
			    }

			// For HOST_AUTH_CAPTURE move the transaction all the way to the "settled" stage
			else if (paymentModel.equalsIgnoreCase(HOST_AUTH_CAPTURE)|| paymentModel.equalsIgnoreCase(HOST_AUTH_CAPTURE_AVS))
			    {
				// A transactionEntry was created above even though this whole process happens at the same time
				// for logging purposes.
				paymentTransaction.settle();
				paymentTransaction.addTransactionEntry(createTransactionEntry(paymentTransaction, amount));
				transactionResponse.setTransactionStatus(getCurrentStatus(paymentTransaction.getStatus()));
			    }
		    }
		catch(IllegalWorkflowTransitionException iwte)
		    {
			throw new SystemException("ebusiness-payment", "workflow.exception", iwte);
		    }
	    }
        // Hard (unrecoverable) failure
        else if (transactionStatus.equals(MSTATUS_FAILURE_HARD))
	    {
		// Since InvalidCreditCardException is an application-level exception, we need to
		// set the transaction for rollback.  Additionally, since the TransactionResponse
		// contains information about an assoicated PaymentTransaction, "rollback" the TransactionResponse.
		getSessionContext().setRollbackOnly();
		transactionResponse = convertPSResponseToTransactionResponse(psResponse);

		// Decide what kind of exceptions should be thrown:
		// - an InvalidCreditCardException is thrown if the card itself is invalid
		// - an AuthorizationException is thrown if the card itself is valid but
		//   there is a problem in authorizing it
		// - a ConfigurationException is thrown if there is a configuration problem
		// All these cases can be determined from the ACTION_CODE returned by Payment Service
		if (transactionActionCode != null)
		    {
			throwFailureHardException(transactionActionCode, transactionResponse);
		    }

		// Check for invalid credit card number
		else if (transactionErrorCode != null && transactionErrorCode.equals(BAD_LUHN_CHECK))
		    {
			throw new InvalidCreditCardException("ebusiness-payment", "invalid.creditcard", transactionResponse);
		    }
		// Otherwise, throw a general authorization exception
		else
		    {
			throw new AuthorizationException("ebusiness-payment", "auth.exception", transactionResponse);
		    }
	    }
        // Soft (recoverable) failure
        else if (transactionStatus.equals(MSTATUS_FAILURE_BAD_MONEY))
	    {
		// Since AuthorizationException is an application-level exception, we need to
		// set the transaction for rollback.  Additionally, since the TransactionResponse
		// contains information about an assoicated PaymentTransaction, "rollback" the TransactionResponse.
		getSessionContext().setRollbackOnly();
		transactionResponse = convertPSResponseToTransactionResponse(psResponse);

		// Decide what kind of exceptions should be thrown:
		// - for now, only AuthorizationException is thrown
		// All these cases can be determined from the ACTION_CODE returned by Payment Service
		if (transactionActionCode != null)
		    {
			throwFailureBadMoneyException(transactionActionCode, transactionResponse);
		    }
		// Otherwise, throw a general authorization exception
		else
		    {
			throw new AuthorizationException("ebusiness-payment", "auth.exception", transactionResponse);
		    }
	    }
        // Communication failure
        else if (transactionStatus.equals(MSTATUS_FAILURE_Q_OR_CANCEL) || transactionStatus.equals(MSTATUS_FAILURE_Q_OR_DISCARD))
	    {
		// Set the state of the payment transaction to RETRY
		try
		    {
			paymentTransaction.systemFailure();
		    }
		catch (IllegalWorkflowTransitionException e)
		    {
			// This indicates that the transaction was already in the RETRY state
		    }

		TransactionEntry transactionEntry = createTransactionEntry(paymentTransaction, amount);
		paymentTransaction.addTransactionEntry(transactionEntry);
		transactionResponse.setTransactionStatus(getCurrentStatus(paymentTransaction.getStatus()));

		ServiceUnavailableException unavailableException = new ServiceUnavailableException("ebusiness-payment", "service.unavailable", transactionResponse);
		Logger.getInstance().log(Log.LOG_WARNING, "CreditCardServiceImpl", unavailableException);

		throw unavailableException;
	    }

        return transactionResponse;
    }


    /**
     * Throw exceptions depending on ACTION_CODE returned with a FAILURE_HARD
     */
    static void throwFailureHardException(String transactionActionCode, TransactionResponse transactionResponse) throws AuthorizationException, InvalidCreditCardException, ServiceConfigurationException, GeneralServiceException
    {
        if(transactionActionCode.equals(GPSA_INVALID_CARD_NUMBER) ||
           transactionActionCode.equals(GPSS_INVALID_CARD_NUMBER) ||
           transactionActionCode.equals(NOVA_INVALID_CARD_NUMBER) ||
           transactionActionCode.equals(PAYMENTTECH_INVALID_CARD_NUMBER) ||
           transactionActionCode.equals(VITAL_INVALID_CARD_NUMBER))
	    {
		// throw an InvalidCreditCardException
		throw new InvalidCreditCardException("ebusiness-payment", "invalid.creditcard", transactionResponse);
	    }
        else if(transactionActionCode.equals(FDC_OUT_OF_BALANCE) ||
                transactionActionCode.equals(GPSA_EXPIRED_CARD) ||
                transactionActionCode.equals(GPSA_INVALID_AMOUNT) ||
                transactionActionCode.equals(GPSS_INVALID_AMOUNT) ||
                transactionActionCode.equals(NOVA_OUT_OF_BALANCE) ||
                transactionActionCode.equals(NOVA_INVALID_AMOUNT) ||
                transactionActionCode.equals(PAYMENTTECH_INVALID_AMOUNT) ||
                transactionActionCode.equals(PAYMENTTECH_CALL1) ||
                transactionActionCode.equals(VITAL_INVALID_AMOUNT))
	    {
		// authorization related problem
		throw new AuthorizationException("ebusiness-payment", "auth.exception", transactionResponse);
	    }
        else if(transactionActionCode.equals(GPSA_INVALID_MERCHANT_ID) ||
                transactionActionCode.equals(GPSA_MERCHANT_CONFIG_ERROR) ||
                transactionActionCode.equals(GPSA_INVALID_BANK_ID) ||
                transactionActionCode.equals(GPSS_INVALID_MERCHANT_ID) ||
                transactionActionCode.equals(GPSS_FORMAT_ERROR) ||
                transactionActionCode.equals(GPSS_TRANSACTION_NOT_PERMITTED) ||
                transactionActionCode.equals(NOVA_INVALID_MERCHANT_ID) ||
                transactionActionCode.equals(NOVA_FORMAT_ERROR) ||
                transactionActionCode.equals(PAYMENTTECH_INVALID_MERCHANT_ID) ||
                transactionActionCode.equals(VITAL_INVALID_MERCHANT_ID))
	    {
		// configuration related errors
		throw new ServiceConfigurationException("ebusiness-payment", "config.exception", transactionResponse);
	    }
        else if(transactionActionCode.equals(FDC_BANK_PROCESSOR_ERROR) ||
                transactionActionCode.equals(GPSS_UNKNOWN_BANK_RESPONSE) ||
                transactionActionCode.equals(WELLS_FARGO_PROCESSOR_ERROR))
	    {
		// general service provider errors
		throw new GeneralServiceException("ebusiness-payment", "service.exception", transactionResponse);
	    }
        else
	    {
		// throw the most general - AuthorizationException
		// this can later be made more specific
		throw new AuthorizationException("ebusiness-payment", "invalid.creditcard", transactionResponse);
	    }
    }

    /**
     * Throw exceptions depending on ACTION_CODE returned with a FAILURE_BAD_MONEY
     */
    static void throwFailureBadMoneyException(String transactionActionCode, TransactionResponse transactionResponse) throws AuthorizationException
    {
        // for now there are no conditions where our interpretation
        // and Payment Service's interpretation of what is a AuthorizationException
        // differs - so we just throw a AuthorizationException
        throw new AuthorizationException("ebusiness-payment", "auth.exception", transactionResponse);
    }

    static TransactionResponse convertPSResponseToTransactionResponse( PSResponse psResponse )
    {
        TransactionResponse transactionResponse = new TransactionResponse();

        if (psResponse == null)
	    {
		return transactionResponse;
	    }

        PSResponseElement[] results = psResponse.getResponseArray();
        if (results == null)
	    {
		return transactionResponse;
	    }

        for (int i = 0; i < results.length; i++)
	    {
		PSResponseElement psResponseElement = results[i];
		transactionResponse.putTransactionResult(psResponseElement.getKey(), psResponseElement.getValue());
	    }

        return transactionResponse;
    }

    /**
     * Returns an instance of the <code>PaymentTransactionHome</code>.
     */
    PaymentTransactionHome getPaymentTransactionHome()
    {
        try
	    {
		return (PaymentTransactionHome)getEnvironmentProperty(PAYMENT_TRANSACTION_HOME);
	    }
        catch (NamingException e)
	    {
		throw new SystemException("ebusiness-payment", "payment.transaction.home", e);
	    }
    }

    // given a PaymentTransaction and status type figure out the latest transaction
    // with of that status performed
    // the TransactionEntryId is used for comparison
    // comparing transaction ids is kinda messy as they are concatenated strings
    // but they are the best indicators of the order in which transaction entries
    // were created
    static TransactionEntry getLastTransactionEntryForStatus(PaymentTransaction pt, String status)
    {
        TransactionEntry te = null;

        try
	    {
		java.util.Iterator tecIterator = pt.getTransactionEntryCollection().iterator();

		if(tecIterator.hasNext())
		    {
			String identifier = null;

			while(tecIterator.hasNext())
			    {
				TransactionEntry tEntry = (TransactionEntry) tecIterator.next();
				if(tEntry.getStatus().equalsIgnoreCase(status))
				    {
					if(te != null)
					    {
						if(compareTransactionEntryIds(tEntry, te))
						    {
							te = tEntry;
						    }
					    }
					else
					    {
						te = tEntry;
					    } // end else
				    } // end if
			    } // end while
		    } //end outer if
		else
		    {
			return null;
		    } // end outer else
	    } // end try
        catch(RemoteException re)
	    {
		throw new RemoteSystemException("foundation", "remote.exception", re);
	    }

        return te;
    }

    // returns true if transactionEntry1 was created after transactionEntry2
    // false otherwise
    static boolean compareTransactionEntryIds(TransactionEntry te1, TransactionEntry te2)
    {
        String te1Id = te1.getIdentifier();
        String te2Id = te2.getIdentifier();

        // a transactionEntryId looks like this - order1-txn12
        // get the index of the "-txn" string and get the substring
        // from its end of the end of the complete string to get the number
        // at the end
        int te1Int = te1Id.lastIndexOf("-txn");
        int te2Int = te2Id.lastIndexOf("-txn");

        int te1Integer = (new Integer(te1.getIdentifier().substring(te1Int+3, te1Id.length()))).intValue();
        int te2Integer = (new Integer(te2.getIdentifier().substring(te2Int+3, te2Id.length()))).intValue();

        if(te1Integer > te2Integer)
            return true;
        else
            return false;

    }
    // this method populates a transactionEntry object that has been used for its transactionEntryId
    // but has all its other fields empty
    static void updateTransactionEntry(TransactionEntry te, PaymentTransaction pt)
    {
        try
	    {
		te.setStatus(pt.getStatus());
		te.setEntryDate(new java.sql.Date(System.currentTimeMillis()));
		te.setTransactionAmount(pt.getTransactionAmount());
	    }
        catch(RemoteException re)
	    {
		throw new RemoteSystemException("foundation", "remote.exception", re);
	    }
    }

    // The amount is provided as a separate argument so that this
    // method may be used in cases where a transaction entry does not
    // have its amount the same as that of the transactions amount
    TransactionEntry createTransactionEntry(PaymentTransaction pt, com.beasys.commerce.axiom.units.Price amount)
    {

        //get the previous transcationEntryId
        String transactionEntryId = getTransactionEntryId(pt);

        TransactionEntry te = TransactionEntryHome.create();

        te.setIdentifier(transactionEntryId);
        te.setEntryDate(new java.sql.Date(System.currentTimeMillis()));
        te.setTransactionAmount(amount);
        try
	    {
		te.setStatus(pt.getStatus());
	    }
        catch(RemoteException re)
	    {
		throw new RemoteSystemException("foundation", "remote.exception", re);
	    }
        return te;
    }

    // This method generates transacitonEntryIds for the transactionEntries of
    // a transaction. The logic used to accomplish this is to find the number of
    // transaction entries and then use the orderId of the transaction as the
    // prefix of the transactionEntryId which in turn is obtained by incrementing the
    // the transaction entry colleciton size by one.
    String getTransactionEntryId(PaymentTransaction pt)
    {
        String transactionEntryId;
        try
	    {
		transactionEntryId = pt.getTransactionId() + '-' + pt.transactionEntryCollectionSize() + 1;
	    }
        catch(RemoteException re)
	    {
		throw new RemoteSystemException("foundation", "remote.exception", re);
	    }

        return transactionEntryId;

    }

    // Get the PaymentModel from the theory.properties file
    String getPaymentModel()
    {
        if (PAYMENT_MODEL != null)
            return PAYMENT_MODEL;
        else
            return (PAYMENT_MODEL = pwsHelper.getPaymentServiceClientMBean().getPaymentModel());
    }

    /* Returns current transaction status in int
     * form corresponding to the status int the
     * work flow
     * For an invalid workflow - returns TransactionResponse.NOSTATUS
     */
    static int getCurrentStatus(String status)
    {
        if(status.equals(AUTHORIZED))
            return TransactionResponse.AUTHORIZED;
        else if(status.equals(SETTLEFAILED))
            return TransactionResponse.SETTLEFAILED;
        else if(status.equals(MARKEDFORSETTLE))
            return TransactionResponse.MARKEDFORSETTLE;
        else if(status.equals(PENDINGSETTLE))
            return TransactionResponse.PENDINGSETTLE;
        else if(status.equals(REJECTED))
            return TransactionResponse.REJECTED;
        else if(status.equals(SETTLED))
            return TransactionResponse.SETTLED;
        else if(status.equals(RETRY))
            return TransactionResponse.RETRY;
        else
            return TransactionResponse.NOSTATUS;
    }

    // returns a PaymentTransaction given an orderId
    PaymentTransaction getPaymentTransaction(String transactionId)
    {
        PaymentTransaction pt;
        try
	    {
		PaymentTransactionHome ptHome = getPaymentTransactionHome();
		pt = ptHome.findByPrimaryKey(new PaymentTransactionPk(transactionId));
	    }
        catch(FinderException fe)
	    {
		throw new FinderSystemException("foundation", "finder.exception", fe);
	    }
        catch(RemoteException re)
	    {
		throw new RemoteSystemException("foundation", "finder.exception", re);
	    }

        return pt;
    }

    // generate a new transactionId using JDBCSequencer
    String generateTransactionId()
    {
        // create a sequencer with name PAYMENTTRANSACTION_SEQUENCE
        // default sequence -- Sequence
        // default cache size -- 2000
        // default step size -- 1
        Sequencer sequencer = SequencerFactory.createSequencer("TRANSACTION_ID_SEQUENCE");
        Connection conn = getConnection();
        Long longSequence;
        try
	    {
		longSequence = new Long(sequencer.getNext(conn));
	    }
        catch(SQLException se)
	    {
		throw new SystemException("ebusiness-payment", "sql.exception", se);
	    }
        return ((new String(longSequence.toString())) + (new Long(System.currentTimeMillis()).toString()));

    }

    // extracts the currency type from a Price object and maps it to a Payment Service specific string
    static String getCurrencyType(com.beasys.commerce.axiom.units.Price amount )
    {
        // if the Price does not specify any currency - default to USD
        if(amount.getCurrency() == null)
            return CurrencyConvertor.convert("USD");


        return CurrencyConvertor.convert(amount.getCurrency());

    }

    /**
     * Returns a decrypted credit card number
     * by using the Decryptor
     */
    String getDecryptedNumber(String encryptedCreditCardNumber) throws InconsistentStateException
    {
        // if ENCRYPTION_PROPERTY == null, encryption is disabled
        // and teh credit card number is plain text - so return it as is
        if((ENCRYPTION_PROPERTY == null) || (ENCRYPTION_PROPERTY.equalsIgnoreCase("false")))  return encryptedCreditCardNumber;
        try
	    {
		DecryptorHome decryptorHome = (DecryptorHome)getEnvironmentProperty(DECRYPTOR_HOME);
		Decryptor decryptor = decryptorHome.create();

		// decrypt the credit card number
		return decryptor.decrypt(encryptedCreditCardNumber);
	    }
        catch (NamingException e)
	    {
		throw new SystemException("ebusiness-payment", "decryptor.home", e);
	    }
        catch(CreateException ce)
	    {
		throw new SystemException("ebusiness-payment", "decryptor.create", ce);
	    }
        catch(RemoteException re)
	    {
		throw new SystemException("ebusiness-payment", "decryption.error", re);
	    }
    }

    /**
     * Get database connection from resource ref
     * -- this needs that the resource ref property
     * be specified in the deployment descriptor
     * for this bean
     */
    java.sql.Connection getConnection()
    {
        try
	    {
		DataSource ds = null;

		ds = (javax.sql.DataSource) this.getEnvironmentProperty("jdbc/commercePool");

		return ds.getConnection();
	    }
        catch(NamingException namingException)
	    {
		Logger.getInstance().log(Log.LOG_ERROR,
					 namingException.getMessage());
		String[] messageParameters = {"jdbc/commercePool"};
		throw new SystemException("ebusiness-payment", "errorGetConnection", messageParameters);
	    }
        catch(java.sql.SQLException sqlException)
	    {
		String[] messageParameters = {"jdbc/commercePool"};
		throw new SystemException("ebusiness-payment", "errorGetConnection", messageParameters, sqlException);
	    }
    }

    private PaymentServicePort getCCWebService() throws PaymentException
	{
	    if (ccWebService == null) {
	       ccWebService = pwsHelper.getPaymentWebService();
	       }

	    return ccWebService;
	}


}







