package examples.e2e.b2c.payment.pipeline; 

/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/src/examples/e2e/b2c/payment/pipeline/CajunBasedPaymentPC.java,v 1.2 2002/05/02 00:22:11 toneill Exp $
 */

import com.beasys.commerce.foundation.exception.*;
import com.beasys.commerce.axiom.contact.*;
import com.beasys.commerce.foundation.*;
import com.beasys.commerce.webflow.PipelineSessionConstants;
import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.exception.PipelineException;


import java.rmi.RemoteException;

import paymentWS.Payment;
import paymentWS.PaymentSoap;
import examples.e2e.b2c.util.B2CPortalConstants;

import com.bea.p13n.util.debug.Debug;
import org.openuri.www.x2002.x04.*;
import org.openuri.www.x2002.x04.soap.conversation.*;

import com.beasys.commerce.axiom.units.Price;

 /**
 * This <code>PipelineComponent</code> calls a Java proxy which lets the pipeline 
 * component call a Payment webservice.  The webservice uses the conversational 
 * aspect of Cajun framework.  The first call is to authorize the credit card 
 * where the credit card number and amount to be authorized are passed as arguments. 
 * After the credit card authorization is complete, a call is made to capture an amount.  
 * Eventually a request is made to settle the amount.
 * 
 * The codes returned by the webservice are as follows:
 *       -1 = Error (Bad argument passed)
 *       -2 = Error (When amount to capture is more than amount the credit is authorized for)
 *       -3 = Error (When amount to settle is more than amount for which the credit card was captured for) 
 *        > 0 = Success 
 * 
 *
 */

public class CajunBasedPaymentPC
    extends com.bea.p13n.appflow.pipeline.PipelineComponentSupport {

    private static final Debug DEBUG =
        Debug.getInstance(CajunBasedPaymentPC.class);
    private String connectString = null;

/**
 * The process method which processes the <code>PipelineSession</code>
 * and returns the processed <code>PipelineSession</code>.
 * <p>
 * Note: This method is not allowed to return null.  Returning null will
 * cause the pipeline in which this component is participating to terminate
 * abnormally.
 * </p>
 * @param  <code>PipelineSession</code> object
 * @param requestContext this gives the current requesting threads context-ID with which
 * this attribute has to be stored in this namespace.  If there are multiple requests
 * using this context id the processes can avoid name collisions of attributes as well
 * as it will be guaranteed that the attributes added in this thread are available no matter
 * what the order of execution of the threads are.  This context id will be provided to all the 
 * processors so that they can pass it along to specific processor types.
 * @return <code>PipelineSession</code> the processed <code>PipelineSession</code> object
 * @throws <code>PipelineException</code> indicates that the component failed to execute.
 * @throws <code>RemoteException</code> on Remote error.
 */
public PipelineSession process(
    PipelineSession pipelineSession,
    Object requestContext)
    throws PipelineException, RemoteException {

    Payment         proxy = null;
    PaymentSoap     proxySoap = null;
    String          conversationID = "" + System.currentTimeMillis() ; 

    if (DEBUG.ON) {
       DEBUG.out("process()"); 
    }

    try {

         weblogic.management.configuration.ServerMBean serverMBean =
                  com.bea.p13n.management.ServerConfiguration.getServerMBean();
                  if (serverMBean == null) {
                      throw new PipelineException("serverMBean is NULL");
                  }
              
          String serverName = (serverMBean.getListenAddress() == null) ? "localhost"
                          : serverMBean.getListenAddress();
                  connectString = "http://" + serverName + ":" + serverMBean.getListenPort();

      } catch (Exception e) {
         e.printStackTrace();
         throw new PipelineException("CajunBasedPaymentPC:error computing provider url"); 
      }


    String namespace = (String) pipelineSession.getDefaultNamespace();

    // Get the shopping cart from the pipeline session
    CreditCard cc = (CreditCard) getSessionAttribute(PipelineSessionConstants.PAYMENT_CREDIT_CARD,
                              namespace, pipelineSession);

    Price amt = (Price) getSessionAttribute(PipelineSessionConstants.PAYMENT_AUTHORIZATION_AMOUNT,
                              namespace, pipelineSession);

    if (cc == null) {
        if (DEBUG.ON) {
            DEBUG.out("cc is null"); 
        }
        throw new PipelineException("CajunBasedPaymentPC:cc is null"); 
    }

    if (amt == null) {
        if (DEBUG.ON) {
            DEBUG.out("amount is null"); 
        }
        throw new PipelineException("CajunBasedPaymentPC:amount is null");
       
    }


    if (DEBUG.ON) {
       DEBUG.out("number:" + cc.getNumber() + ", amount:" + amt.getValue()); 
    }

    // Use Weblogic's jax-rpc impl
    //System.setProperty( "javax.xml.soap.MessageFactory", "weblogic.webservice.core.soap.MessageFactoryImpl");
    //if (DEBUG.ON) { DEBUG.out("jax-rpc impl set"); }

    try {
       proxy = new Payment(connectString + "/workshop/Payment.jws?WSDL");
       if (DEBUG.ON) {
          DEBUG.out("proxy instantiated"); 
       }
    }
    catch (Exception ex) {
        if (DEBUG.ON) {
            DEBUG.out("error gettting proxy handle"); 
        }
        throw new PipelineException("CajunBasedPaymentPC:amount is null");
    }

    proxySoap = proxy.getPaymentSoap();
 
    try {
       // Set up the header objects
       if (DEBUG.ON) {
          DEBUG.out("setting up header objects"); 
       }
       StartHeader startHeader = new StartHeader( conversationID, connectString + "/workshop/Payment.jws");
       ContinueHeader continueHeader = new ContinueHeader( conversationID );

       // Start the conversation
       Authorize auth = new Authorize(cc.getNumber(), amt.getValue());
       long result = proxySoap.Authorize(auth, startHeader).getReturn0() ;

       if (result < 0) {

            //set error code and return
            setSessionAttribute(
                B2CPortalConstants.PAYMENT_AUTH_CAPTURE_SETTLE_RESULT,
                new Long(result),
                namespace,
                pipelineSession);

            return pipelineSession;

        }


       if (DEBUG.ON) {
          DEBUG.out("auth response: " + result); 
       }
       // Continue the conversation
       Capture capture = new Capture(amt.getValue());

       result = proxySoap.Capture(capture, continueHeader).getReturn0();
       if (DEBUG.ON) {
          DEBUG.out("capture response: " + result); 
       }

       if (result < 0) {
            //set error code and return
            setSessionAttribute(
                B2CPortalConstants.PAYMENT_AUTH_CAPTURE_SETTLE_RESULT,
                new Long(result),
                namespace,
                pipelineSession);

            return pipelineSession;
        }


       // Finish the conversation
       Settle settle = new Settle(amt.getValue());
       result = proxySoap.Settle(settle, continueHeader).getReturn0();
       if (DEBUG.ON) {
          DEBUG.out("settle response: " + result); 
       }

           //set return code and return
        setSessionAttribute(
            B2CPortalConstants.PAYMENT_AUTH_CAPTURE_SETTLE_RESULT,
            new Long(result),
            namespace,
            pipelineSession);

      return pipelineSession;
            
   }
   catch (RemoteException ex)
   {
       ex.printStackTrace();
       if (DEBUG.ON) {
        DEBUG.out("remote exception"); 
       }
    throw new PipelineException("CajunBasedPaymentPC:remote exception");
   }
 } //process
}
