/*
 * B E A   S Y S T E M S
 *
 * C O M M E R C E   C O M P O N E N T S
 *
 * Copyright (c) 2000-2002  BEA Systems, Inc.
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws
 * of the United States. The software contained on this media is proprietary
 * to and embodies the confidential technology of BEA Systems, Inc. The
 * possession or receipt of this information does not convey any right to disclose
 * its contents, reproduce it, or use,  or license the use, for manufacture or
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/src/examples/e2e/b2c/payment/pipeline/PaymentAuthorizationPC.java,v 1.2 2002/05/02 00:22:11 toneill Exp $
 */
package examples.e2e.b2c.payment.pipeline;

import java.rmi.*;
import java.util.*;
import javax.sql.*;
import java.sql.*;
import javax.naming.*;
import javax.ejb.*;

import examples.e2e.b2c.payment.CreditCardService;

import com.beasys.commerce.foundation.exception.*;
import com.beasys.commerce.ebusiness.customer.*;
import com.beasys.commerce.ebusiness.payment.*;
import com.beasys.commerce.ebusiness.order.*;
import com.beasys.commerce.axiom.contact.*;
import com.beasys.commerce.axiom.units.*;
import com.beasys.commerce.util.*;
import com.beasys.commerce.axiom.util.helper.*;
import com.beasys.commerce.axiom.util.ToolkitException;
import com.beasys.commerce.foundation.*;
import com.beasys.commerce.webflow.PipelineSessionConstants;
import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.exception.PipelineException;

/**
 * This <code>PipelineComponent</code> authorizes a credit card for a
 * purchase of a given amount.  This class is an abstract class that
 * must be extended to implement the various authorization modes.  The
 * credit card that is authorized must be specified using the
 * <code>PipelineSessionConstants.PAYMENT_CREDIT_CARD</code>
 * <code>PipelineSession</code> attribute and the amount to the
 * authorized must be specified as the
 * <code>PipelineSessionConstants.PAYMENT_AUTHORIZATION_AMOUNT</code>
 * <code>PipelineSession</code> attribute.
 *
 * @see com.beasys.commerce.ebusiness.payment.pipeline.PaymentPC
 */
public abstract class PaymentAuthorizationPC extends PaymentPC
{
    protected static String PAYMENT_TRANSACTION_HOME = "/ejb/PaymentTransaction";

    /**
     * Authorize a credit card for a purchase of a given amount.
     *
     * @param creditCardService The service with which to authorize
     * @param creditCard The credit card to authorize
     * @param amount The amount of the purchase
     * @throws PipelineNonFatalException if the credit card service is unavailable or incorrectly configured.
     * @throws PipelineFatalException on general service error.
     * @throws RemoteException on remote error.
     */
    public abstract TransactionResponse authorize(CreditCardService creditCardService,
                                                  CreditCard creditCard,
                                                  Price amount,
                                                  Order order) throws PipelineException, RemoteException;
    
    public PipelineSession process(PipelineSession pipelineSession, Object requestContext) 
	throws PipelineException, RemoteException
    {
	String namespace = (String) pipelineSession.getDefaultNamespace();
	
        // Get the credit card and authorization amount from the PipelineSession
        CreditCard creditCard = (CreditCard) getSessionAttribute(PipelineSessionConstants.PAYMENT_CREDIT_CARD, 
								 namespace, pipelineSession);
        Price amount = (Price) getSessionAttribute(PipelineSessionConstants.PAYMENT_AUTHORIZATION_AMOUNT, 
						   namespace, pipelineSession);

        // Get a CreditCardService
        CreditCardService service = getCreditCardService(pipelineSession);

        // Get the Order
        Order order = (Order)((Handle) getRequestAttribute(PipelineSessionConstants.ORDER_HANDLE, namespace, requestContext, pipelineSession )).getEJBObject();

        // Authorize the credit card for the specified amount
        TransactionResponse response = authorize(service, creditCard, amount, order);

        if(DEBUG) {
            StringBuffer buffer = new StringBuffer();
            Enumeration keys = response.getAllResults().keys();
            Enumeration values = response.getAllResults().elements();
            while (keys.hasMoreElements() && values.hasMoreElements())
            {
                buffer.append("\n\t" + "(" + keys.nextElement() + ", \t" + values.nextElement() + ")");
            }

            Logger.getInstance().debug("In PaymentAuthorizationPC: received response from authorize method " + buffer.toString());

            Logger.getInstance().debug("In PaymentAuthorizationPC : successfully authorized credit card.");
        }
	
        // Add a handle to the PaymentTransaction to the Order
        order.setPaymentTransaction((PaymentTransaction)response.getPaymentTransactionHandle().getEJBObject());
	
        if(DEBUG) {
            Logger.getInstance().debug("In PaymentAuthorizationPC : added a payment transaction handle to the order.");
        }
	
        return pipelineSession;
    }
}




