/*
 * B E A   S Y S T E M S
 *
 * C O M M E R C E   C O M P O N E N T S
 *
 * Copyright (c) 2000-2002  BEA Systems, Inc.
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws
 * of the United States. The software contained on this media is proprietary
 * to and embodies the confidential technology of BEA Systems, Inc. The
 * possession or receipt of this information does not convey any right to disclose
 * its contents, reproduce it, or use,  or license the use, for manufacture or
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/src/examples/e2e/b2c/payment/pipeline/PaymentPC.java,v 1.2 2002/05/02 00:22:11 toneill Exp $
 */

package examples.e2e.b2c.payment.pipeline;

// java imports
import java.rmi.*;
import javax.ejb.*;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.util.Properties;

// javax imports

// com.beasys imports
import com.beasys.commerce.axiom.util.*;
import com.beasys.commerce.axiom.util.helper.*;
import com.beasys.commerce.ebusiness.payment.*;
import examples.e2e.b2c.payment.CreditCardService;
import examples.e2e.b2c.payment.CreditCardServiceHome;
import com.beasys.commerce.util.*;
import examples.e2e.b2c.webflow.CommercePipelineComponentSupport;

import com.bea.p13n.appflow.exception.PipelineException;
import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.util.JndiHelper;
import com.bea.p13n.management.ApplicationHelper;


// weblogic imports

// other imports

/**
 * This <code>PipelineComponent</code> is the base class for all Payment related
 * <code>PipelineComponent</code>s components.  It contains Payment related utility methods.
 *
 * @see com.beasys.commerce.ebusiness.payment.CreditCardService
 * @see com.beasys.commerce.foundation.pipeline.PipelineComponent
 * @see com.beasys.commerce.foundation.pipeline.PipelineSession
 * @see com.beasys.commerce.webflow.PipelineSessionConstants
 */
public abstract class PaymentPC extends CommercePipelineComponentSupport
{
    /**
     * JNDI name of the CreditCardService Session EJB
     */
    private static final String CREDIT_CARD_SERVICE_JNDI_NAME = "BEA_commerce.CreditCardService";

    /**
     * Debug flag - prevents compilation of debug tracing messages when false
     */
    static final boolean DEBUG = false;

    /**
     * Returns an instance of the <code>CreditCardService</code>.
     *
     * @param pipelineSession The PipelineSession
     * @throws PipelineFatalException on service creation error
     * @throws RemoteException on remote error
     */
    public static CreditCardService getCreditCardService(PipelineSession pipelineSession) 
	throws PipelineException, RemoteException
    {
        // Create the CreditCardService
        try {
	    String jndiName = ApplicationHelper.getApplicationName() + ".";
            jndiName = jndiName + CREDIT_CARD_SERVICE_JNDI_NAME;
            CreditCardServiceHome home = (CreditCardServiceHome) JndiHelper.lookupNarrow(jndiName, 
											 CreditCardServiceHome.class);
            return home.create();
        }
        catch(NamingException e) {
            throw new PipelineException("ebusiness-payment:toolkit.exception");
        }
        catch(CreateException e) {
            throw new PipelineException("ebusiness-payment:toolkit.exception");
        }
    }
}
