/*
 * B E A   S Y S T E M S
 *
 * C O M M E R C E   C O M P O N E N T S
 *
 * Copyright (c) 2000-2002  BEA Systems, Inc.
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws
 * of the United States. The software contained on this media is proprietary
 * to and embodies the confidential technology of BEA Systems, Inc. The
 * possession or receipt of this information does not convey any right to disclose
 * its contents, reproduce it, or use,  or license the use, for manufacture or
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/src/examples/e2e/b2c/shoppingcart/pipeline/AddProductItemToShoppingCartPC.java,v 1.2 2002/05/02 00:22:11 toneill Exp $
 */
package examples.e2e.b2c.shoppingcart.pipeline;

import com.beasys.commerce.ebusiness.catalog.ProductItem;
import com.beasys.commerce.ebusiness.shoppingcart.ShoppingCart;
import com.beasys.commerce.ebusiness.shoppingcart.ShoppingCartHome;
import com.beasys.commerce.util.Logger;

import com.beasys.commerce.webflow.PipelineSessionConstants;

import com.beasys.commerce.foundation.exception.InvalidArgumentException;
import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.pipeline.PipelineComponent;
import com.bea.p13n.appflow.exception.PipelineException;
import com.bea.p13n.appflow.pipeline.PipelineComponentSupport;

/**
 * This <code>PipelineComponent</code> extracts a
 * <code>ProductItem</code> and an optional quantity from the
 * <code>PipelineSession</code> and adds the appropriate quantity of
 * that item to the customer's shopping cart.  If no quantity is
 * specified, a default quantity of '1' is assumed.
 *
 * @see com.bea.p13n.appflow.pipeline.PipelineComponent
 * @see com.bea.p13n.appflow.pipeline.PipelineSession
 * @see com.beasys.commerce.webflow.PipelineSessionConstants
 * @see com.beasys.commerce.ebusiness.catalog.ProductItem
 * @see com.beasys.commerce.ebusiness.shoppingcart.ShoppingCart
 */
public class AddProductItemToShoppingCartPC extends PipelineComponentSupport
{
    private static final boolean DEBUG = true;
    private static final Integer DEFAULT_QUANTITY = new Integer(1);

    /**
     * See class documentation above.
     *
     * @param pipelineSession <code>PiplineSession</code> object that
     * the component will operate on
     * @return Modified <code>PipelineSession</code> object
     * @throws PipelineFatalException If
     * <code>PipelineSessionConstants.CATALOG_ITEM</code> attribute
     * cannot be found in the pipeline session
     */
    public PipelineSession process( PipelineSession pipelineSession, Object requestContext )
        throws PipelineException
    {
	String namespace = (String) pipelineSession.getDefaultNamespace();

        // Get the ProductItem from the PipelineSession
        ProductItem item = (ProductItem) getSessionAttribute(PipelineSessionConstants.CATALOG_ITEM, 
							     namespace, pipelineSession);
        if(item == null) {
            throw new PipelineException("ebusiness-shoppingcart:catalog.item.was.null" + 
					PipelineSessionConstants.CATALOG_ITEM);
        }

        // In case you'd like to put more in the preceding JSP, you'd retrieve a
        // quantity here from the session attribute.
        if(DEBUG) {
            Logger.getInstance().debug("In AddProductItemToShoppingCartPC : adding 1 item with key " + 
				       item.getKey().getIdentifier() + " to shopping cart.");
        }

        // Get the shopping cart from the pipeline session
        ShoppingCart shoppingCart = (ShoppingCart) getSessionAttribute(PipelineSessionConstants.SHOPPING_CART,
								       namespace, pipelineSession);

        // If the shopping cart is not in the pipeline session, create one
        if(shoppingCart == null) {
            shoppingCart = ShoppingCartHome.create();
            setSessionAttribute(PipelineSessionConstants.SHOPPING_CART, shoppingCart, namespace, pipelineSession);
        }

        // Add the item to the shopping cart
        try {
            shoppingCart.addProductItem(item, 1);
        }
        catch (InvalidArgumentException iae) {
            throw new PipelineException("ebusiness-shoppingcart:invalid.argument.exception.in.add.product.item.to.shoppingcart",iae);
        }

        // Return modified pipeline session object
        return pipelineSession;
    }
}
