/*
 * B E A S Y S T E M S
 *
 * C O M M E R C E   C O M P O N E N T S
 *
 * Copyright (c) 2000-2002  BEA Systems, Inc.
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws
 * of the United States. The software contained on this media is proprietary
 * to and embodies the confidential technology of BEA Systems, Inc. The
 * possession or receipt of this information does not convey any right to disclose
 * its contents, reproduce it, or use,  or license the use, for manufacture or
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/src/examples/e2e/b2c/shoppingcart/pipeline/CheckInventoryPC.java,v 1.2 2002/05/02 00:22:11 toneill Exp $
 */

package examples.e2e.b2c.shoppingcart.pipeline;

import examples.e2e.common.inventory.spi.*;
import examples.e2e.b2c.util.B2CPortalConstants;

import java.util.*;
import java.rmi.*;
import javax.ejb.*;
import javax.naming.*;

import com.beasys.commerce.util.*;
import com.beasys.commerce.axiom.util.helper.*;
import com.beasys.commerce.ebusiness.shoppingcart.*;
import com.beasys.commerce.ebusiness.catalog.ProductItem;

import com.beasys.commerce.webflow.PipelineSessionConstants;

import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.pipeline.PipelineComponent;
import com.bea.p13n.appflow.exception.PipelineException;
import com.bea.p13n.exceptions.ApplicationException;
import com.bea.p13n.appflow.pipeline.PipelineComponentSupport;
import com.bea.p13n.management.ApplicationHelper;
import com.bea.p13n.util.JndiHelper;


/**
  * This PC checks the Inventory and places an additional
  * value; it needs to be invoked everytime the shopping cart
  * content is changed, which should correspond to the
  * invocation of the PriceOrderPC
  * <br>
  * For this example, we redo all inventory checking EVERY
  * TIME the content changes. 
  * <br>
  * You may want to do it differently for performance reason,
  * based on your business logic; for instance, only the
  * first time an item is placed in the shopping cart by keeping
  * the actual quantity in stock, and showing different messages
  * based on the difference between what is in stock and 
  * what is in the shopping cart.
  * <br>
  * Since we want to use the shopping cart as is, as opposed
  * to modifying it just for putting new attributes for each
  * shopping cart line, we use a hashtable to store the inventory values
  * of each product items.
  * <br>
  * @see com.bea.p13n.appflow.pipeline.PipelineComponent
  * @see com.bea.p13n.appflow.pipeline.PipelineSession
  * @see com.beasys.commerce.webflow.PipelineSessionConstants
  * @see com.beasys.commerce.ebusiness.shoppingcart.ShoppingCart
  */
public class CheckInventoryPC extends PipelineComponentSupport 
{

    private static final String INVENTORY_PROVIDER_JNDI = ".BEA_commerce.InventoryProvider";
    private static final boolean DEBUG = false;

    public PipelineSession process(PipelineSession pipelineSession, Object requestContext) throws PipelineException
    {
	String namespace = (String) pipelineSession.getDefaultNamespace();
	
        // Get the shopping cart from the pipeline session
        ShoppingCart shoppingCart = (ShoppingCart) getSessionAttribute(PipelineSessionConstants.SHOPPING_CART, 
								       namespace, pipelineSession);

        // Nothing to do if no shopping cart or an empty on.
        if(shoppingCart == null) {
            return pipelineSession; 
        }

        // Iterate of the shopping cart lines to get the inventory
        Iterator lineIterator = shoppingCart.shoppingCartLineCollectionIterator();
        int[] inventoryCount = new int[shoppingCart.shoppingCartLineCollectionSize( )];
        
        // Get the inventory provider.
        InventoryProvider invProvider = null;
        try {

            String ref = ApplicationHelper.getApplicationName() + INVENTORY_PROVIDER_JNDI;
            InventoryProviderHome invProviderHome = (InventoryProviderHome) JndiHelper.lookupNarrow(ref, InventoryProviderHome.class);
            invProvider = invProviderHome.create();

            // Fill the lineSkus[] as a prep for the inventory calls
            int k=0;
            String lineSku = null;
            while(lineIterator.hasNext()) {
                ShoppingCartLine scl = (ShoppingCartLine) lineIterator.next();
                lineSku = scl.getProductItem().getKey().getIdentifier();
                inventoryCount[k] = invProvider.checkInventory(lineSku);

                //If any requested cound is too little, we do something for 
                // the doc portlet. Really, this should NOT be here, but
                // this is only for the doc portlet demand.
                if ( inventoryCount[k] <= scl.getQuantity() ) {
                    // Get the portlet webflow event from the HTTP request 
                    setSessionAttribute( B2CPortalConstants.DOC_PORTLET_LOCATION, "step1.inventory", namespace, pipelineSession);
                }
                k++;
            }

        } catch (NamingException ne) {
                throw new PipelineException("ebusiness-order:naming.exception", ne);
        } catch (CreateException ce) {
                throw new PipelineException("ebusiness-order:create.exception", ce);
        } catch (IllegalArgumentException ie) {
                throw new PipelineException("ebusiness-order:illegal.argument.exception", ie);
        } catch (ApplicationException ae) {
                throw new PipelineException("ebusiness-order:application.exception.exception", ae);
        } catch (RemoteException re) {
                throw new PipelineException("ebusiness-order:remote.exception", re);
        }

        // Put the data for each of the lines in the pipeline session.
        setSessionAttribute(B2CPortalConstants.INVENTORY_CHECK, inventoryCount, namespace, pipelineSession);

        return pipelineSession;
    }

}
