/*
 * B E A   S Y S T E M S
 *
 * C O M M E R C E   C O M P O N E N T S
 *
 * Copyright (c) 2000-2002  BEA Systems, Inc.
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws
 * of the United States. The software contained on this media is proprietary
 * to and embodies the confidential technology of BEA Systems, Inc. The
 * possession or receipt of this information does not convey any right to disclose
 * its contents, reproduce it, or use,  or license the use, for manufacture or
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/src/examples/e2e/b2c/shoppingcart/pipeline/DeleteProductItemFromSavedListPCImpl.java,v 1.2 2002/05/02 00:22:11 toneill Exp $
 */


package examples.e2e.b2c.shoppingcart.pipeline;

import com.beasys.commerce.foundation.SessionImpl;
import com.beasys.commerce.util.Logger;

import java.rmi.RemoteException;
import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.SessionContext;
import javax.naming.NamingException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import javax.sql.DataSource;

import com.beasys.commerce.foundation.exception.SystemException;
import com.beasys.commerce.ebusiness.shoppingcart.ShoppingCart;
import com.beasys.commerce.foundation.pipeline.PipelineConstants;
import com.beasys.commerce.webflow.PipelineSessionConstants;
import com.beasys.commerce.webflow.HttpRequestConstants;

import com.bea.p13n.appflow.exception.*;
import com.bea.p13n.appflow.common.PipelineSession;

/**
 * DeleteProductItemFromSavedListPCImpl is the implementation for the
 * DeleteProductItemFromSavedListPC interface.
 *
 * DeleteProductItemFromSavedListPC is a pipeline component that
 * removes a <code>ProductItem</code> from the list of saved items for
 * a given customer.  The customer login id is specified by the
 * <code>PipelineSessionConstants.USER_NAME</code> attribute in the
 * Pipeline Session.  The sku of the item to delete is specified by
 * the <code>PipelineSessionConstants.CATALOG_ITEM_SKU</code>.  The
 * <code>PipelineSessionConstants.SAVED_SHOPPING_CART</code> pipeline
 * session attribute, and the WLCS_SAVED_ITEM_LIST table are updated
 * to reflect the change.
 *
 *
 * @see com.beasys.commerce.ebusiness.shoppingcart.pipeline.DeleteProductItemFromSavedListPC
 * @see com.beasys.commerce.ebusiness.shoppingcart.pipeline.DeleteProductItemFromSavedListPCHome
 * @see com.bea.p13n.appflow.pipeline.PipelineComponent
 * @see com.bea.p13n.appflow.pipeline.PipelineSession
 * @see com.beasys.commerce.webflow.PipelineSessionConstants
 * @see com.beasys.commerce.ebusiness.shoppingcart.ShoppingCart
 */
public class DeleteProductItemFromSavedListPCImpl extends SessionImpl
{
    private static final boolean DEBUG = false;

    /**
     * Default constuctor.
     */
    public DeleteProductItemFromSavedListPCImpl()
    {
        super();
    }

    /**
     * ejbCreate() method.
     */
    public void ejbCreate() throws CreateException
    {
        super.ejbCreate();
    }

    /**
     * ejbPostCreate() method.
     */
    public void ejbPostCreate() throws CreateException
    {
        super.ejbPostCreate();
    }

    /**
     * ejbActivate() method.
     */
    public void ejbActivate() throws EJBException
    {
        super.ejbActivate();
    }

    /**
     * ejbPassivate() method.
     */
    public void ejbPassivate() throws EJBException
    {
        super.ejbPassivate();
    }

    /**
     * ejbRemove() method.
     */
    public void ejbRemove() throws EJBException
    {
        super.ejbRemove();
    }

    /**
     * setSessionContext() method.
     */
    public void setSessionContext(SessionContext ctx) throws EJBException
    {
        super.setSessionContext(ctx);
    }

    /**
     * See class documentation above.
     *
     * @param pipelineSession <code>PipelineSession</code> object that
     * the pipeline component will use
     * @return The updated <code>PipelineSession</code> object
     * @throws com.beasys.commerce.foundation.exception.PipelineFatalException If any of the following attributes do not exist in the
     *        pipeline session : <ul>
     *        <li><code>PipelineSessionConstants.USER_NAME</code></li>
     *        <li><code>PipelineSessionConstants.CATALOG_ITEM_SKU</code></li>
     *        <li><code>PipelineSessionConstants.SAVED_SHOPPING_CART</code></li></ul>
     */
    public PipelineSession process(PipelineSession pipelineSession, Object requestContext) throws PipelineException
    {
        Connection connection = null;

        if(DEBUG) {
            Logger.getInstance().debug("Entering DeleteProductItemFromSavedListPCImpl.process()");
        }

	String namespace = (String) pipelineSession.getDefaultNamespace();
	
        // Get the customerId, sku, and saved list attributes from the pipeline session
	String customerId = null;
	
	try {
	    customerId = (String)pipelineSession.getSessionScopedAttribute(PipelineSessionConstants.USER_NAME, 
									   namespace);
	} catch (InvalidArgumentException e) {
            throw new PipelineException("MoveProductItemToSavedListPCImpl:Invalid argument:USERNAME", e);
        }

        if(customerId == null) {
            throw new PipelineException("ebusiness-shoppingcart:username.was.null:" + PipelineSessionConstants.USER_NAME);
        }
	
	String sku = null;
	try {
	    sku = (String)pipelineSession.getSessionScopedAttribute(PipelineSessionConstants.CATALOG_ITEM_SKU, 
								    namespace);
	} catch (InvalidArgumentException e) {
            throw new PipelineException("MoveProductItemToSavedListPCImpl:Invalid argument:SKU", e);
        }
        if(sku == null) {
            throw new PipelineException("ebusiness-shoppingcart:sku.was.null:" + 
					PipelineSessionConstants.CATALOG_ITEM_SKU);
        }

	ShoppingCart savedShoppingCart = null;
	try {
	    savedShoppingCart = (ShoppingCart) pipelineSession.getSessionScopedAttribute(PipelineSessionConstants.SAVED_SHOPPING_CART, namespace);
	} catch (InvalidArgumentException e) {
            throw new PipelineException("MoveProductItemToSavedListPCImpl:Invalid argument:saved_shoppingcart", e);
        }
        if(savedShoppingCart == null) {
            throw new PipelineException("ebusiness-shoppingcart:savedshoppingcart.was.null:" + PipelineSessionConstants.SAVED_SHOPPING_CART);
        }

        try {
            if(DEBUG) {
                Logger.getInstance().debug("In DeleteProductItemFromSavedListPCImpl.process() - removing item " + sku + " from saved shopping cart");
            }

            // Remove the line from the shopping cart
            savedShoppingCart.removeProductItem(sku);

            // Get a connection to the database
            DataSource dataSource = (javax.sql.DataSource)getEnvironmentProperty("jdbc/commercePool");
            connection = dataSource.getConnection();

            // Create a prepared statement
            PreparedStatement deleteStatement = connection.prepareStatement("DELETE FROM WLCS_SAVED_ITEM_LIST WHERE CUSTOMER_ID = ? AND SKU = ?");
            deleteStatement.setString(1, customerId);
            deleteStatement.setString(2, sku);

            // Add an item from the save list
            deleteStatement.executeUpdate();
        }
        catch(java.sql.SQLException sqle) {
            throw new SystemException("ebusiness-shoppingcart","system.exception.in.delete.product.item.from.saved.list",sqle);
        }
        catch(javax.naming.NamingException ne) {
            throw new SystemException("ebusiness-shoppingcart","system.exception.in.delete.product.item.from.saved.list",ne);
        }
        catch(com.beasys.commerce.foundation.exception.InvalidArgumentException iae) {
            throw new PipelineException("ebusiness-shoppingcart:invalid.argument.exception.in.delete.product.item.from.saved.list",iae);
        }
        finally
        {
            // Close the database connection
            try {
                connection.close();
            }
            catch(java.sql.SQLException e) {
                Logger.getInstance().debug("SQLException occurred while trying to close connection inside finally block : " + e);
            }
        }

        if(DEBUG) {
            Logger.getInstance().debug("Leaving DeleteProductItemFromSavedListPCImpl.process()");
        }

        return pipelineSession;
    }

}
