/*
 * B E A   S Y S T E M S
 *
 * C O M M E R C E   C O M P O N E N T S
 *
 * Copyright (c) 2000-2002  BEA Systems, Inc.
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws
 * of the United States. The software contained on this media is proprietary
 * to and embodies the confidential technology of BEA Systems, Inc. The
 * possession or receipt of this information does not convey any right to disclose
 * its contents, reproduce it, or use,  or license the use, for manufacture or
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/src/examples/e2e/b2c/shoppingcart/pipeline/MoveProductItemToSavedListPCImpl.java,v 1.2 2002/05/02 00:22:11 toneill Exp $
 */


package examples.e2e.b2c.shoppingcart.pipeline;

import java.rmi.RemoteException;
import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.SessionContext;
import javax.naming.NamingException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import javax.sql.DataSource;

import com.beasys.commerce.foundation.SessionImpl;
import com.beasys.commerce.util.Logger;

import com.beasys.commerce.ebusiness.catalog.*;
import com.beasys.commerce.ebusiness.customer.*;
import com.beasys.commerce.ebusiness.shoppingcart.*;
import com.beasys.commerce.ebusiness.catalog.service.item.*;
import com.beasys.commerce.foundation.exception.SystemException;
import com.beasys.commerce.ebusiness.shoppingcart.ShoppingCart;
import com.beasys.commerce.ebusiness.catalog.ProductItem;

import com.beasys.commerce.foundation.pipeline.PipelineConstants;
import com.beasys.commerce.webflow.PipelineSessionConstants;
import com.beasys.commerce.webflow.HttpRequestConstants;

import com.bea.p13n.appflow.exception.*;
import com.bea.p13n.appflow.common.PipelineSession;

/**
 * MoveProductItemToSavedListPCImpl is the implementation for the
 * MoveProductItemToSavedListPC interface.
 *
 * MoveProductItemToSavedListPC is a pipeline component that removes a
 * <code>ProductItem</code> from a customer's shopping cart and adds
 * it to their list of saved items.  The customer's login id is
 * specified by the <code>PipelineSessionConstants.USER_NAME</code>
 * attribute in the Pipeline Session.  The sku of the item to move is
 * specified by the
 * <code>PipelineSessionConstants.CATALOG_ITEM_SKU</code> pipeline
 * session attribute.  The
 * <code>PipelineSessionConstants.SAVED_SHOPPING_CART</code> and
 * <code>PipelineSessionConstants.SHOPPING_CART</code> pipeline
 * session attributes, and the WLCS_SAVED_ITEM_LIST table are updated
 * to reflect the change.
 *
 * @see com.beasys.commerce.ebusiness.shoppingcart.pipeline.MoveProductItemToSavedListPC
 * @see com.beasys.commerce.ebusiness.shoppingcart.pipeline.MoveProductItemToSavedListPCHome
 * @see com.bea.p13n.appflow.pipeline.PipelineComponent
 * @see com.bea.p13n.appflow.pipeline.PipelineSession
 * @see com.beasys.commerce.webflow.PipelineSessionConstants
 * @see com.beasys.commerce.ebusiness.shoppingcart.ShoppingCart
 */
public class MoveProductItemToSavedListPCImpl extends SessionImpl
{

    private static final boolean DEBUG = false;

    /**
     * Default constructor.
     */
    public MoveProductItemToSavedListPCImpl()
    {
        super();
    }

    /**
     * ejbCreate() method.
     */
    public void ejbCreate() throws CreateException
    {
        super.ejbCreate();
    }

    /**
     * ejbPostCreate() method.
     */
    public void ejbPostCreate() throws CreateException
    {
        super.ejbPostCreate();
    }

    /**
     * ejbActivate() method.
     */
    public void ejbActivate() throws EJBException
    {
        super.ejbActivate();
    }

    /**
     * ejbPassivate() method.
     */
    public void ejbPassivate() throws EJBException
    {
        super.ejbPassivate();
    }

    /**
     * ejbRemove() method.
     */
    public void ejbRemove() throws EJBException
    {
        super.ejbRemove();
    }

    /**
     * setSessionContext() method.
     */
    public void setSessionContext(SessionContext ctx) throws EJBException
    {
        super.setSessionContext(ctx);
    }


    /**
     * See class documentation above.
     *
     * @param pipelineSession <code>PipelineSession</code> object that
     * the pipeline component will use
     *
     * @return The updated <code>PipelineSession</code> object
     *
     * @throws
     * com.beasys.commerce.foundation.exception.PipelineFatalException
     * If any of the following attributes do not exist in the pipeline
     * session : <ul>
     * <li><code>PipelineSessionConstants.USER_NAME</code></li>
     * <li><code>PipelineSessionConstants.CATALOG_ITEM_SKU</code></li>
     * <li><code>PipelineSessionConstants.SHOPPING_CART</code></li>
     * <li><code>PipelineSessionConstants.SAVED_SHOPPING_CART</code></li></ul>
     */
    public PipelineSession process(PipelineSession pipelineSession, Object requestContext) throws PipelineException
    {
        Connection connection = null;

	if(DEBUG) {
            Logger.getInstance().debug("Entering MoveProductItemToSavedListPCImpl.process()");
        }

	String namespace = (String) pipelineSession.getDefaultNamespace();
	
        // Get the customerId, sku, shopping cart and saved list attributes from the pipeline session
	String customerId = null;
	try {
	    customerId = (String)pipelineSession.getSessionScopedAttribute(PipelineSessionConstants.USER_NAME, 
									   namespace);
	} catch (InvalidArgumentException e) {
            throw new PipelineException("MoveProductItemToSavedListPCImpl:Invalid argument:USERNAME", e);
        }

        if(customerId == null) {
            throw new PipelineException("ebusiness-shoppingcart:username.was.null:" +
					PipelineSessionConstants.USER_NAME);
        }

	String sku = null;
	try {
	    sku = (String)pipelineSession.getSessionScopedAttribute(PipelineSessionConstants.CATALOG_ITEM_SKU, 
								    namespace);
	} catch (InvalidArgumentException e) {
            throw new PipelineException("MoveProductItemToSavedListPCImpl:Invalid argument:SKU", e);
        }
        if(sku == null) {
            throw new PipelineException("ebusiness-shoppingcart:sku.was.null:" + 
					PipelineSessionConstants.CATALOG_ITEM_SKU);
        }

	ShoppingCart shoppingCart = null;
	try {
	    shoppingCart = (ShoppingCart) pipelineSession.getSessionScopedAttribute(PipelineSessionConstants.SHOPPING_CART, namespace);
	} catch (InvalidArgumentException e) {
            throw new PipelineException("MoveProductItemToSavedListPCImpl:Invalid argument:shoppingcart", e);
        }

        if(shoppingCart == null) {
            throw new PipelineException("ebusiness-shoppingcart:shoppingcart.was.null:" + PipelineSessionConstants.SHOPPING_CART);
        }

	ShoppingCart savedShoppingCart = null;
	try {
	    savedShoppingCart = (ShoppingCart) pipelineSession.getSessionScopedAttribute(PipelineSessionConstants.SAVED_SHOPPING_CART, namespace);
	} catch (InvalidArgumentException e) {
            throw new PipelineException("MoveProductItemToSavedListPCImpl:Invalid argument:saved_shoppingcart", e);
        }

        if(savedShoppingCart == null) {
            throw new PipelineException("ebusiness-shoppingcart:savedshoppingcart.was.null:" + PipelineSessionConstants.SAVED_SHOPPING_CART);
        }
	
        try {
            if(DEBUG) {
                Logger.getInstance().debug("In MoveProductItemToSavedListPCImpl.process() - removing item " + sku + " from shopping cart");
            }
	    
            Integer currentQuantity = new Integer(1);
            try {
		double tmp = shoppingCart.findShoppingCartLine(sku).getQuantity();
               currentQuantity = new Integer((int) tmp );
            } catch (Exception ignore) {
            }

            // Remove the item from the shopping cart
            ProductItem item = shoppingCart.removeProductItem(sku);

            if(DEBUG) {
                Logger.getInstance().debug("In MoveProductItemToSavedListPCImpl.process() - adding item " + sku + " to saved shopping cart");
            }
	    
            // Add the item to the saved list
            savedShoppingCart.addProductItem(item,1);

            // For the events firing, we need a product item
            // on CATALOG_ITEM
	    try {
            	pipelineSession.setRequestScopedAttribute( PipelineSessionConstants.CATALOG_ITEM,
							   item,
							   namespace,
							   requestContext );
            	pipelineSession.setRequestScopedAttribute( PipelineSessionConstants.QUANTITY,
							   currentQuantity,
							   namespace,
							   requestContext );
	    } catch (InvalidArgumentException e) {
            	throw new PipelineException("MoveProductItemToSavedListPCImpl:Invalid argument:item/quantity", e);
	    }
	    
            // Get a connection to the database
            DataSource dataSource = (javax.sql.DataSource) getEnvironmentProperty("jdbc/commercePool");
            connection = dataSource.getConnection();
	    
            // Create a prepared statement
            PreparedStatement insertStatement = connection.prepareStatement("INSERT INTO WLCS_SAVED_ITEM_LIST(CUSTOMER_ID, SKU) values( ?, ?)");
            insertStatement.setString(1, customerId);
            insertStatement.setString(2, sku);
	    
            // Add an item from the save list
            insertStatement.executeUpdate();
        }
        catch(java.sql.SQLException sqle) {
            throw new SystemException("ebusiness-shoppingcart",
				      "system.exception.in.move.product.item.to.saved.list",sqle);
        }
        catch(javax.naming.NamingException ne) {
            throw new SystemException("ebusiness-shoppingcart",
				      "system.exception.in.move.product.item.to.saved.list",ne);
        }
        catch(com.beasys.commerce.foundation.exception.InvalidArgumentException iae) {
            throw new PipelineException("ebusiness-shoppingcart:invalid.argument.exception.in.move.product.item.to.saved.list",iae);
        }
        finally {
            // Close the database connection
            try {
                connection.close();
            }
            catch(java.sql.SQLException e) {
            }
        }

	if(DEBUG) {
            Logger.getInstance().debug("Leaving MoveProductItemToSavedListPCImpl.process()");
        }

        return pipelineSession;
    }

}
