/*
 * B E A S Y S T E M S
 *
 * C O M M E R C E   C O M P O N E N T S
 *
 * Copyright (c) 2000-2002  BEA Systems, Inc.
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws
 * of the United States. The software contained on this media is proprietary
 * to and embodies the confidential technology of BEA Systems, Inc. The
 * possession or receipt of this information does not convey any right to disclose
 * its contents, reproduce it, or use,  or license the use, for manufacture or
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/src/examples/e2e/b2c/shoppingcart/pipeline/PriceShoppingCartPC.java,v 1.2 2002/05/02 00:22:11 toneill Exp $
 */

package examples.e2e.b2c.shoppingcart.pipeline;

import java.util.*;
import java.rmi.*;
import javax.ejb.*;
import javax.naming.*;

import com.bea.commerce.ebusiness.price.service.*;
import com.bea.commerce.ebusiness.price.quote.*;

import com.beasys.commerce.foundation.*;
import com.beasys.commerce.util.*;
import com.beasys.commerce.axiom.util.helper.*;
import com.beasys.commerce.ebusiness.catalog.*;
import com.beasys.commerce.ebusiness.shoppingcart.*;
import com.beasys.commerce.ebusiness.customer.*;
import com.beasys.commerce.ebusiness.catalog.service.item.*;
import com.beasys.commerce.ebusiness.order.*;
import com.beasys.commerce.axiom.units.Money;  

import com.beasys.commerce.webflow.PipelineSessionConstants;

import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.pipeline.PipelineComponent;
import com.bea.p13n.appflow.exception.PipelineException;
import com.bea.p13n.appflow.pipeline.PipelineComponentSupport;
import com.bea.p13n.management.ApplicationHelper;
import com.bea.p13n.util.JndiHelper;

/**
 * This pipeline component invokes the Pricing Service to price each of
 * the shopping cart lines and compute the subtotal.
 *
 * @see com.beasys.commerce.ebusiness.shoppingcart.pipeline.DeleteProductItemFromSavedListPCHome
 * @see com.beasys.commerce.ebusiness.shoppingcart.pipeline.DeleteProductItemFromSavedListPCImpl
 * @see com.bea.p13n.appflow.pipeline.PipelineComponent
 * @see com.bea.p13n.appflow.pipeline.PipelineSession
 * @see com.beasys.commerce.webflow.PipelineSessionConstants
 * @see com.beasys.commerce.ebusiness.shoppingcart.ShoppingCart
 */
public class PriceShoppingCartPC extends PipelineComponentSupport 
{

    private static final boolean DEBUG = false;
    private final String PRICE_SERVICE_HOME = ".BEA_commerce.PriceService";

    public PipelineSession process(PipelineSession pipelineSession, Object requestContext) throws PipelineException
    {
	String namespace = (String) pipelineSession.getDefaultNamespace();
	
        if(DEBUG) {
            Logger.getInstance().debug("Entering PriceShoppingCartPC.process()");
        }

        // Get the shopping cart from the pipeline session
        ShoppingCart shoppingCart = (ShoppingCart) getSessionAttribute(PipelineSessionConstants.SHOPPING_CART, 
								       namespace, pipelineSession);

        // Nothing to do if no shopping cart or an empty on.
        if(shoppingCart == null) {
            return pipelineSession; 
        }

        // Make sure the total and subtotal are ok
        if(shoppingCart.isEmpty()) {
            shoppingCart.setTotal(com.beasys.commerce.axiom.units.MoneyHome.create());
            shoppingCart.setSubtotal(com.beasys.commerce.axiom.units.MoneyHome.create());
            return pipelineSession;
        }

        // Prepare the argument to the pricing engine
        PricingRequestImpl pricingRequest = new PricingRequestImpl();

        // Get the customerId, sku, and saved list attributes from the pipeline session
        String customerId = (String) getSessionAttribute(PipelineSessionConstants.USER_NAME, namespace,
							 pipelineSession);

        // If the customerId is null, the PricingRequest's customerId is null
        if(customerId != null) {
            pricingRequest.setCustomerPK(new CustomerPk(customerId) );
        }

        // Iterate of the shopping cart lines to set the price & stuff.
        Iterator lineIterator = shoppingCart.shoppingCartLineCollectionIterator();
        Line[] lines = new Line[shoppingCart.shoppingCartLineCollectionSize( )];
        int i = 0;

        // Since the Shopping Cart does not have an associated currency, 
        // we need to extract it indirectly; We use a product item's current price
        // to do so. It is sub-optimal, but one reliable way to do so.
        // We need the currency to set the baseShipping to 0 of that currency.
        String currency = "USD"; // the default currency, in case we don't get it next

        // Fill the Line[] of the PricingRequest.
        while(lineIterator.hasNext()) {
            ShoppingCartLine scl = (ShoppingCartLine) lineIterator.next();
            ProductItem pi = scl.getProductItem();
            currency = pi.getCurrentPrice().getCurrency();
            double q = scl.getQuantity();
            lines[i] = new Line(pi, q);
            i++;
        }
        pricingRequest.setLines(lines);  

	// Put in a dummy shipping, since shipping only happens
	// after the shopping cart pricing.
        pricingRequest.setBaseShipping(com.beasys.commerce.axiom.units.MoneyHome.create(0.0d, currency) );


        // Make the call to the Pricing Service!
        PricingReply pricingReply = null;
        String ref = ApplicationHelper.getApplicationName() +
          PRICE_SERVICE_HOME;
        try {
            PriceServiceHome psHome = (PriceServiceHome)JndiHelper.lookupNarrow(ref,
										PriceServiceHome.class);
            PriceService ps = psHome.create();
            pricingReply = ps.computeSubtotal(pricingRequest);
        }
        catch(PricingException pe) {
            throw new PipelineException("ebusiness-order:order.lookup.exception" + ref, pe);
        }
        catch(javax.ejb.CreateException ce) {
            throw new PipelineException("ebusiness-order:order.lookup.exception" + ref, ce);
        }
        catch(javax.naming.NamingException ne) {
            throw new PipelineException("ebusiness-order:order.lookup.exception" + ref, ne);
        }
        catch(RemoteException re) {
            throw new PipelineException("ebusiness-order:create.priceservice.remote.exception", re);
        }

        // Use the returned values to set the attributes.
        Line[] returnLines = pricingReply.getLines();
        lineIterator = shoppingCart.shoppingCartLineCollectionIterator();

        // Use the returned value from the Pricing Service to
        // fill the discounted price
        i = 0;
        while(lineIterator.hasNext()) {
            ShoppingCartLine scl = (ShoppingCartLine) lineIterator.next();
            scl.setDiscountPresentations( (ArrayList) returnLines[i].getDiscountDescriptions() );
            scl.setDiscountedPrice(returnLines[i].getTotal());
            i++;
        }

        // Set the total and subtotal for the shopping cart
        Money subtotal = pricingReply.getSubtotal();
        shoppingCart.setSubtotal (subtotal);
        shoppingCart.setTotal( pricingReply.getTotal() );

        shoppingCart.setOrderDiscountPresentations((ArrayList) pricingReply.getOrderDiscounts() );
        shoppingCart.setShippingDiscountPresentations( (ArrayList) pricingReply.getShippingDiscounts() );

        if(DEBUG) {
            Logger.getInstance().debug("Leaving PriceShoppingCartPCImpl.process()");
        }

        return pipelineSession;
    }

}
