/*
 * B E A   S Y S T E M S
 *
 * C O M M E R C E   C O M P O N E N T S
 *
 * Copyright (c) 2000-2002  BEA Systems, Inc.
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws
 * of the United States. The software contained on this media is proprietary
 * to and embodies the confidential technology of BEA Systems, Inc. The
 * possession or receipt of this information does not convey any right to disclose
 * its contents, reproduce it, or use,  or license the use, for manufacture or
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/src/examples/e2e/b2c/shoppingcart/pipeline/RefreshSavedListPCImpl.java,v 1.2 2002/05/02 00:22:11 toneill Exp $
 */


package examples.e2e.b2c.shoppingcart.pipeline;

import java.rmi.RemoteException;
import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.SessionContext;
import javax.naming.NamingException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import javax.sql.DataSource;

import com.beasys.commerce.foundation.SessionImpl;
import com.beasys.commerce.util.Logger;
import com.beasys.commerce.foundation.exception.SystemException;
import com.beasys.commerce.foundation.pipeline.PipelineConstants;
import com.beasys.commerce.ebusiness.shoppingcart.ShoppingCart;
import com.beasys.commerce.ebusiness.shoppingcart.ShoppingCartHome;
import com.beasys.commerce.ebusiness.catalog.CatalogRequest;
import com.beasys.commerce.ebusiness.catalog.CatalogManager;
import com.beasys.commerce.ebusiness.catalog.ProductItem;
import com.beasys.commerce.ebusiness.catalog.ProductItemKey;
import com.beasys.commerce.ebusiness.catalog.service.item.ProductItemManager;
import com.beasys.commerce.webflow.PipelineSessionConstants;
import com.beasys.commerce.webflow.HttpRequestConstants;

import com.bea.p13n.appflow.exception.*;
import com.bea.p13n.appflow.common.PipelineSession;

import examples.e2e.b2c.catalog.pipeline.CatalogPC;

/**
 * <p>RefreshSavedListPCImpl is the implementation for the
 * RefreshSavedListPC interface.
 *
 * <p>RefreshSavedListPC updates a customer's list of saved items in
 * the pipeline session.  It queries the WLCS_SAVED_LIST table to get
 * the customer's list of saved items, creates a
 * <code>ShoppingCart</code> object that contains them, and updates
 * the <code>PipelineSessionConstants.SAVED_SHOPPING_CART</code>
 * pipeline session attribute with this object.  The customer's login
 * id is specified by the
 * <code>PipelineSessionConstants.USER_NAME</code> pipeline session
 * attribute.
 *
 * @see com.beasys.commerce.ebusiness.shoppingcart.pipeline.RefreshSavedListPCHome
 * @see com.beasys.commerce.ebusiness.shoppingcart.pipeline.RefreshSavedListPCImpl
 * @see com.bea.p13n.appflow.pipeline.PipelineComponent
 * @see com.bea.p13n.appflow.pipeline.PipelineSession
 * @see com.beasys.commerce.webflow.PipelineSessionConstants
 * @see com.beasys.commerce.ebusiness.shoppingcart.ShoppingCart
 */
public class RefreshSavedListPCImpl extends SessionImpl
{

    private static final boolean DEBUG = false;

    /**
     * Default constructor.
     */
    public RefreshSavedListPCImpl()
    {
        super();
    }

    /**
     * ejbCreate() method.
     */
    public void ejbCreate() throws CreateException
    {
        super.ejbCreate();
    }

    /**
     * ejbPostCreate() method.
     */
    public void ejbPostCreate() throws CreateException
    {
        super.ejbPostCreate();
    }

    /**
     * ejbActivate() method.
     */
    public void ejbActivate() throws EJBException
    {
        super.ejbActivate();
    }

    /**
     * ejbPassivate() method.
     */
    public void ejbPassivate() throws EJBException
    {
        super.ejbPassivate();
    }

    /**
     * ejbRemove() method.
     */
    public void ejbRemove() throws EJBException
    {
        super.ejbRemove();
    }

    /**
     * setSessionContext() method.
     */
    public void setSessionContext(SessionContext ctx) throws EJBException
    {
        super.setSessionContext(ctx);
    }

    /**
     * See class documentation above.
     *
     * @param pipelineSession <code>PipelineSession</code> object that
     * the pipeline component will use
     *
     * @return The updated <code>PipelineSession</code> object
     *
     * @throws
     * com.beasys.commerce.foundation.exception.PipelineFatalException
     * If the user is logged in but the
     * <code>PipelineSessionConstants.USER_NAME</code> attribute is
     * not in the pipeline session, or if there is a problem locating
     * any of the saved items in the catalog
     */
    public PipelineSession process(PipelineSession pipelineSession, Object requestContext) throws PipelineException
    {
        CatalogRequest catalogRequest = null;
        ProductItemManager productItemManager = null;
        Connection connection = null;

	String namespace = (String) pipelineSession.getDefaultNamespace();
	
        if(DEBUG) {
            Logger.getInstance().debug("Entering RefreshSavedListPCImpl.process(): namespace = " + namespace);
        }

	boolean refreshTest = false;
        String customerId = null;
	
	try {
            customerId = (String)pipelineSession.getSessionScopedAttribute(PipelineSessionConstants.USER_NAME, namespace);
	    if (customerId != null &&
		pipelineSession.getSessionScopedAttribute(PipelineSessionConstants.SAVED_SHOPPING_CART, 
							  namespace) == null)
		refreshTest = true;
	}
	catch (InvalidArgumentException e) {
            throw new PipelineException("RefreshSavedListPCImpl:Invalid argument", e);
        }

        if(DEBUG) {
            Logger.getInstance().debug("RefreshSavedListPCImpl.process(): customerId = " + customerId + "refreshTest = " + refreshTest);
            Logger.getInstance().debug("RefreshSavedListPCImpl.process(): pipelineSession SAVED SHOPPING CART = = " + 
                                       pipelineSession.getSessionScopedAttribute(PipelineSessionConstants.SAVED_SHOPPING_CART, namespace)
                                       );
        }

        // check if we need to refresh the saved list
        if(refreshTest) {

            // Get the customerId from the pipeline session
	    try {
            	customerId = (String)pipelineSession.getSessionScopedAttribute(PipelineSessionConstants.USER_NAME, 
									       namespace);
	    }
	    catch (InvalidArgumentException e) {
            	throw new PipelineException("RefreshSavedListPCImpl:Invalid argument", e);
	    }
	    
            if(customerId == null) {
                throw new PipelineException("ebusiness-shoppingcart:username.was.null" + PipelineSessionConstants.USER_NAME);
            }
	    
            try {
                // Get a connection to the database
                DataSource dataSource = (javax.sql.DataSource) getEnvironmentProperty("jdbc/commercePool");
                connection  = dataSource.getConnection();
		
                // Create a prepared statement object for the query we're about to do
                PreparedStatement queryStatement = connection.prepareStatement("SELECT SKU FROM WLCS_SAVED_ITEM_LIST WHERE CUSTOMER_ID = ?");
                queryStatement.setString(1, customerId);

                // To add an item from the save list
                ResultSet resultSet = queryStatement.executeQuery();

                // Create a shopping cart object
                ShoppingCart savedShoppingCart = ShoppingCartHome.create();

                // Check if there are any items in the saved list
                boolean more = resultSet.next();

                // If the result set is not empty, set up the product item manager and catalog manager
                if(more == true) {
                    CatalogManager catalogManager = CatalogPC.getCatalogManager(pipelineSession);
                    catalogRequest = CatalogPC.getCatalogRequest(pipelineSession);
                    productItemManager = catalogManager.getProductItemManager(catalogRequest);
                }

                if(DEBUG) {
                    Logger.getInstance().debug("In RefreshSavedListPCImpl.process() - about to populate saved shopping cart");
                }

                // iterate through the result set and add the product items to the shopping cart
                while(more == true) {
                    // Get the sku
                    String sku = (String)resultSet.getObject(1);
		    
                    // Use the sku to create the product item
                    ProductItem item = productItemManager.getItem(catalogRequest, new ProductItemKey(sku));

                    // Add the item to the saved shopping cart
                    savedShoppingCart.addProductItem(item,1);

                    // Move on to the next line of the saved shopping cart
                    more = resultSet.next();
                }

                if(DEBUG) {
                    Logger.getInstance().debug("In RefreshSavedListPCImpl.process() - updating saved shopping cart in pipeline session with " + savedShoppingCart.toString() );
                }

		try {
		    // update the saved list in the pipeline session
    	            pipelineSession.setSessionScopedAttribute(PipelineSessionConstants.SAVED_SHOPPING_CART, 
							      savedShoppingCart, namespace);
		}
		catch(InvalidArgumentException e) {
		    throw new PipelineException("RefreshSavedListPCImpl:Invalid argument", e);
		}
            }
            catch(java.rmi.RemoteException re) {
                throw new SystemException("ebusiness-shoppingcart","system.exception.in.refresh.saved.list", 
					  re);
            }
            catch(java.sql.SQLException sqle) {
                throw new SystemException("ebusiness-shoppingcart","system.exception.in.refresh.saved.list", 
					  sqle);
            }
            catch(javax.ejb.CreateException ce) {
                throw new SystemException("ebusiness-shoppingcart","system.exception.in.refresh.saved.list",
					  ce);
            }
            catch(javax.naming.NamingException ne) {
                throw new SystemException("ebusiness-shoppingcart","system.exception.in.refresh.saved.list",
					  ne);
            }
            catch(com.beasys.commerce.ebusiness.catalog.CatalogFinderException cfe) {
                throw new PipelineException("ebusiness-shoppingcart:catalog.finder.exception.in.refresh.saved.list",
					    cfe);
            }
            catch(com.beasys.commerce.ebusiness.catalog.CatalogException ce) {
                throw new PipelineException("ebusiness-shoppingcart:catalog.exception.in.refresh.saved.list",
					    ce);
            }
            catch(com.beasys.commerce.foundation.exception.InvalidArgumentException iae) {
                throw new PipelineException("ebusiness-shoppingcart:invalid.argument.exception.in.refresh.saved.list",
					    iae);
            }
            finally {
                // Close the database connection
                try {
                    connection.close();
                }
                catch(java.sql.SQLException e) {
                }
            }
        }
        else
        {
            if(DEBUG) {
                Logger.getInstance().debug("In RefreshSavedListPCImpl.process() - did not refresh saved shopping cart - already exists in pipeline session or user not logged in");
            }
        }

        if(DEBUG) {
            Logger.getInstance().debug("Leaving RefreshSavedListPCImpl.process()");
        }

        return pipelineSession;
    }

}
