/*
 * B E A   S Y S T E M S
 *
 * C O M M E R C E   C O M P O N E N T S
 *
 * Copyright (c) 2000-2002  BEA Systems, Inc.
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws
 * of the United States. The software contained on this media is proprietary
 * to and embodies the confidential technology of BEA Systems, Inc. The
 * possession or receipt of this information does not convey any right to disclose
 * its contents, reproduce it, or use,  or license the use, for manufacture or
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/src/examples/e2e/b2c/shoppingcart/pipeline/SaveProductItemPCImpl.java,v 1.2 2002/05/02 00:22:11 toneill Exp $
 */


package examples.e2e.b2c.shoppingcart.pipeline;

import java.rmi.RemoteException;
import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.SessionContext;
import javax.naming.NamingException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import javax.sql.DataSource;

import com.beasys.commerce.foundation.SessionImpl;
import com.beasys.commerce.util.Logger;

import com.beasys.commerce.ebusiness.catalog.*;
import com.beasys.commerce.ebusiness.customer.*;
import com.beasys.commerce.ebusiness.shoppingcart.*;
import com.beasys.commerce.ebusiness.catalog.service.item.*;
import com.beasys.commerce.foundation.exception.SystemException;
import com.beasys.commerce.ebusiness.shoppingcart.ShoppingCart;

import com.beasys.commerce.foundation.pipeline.PipelineConstants;
import com.beasys.commerce.webflow.PipelineSessionConstants;
import com.beasys.commerce.webflow.HttpRequestConstants;

import com.bea.p13n.appflow.exception.*;
import com.bea.p13n.appflow.common.PipelineSession;

/**
 * SaveProductItemPC is a pipeline component that uses a
 * the sku of the item, creates that ProductItem, places it
 * in the savedShoppingCart, and persists the change to the database
 * Needs:
 * <code>PipelineSessionConstants.CATALOG_ITEM_SKU</code> pipeline
 * session attribute.  The
 * <code>PipelineSessionConstants.SAVED_SHOPPING_CART</code> and
 * <code>PipelineSessionConstants.SHOPPING_CART</code> pipeline
 * session attributes, and the WLCS_SAVED_ITEM_LIST table are updated
 * to reflect the change.
 *
 *
 * @see com.beasys.commerce.ebusiness.shoppingcart.pipeline.SaveProductItemPCHome
 * @see com.beasys.commerce.ebusiness.shoppingcart.pipeline.SaveProductItemPCImpl
 * @see com.bea.p13n.appflow.pipeline.PipelineComponent
 * @see com.bea.p13n.appflow.pipeline.PipelineSession
 * @see com.beasys.commerce.webflow.PipelineSessionConstants
 * @see com.beasys.commerce.ebusiness.shoppingcart.ShoppingCart
 */
public class SaveProductItemPCImpl extends SessionImpl
{

    private static final boolean DEBUG = true;

    /**
     * Default constructor.
     */
    public SaveProductItemPCImpl()
    {
        super();
    }

    /**
     * ejbCreate() method.
     */
    public void ejbCreate() throws CreateException
    {
        super.ejbCreate();
    }

    /**
     * ejbPostCreate() method.
     */
    public void ejbPostCreate() throws CreateException
    {
        super.ejbPostCreate();
    }

    /**
     * ejbActivate() method.
     */
    public void ejbActivate() throws EJBException
    {
        super.ejbActivate();
    }

    /**
     * ejbPassivate() method.
     */
    public void ejbPassivate() throws EJBException
    {
        super.ejbPassivate();
    }

    /**
     * ejbRemove() method.
     */
    public void ejbRemove() throws EJBException
    {
        super.ejbRemove();
    }

    /**
     * setSessionContext() method.
     */
    public void setSessionContext(SessionContext ctx) throws EJBException
    {
        super.setSessionContext(ctx);
    }


    /**
     * See class documentation above.
     *
     * @param pipelineSession <code>PipelineSession</code> object that
     * the pipeline component will use
     *
     * @return The updated <code>PipelineSession</code> object
     *
     * @throws
     * com.beasys.commerce.foundation.exception.PipelineFatalException
     * If any of the following attributes do not exist in the pipeline
     * session : <ul>
     * <li><code>PipelineSessionConstants.USER_NAME</code></li>
     * <li><code>PipelineSessionConstants.CATALOG_ITEM_SKU</code></li>
    ** <li><code>PipelineSessionConstants.CATALOG_ITEM</code></li>
     * <li><code>PipelineSessionConstants.SAVED_SHOPPING_CART</code></li>
     * </ul>
     */
    public PipelineSession process(PipelineSession pipelineSession, Object requestContext) throws PipelineException
    {
        Connection connection = null;

	if(DEBUG) {
            Logger.getInstance().debug("Entering SaveProductItemPCImpl.process()");
        }

	String namespace = (String) pipelineSession.getDefaultNamespace();
	
        // 1. Get the customerId -----------------
	String customerId = null;
	try {
	    customerId = (String)pipelineSession.getSessionScopedAttribute(PipelineSessionConstants.USER_NAME, 
									   namespace);
	} catch (InvalidArgumentException e) {
            throw new PipelineException("SaveProductItemPCImpl:Invalid argument:USERNAME", e);
        }

        if(customerId == null) {
            throw new PipelineException("ebusiness-shoppingcart:username.was.null:" +
					PipelineSessionConstants.USER_NAME);
        }
	
        // 2. Get the sku         -----------------
	String sku = null;
	try {
            ProductItemKey key = (ProductItemKey) pipelineSession.getSessionScopedAttribute(PipelineSessionConstants.CATALOG_ITEM_KEY, 
                                                                                            namespace);
            sku = key.getIdentifier();
	} 
	catch (InvalidArgumentException e) {
            throw new PipelineException("SaveProductItemPCImpl:Invalid argument:SKU", e);
        }
        if(sku == null) {
            throw new PipelineException("ebusiness-shoppingcart:sku.was.null:" + 
					PipelineSessionConstants.CATALOG_ITEM_SKU);
        }

        // 3. Get the product item ----------------
	ProductItem item = null;
	try {
	    item = (ProductItem)pipelineSession.getSessionScopedAttribute(PipelineSessionConstants.CATALOG_ITEM, 
								    namespace);
	} catch (InvalidArgumentException e) {
            throw new PipelineException("SaveProductItemPCImpl:Invalid argument:CATALOG_ITEM", e);
        }
        if(item == null) {
            throw new PipelineException("ebusiness-shoppingcart:item.was.null:" + 
					PipelineSessionConstants.CATALOG_ITEM);
        }

        // 4. Get the saved list shopping cart ----
	ShoppingCart savedShoppingCart = null;
	try {
	    savedShoppingCart = (ShoppingCart) pipelineSession.getSessionScopedAttribute(PipelineSessionConstants.SAVED_SHOPPING_CART, namespace);
	} catch (InvalidArgumentException e) {
            throw new PipelineException("SaveProductItemPCImpl:Invalid argument:saved_shoppingcart", e);
        }
	
        // If shopping cart is not in the pipeline session, create one
        if(savedShoppingCart == null) {
            savedShoppingCart = ShoppingCartHome.create();
        }
	
        try {
	    
            // Add the ProductItem to the saved list
            if(DEBUG) {
                Logger.getInstance().debug("In SaveProductItemPCImpl.process() - adding item " + item + " to saved shopping cart");
            }
	    
            // Add the item to the saved list
            savedShoppingCart.addProductItem(item,1);
	    
            // Get a connection to the database
            DataSource dataSource = (javax.sql.DataSource) getEnvironmentProperty("jdbc/commercePool");
            connection = dataSource.getConnection();
	    
            // Create a prepared statement
            PreparedStatement insertStatement = connection.prepareStatement("INSERT INTO WLCS_SAVED_ITEM_LIST(CUSTOMER_ID, SKU) values( ?, ?)");
            insertStatement.setString(1, customerId);
            insertStatement.setString(2, sku);
	    
            // Add an item from the save list
            insertStatement.executeUpdate();

            // Update modified pipeline attributes
	    try {
		pipelineSession.setSessionScopedAttribute(PipelineSessionConstants.SAVED_SHOPPING_CART, 
							  savedShoppingCart, namespace);
	    } catch (InvalidArgumentException e) {
            	throw new PipelineException("SaveProductItemPCImpl:Invalid argument:shoppingcart/saved_shoppingcart", e);
	    }
        }
        catch(java.sql.SQLException sqle) {
            throw new SystemException("ebusiness-shoppingcart",
				      "system.exception.in.move.product.item.to.saved.list",sqle);
        }
        catch(javax.naming.NamingException ne) {
            throw new SystemException("ebusiness-shoppingcart",
				      "system.exception.in.move.product.item.to.saved.list",ne);
        }
        catch(com.beasys.commerce.foundation.exception.InvalidArgumentException iae) {
            throw new PipelineException("ebusiness-shoppingcart:invalid.argument.exception.in.move.product.item.to.saved.list",iae);
        }
        finally {
            // Close the database connection
            try {
                connection.close();
            }
            catch(java.sql.SQLException e) {
            }
        }

	if(DEBUG) {
            Logger.getInstance().debug("Leaving MoveProductItemToSavedListPCImpl.process()");
        }

        return pipelineSession;
    }
}
