/*
 * B E A   S Y S T E M S
 *
 * C O M M E R C E   C O M P O N E N T S
 *
 * Copyright (c) 2000-2002  BEA Systems, Inc.
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws
 * of the United States. The software contained on this media is proprietary
 * to and embodies the confidential technology of BEA Systems, Inc. The
 * possession or receipt of this information does not convey any right to disclose
 * its contents, reproduce it, or use,  or license the use, for manufacture or
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 */

package examples.e2e.b2c.tax;

// java imports
import java.util.*;
import java.rmi.*;

// javax imports
import javax.ejb.*;
import javax.naming.*;

// com.beasys imports
import com.beasys.commerce.foundation.*;
import com.beasys.commerce.foundation.exception.*;
import com.beasys.commerce.axiom.units.*;
import com.beasys.commerce.axiom.contact.*;
import com.beasys.commerce.ebusiness.tax.*;
import com.beasys.commerce.util.*;

// com.bea imports
import com.bea.p13n.license.LicenseHelper;
import com.bea.commerce.ebusiness.tax.*;

import examples.wlcs.sampleapp.tax.AVSResponseElement;
import examples.wlcs.sampleapp.tax.AVSResponse;
import examples.wlcs.sampleapp.tax.AVSRequest;

import examples.wlcs.sampleapp.tax.TaxResponseElement;
import examples.wlcs.sampleapp.tax.TaxResponse;
import examples.wlcs.sampleapp.tax.TaxRequestElement;
import examples.wlcs.sampleapp.tax.TaxRequest;

import examples.wlcs.sampleapp.tax.TaxWebServiceHelper;
import examples.wlcs.sampleapp.tax.TaxServicePort;

import weblogic.common.*;

/**
 * A generic Tax Calculator.
 *
 * @see com.beasys.commerce.ebusiness.tax.TaxCalculator
 * @see com.beasys.commerce.ebusiness.tax.TaxCalculatorHome
 */
public class TaxCalculatorImpl extends SessionImpl
{
    private String	CURRENCY = null;

    public TaxCalculatorImpl()
    {
		super();
		LicenseHelper.getInstance().checkLicense( com.bea.commerce.common.CommerceProduct.CORE );
    }

    public void ejbActivate() throws EJBException
    {
        super.ejbActivate();
    }

    public void ejbPassivate() throws EJBException
    {
        super.ejbPassivate();
    }

    public void ejbRemove() throws EJBException
    {
        super.ejbRemove();
    }

    public void setSessionContext(SessionContext ctx) throws EJBException
    {
        super.setSessionContext(ctx);
    }

    /**
     * Given an address (with the stipulation that the Zipcode/postalcode is correct) return a set of
     * addresses that may be the correct address (city, county combinations)  Only works for Canadian
     * and USA addresses.  Non US/Canadian addresses are returned as is.
     *
     * @param address Address which needs to be checked
     * @throws SystemException, TaxException
     */
    public Address[] getAlternateAddresses( Address address ) throws SystemException, TaxException
    {
		AVSRequest	avsRequest = new AVSRequest();
		AVSResponse	avsResponse = null;

		avsRequest.setStreetAddress1( address.getStreet1() );
		avsRequest.setStreetAddress2( address.getStreet2() );
		avsRequest.setCityName( address.getCity() );
		avsRequest.setProvinceName( address.getState() );
		avsRequest.setPostalCode( address.getPostalCode() );
		avsRequest.setCountyName( address.getCounty() );
		avsRequest.setCountyCode( address.getCountyCode() );
		avsRequest.setCountryName( address.getCountry() );

  		try
       		{

       		TaxWebServiceHelper twsHelper = new TaxWebServiceHelper();
            TaxServicePort taxPort = twsHelper.getTaxWebService();
            avsResponse = taxPort.getAlternateAddresses( avsRequest );
            // avsResponse = localGetAlternateAddresses(avsRequest); // for testing w/o web service

		}
        catch (TaxException te)
        {
            throw te;
        }
		catch( Exception re )
		{
			throw new TaxException("ebusiness-tax", "tax.pipeline.error.calculate");
		}

		Address[]	alternateAddresses = new Address[avsResponse.getResponseArray().length];
		Object[]	responses = avsResponse.getResponseArray();

		for( int i = 0; i < responses.length; i++ )
		{
			alternateAddresses[i] = AddressHome.create();
			AVSResponseElement	nextResponseElement = (AVSResponseElement)responses[i];

			alternateAddresses[i].setStreet1( nextResponseElement.getStreetAddress1() );
			alternateAddresses[i].setStreet2( nextResponseElement.getStreetAddress2() );
			alternateAddresses[i].setCity( nextResponseElement.getCityName() );
			alternateAddresses[i].setState( nextResponseElement.getProvinceName() );
			alternateAddresses[i].setPostalCode( nextResponseElement.getPostalCode1() );
			alternateAddresses[i].setGeoCode( nextResponseElement.getLocationCode() );
			alternateAddresses[i].setCounty( nextResponseElement.getCountyName() );
			alternateAddresses[i].setCountyCode( nextResponseElement.getCountyCode() );
			alternateAddresses[i].setCountry( nextResponseElement.getCountryName() );
		}

    	return( alternateAddresses );
    }

    /**
     * This essentially creates an itemLine with no taxcode, 1 unit, and calls
     * calculateTax with the line and parameters.
     *
     * @param params {@link TaxParameters} provided for tax calculation
     * @param total Total amount on which tax is to be calculated
     * @throws com.beasys.commerce.ebusiness.tax.TaxException
     */
    public Price calculateTotalTax( TaxParameters params, Price total ) throws SystemException, TaxException
    {
        TaxLineList tll = TaxLineListHome.create();
        TaxLine tl = TaxLineHome.create();


        // Create a TaxLine for the provided Gross Amount

        Price holder = PriceHome.create();
        holder.setValue( 0 );
        CURRENCY = (String)params.getAttrib( TaxParameters.CURRENCY );

        try
        {
            holder.setCurrency( CURRENCY );
        }
        catch( com.beasys.commerce.foundation.exception.InvalidArgumentException iae )
        {
	    System.out.println( "can\'t set currency to \"" + CURRENCY + "\"" );
	    return( null );
        }

        Price freightCost = (Price)params.getAttrib( TaxParameters.FREIGHT );
        if ( freightCost == null )
        {
            freightCost = holder;
        }
        tl.setFreight( freightCost );
        tl.setPrice( total );
        com.beasys.commerce.axiom.units.Quantity quantity = com.beasys.commerce.axiom.units.QuantityHome.create();
        quantity.setCount( 1 );
        tl.setQuantity( quantity );
        tl.setTaxCode( "" );

        LinkedList list  = new LinkedList( );
        list.add( tl );
        tll.setTaxLineList( list );

        return calculateTax( params, tll ).getTotal( ).getTotal( );
    }


    /* This method retreives the calculated taxes at all levels from the outputRecord
     * and represents as a TaxInformation per UNIT line item
     */
    private TaxInformation getCalculatedTaxes( TaxResponseElement responseElement ) throws TaxException
    {
        TaxInformation result = TaxInformationHome.create( );
        TaxLineResult tlr;
        Price total = PriceHome.create( );
        try
        {
            total.setCurrency( CURRENCY );
        }
        catch( com.beasys.commerce.foundation.exception.InvalidArgumentException iae )
        {
	    System.out.println( "getCalculatedTaxes() - can\'t set currency to \"" + CURRENCY + "\"" );
	    return( null );
        }

        total.setValue( 0 );

        Double taxState = responseElement.getProvinceCalculatedTaxAmount();
        if( (taxState != null) && (taxState.doubleValue() > 0.0) )
        {
            Price statePrice = PriceHome.create( );
            statePrice.setValue( taxState.doubleValue() );
            try
            {
                statePrice.setCurrency( CURRENCY );
            }
            catch( com.beasys.commerce.foundation.exception.InvalidArgumentException iae )
            {
		System.out.println( "getCalculatedTaxes() - statePrice - can\'t set currency to \"" + CURRENCY + "\"" );
		return( null );
            }
            tlr = TaxLineResultHome.create( );
            tlr.setTax( statePrice );
            result.addEntityTax( tlr );
            total.setValue( total.getValue( ) + statePrice.getValue( ) );
        }

        Double taxCounty = responseElement.getCountyCalculatedTaxAmount();
        if( (taxCounty != null) && (taxCounty.doubleValue() > 0.0) )
        {
            Price countyPrice = PriceHome.create( );
            countyPrice.setValue( taxCounty.doubleValue() );
            try
            {
                countyPrice.setCurrency( CURRENCY );
            }
            catch( com.beasys.commerce.foundation.exception.InvalidArgumentException iae )
            {
		System.out.println( "getCalculatedTaxes() - countryPrice - can\'t set currency to \"" + CURRENCY + "\"" );
		return( null );
            }
            tlr = TaxLineResultHome.create( );
            tlr.setTax( countyPrice );
            result.addEntityTax( tlr );
            total.setValue( total.getValue( ) + countyPrice.getValue( ) );
        }

        Double taxCity = responseElement.getCityCalculatedTaxAmount();
        if( (taxCity != null) && (taxCity.doubleValue() > 0.0) )
        {
            Price cityPrice = PriceHome.create( );
            cityPrice.setValue( taxCity.doubleValue() );
            try
            {
                cityPrice.setCurrency( CURRENCY );
            }
            catch( com.beasys.commerce.foundation.exception.InvalidArgumentException iae )
            {
		System.out.println( "getCalculatedTaxes() - cityPrice - can\'t set currency to \"" + CURRENCY + "\"" );
		return( null );
            }
            tlr = TaxLineResultHome.create( );
            tlr.setTax( cityPrice );
            result.addEntityTax( tlr );
            total.setValue( total.getValue( ) + cityPrice.getValue( ) );
        }

        result.setTotal( total );

        return result;
    }

     /**
    * @return the name of the message bundle used when
    * reporting errors.
    *
    */
    private String getMessageBundleName( )
    {
        return "ebusiness-tax";
    }


    public String getString( String id )
    {
        return MessageCatalog.getUserMessage( getMessageBundleName(), id );
    }

    /*
     * The setParams method sets all the fields in the UTLInputRecord with the
     * values provided in TaxParameters
     */
    private void setParams( TaxParameters params,
						    TaxRequestElement requestElement ) throws SystemException, TaxException
    {
        Map				locations = (Map)params.getLocationMap();
        Address			address = null;
		Country			country = null;
		Province		province = null;


        address = (Address) locations.get( TaxParameters.SHIPTO );

        if ( address == null )
        {
			logError( "ship to address parameter is null" );
            throw new TaxUserException( "ebusiness-tax", "tax.ship.exception" );
        }
		try
		{
			country = CountryInformation.getCountry( address.getCountry() );
		}
		catch( Exception e )
		{
			logError( "exception getting ship to country - input country is \"" + address.getCountry() + "\"" );
            throw new TaxUserException( "ebusiness-tax", "tax.country.exception",
											address.getCountry() );
		}

		if( country == null )
		{
			logError( "ship to country information is null - input country is \"" + address.getCountry() + "\"" );

            throw new TaxUserException( "ebusiness-tax", "tax.country.exception",
											address.getCountry() );
		}

		if( address.getState().trim().length() > 0 )
		{
			try
			{
				province = country.getProvince( address.getState() );
			}
			catch( Exception e )
			{
				logError( "exception getting ship to province - input province is \"" + address.getState() + "\"" );

            	throw new TaxUserException( "ebusiness-tax", "tax.country.exception",
												address.getState() );
			}
		}

		requestElement.setShipToCountryCode( country.getTwoCharacterAbbreviation() );
		requestElement.setPOACountryCode( country.getTwoCharacterAbbreviation() );
		if( province != null )
		{
			requestElement.setShipToProvinceCode( province.getTwoCharacterAbbreviation() );
			requestElement.setPOAProvinceCode( province.getTwoCharacterAbbreviation() );
		}
		requestElement.setShipToCountyName( address.getCounty() );
		requestElement.setShipToCountyCode( address.getCountyCode() );
		requestElement.setShipToCityName( address.getCity() );
		requestElement.setShipToPostalCode( address.getPostalCode() );
		requestElement.setShipToLocationCode( address.getGeoCode() );

		requestElement.setPOACountyCode( address.getCountyCode() );
		requestElement.setPOACityName( address.getCity() );
		requestElement.setPOAPostalCode( address.getPostalCode() );
		requestElement.setPOALocationCode( address.getGeoCode() );


        address = (Address) locations.get( TaxParameters.SHIPFROM );
        if ( address == null )
        {
	    logError( "ship from address parameter is null" );
            throw new TaxUserException( "ebusiness-tax", "tax.ship.exception" );
        }

	try {
	    country = CountryInformation.getCountry( address.getCountry() );
            }
	catch( Exception e )
	    {
	    logError( "exception getting ship from country - input country is \"" + address.getCountry() + "\"" );
            throw new TaxUserException( "ebusiness-tax", "tax.country.exception", address.getCountry() );
	    }

	if( country == null )
	  {
	  logError( "ship from country information is null - input country is \"" + address.getCountry() + "\"" );
          throw new TaxUserException( "ebusiness-tax", "tax.country.exception", address.getCountry() );
          }

        if( (address.getState() != null) && (address.getState().trim().length() > 0) )
	  {
	  try {
	      province = country.getProvince( address.getState() );
	      }
	  catch( Exception e )
	      {
	      logError( "exception getting ship from province - input province is \"" + address.getState() + "\"" );
              throw new TaxUserException( "ebusiness-tax", "tax.country.exception", address.getState() );
              }
	  }

	requestElement.setPrimaryCurrencyCode( CURRENCY );
        requestElement.setShipFromCountryCode( country.getTwoCharacterAbbreviation() );
	if( province != null )
	{
		requestElement.setShipFromProvinceCode( province.getTwoCharacterAbbreviation() );
	}
	requestElement.setShipFromCountyName( address.getCounty() );
	requestElement.setShipFromCountyCode( address.getCountyCode() );
	requestElement.setShipFromCityName( address.getCity() );
	requestElement.setShipFromPostalCode( address.getPostalCode() );
	requestElement.setShipFromLocationCode( address.getGeoCode() );

//		requestElement.setCountryOfOrigin( country.getISOCode() );

	String	seller_registration_number = (String)params.getAttrib( TaxParameters.SELLERREGISTRATIONNUMBER );
	if( seller_registration_number != null )
	{
		requestElement.setSellerRegistrationNumber( country.getTwoCharacterAbbreviation() +
								  seller_registration_number );
	}
	String	buyer_registration_number = (String)params.getAttrib( TaxParameters.BUYERREGISTRATIONNUMBER );
	if( buyer_registration_number != null )
	{
		requestElement.setBuyerRegistrationNumber( country.getTwoCharacterAbbreviation() +
												 buyer_registration_number );
	}

        address = (Address) locations.get( TaxParameters.ORDERORIGIN );
        if ( address != null )
        {
		try
		{
		        country = CountryInformation.getCountry( address.getCountry() );
		}
		catch( Exception e )
		{
			logError( "exception getting order origin country - input country is \"" + address.getCountry() + "\"" );
            	        throw new TaxUserException( "ebusiness-tax", "tax.country.exception", address.getCountry() );
		}
		if( country == null )
		{
		        logError( "order origin country information is null - input country is \"" + address.getCountry() + "\"" );
               	        throw new TaxUserException( "ebusiness-tax", "tax.country.exception", address.getCountry() );
		}

		requestElement.setOriginCountryCode( country.getTwoCharacterAbbreviation() );
		if( (address.getState() != null) && (address.getState().trim().length() > 0) )
		{
			try
			{
				province = country.getProvince( address.getState() );
			}
			catch( Exception e )
			{
                                logError( "exception getting origin province - input province is \"" + address.getState() + "\"" );
                                throw new TaxUserException( "ebusiness-tax", "tax.country.exception", address.getCountry() );

			}
		}
		if( province != null )
		{
			requestElement.setOriginProvinceCode( province.getTwoCharacterAbbreviation() );
		}
		requestElement.setOriginCountyName( address.getCounty() );
		requestElement.setOriginCountyCode( address.getCountyCode() );
		requestElement.setOriginCityName( address.getCity() );
		requestElement.setOriginPostalCode( address.getPostalCode() );
		requestElement.setOriginLocationCode( address.getGeoCode() );
        }
	else
	{
		requestElement.setOriginCountryCode( requestElement.getShipFromCountryCode() );
		requestElement.setOriginCountyName( requestElement.getShipFromCountyName() );
		requestElement.setOriginCountyCode( requestElement.getShipFromCountyCode() );
		requestElement.setOriginCityName( requestElement.getShipFromCityName() );
		requestElement.setOriginPostalCode( requestElement.getShipFromPostalCode() );
		requestElement.setOriginLocationCode( requestElement.getShipFromLocationCode() );
	}

        Boolean boolCheck = (Boolean)params.getAttrib( TaxParameters.CREDIT );
        if ( boolCheck!= null && boolCheck.booleanValue( ) )
        {
		requestElement.setIsCredit( new Boolean(true) );
	}
	else
	{
		requestElement.setIsCredit( new Boolean(false) );
	}

        String strCheck = (String) params.getAttrib( TaxParameters.TITLEPASSAGE );
        if ( strCheck != null )
        {
		requestElement.setPointOfTitlePassage( strCheck );
        }

        strCheck = (String)params.getAttrib( TaxParameters.COMPANYID );

        if ( strCheck != null )
        {
        	requestElement.setCompanyID( strCheck );
        }
	else
	{
		requestElement.setCompanyID( "Widgets R Us" );
	}

        strCheck = (String)params.getAttrib( TaxParameters.BUSINESSLOCATIONCODE );
        if ( strCheck != null )
        {
        	requestElement.setDivisionCode( strCheck );
        }
	else
	{
                requestElement.setDivisionCode( "Boulder Loc" );
	}

        java.sql.Date dateCheck = (java.sql.Date)params.getAttrib( TaxParameters.DATE );
        if ( dateCheck == null )
        {
            dateCheck = new java.sql.Date( System.currentTimeMillis( ) );
        }
        requestElement.saveInvoiceDate( dateCheck );
        requestElement.saveDeliveryDate( dateCheck );

        strCheck = (String) params.getAttrib( TaxParameters.TAXTYPE );
        if ( strCheck != null )
        {
		try
		{
            	requestElement.setFederalTaxType( strCheck );
            	requestElement.setProvinceTaxType( strCheck );
            	requestElement.setCountyTaxType( strCheck );
            	requestElement.setCityTaxType( strCheck );
		}
		catch( Exception e )
		{
		e.printStackTrace();
		}
        }

	String	tempString = (String)params.getAttrib( TaxParameters.TRANSACTIONTYPE );
	if( tempString != null )
	{
		requestElement.setTransactionType( tempString );
	}
	tempString = (String)params.getAttrib( TaxParameters.WTCODE );
	if( tempString != null )
	{
		requestElement.setTaxCode( tempString );
	}
	tempString = (String)params.getAttrib( TaxParameters.DOCUMENTNUMBER );
	if( tempString != null )
	{
		requestElement.setDocumentNumber( tempString );
	}
	tempString = (String)params.getAttrib( TaxParameters.MODEOFTRANSPORT );
	if( tempString != null )
	{
		requestElement.setModeOfTransport( tempString );
	}
    }


    /**
     * Calculate tax based on the parameters given.  The basic TaxCalculator requires an override
     * percentage.  If one of the additional modules are used, then additional functionality will
     * be provided.  Addresses should go through getAlternateAddresses() before being passed in.
     * Otherwise accurate tax calculation may not be performed.
     *
     * @param params {@link TaxParameters} provided for tax calculation
     * @param lines {@link TaxLineList} provided for tax calculation
     * @throws com.beasys.commerce.foundation.exception.SystemException
     * @throws com.beasys.commerce.ebusiness.tax.TaxException
     */
    public TaxResult calculateTax( TaxParameters params, TaxLineList taxLineList ) throws SystemException, TaxException
    {
	TaxRequest		request = new TaxRequest();
        TaxResult		result = null;
        List			lines = taxLineList.getTaxLineList();
	String			fallback_document_number = null;
	TaxRequestElement[] reqElements = new TaxRequestElement[ lines.size() ];

        CURRENCY = (String)params.getAttrib( TaxParameters.CURRENCY );

        TaxLine tl;



        for ( int i = 0; i < lines.size( ); i++ )
        {
            TaxRequestElement requestElement = new TaxRequestElement();

            setParams( params, requestElement );

            tl = (TaxLine)lines.get( i );

            requestElement.setLineItemAmount( new Double( tl.getPrice().getValue() ) );
            requestElement.setDiscountedLineItemAmount( new Double( tl.getDiscountedPriceAmount().getValue() ) );
            Double items = new Double( tl.getQuantity().getCount() );
            requestElement.setNumberOfItems( new Integer (items.intValue()) );
            requestElement.setCommodityCode( tl.getTaxCode() );
            requestElement.setFreightAmount( new Double( tl.getFreight().getValue() ) );

			if( requestElement.getDocumentNumber() == null )
			{
				if( fallback_document_number == null )
				{
					fallback_document_number = new Long( System.currentTimeMillis() ).toString();
				}

				requestElement.setDocumentNumber( fallback_document_number );
			}

			reqElements[i] = requestElement;
        }
        request.setRequestArray( reqElements );

        TaxResponse taxResponse = null;

        try
	    {
	        TaxWebServiceHelper twsHelper = new TaxWebServiceHelper();
            TaxServicePort taxPort = twsHelper.getTaxWebService();
            taxResponse = taxPort.calculateTax( request );
            // taxResponse = localCalculateTax( request ); //- test using local non-webservice tax calc
	    }
        catch (TaxException te)
        {
            throw te;
        }
       	catch( Exception e )
       	{
       	    throw new TaxException("ebusiness-tax", "tax.pipeline.error.calculate");
	}

        //Calculate the total tax per an output record
        LinkedList list = new LinkedList();
	TaxResponseElement[] outElementVector = taxResponse.getResponseArray();

        for ( int i = 0; i < lines.size(); i++ )
        {
            TaxResponseElement responseElement = outElementVector[i];

            tl = (TaxLine)lines.get( i );

            TaxInformation recInfo = getCalculatedTaxes( responseElement );

            list.add( recInfo );
        }

	result = TaxResultHome.create();

        result.setLine( list );
        result.calculateTotals();

        try
        {
            Price price = result.getTotal().getTotal();
            price.setCurrency( CURRENCY );
            result.getTotal().setTotal( price );

        }
        catch( Exception e )
        {
	    e.printStackTrace();
        }

        return result;
    }

	private void logError( String message )
	{
		logit( true, message );
	}

	private void logInfo( String message )
	{
		logit( false, message );
	}

	private void logit( boolean error, String message )
	{
		LogServicesDef	log = T3Services.getT3Services().log();

		try
		{
			if( error )
			{
				log.error( message );
			}
			else
			{
				log.info( message );
			}
		}
		catch( weblogic.common.T3Exception e )
		{
			System.err.println( "Can't log message to weblogic log.  Message is" );
			System.err.println( message );
			System.err.println( "weblogic log stack trace is" );
			e.printStackTrace();
		}
	}

	private TaxResponse localCalculateTax(TaxRequest taxRequest)
	{
		TaxResponse	taxResponse = new TaxResponse();
		TaxRequestElement[] requestArray = taxRequest.getRequestArray();
		TaxResponseElement[] responseArray = new TaxResponseElement[ requestArray.length ];

		for( int i = 0; i < requestArray.length; i++ )
		{
			TaxResponseElement	nextResponseElement = new TaxResponseElement();
			TaxRequestElement	nextRequestElement = requestArray[i];
			Double			lineItemAmount = nextRequestElement.getLineItemAmount();


			nextResponseElement.setProvinceCalculatedTaxAmount( new Double( lineItemAmount.doubleValue() * 0.05 ) );

			responseArray[i] = nextResponseElement;
		}
                taxResponse.setResponseArray( responseArray );

		return( taxResponse );
	}


	private AVSResponse localGetAlternateAddresses(AVSRequest avsRequest)
	{
		AVSResponse avsResponse = new AVSResponse();
		AVSResponseElement addrElement = new AVSResponseElement();

		if (avsRequest == null)
		{
			avsResponse.setResponseCode(AVSResponse.AVS_INVALID_ADDRESS);
			avsResponse.setResponseMessage("AVS Request is NULL.");
			return avsResponse;
		}

		addrElement.setStreetAddress1(avsRequest.getStreetAddress1());
		addrElement.setStreetAddress2(avsRequest.getStreetAddress2());
		addrElement.setCityName(avsRequest.getCityName());
		addrElement.setProvinceName(avsRequest.getProvinceName());
		addrElement.setProvinceCode(avsRequest.getProvinceCode());
		addrElement.setPostalCode1(avsRequest.getPostalCode());
		addrElement.setCountyName(avsRequest.getCountyName());
		addrElement.setCountyCode(avsRequest.getCountyCode());
		addrElement.setCountryName(avsRequest.getCountryName());
		addrElement.setCountryCode(avsRequest.getCountryCode());
                addrElement.setLocationCode( "00" );

		AVSResponseElement resp[] = new AVSResponseElement[1];
		resp[0] = addrElement;
		avsResponse.setResponseArray( resp );

		// avsResponse.addResponseElement(addrElement);
		avsResponse.makeResponseOK();

		return avsResponse;
	}
}
