/**
 *
 * Copyright (c) 2000-2002  BEA Systems, Inc.   All Rights Reserved
 *
 */
package examples.e2e.b2c.tax.pipeline;

import java.rmi.RemoteException;

import com.beasys.commerce.ebusiness.shoppingcart.ShoppingCart;
import com.beasys.commerce.ebusiness.shoppingcart.ShoppingCartConstants;
import com.beasys.commerce.ebusiness.tax.TaxLineList;
import com.beasys.commerce.ebusiness.tax.TaxLineListHome;
import com.beasys.commerce.ebusiness.tax.TaxParameters;
import com.beasys.commerce.ebusiness.tax.TaxParametersHome;
import com.beasys.commerce.ebusiness.tax.TaxResult;
import com.beasys.commerce.ebusiness.tax.TaxException;
import com.beasys.commerce.axiom.contact.Address;
import com.beasys.commerce.axiom.units.Money;

import com.beasys.commerce.webflow.PipelineSessionConstants;

import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.pipeline.PipelineComponent;
import com.bea.p13n.appflow.exception.PipelineException;
import com.bea.p13n.appflow.pipeline.PipelineComponentSupport;

/**
 * <code>TaxCalculateAndCommitLineLevelPC</code> calculates the tax
 * for the shopping cart and commits the result back into the shopping
 * cart.
 */
public class TaxCalculateAndCommitLineLevelPC extends TaxPC
{
    /**
    * Calculates the tax for the shopping cart and 
    * commits the result back into the shopping cart.
    * <br>
    * INPUT: PipelineSessionConstants.SHIPPING_ADDRESS <br>
    * INPUT: PipelineSessionConstants.SHOPPING_CART <br>
    * OUTPUT: PipelineSessionConstants.SHOPPING_CART <br>
    *
    * @param PipelineSession
    * @return PipelineSession
    */
    public PipelineSession process( PipelineSession pipelineSession, Object requestContext )
        throws PipelineException, RemoteException
    {
	String namespace = (String) pipelineSession.getDefaultNamespace();

        // Get the shipping address from the pipeline session
        Address shippingAddress = (Address) requireDefined(pipelineSession,
							   PipelineSessionConstants.SHIPPING_ADDRESS);

        // Get the shopping cart from the pipeline session
        ShoppingCart shoppingCart = (ShoppingCart) requireDefined(pipelineSession, 
								  PipelineSessionConstants.SHOPPING_CART);

        // If the cart is empty then do nothing...
        if(shoppingCart.shoppingCartLineCollectionSize( ) > 0) {
            // Create the service to process the shopping cart and
            // create a tax line list from the shopping cart
            TaxPipelineService tls = new TaxPipelineService( );
            TaxLineList taxLineList = tls.getTaxLineList( shoppingCart );
            TaxParameters taxParameters = tls.getTaxParameters(shippingAddress, getCommit());

            try {
                // Calculate the tax for the items in the cart
                TaxResult taxResult = getTaxCalculator().calculateTax( taxParameters, taxLineList );
                
                
                // Write the updated cart into the pipeline session
                setSessionAttribute( PipelineSessionConstants.SHOPPING_CART, 
				     tls.updateShoppingCart(taxResult, shoppingCart), 
				     namespace, pipelineSession);
                // set the total, not forgetting taxes!
                Money total = shoppingCart.getTotal();
                double totalTax = 0;
                try {
                    totalTax  = shoppingCart.getTotal(ShoppingCartConstants.LINE_TAX).getValue(); 
                }
                catch ( Exception e ) {
                    throw new PipelineException("ebusiness-order:order.tax.exception", e);
                }
                // Adds the tax to the total
                total.setValue( totalTax + total.getValue());
                shoppingCart.setTotal(total);    
            }
            catch(TaxException te) {
                throwFatal(te.getMessage(), te);  // "tax.pipeline.error.calculate"
            }
        }
        
        // Always return the updated pipeline session
        // so that it can be written back into the HttpSession
        return pipelineSession;
    }

    /**
    * @return true if the calculated tax is to be commited
    */    
    boolean getCommit()
    {
        return true;
    }
}












