/**
 *
 * Copyright (c) 2000 BEA Systems, Inc.   All Rights Reserved
 *
 */
package examples.e2e.b2c.tax.pipeline;

import java.rmi.RemoteException;
import javax.management.InstanceNotFoundException;

import com.beasys.commerce.axiom.contact.Address;
import com.beasys.commerce.ebusiness.tax.TaxException;
import com.beasys.commerce.ebusiness.tax.TaxUserException;
import com.beasys.commerce.webflow.PipelineSessionConstants;
import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.exception.PipelineException;

import com.bea.p13n.management.MBeanAccess;
import com.bea.p13n.management.ServiceConfigurationMBean;
import com.bea.p13n.management.ApplicationHelper;
import com.bea.commerce.mbeans.TaxServiceClientMBean;

import examples.e2e.b2c.tax.TaxCalculator;
import examples.e2e.b2c.tax.pipeline.TaxPipelineService;


/**
 * <code>TaxVerifyShippingAddressPC</code> verifies that the specified
 * shipping address is unambigious with regard to tax calculation.  If
 * not it returns a collection of possible addresses for the user to
 * select from.
 *
 */
public class TaxVerifyShippingAddressPC extends TaxPC
{
    /**
     * Calculates a tex code address from a Shipping Address.
     * <br>
     * INPUT: PipelineSessionConstants.SHIPPING_ADDRESS <br>
     * OUTPUT: PipelineSessionConstants.AVS_SHIPPING_ADDRESSES <br>
     * <br>
     * If the shipping address is ambigious an array of the potential
     * addresses is written into the Pipeline session.
     *
     * @param PipelineSession
     * @return PipelineSession
     */
    public PipelineSession process(PipelineSession pipelineSession , Object requestContext)
        throws PipelineException, RemoteException, TaxPipelineUserException, TaxPipelineSystemException
    {
	String namespace = (String) pipelineSession.getDefaultNamespace();	
	
        // Get the shipping address from the pipeline session
        Address shippingAddress = (Address) requireDefined(pipelineSession, 
							   PipelineSessionConstants.SHIPPING_ADDRESS);
	
        // Get the tax calculator
        TaxCalculator taxCalculator = getTaxCalculator();

        try {
	    String returnZipString = TaxVerifyShippingAddressPC.getTaxServiceClientMBean().getAVSReturnListOnZipFailure();
            boolean returnZipCodeList = false;
	    
	    if ( (returnZipString != null) && (returnZipString.compareToIgnoreCase("true") == 0) ) {
		returnZipCodeList = true;
	    }
	    
            // Get the array of possible tax addresses for the shipping address
            Address[] possibleShippingAddresses = taxCalculator.getAlternateAddresses(shippingAddress);
            
            // Save the possible addresses back into the pipeline session
            setSessionAttribute(PipelineSessionConstants.AVS_SHIPPING_ADDRESSES,
				 possibleShippingAddresses,
				 namespace,
				 pipelineSession);            
	    
            // If the user has chosen to have a list of zip codes returned when an avs
            // fails to verify a zip code, put the list of possible addresses in a
            // PipelineSession attribute especially for this case. This is necessary
            // to keep the state of the HTTP session consistent if the user hits the
            // "back" button on their Web browser.
            if (returnZipCodeList) {
                setSessionAttribute(PipelineSessionConstants.AVS_POSSIBLE_ZIPCODES, 
				     possibleShippingAddresses, 
				     namespace,
				     pipelineSession);
            }
        }
        catch (TaxException te) {
   	    throw new PipelineException(te.getMessage(), te);
	}
        
        // Always return the modified PipelineSession
        // so that the changes can be saved into the HttpSession
        return pipelineSession;
    }


    private static TaxServiceClientMBean getTaxServiceClientMBean()
    {
	TaxServiceClientMBean taxMBean = null;
	
	if(taxMBean == null) {
	    try {
		taxMBean = (TaxServiceClientMBean)ApplicationHelper.getServiceConfigurationMBean("TaxServiceClient");
	    }
	    catch(Exception e) {
		e.printStackTrace();
	    }
	}
	
	return(taxMBean);
    }
}
