/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/src/examples/e2e/b2c/webflow/CommercePipelineComponentSupport.java,v 1.2 2002/05/02 00:22:12 toneill Exp $
 */
package examples.e2e.b2c.webflow;

import com.bea.p13n.appflow.common.PipelineSession;
import com.bea.p13n.appflow.common.internal.AppflowConstants;
import com.bea.p13n.appflow.exception.InvalidArgumentException;
import com.bea.p13n.appflow.exception.PipelineException;
import com.bea.p13n.appflow.pipeline.PipelineComponent;
import com.bea.p13n.i18n.PipelineExceptionTextFormatter;
import com.bea.p13n.management.ApplicationHelper;

import com.bea.commerce.mbeans.CommercePipelineComponentSupportMBean;
import com.beasys.commerce.foundation.exception.SystemException;
import com.beasys.commerce.axiom.util.helper.CommercePropertiesHelper;

import com.bea.p13n.util.JndiHelper;

import java.sql.Connection;
import java.sql.SQLException;
import javax.sql.DataSource;
import javax.naming.NamingException;

/**
 * This class provides additional support over implementing
 * PipelineComponent directly. 
 */
public abstract class CommercePipelineComponentSupport implements PipelineComponent
{
    /** 
     * Exception message formatter class, conforming to WLS logger API
     */
    public static final PipelineExceptionTextFormatter exFmt = new PipelineExceptionTextFormatter();

    /** Get request scoped attributes from the pipeline session */
    public Object getRequestAttribute(
				      String          key,
				      String          namespace,
				      Object          reqContext,
				      PipelineSession pSession) throws PipelineException
    {
        try
        {
            return pSession.getRequestScopedAttribute(key, namespace, reqContext);
        }
        catch (InvalidArgumentException e)
        {
            throw new PipelineException(exFmt.getInvalidArgumentException(key, namespace), e);
        }
    }

    /** Set request scoped attributes in the pipeline session */
    public void setRequestAttribute(
				    String          key,
				    Object          obj,
				    String          namespace,
				    Object          reqContext,
				    PipelineSession pSession) throws PipelineException
    {
        try
        {
            pSession.setRequestScopedAttribute(key, obj, namespace, reqContext);
        }
        catch (InvalidArgumentException e)
        {
            throw new PipelineException(exFmt.setInvalidArgumentException(key, obj, namespace), e);
        }
    }

    /** Get session scoped attributes from the pipeline session */
    public Object getSessionAttribute(
				      String          key,
				      String          namespace,
				      PipelineSession pSession) throws PipelineException
    {
        try
        {
            return pSession.getSessionScopedAttribute(key, namespace);
        }
        catch (InvalidArgumentException e)
        {
            throw new PipelineException(exFmt.getInvalidArgumentException(key, namespace), e);
        }
    }

    /** Set session scoped attributes in the pipeline session */
    public void setSessionAttribute(
				    String          key,
				    Object          obj,
				    String          namespace,
				    PipelineSession pSession) throws PipelineException
    {
        try
        {
            pSession.setSessionScopedAttribute(key, obj, namespace);
        }
        catch (com.bea.p13n.appflow.exception.InvalidArgumentException e)
        {
            throw new PipelineException(exFmt.setInvalidArgumentException(key, obj, namespace), e);
        }
    }

    public Connection getConnection() throws SystemException
    {
        Connection connection = null;
        
        String persistenceURL= CommercePropertiesHelper.getInstance().getProperty("commerce.jdbc.pool.url");
        
        try
        {
	    DataSource ds = (DataSource) JndiHelper.lookupNarrow("java:comp/env/" + persistenceURL,
								 javax.sql.DataSource.class);
            connection = ds.getConnection();
        }
         catch(NamingException namingException)
        {
            String[] messageParameters = {"jdbc/commercePool"};
            // The following property should be moved into an appropriate place.
            throw new SystemException("orMap", "errorGetConnection", messageParameters);
        }
        catch(java.sql.SQLException sqlException)
        { 
            String[] messageParameters = {"jdbc/commercePool"};
            // The following property should be moved into an appropriate place.
            throw new SystemException("orMap", "errorGetConnection", messageParameters, sqlException);
        }

        return connection;
        
    }

    public DataSource getDataSource() throws SystemException, PipelineException
    {
        DataSource ds = null;
            
        try
        {
            CommercePipelineComponentSupportMBean commercePipelineMBean = (CommercePipelineComponentSupportMBean) ApplicationHelper.getServiceConfigurationMBean( "CommercePipelineComponentSupport" );
            String persistenceURL = commercePipelineMBean.getJdbcPoolName();    
	    ds = (DataSource) JndiHelper.lookupNarrow(persistenceURL,
						      javax.sql.DataSource.class);

        }
        catch (javax.management.InstanceNotFoundException ex)
        {
            throw new PipelineException(ex.getMessage(), ex);
        }
	catch(NamingException namingException)
        {
            String[] messageParameters = {"jdbc/commercePool"};
            throw new SystemException("orMap", "errorGetConnection", messageParameters);
        }

        return ds;
        
    }

}
