

/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/src/examples/e2e/common/EjbHelper.java,v 1.2 2002/05/02 00:22:12 toneill Exp $
 */

package examples.e2e.common;

import examples.e2e.common.inventory.spi.InventoryProviderHome;
import examples.e2e.common.inventory.spi.InventoryProvider;
import examples.e2e.common.purchase.spi.PurchaseManagerHome;
import examples.e2e.common.purchase.spi.PurchaseManager;

import com.bea.p13n.util.JndiHelper;
import com.bea.p13n.util.debug.Debug;
import com.bea.p13n.management.ApplicationHelper;

import java.rmi.RemoteException;
import javax.ejb.CreateException;
import javax.naming.NamingException;


public class EjbHelper {

    private static Debug debug =
            Debug.getInstance( EjbHelper.class );

    public static final String INV_EJB_JNDI = ".BEA_commerce.InventoryProvider";
    public static InventoryProviderHome invEjbHome = null;

    public static final String PURCH_EJB_JNDI = ".BEA_commerce.PurchaseManager";
    public static PurchaseManagerHome purchEjbHome = null;

    public static String appName = null;


    private EjbHelper() {
    } 


    private static String getApplicationName() {

        if ( appName == null ) {

            // may throw IllegalStateException
            appName = ApplicationHelper.getApplicationName();
        }

        return appName;
    }

    private static InventoryProviderHome getInventoryHome()
    throws NamingException, ClassCastException {

        String jndiName = getApplicationName() + INV_EJB_JNDI;

        if ( debug.ON ) {
            debug.out( "Looking up InventoryProviderHome, using JNDI name( " +
                    jndiName + " )" );
        }

        if ( invEjbHome == null ) {
            invEjbHome = (InventoryProviderHome) JndiHelper.lookupNarrow(
                    jndiName, InventoryProviderHome.class );
        }

        return invEjbHome;
    }

    public static InventoryProvider getInventoryProvider()
    throws RemoteException, CreateException, NamingException {

        InventoryProvider bean = getInventoryHome().create();

        if ( debug.ON ) {
            debug.out( "bean(" + bean + ")" );
        }

        return bean;
    }

    private static PurchaseManagerHome getPurchaseManagerHome()
    throws NamingException, ClassCastException {

        String jndiName = getApplicationName() + PURCH_EJB_JNDI;

        if ( debug.ON ) {
            debug.out( "Looking up PurchaseManagerHome, using JNDI name( " +
                    jndiName + " )" );
        }

        if ( purchEjbHome == null ) {
            purchEjbHome = (PurchaseManagerHome) JndiHelper.lookupNarrow(
                    jndiName, PurchaseManagerHome.class );
        }

        return purchEjbHome;
    }

    public static PurchaseManager getPurchaseManager()
    throws RemoteException, CreateException, NamingException {

        PurchaseManager bean = getPurchaseManagerHome().create();

        if ( debug.ON ) {
            debug.out( "bean(" + bean + ")" );
        }

        return bean;
    }
}
