
/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 */

package examples.e2e.common;

import java.io.Serializable;


public interface Inventory extends Serializable {

    public static final int STD_IMAGE = 0;
    public static final int WIDE_IMAGE = 1;

    /**
     * Returns the identifier for the inventory object.
     */
    public String id();

    /**
     * Returns a description for the inventory object.
     */
    public String description();
    
    /**
     * Returns the minimum quantity of inventory that
     * should be maintained in stock for this item.
     */
    public long min();
    
    /**
     * Returns the maximum quantity of inventory that
     * should be maintained in stock for this item.
     */
    public long max();

    /**
     * Returns the current available quantity of inventory for this item.
     */
    public long available();

    /**
     * Returns the URL to an image for the product.
     */
    public String imageURL( int image );
    
    /**
     * Returns true if the available stock is less than the minimum
     * requirement for the product.
     */
    public boolean isBelowMinimum();
    
    /**
     * Returns the quantity required to bring the available stock
     * to the minimum specifed for the product.  Returns zero if
     * available >= minimum.
     */
    public long qtyBelowMinimum();

}
