
/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 */

package examples.e2e.common;


public class InventoryImpl implements Inventory {

    public static final String EMPTY_STRING = "";

    private String id;
    private String description;
    private long min;
    private long max;
    private long available;
    private String stdImageURL;
    private String wideImageURL;


    // product
    public InventoryImpl( String id, long min, long max,
                          long available, String stdImageURL, String wideImageURL ) {

        this.id = id;
        this.description = EMPTY_STRING;
        this.min = min;
        this.max = max;
        this.available = available;
        this.stdImageURL = stdImageURL;
        this.wideImageURL = wideImageURL;
    }
    
    // part
    public InventoryImpl( String id, String description,
                          long min, long max, long available ) {

        this.id = id;
        this.description = description;
        this.min = min;
        this.max = max;
        this.available = available;
        this.stdImageURL = null;
        this.wideImageURL = null;
    }

    /**
     * Returns the identifier for the inventory object.
     */
    public String id() {
        
        return id;
    }

    /**
     * Returns a description for the inventory object.
     */
    public String description() {
        
        return description;
    }
    
    /**
     * Returns the minimum quantity of inventory that
     * should be maintained in stock for this item.
     */
    public long min() {
        
        return min;
    }
    
    /**
     * Returns the maximum quantity of inventory that
     * should be maintained in stock for this item.
     */
    public long max() {
        
        return max;
    }

    /**
     * Returns the current available quantity of inventory for this item.
     */
    public long available() {

        return available;
    }

    /**
     * Returns the URL to an image for the product.
     */
    public String imageURL( int image ) {

        String rtnVal = null;

        switch ( image ) {
            case STD_IMAGE: {
                rtnVal = stdImageURL;
                break;
            }

            case WIDE_IMAGE: {
                rtnVal = wideImageURL;
                break;
            }

            default: throw new RuntimeException(
                "A valid image must be requested" );
        }

        return rtnVal;
    }

    /**
     * Returns true if the available stock is less than the minimum
     * requirement for the product.
     */
    public boolean isBelowMinimum() {

        return ( available < min ) ? true : false;
    }

    /**
     * Returns the quantity required to bring the available stock
     * to the minimum specifed for the product.  Returns zero if
     * available >= minimum.
     */
    public long qtyBelowMinimum() {

        long rtnVal = 0L;

        if ( isBelowMinimum() ) {
            rtnVal = min() - available();
        }

        return rtnVal;
    }

/**
 * Returns a String that represents the value of this object.
 * @return a string representation of the receiver
 */
    public String toString() {
        // Insert code to print the receiver here.
        StringBuffer sb = new StringBuffer( 64 );

        sb.append( this.getClass().getName() + "[ " );
        sb.append( "id( " + id() + " )" );
        sb.append( ", min( " + min() + " )" );
        sb.append( ", max( " + max() + " )" );
        sb.append( ", available( " + available() + " )" );
        sb.append( " ]" );

        return sb.toString();

    }


}
