
/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 */

package examples.e2e.common;

import com.bea.commerce.ebusiness.price.quote.Money;

import java.util.Calendar;
import java.io.Serializable;


/**
 * Represents a purchase order that is to be communicated
 * to supplier(s).
 *
 * @author Copyright (c) 2000-2002  BEA Systems, Inc. All Rights Reserved.
 */
public interface PO extends Serializable {

    /**
     * Returns the identifier for this purchase order.
     */
    public String poNumber();
        
    /**
     * Returns the date this PO was created.
     */
    public Calendar creationDate();
    
    /**
     * Returns the status of this PO.  This value is generally
     * one of Pending, Acknowledged, Shipped or Received.
     */
    public POStatus status();
    
    /**
     * Returns the status of this PO.  This value is generally
     * one of Pending, Acknowledged, Shipped or Received.
     */
    public String statusString();

    /**
     * Returns the name of the supplier associated with this PO.
     */
    public String supplierName();

    /**
     * Returns the identifier for the product supplied.
     */
    public String productId();
    
    /**
     * Returns a description for the product supplied.
     */
    public String description();
    
    /**
     * Returns the quantity of the product supplied.
     */
    public long qty();
    
    /**
     * Returns the unit price for each item supplied.
     */
    public Money unitPrice();

    /**
     * Returns the date promised by the supplier.
     */
    public Calendar promiseDate();
    
    /**
     * Returns the total cost for the PO.
     */
    public Money total();
    
}
