
/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 */

package examples.e2e.common;

import java.io.Serializable;
import java.util.Map;
import java.util.HashMap;


public final class POElement implements Serializable {

    private int intValue;
    private String name;

    public static final int SIZE = 16;
    // set the size to prevent rehashing
    private static final Map stringToObject =
            new HashMap( (int)( SIZE / 0.75f ) + 1 );

    public static final int I_PURCHASE_ORDER = 0;
    public static final POElement PURCHASE_ORDER =
            new POElement( I_PURCHASE_ORDER, "E2E_PurchaseOrder" );

    public static final int I_PO_ACK = 1;
    public static final POElement PO_ACK =
            new POElement( I_PO_ACK, "E2E_PurchaseOrderAcknowledgement" );

    public static final int I_PO_NUMBER = 2;
    public static final POElement PO_NUMBER =
            new POElement( I_PO_NUMBER, "PONumber" );

    public static final int I_CREATION_DATE = 3;
    public static final POElement CREATION_DATE =
            new POElement( I_CREATION_DATE, "CreationDate" );

    public static final int I_PO_CREATION_DATE = 4;
    public static final POElement PO_CREATION_DATE =
            new POElement( I_PO_CREATION_DATE, "POCreationDate" );

    public static final int I_STATUS = 5;
    public static final POElement STATUS =
            new POElement( I_STATUS, "Status" );

    public static final int I_SUPPLIER_NAME = 6;
    public static final POElement SUPPLIER_NAME =
            new POElement( I_SUPPLIER_NAME, "SupplierName" );

    public static final int I_LINE_ITEM = 7;
    public static final POElement LINE_ITEM =
            new POElement( I_LINE_ITEM, "LineItem" );

    public static final int I_PART_ID = 8;
    public static final POElement PART_ID =
            new POElement( I_PART_ID, "PartId" );

    public static final int I_PART_DESCRIPTION = 9;
    public static final POElement PART_DESCRIPTION =
            new POElement( I_PART_DESCRIPTION, "PartDescription" );

    public static final int I_QUANTITY = 10;
    public static final POElement QUANTITY =
            new POElement( I_QUANTITY, "Quantity" );

    public static final int I_UNIT_PRICE = 11;
    public static final POElement UNIT_PRICE =
            new POElement( I_UNIT_PRICE, "UnitPrice" );

    public static final int I_UNIT_CURRENCY = 12;
    public static final POElement UNIT_CURRENCY =
            new POElement( I_UNIT_CURRENCY, "UnitCurrency" );

    public static final int I_DELIVERY_DATE = 13;
    public static final POElement DELIVERY_DATE =
            new POElement( I_DELIVERY_DATE, "DeliveryDate" );

    public static final int I_NOTE = 14;
    public static final POElement NOTE =
            new POElement( I_NOTE, "Note" );

    public static final int I_TOTAL = 15;
    public static final POElement TOTAL =
            new POElement( I_TOTAL, "TotalAmount" );

    static {

        stringToObject.put( PURCHASE_ORDER.getString(), PURCHASE_ORDER );
        stringToObject.put( PO_ACK.getString(), PO_ACK );
        stringToObject.put( PO_NUMBER.getString(), PO_NUMBER );
        stringToObject.put( CREATION_DATE.getString(), CREATION_DATE );
        stringToObject.put( PO_CREATION_DATE.getString(), PO_CREATION_DATE );
        stringToObject.put( STATUS.getString(), STATUS );
        stringToObject.put( SUPPLIER_NAME.getString(), SUPPLIER_NAME );
        stringToObject.put( LINE_ITEM.getString(), LINE_ITEM );
        stringToObject.put( PART_ID.getString(), PART_ID );
        stringToObject.put( PART_DESCRIPTION.getString(), PART_DESCRIPTION );
        stringToObject.put( QUANTITY.getString(), QUANTITY );
        stringToObject.put( UNIT_PRICE.getString(), UNIT_PRICE );
        stringToObject.put( UNIT_CURRENCY.getString(), UNIT_CURRENCY );
        stringToObject.put( DELIVERY_DATE.getString(), DELIVERY_DATE );
        stringToObject.put( NOTE.getString(), NOTE );
        stringToObject.put( TOTAL.getString(), TOTAL );
    }


    /**
     */
    private POElement( int intValue, String name ) {

        this.intValue = intValue;
        this.name = name;
    }

    public String toString() {

        return getString();
    }

    public String getString() {

        return name;
    }

    public int getInt() {

        return intValue;
    }

    public static POElement getObject( String token ) {

        return(POElement) stringToObject.get( token );
    }

    public boolean equals( Object object ) {
        if ( this == object ) {
            return true;
        }
        else if ( object == null || getClass() != object.getClass() ) {
            return false;
        }

        POElement other = (POElement) object;
        return this.intValue == other.intValue;
    }

    private int hashCode;
    private boolean hashCodeValid = false;
    private static final int HASHCODE_SEED = 1000003;

    public int hashCode() {

        if ( hashCodeValid == true ) {
            return hashCode;
        }

        computeHashCode();
        return hashCode;
    }
    private void computeHashCode() {

        hashCode = intValue * HASHCODE_SEED;
        hashCodeValid = true;
    }

}
