
/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/src/examples/e2e/common/POImpl.java,v 1.2 2002/05/02 00:22:12 toneill Exp $
 */

package examples.e2e.common;

import com.bea.commerce.ebusiness.price.quote.Money;
import com.bea.commerce.ebusiness.price.quote.ImmutableMoney;
import com.bea.p13n.util.debug.Debug;

import java.util.Calendar;


/**
 * Represents a purchase order that is to be communicated
 * to supplier(s).
 *
 * @author Copyright (c) 2000-2002  BEA Systems, Inc. All Rights Reserved.
 */
public class POImpl implements PO {

    private static Debug debug =
            Debug.getInstance( POImpl.class );

    private String poNumber;
    private Calendar creationDate;
    private POStatus status;
    private String supplierName;
    private String productId;
    private String description;
    private long qty;
    private Money unitPrice;
    private Calendar promiseDate;

    // derived value
    private Money total;


    POImpl() {
    }

    public POImpl(
            String poNumber, Calendar creationDate,
            POStatus status, String supplierName,
            String productId, String description,
            long qty, Money unitPrice, Calendar promiseDate ) {

        this.poNumber = poNumber;
        this.creationDate = creationDate;
        this.status = status;
        this.supplierName = supplierName;
        this.productId = productId;
        this.description = description;
        this.qty = qty;
        this.unitPrice = unitPrice;
        this.promiseDate = promiseDate;
    } 


    /**
     * Returns the identifier for this purchase order.
     */
    public String poNumber() {

        return poNumber;
    }

    /**
     * Returns the date this PO was created.
     */
    public Calendar creationDate() {

        return creationDate;
    }

    /**
     * Returns the status of this PO.  This value is generally
     * one of Pending, Acknowledged, Shipped or Received.
     */
    public POStatus status() {

        return status;
    }

    /**
     * Returns the status of this PO.  This value is generally
     * one of Pending, Acknowledged, Shipped or Received.
     */
    public String statusString() {

        return status.getString();
    }

    /**
     * Returns the name of the supplier associated with this PO.
     */
    public String supplierName() {

        return supplierName;
    }

    /**
     * Returns the identifier for the product supplied.
     */
    public String productId() {

        return productId;
    }

    /**
     * Returns a description for the product supplied.
     */
    public String description() {

        return description;
    }

    /**
     * Returns the quantity of the product supplied.
     */
    public long qty() {

        return qty;
    }

    /**
     * Returns the unit price for each item supplied.
     */
    public Money unitPrice() {

        return unitPrice;
    }

    /**
     * Returns the date promised by the supplier.
     */
    public Calendar promiseDate() {

        return promiseDate;
    }

    /**
     * Returns the total cost for the PO.
     */
    public Money total() {

        if ( total == null ) {
            total = calculateTotal( unitPrice, qty );
        }

        return total;
    }


    /**
     * Sets the identifier for this purchase order.
     */
    public String setPoNumber( String poNumber ) {

        this.poNumber = poNumber;
        return this.poNumber;
    }

    /**
     * Sets the date this PO was created.
     */
    public Calendar setCreationDate( Calendar date ) {

        this.creationDate = date;
        return creationDate;
    }

    /**
     * Sets the status of this PO.  This value is generally
     * one of Pending, Acknowledged, Shipped or Received.
     */
    public POStatus setStatus( POStatus status ) {

        this.status = status;
        return this.status;
    }

    /**
     * Sets the name of the supplier associated with this PO.
     */
    public String setSupplierName( String name ) {

        supplierName = name;
        return supplierName;
    }

    /**
     * Sets the identifier for the product supplied.
     */
    public String setProductId( String id ) {

        productId = id;
        return productId;
    }

    /**
     * Sets a description for the product supplied.
     */
    public String setDescription( String description ) {

        this.description = description;
        return this.description;
    }

    /**
     * Sets the quantity of the product supplied.
     */
    public long setQty( long qty ) {

        this.qty = qty;
        return this.qty;
    }

    /**
     * Sets the unit price for each item supplied.
     */
    public Money setUnitPrice( Money unitPrice ) {

        this.unitPrice = unitPrice;
        return this.unitPrice;
    }

    /**
     * Sets the date promised by the supplier.
     */
    public Calendar setPromiseDate( Calendar promiseDate ) {

        this.promiseDate = promiseDate;
        return this.promiseDate;
    }

    private static Money calculateTotal( Money unitPrice, long qty ) {

        return(Money) new ImmutableMoney( unitPrice.getValue() * qty,
                unitPrice.getCurrency() );
    }
}   
