
/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/src/examples/e2e/common/POParser.java,v 1.2 2002/05/02 00:22:12 toneill Exp $
 */

package examples.e2e.common;

import com.bea.p13n.util.debug.Debug;
import com.bea.p13n.util.DefaultEntityResolver;
import com.bea.commerce.ebusiness.price.quote.Money;
import com.bea.commerce.ebusiness.price.quote.ImmutableMoney;

import weblogic.apache.xerces.parsers.SAXParser;
import org.xml.sax.ContentHandler;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.XMLReader;
import org.xml.sax.Locator;
import org.xml.sax.Attributes;

import java.io.InputStream;
import java.io.IOException;
import java.util.List;
import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;
import java.util.Calendar;
import java.util.Date;
import java.text.SimpleDateFormat;


public class POParser implements ContentHandler, ErrorHandler {

    public static final String EMPTY_STRING = "";

    private static final SimpleDateFormat xmlDateFormat =
            new SimpleDateFormat( BuyerFactory.xmlDateFormatString );

    private static Debug debug =
            Debug.getInstance( POParser.class );

    private POElement element;
    private InputSource source;
    private Locator locator;
    private boolean errorFlag = false;
    private PO po;
    private String cValue;
    private int lineItemCnt = 0;

    private final Map fields =
            new HashMap( (int)( POElement.SIZE / 0.75f ) + 1 );


    // set the system property to turn off XML validation
    private static final String VALIDATION_PROP =
            "samples.e2e.b2b.xml.validate";
    private static boolean validate = false;

    static {

        try {
            validate = Boolean.getBoolean( VALIDATION_PROP );

            if ( debug.ON ) {
                debug.out( "validate(" + validate + ")" );
            }
        }
        catch ( Exception e ) {
            debug.out( "validate(" + validate + ")", e );
        }
    }


    public POParser( InputSource source ) {

        this.source = source;
    } 


    public PO parse() throws ParsingException {

        if ( source == null ) {

            return null;
        }

        if ( debug.ON ) {
            debug.out( "validate( " + validate + " )" );
        }

        try {

            XMLReader xmlReader = new SAXParser();

            if ( validate ) {

                // configure for validation

                xmlReader.setEntityResolver(
                        new DefaultEntityResolver( "/examples/e2e/common/dtd" ) );

                xmlReader.setFeature(
                        "http://xml.org/sax/features/validation", true );
            }

            xmlReader.setContentHandler( this );
            xmlReader.setErrorHandler( this );

            xmlReader.parse( source );

            InputStream is = source.getByteStream();
            // is.close();

            if ( errorFlag ) {

                if ( debug.ON ) {
                    debug.out( "errorFlag(" + errorFlag + ")" );
                }

                throw new ParsingException( "parsing error encountered, see log" );
            }

        }
        catch ( SAXParseException ex ) {

            throw (ParsingException) ex.getException();
        }
        catch ( SAXException ex ) {

            throw new ParsingException( "SAXException(" + ex.getMessage() +
                    ") encountered processing PO XML", ex );
        }
        catch ( IOException ex ) {

            throw new ParsingException( "IOException(" + ex.getMessage() +
                    ") encountered processing PO XML", ex );
        }

        return po;
    }


    private void start( POElement element ) {

        switch ( element.getInt() ) {

        // we recognize only a single line item in a PO
        // ignore additional lineItems
        case POElement.I_LINE_ITEM: {

                lineItemCnt++;

                if ( debug.ON ) {
                    debug.out( "lineItemCnt(" + lineItemCnt + ")" );
                }

                break;
            }
        }
    }

    private void end( POElement element ) throws ParsingException {

        // more than 1 line item for a PO not supported
        if ( lineItemCnt > 1 ) {
            return;
        }

        switch ( element.getInt() ) {

        // end of PO
        case POElement.I_PO_ACK:
        case POElement.I_PURCHASE_ORDER: {

                // create object
                po = createPO();
                break;
            }

        case POElement.I_PO_NUMBER:
        case POElement.I_STATUS:
        case POElement.I_SUPPLIER_NAME:
        case POElement.I_PART_ID:
        case POElement.I_PART_DESCRIPTION:
        case POElement.I_UNIT_CURRENCY:
        case POElement.I_NOTE: {

                fields.put( element, getString() );
                break;
            }

        case POElement.I_CREATION_DATE: 
        case POElement.I_PO_CREATION_DATE: 
        case POElement.I_DELIVERY_DATE: {

                fields.put( element, getCalendar() );
                break;
            }

        case POElement.I_QUANTITY: {

                fields.put( element, getLong() );
                break;
            }

        case POElement.I_UNIT_PRICE:
        case POElement.I_TOTAL: {

                fields.put( element, getDouble() );
                break;
            }
        }
    }


    private String getString() {

        return cValue.trim();
    }

    private Calendar getCalendar() throws ParsingException {

        Date date = null;

        try {

            date = xmlDateFormat.parse( cValue );
        }
        catch ( java.text.ParseException ex ) {

            throw new ParsingException( "failed parsing Date for element( " +
                    element + " ) with value( " +
                    cValue + " ), message( " +
                    ex.getMessage() + " )", ex );
        }

        Calendar calendar = (Calendar) xmlDateFormat.getCalendar().clone();
        calendar.setTime( date );

        return calendar;
    }

    private Double getDouble() throws ParsingException {

        try {

            return new Double( cValue );
        }
        catch ( NumberFormatException ex ) {

            throw new ParsingException( "failed parsing Double for element( " +
                    element + " ) with value( " +
                    cValue + " ), message( " +
                    ex.getMessage() + " )", ex );
        }
    }

    private Long getLong() throws ParsingException {

        try {

            return new Long( cValue );
        }
        catch ( NumberFormatException ex ) {

            throw new ParsingException( "failed parsing Long for element( " +
                    element + " ) with value( " +
                    cValue + " ), message( " +
                    ex.getMessage() + " )", ex );
        }
    }

    private PO createPO() throws ParsingException {

        POImpl po = new POImpl();

        // temp variables
        String string = null;
        POStatus status = null;
        long longVal = 0;
        double doubleVal = 0.0d;
        Calendar calendar = null;


        string = (String) fields.get( POElement.PO_NUMBER );
        if ( string == null ) {

            throw new ParsingException(
                    "failed creating po[ " +
                    POElement.PO_NUMBER +
                    "( " + string + " ) ]" );
        }
        po.setPoNumber( string );


        // po element
        calendar = (Calendar) fields.get( POElement.CREATION_DATE );
        if ( calendar == null ) {

            // ack element
            calendar = (Calendar) fields.get( POElement.PO_CREATION_DATE );
        }

        if ( calendar == null ) {

            throw new ParsingException(
                    "failed creating po[ " +
                    POElement.CREATION_DATE +
                    "( " + calendar + " ) ]" );
        }
        po.setCreationDate( calendar );


        string = (String) fields.get( POElement.STATUS );

        // no status field in acknowledgement
        if ( string == null ) {

            status = POStatus.ACKNOWLEDGED;
        }
        else {

            status = POStatus.getObject( string );
        }

        if ( status == null ) {

            throw new ParsingException(
                    "failed creating po[ " +
                    POElement.STATUS +
                    "( " + string + " ) ]" );
        }
        po.setStatus( status );


        string = (String) fields.get( POElement.SUPPLIER_NAME );
        if ( string == null ) {

            throw new ParsingException(
                    "failed creating po[ " +
                    POElement.SUPPLIER_NAME +
                    "( " + string + " ) ]" );
        }
        po.setSupplierName( string );


        string = (String) fields.get( POElement.PART_ID );
        if ( string == null ) {

            throw new ParsingException(
                    "failed creating po[ " +
                    POElement.PART_ID +
                    "( " + string + " ) ]" );
        }
        po.setProductId( string );


        string = (String) fields.get( POElement.PART_DESCRIPTION );
        if ( string == null ) {

            throw new ParsingException(
                    "failed creating po[ " +
                    POElement.PART_DESCRIPTION +
                    "( " + string + " ) ]" );
        }
        po.setDescription( string );


        longVal = ((Long) fields.get( POElement.QUANTITY )).longValue();
        if ( longVal < 0.0 ) {

            throw new ParsingException(
                    "failed creating po[ " +
                    POElement.QUANTITY +
                    "( " + longVal + " ) ]" );
        }
        po.setQty( longVal );


        // money
        doubleVal = ((Double) fields.get( POElement.UNIT_PRICE )).doubleValue();
        if ( doubleVal < 0.0d ) {

            throw new ParsingException(
                    "failed creating po[ " +
                    POElement.UNIT_PRICE +
                    "( " + doubleVal + " ) ]" );
        }

        string = (String) fields.get( POElement.UNIT_CURRENCY );
        if ( string == null ) {

            throw new ParsingException(
                    "failed creating po[ " +
                    POElement.UNIT_CURRENCY +
                    "( " + string + " ) ]" );
        }
        po.setUnitPrice( (Money) new ImmutableMoney( doubleVal, string ) );


        calendar = (Calendar) fields.get( POElement.DELIVERY_DATE );
        if ( calendar == null ) {

            throw new ParsingException(
                    "failed creating po[ " +
                    POElement.DELIVERY_DATE +
                    "( " + calendar + " ) ]" );
        }
        po.setPromiseDate( calendar );


        return(PO) po;
    }

    // clear values out of the hash prior to parsing a new po.
    private void clear() {

        fields.clear();
    }


    //////////////////////////////////////////////
    // IMPLEMENTATION OF CONTENT HANDLER INTERFACE
    //////////////////////////////////////////////

    public void startElement(String namespaceURI,
            String localName,
            String qName,
            Attributes attributes) throws SAXException {

        if ( debug.ON ) {
            debug.out( "localName(" + localName + ")" );
            debug.out( "namespaceURI(" + namespaceURI + ")" );
            debug.out( "qName(" + qName + ")" );
        }

        element = POElement.getObject( localName );

        if ( debug.ON ) {
            debug.out( "element(" + element + ")" );
        }

        if ( element == null ) {

            if ( debug.ON ) {
                debug.out( "unrecognized localName(" + localName + ")" );
            }
        }
        else {

            start( element );
        }
    }

    public void characters( char[] chars, int start, int length )
    throws SAXException {

        cValue = new String( chars, start, length );

        if ( debug.ON ) {
            debug.out( "cValue(" + cValue + ")" );
        }

    }

    public void endElement(String namespaceURI, String localName, String qName)
    throws SAXException {

        if ( debug.ON ) {
            debug.out( "localName(" + localName + ")" );
            debug.out( "namespaceURI(" + namespaceURI + ")" );
            debug.out( "qName(" + qName + ")" );
        }

        element = POElement.getObject( localName );

        if ( debug.ON ) {
            debug.out( "element(" + element + ")" );
        }

        if ( element == null ) {

            if ( debug.ON ) {
                debug.out( "unrecognized localName(" + localName + ")" );
            }
        }
        else {

            try {
                end( element );
            }
            catch ( ParsingException ex ) {

                if ( debug.ON ) {
                    debug.out( "ex(" + ex + ")" );
                }

                throw new SAXParseException( ex.getMessage(), locator, ex );
            }
        }
    }

    public void ignorableWhitespace(char[] chars, int start, int length)
    throws SAXException {

        String wValue = new String( chars, start, length );

        if ( debug.ON ) {
            debug.out( "wValue(" + wValue + ")" );
        }

    }

    public void processingInstruction( String target, String data )
    throws SAXException {

        if ( debug.ON ) {
            debug.out( "target(" + target + ")" );
            debug.out( "data(" + data + ")" );
        }

    }

    public void setDocumentLocator( Locator locator ) {

        if ( debug.ON ) {
            debug.out( "locator(" + locator + ")" );
        }

        this.locator = locator;
    }

    public void skippedEntity(String name)
    throws SAXException {

        if ( debug.ON ) {
            debug.out( "name(" + name + ")" );
        }

    }

    public void startDocument()
    throws SAXException {

        if ( debug.ON ) {
            debug.out( EMPTY_STRING );
        }

    }

    public void endDocument()
    throws SAXException {

        if ( debug.ON ) {
            debug.out( EMPTY_STRING );
        }

    }

    public void startPrefixMapping(String prefix, String uri)
    throws SAXException {

        if ( debug.ON ) {
            debug.out( "prefix(" + prefix + ")" );
            debug.out( "uri(" + uri + ")" );
        }

    }

    public void endPrefixMapping(String prefix)
    throws SAXException {

        if ( debug.ON ) {
            debug.out( "prefix(" + prefix + ")" );
        }

    }


    ////////////////////////////////////////////
    // IMPLEMENTATION OF ERROR HANDLER INTERFACE
    ////////////////////////////////////////////

    public void error(SAXParseException e)
    throws SAXException {
        errorFlag = true;

        if ( debug.ON ) {

            debug.out( "PARSE ERROR[ " +
                    "Public ID( " + e.getPublicId() + " ), " +
                    "System ID( " + e.getSystemId() + " ), " +
                    "Line number( " + e.getLineNumber() + " ), " +
                    "Column number(" + e.getColumnNumber() + " ) ]",
                    e );

        }
    }

    public void fatalError(SAXParseException e)
    throws SAXException {

        errorFlag = true;

        if ( debug.ON ) {

            debug.out( "FATAL PARSE ERROR[ " +
                    "Public ID( " + e.getPublicId() + " ), " +
                    "System ID( " + e.getSystemId() + " ), " +
                    "Line number( " + e.getLineNumber() + " ), " +
                    "Column number(" + e.getColumnNumber() + " ) ]",
                    e );

        }
    }

    public void warning(SAXParseException e)
    throws SAXException {

        errorFlag = true;

        if ( debug.ON ) {

            debug.out( "PARSE WARNING[ " +
                    "Public ID( " + e.getPublicId() + " ), " +
                    "System ID( " + e.getSystemId() + " ), " +
                    "Line number( " + e.getLineNumber() + " ), " +
                    "Column number(" + e.getColumnNumber() + " ) ]",
                    e );

        }
    }


}
