
/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 */

package examples.e2e.common;

import java.io.Serializable;


public final class POStatus implements Serializable {

    private int intValue;

    public static final int I_PENDING = 1;
    public static final int I_ACKNOWLEDGED = 2;
    public static final int I_SHIPPED = 3;
    public static final int I_RECEIVED = 4;
    
    public static final POStatus PENDING =
            new POStatus( I_PENDING );

    public static final POStatus ACKNOWLEDGED =
            new POStatus( I_ACKNOWLEDGED );

    public static final POStatus SHIPPED =
            new POStatus( I_SHIPPED );

    public static final POStatus RECEIVED =
        new POStatus( I_RECEIVED );


    public static final String S_PENDING = "Pending";
    public static final String S_ACKNOWLEDGED = "Acknowledged";
    public static final String S_SHIPPED = "Shipped";
    public static final String S_RECEIVED = "Received";


    /**
     */
    private POStatus( int intValue ) {
        init( intValue );
    }

    public String toString() {
        return getString();
    }

    public String getString() {
        switch ( intValue ) {
            case I_PENDING: 
                { return S_PENDING;}
            case I_ACKNOWLEDGED: 
                { return S_ACKNOWLEDGED;}
            case I_SHIPPED: 
                { return S_SHIPPED;}
            case I_RECEIVED: 
                { return S_RECEIVED;}
        }

        // should never get here (as long as the constructor is private)
        throw new RuntimeException(
                "PROGRAMMER ERROR"  );
    }

    public int getInt() {
        return intValue;
    }

    public static POStatus getObject( String token ) {
        
        if ( token.equalsIgnoreCase( POStatus.S_PENDING ) ) {
            return PENDING;
        }
        if ( token.equalsIgnoreCase( POStatus.S_ACKNOWLEDGED ) ) {
            return ACKNOWLEDGED;
        }
        if ( token.equalsIgnoreCase( POStatus.S_SHIPPED ) ) {
            return SHIPPED;
        }
        if ( token.equalsIgnoreCase( POStatus.S_RECEIVED ) ) {
            return RECEIVED;
        }

        return null;
    }

    private void init( int intValue ) {
        this.intValue = intValue;
    }

    public boolean equals( Object object ) {
        if ( this == object ) {
            return true;
        }
        else if ( object == null || getClass() != object.getClass() ) {
            return false;
        }

        POStatus other = (POStatus) object;
        return this.intValue == other.intValue;
    }

    private int hashCode;
    private boolean hashCodeValid = false;
    private static final int HASHCODE_SEED = 1000003;

    public int hashCode() {
        
        if ( hashCodeValid == true ) {
            return hashCode;
        }

        computeHashCode();
        return hashCode;
    }
    private void computeHashCode() {
        
        hashCode = intValue * HASHCODE_SEED;
        hashCodeValid = true;
    }

}
