
/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 */

package examples.e2e.common;

import com.bea.commerce.ebusiness.price.quote.Money;

import java.util.Calendar;
import java.io.Serializable;


/**
 * Represents a query for price and availability that is to be communicated
 * to supplier(s).
 *
 * @author Copyright (c) 2000-2002  BEA Systems, Inc. All Rights Reserved.
 */
public interface PurchaseQuote extends Serializable {

    /**
     * Returns the identifier for the QPA that precipitated this quote.
     */
    public String requestId();
    
    /**
     * Returns the identifier for this quote.
     */
    public String quoteId();
    
    /**
     * Returns the date this QPA was created.
     */
    public Calendar qpaCreationDate();
    
    /**
     * Returns the date this quote was generated by the supplier.
     */
    public Calendar quoteDate();
    
    /**
     * Returns the name of the supplier who sent this quote.
     */
    public String supplierName();

    /**
     * Returns the identifier for the product quoted.
     */
    public String productId();
    
    /**
     * Returns a description for the product quoted.
     */
    public String description();
    
    /**
     * Returns the quantity of the product promised by the supplier.
     */
    public long quotedQty();
    
    /**
     * Returns the unit price quoted by the supplier.
     */
    public Money quotedUnitPrice();

    /**
     * Returns the date promised by the supplier.
     */
    public Calendar promiseDate();

    /**
     * Returns the total cost for the quote.
     */
    public Money total();
    
}
