package examples.e2e.common;

/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 1997-2001 BEA Systems, Inc. 
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 */

import com.bea.commerce.ebusiness.price.quote.Money;
import com.bea.commerce.ebusiness.price.quote.ImmutableMoney;

import java.util.Calendar;
import java.text.SimpleDateFormat;


public class PurchaseQuoteImpl implements PurchaseQuote {

    private static final SimpleDateFormat format = 
            new SimpleDateFormat( "dd MMM yyyy" );

    public static final String EMPTY_STRING = "";

    private String quoteId;
    private String description;
    private String productId;
    private Calendar promiseDate;
    private Calendar qpaCreationDate;
    private Calendar quoteDate;
    private long quotedQty;
    private Money quotedUnitPrice;
    private String requestId;
    private String supplierName;


    /**
     * Create an empty PurchaseQuote object.
     */
    PurchaseQuoteImpl() {

        //which instance variables need to get a default value?
    }

    /**
     * Construct a PurchaseQuote object.
     */
    public PurchaseQuoteImpl(
            String requestId, String quoteId,
            Calendar qpaCreationDate, Calendar quoteDate,
            String supplierName, String productId,
            String description, long quotedQty,
            Money quotedUnitPrice, Calendar promiseDate ) {

        this.requestId = requestId;
        this.quoteId = quoteId;
        this.qpaCreationDate = qpaCreationDate;
        this.quoteDate = quoteDate;
        this.supplierName = supplierName;
        this.productId = productId;
        this.description = description;
        this.quotedQty = quotedQty;
        this.quotedUnitPrice = quotedUnitPrice;
        this.promiseDate = promiseDate;
    }


    /**
     * Returns a description for the product quoted.
     */
    public String description() {
        return description;
    }

    /**
     * Compares two objects for equality. Returns a boolean that indicates
     * whether this object is equivalent to the specified object. This method
     * is used when an object is stored in a hashtable.
     * @param obj the Object to compare with
     * @return true if these Objects are equal; false otherwise.
     * @see java.util.Hashtable
     */
    public boolean equals(Object obj) {

        if ( obj == this ) {

            return true;
        }
        else if ( obj == null || getClass() != obj.getClass() ) {

            return false;
        }

        PurchaseQuote other = (PurchaseQuote) obj;
        return this.requestId().equals( other.quoteId() ); 
    }

/**
 * Code to perform when this object is garbage collected.
 * 
 * Any exception thrown by a finalize method causes the finalization to
 * halt. But otherwise, it is ignored.
 */
    protected void finalize() throws Throwable {
        super.finalize();
    }
/**
 * Generates a hash code for the receiver.
 * This method is supported primarily for
 * hash tables, such as those provided in java.util.
 * @return an integer hash code for the receiver
 * @see java.util.Hashtable
 */
    public int hashCode() {
        /** 
         * override hashCode() to get consistent
         * results when instances of the class with overridden equals() are used as
         * keys in a java.util.Hashtable 
         */

        return this.toString().hashCode();
    }
/**
 * Returns the identifier for the product quoted.
 */
    public String productId() {
        return productId;
    }
/**
 * Returns the date promised by the supplier.
 */
    public java.util.Calendar promiseDate() {
        return promiseDate;
    }
/**
 * Returns the date this QPA was created.
 */
    public java.util.Calendar qpaCreationDate() {
        return qpaCreationDate;
    }
/**
 * Returns the date this quote was generated by the supplier.
 */
    public java.util.Calendar quoteDate() {
        return quoteDate;
    }
/**
 * Returns the quantity of the product promised by the supplier.
 */
    public long quotedQty() {
        return quotedQty;
    }
/**
 * Returns the unit price quoted by the supplier.
 */
    public com.bea.commerce.ebusiness.price.quote.Money quotedUnitPrice() {
        return quotedUnitPrice;
    }
/**
 * Returns the identifier for this quote.
 */
    public String quoteId() {
        return quoteId;
    }
/**
 * Returns the identifier for the QPA that precipitated this quote.
 */
    public String requestId() {
        return requestId;
    }

    /**
     * Returns the total cost for the quote.
     */
    public Money total() {

        return(Money) new ImmutableMoney(
                quotedUnitPrice().getValue() * quotedQty(),
                quotedUnitPrice.getCurrency() );
    }


/**
 * Sets a description for the product quoted.
 */
    public void setDescription(String description) {
        this.description = description;
    }
/**
 * Sets the identifier for the product quoted.
 */
    public void setProductId(String productId) {
        this.productId = productId;
    }
/**
 * Sets the date promised by the supplier.
 */
    public void setPromiseDate(Calendar promiseDate) {
        this.promiseDate = promiseDate;
    }
/**
 * Sets the date this QPA was created.
 */
    public void setQpaCreationDate(Calendar qpaCreationDate) {
        this.qpaCreationDate = qpaCreationDate;
    }
/**
 * Sets the date this quote was generated by the supplier.
 */
    public void setQuoteDate(Calendar quoteDate) {
        this.quoteDate = quoteDate;
    }
/**
 * Sets the quantity of the product promised by the supplier.
 */
    public void setQuotedQty(long quotedQty) {
        this.quotedQty = quotedQty;
    }
/**
 * Sets the unit price quoted by the supplier.
 */
    public void setQuotedUnitPrice(Money quotedUnitPrice) {
        this.quotedUnitPrice = quotedUnitPrice;
    }
    public void setQuoteId(String quoteId) {
        this.quoteId = quoteId;
    }
/**
 * Sets the identifier for the QPA that precipitated this quote.
 */
    public void setRequestId(String requestId) {
        this.requestId = requestId;
    }
/**
 * Sets the name of the supplier who sent this quote.
 */
    public void setSupplierName(String supplierName) {
        this.supplierName = supplierName;
    }
/**
 * Returns the name of the supplier who sent this quote.
 */
    public String supplierName() {
        return supplierName;
    }
/**
 * Returns a String that represents the value of this object.
 * @return a string representation of the receiver
 */
    public String toString() {
        // Insert code to print the receiver here.
        StringBuffer sb = new StringBuffer( 256 );

        sb.append( this.getClass().getName() + "[ " );

        sb.append( "requestId( " + requestId() + " )" );

        sb.append( ", qpa creation date( " +
        format.format( qpaCreationDate().getTime() ) + " )" );

        sb.append( ", quoteId( " + quoteId() + " )" );

        sb.append( ", responseDate( " + 
                format.format( quoteDate().getTime() ) + " )" );

        sb.append( ", supplierName( " + supplierName() + " )" );

        sb.append( ", productID( " + productId() + " )" );

        sb.append( ", quotedQty( " + quotedQty() + " )" );

        sb.append( ", unit price( " + quotedUnitPrice() + " )" );

        sb.append( ", promiseDate( " + 
                format.format( promiseDate().getTime() ) + " )" );

        sb.append( ", description( " + description() + " )" );

        sb.append( " ]" );

        return sb.toString();
    }
}
