/*
 * B E A   S Y S T E M S
 *
 * C O M M E R C E   C O M P O N E N T S
 *
 * Copyright (c) 2000-2002  BEA Systems, Inc.
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws
 * of the United States. The software contained on this media is proprietary
 * to and embodies the confidential technology of BEA Systems, Inc. The
 * possession or receipt of this information does not convey any right to disclose
 * its contents, reproduce it, or use,  or license the use, for manufacture or
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 */

package examples.e2e.common;

import java.util.Iterator;

import java.io.ByteArrayOutputStream;
import java.io.OutputStream;

import org.w3c.dom.*;

import com.beasys.commerce.axiom.contact.Address;
import com.beasys.commerce.axiom.units.Money;
import com.beasys.commerce.ebusiness.customer.Customer;
import com.beasys.commerce.ebusiness.order.OrderValue;
import com.beasys.commerce.ebusiness.order.OrderLine;
import com.bea.commerce.ebusiness.price.quote.Quote;
import com.bea.p13n.appflow.exception.*;

import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.Result;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.parsers.*;

import weblogic.apache.xalan.processor.*;

import java.rmi.RemoteException;
import javax.rmi.PortableRemoteObject;

import examples.e2e.common.*;

public class PurchaseXMLHelper {

    //commerce order tags
    private static final String AMOUNT = "amount";
    private static final String CITY = "city";
    private static final String COUNTRY = "country";
    private static final String COUNTY = "county";
    private static final String CURRENCY = "currency";
    private static final String CUSTOMER_ID = "customer_id";
    private static final String DESC = "description";
    private static final String GEOCODE = "geocode";
    private static final String ID = "id";
    private static final String LINE = "line";
    private static final String LINES = "lines";
    private static final String METHOD = "method";
    private static final String MSRP = "msrp";
    private static final String ORDER_DATE = "order_date";
    private static final String POBOX = "pobox";
    private static final String POSTAL_CODE = "postal_code";
    private static final String POSTAL_CODE_TYPE = "postal_code_type";
    private static final String PRICE = "price";
    private static final String PRODUCT_ID = "product_id";
    private static final String QUANTITY = "quantity";
    private static final String SHIPPING = "shipping";
    private static final String SPECIAL_INSTR = "special_instruction";
    private static final String SPLITTING_PREF = "splitting_preference";
    private static final String STATE = "state";
    private static final String STATUS = "status";
    private static final String STREET1 = "street1";
    private static final String STREET2 = "street2";
    private static final String SUBTOTAL = "subtotal";
    private static final String TAX = "tax";
    private static final String TOTAL_AMOUNT = "total_amount";
    private static final String UNIT_PRICE = "unit_price";
    private static final String WLCS_ORDER = "E2E_wlcs_order";
    
    //QPA-PO request tags
    private static final String AVAILABILITY = "Availability";
    private static final String CREATION_DATE = "CreationDate";
    private static final String DELIVERY_DATE = "DeliveryDate";
    private static final String PART_ID = "PartId";
    private static final String PART_DESCRIPTION = "PartDescription";
    private static final String PO_ITEMS = "POItems";
    private static final String PO_REQUEST = "E2E_PORequest";
    private static final String QPA_REQUEST = "E2E_QPARequest";
    private static final String QPA_REQUEST_ID = "QPARequestId";
    private static final String QPA_RESPONSE_ID = "QPAResponseId";
    private static final String REQUIRED_DATE = "RequiredDate";
    private static final String SUPPLIER_NAME = "SupplierName";
    private static final String TRANSACTION_ID = "transaction_id";
    private static final String UNIT_CURRENCY = "UnitCurrency";
    private static final String WLI_QUANTITY = "Quantity";
    private static final String WLI_UNIT_PRICE = "UnitPrice";

public static String transform(Document newDoc)
    throws TransformerConfigurationException, TransformerException {

    TransformerFactory tf = TransformerFactory.newInstance();
    Transformer t = tf.newTransformer();

    t.setOutputProperty("indent", "yes");
    t.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "3");

    Source src = new DOMSource(newDoc.getDocumentElement());

    OutputStream o = new ByteArrayOutputStream();

    Result res = new StreamResult(o);
    t.transform(src, res);

    return o.toString();
}

public static String generateXML(OrderValue ov)
    throws
        ParserConfigurationException,
        TransformerConfigurationException,
        TransformerException,
        RemoteException {
    StringBuffer sb = new StringBuffer();
    Document doc = null;

    DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
    DocumentBuilder db = dbf.newDocumentBuilder();
    doc = db.newDocument();

    doc.appendChild(doc.createElement(WLCS_ORDER));

    //Setting Order's attrs/elements
    Element root = doc.getDocumentElement();
    root.setAttribute(ID, ov.identifier);

    Customer orderValueCustomer = 
       ( Customer ) PortableRemoteObject.narrow( ov.customerHandle.getEJBObject(), Customer.class );
    root.setAttribute(CUSTOMER_ID, orderValueCustomer.getIdentifier());

    root.setAttribute(STATUS, "SUBMITTED");

    //add order date

    root.setAttribute(ORDER_DATE, getFormattedCurrentDate("yyyyMMdd"));

    //add transaction id
    if ( ov.paymentTransaction == null ) {
         root.setAttribute(TRANSACTION_ID, "");
    }  else
         root.setAttribute(TRANSACTION_ID, ov.paymentTransaction.getTransactionId());

    Double price = new Double(ov.price.getValue());

    Element orderAmt = doc.createElement(AMOUNT);
    orderAmt.appendChild(doc.createTextNode(price.toString()));

    Element orderCurr = doc.createElement(CURRENCY);
    orderCurr.appendChild(doc.createTextNode(ov.price.getCurrency()));

    Element orderPrice = doc.createElement(PRICE);
    orderPrice.appendChild(orderAmt);
    orderPrice.appendChild(orderCurr);

    root.appendChild(orderPrice);

    Address add = ov.shippingAddress;
    Money shippingMoney = ov.shipping;

    Element orderShipping = doc.createElement(SHIPPING);

    Element orderShippingMethod = doc.createElement(METHOD);
    orderShippingMethod.appendChild(doc.createTextNode(ov.shippingMethod));

    Element orderShippingAmt = doc.createElement(AMOUNT);
    orderShippingAmt.appendChild(
        doc.createTextNode(new Double(shippingMoney.getValue()).toString()));

    Element orderShippingCurr = doc.createElement(CURRENCY);
    orderShippingCurr.appendChild(doc.createTextNode(shippingMoney.getCurrency()));

    Element orderShippingGeoCode = doc.createElement(GEOCODE);
    orderShippingGeoCode.appendChild(doc.createTextNode(add.getGeoCode()));

    Element orderShippingStr1 = doc.createElement(STREET1);
    orderShippingStr1.appendChild(doc.createTextNode(add.getStreet1()));

    String str2 = add.getStreet2();
    if (str2 != null) {
        Element orderShippingStr2 = doc.createElement(STREET2);
        orderShippingStr2.appendChild(doc.createTextNode(str2));
        orderShipping.appendChild(orderShippingStr2);
    }

    Element orderShippingCity = doc.createElement(CITY);
    orderShippingCity.appendChild(doc.createTextNode(add.getCity()));

    Element orderShippingState = doc.createElement(STATE);
    orderShippingState.appendChild(doc.createTextNode(add.getState()));

    Element orderShippingCountry = doc.createElement(COUNTRY);
    orderShippingCountry.appendChild(doc.createTextNode(add.getCountry()));

    String poBox = add.getPoBox();
    if (poBox != null) {
        Element orderShippingPOBox = doc.createElement(POBOX);
        orderShippingPOBox.appendChild(doc.createTextNode(poBox));
        orderShipping.appendChild(orderShippingPOBox);
    }

    String specialInstructions = ov.specialInstructions;
    if (specialInstructions != null) {
        Element specialInstr = doc.createElement(SPECIAL_INSTR);
        specialInstr.appendChild(doc.createTextNode(specialInstructions));
        root.appendChild(specialInstr);
    }

    Element orderShippingCounty = doc.createElement(COUNTY);
    orderShippingCounty.appendChild(doc.createTextNode(add.getCounty()));

    Element postalCode = doc.createElement(POSTAL_CODE);
    postalCode.appendChild(doc.createTextNode(add.getPostalCode()));

    Element postalCodeType = doc.createElement(POSTAL_CODE_TYPE);
    postalCodeType.appendChild(doc.createTextNode(add.getPostalCodeType()));

    orderShipping.appendChild(orderShippingMethod);
    orderShipping.appendChild(orderShippingAmt);
    orderShipping.appendChild(orderShippingCurr);
    orderShipping.appendChild(orderShippingGeoCode);
    orderShipping.appendChild(orderShippingStr1);
    orderShipping.appendChild(orderShippingCity);
    orderShipping.appendChild(orderShippingState);
    orderShipping.appendChild(orderShippingCountry);
    orderShipping.appendChild(orderShippingCounty);
    orderShipping.appendChild(postalCode);
    orderShipping.appendChild(postalCodeType);

    root.appendChild(orderShipping);

    Element splittingPref = doc.createElement(SPLITTING_PREF);
    splittingPref.appendChild(doc.createTextNode(ov.splittingPreference));

    Element subTotal = doc.createElement(SUBTOTAL);
    subTotal.appendChild(doc.createTextNode((new Double(ov.subtotal)).toString()));

    root.appendChild(splittingPref);
    root.appendChild(subTotal);

    Iterator lines = ov.orderLines.iterator();

    //prepare order lines
    Element orderLines = doc.createElement(LINES);
    while (lines.hasNext()) {
        OrderLine line = (OrderLine) lines.next();

        Long orderLineId = new Long(line.getId());
        Double quantity = new Double(line.getQuantity());
        Money tax = line.getTax();
        Double taxAmt = new Double(tax.getValue());
        Money shipping = line.getShipping();
        Double shippingAmt = new Double(shipping.getValue());
        Money up = line.getUnitPrice();
        Double unitPriceAmt = new Double(up.getValue());
        Money msrp = line.getMsrp();
        Double msrpAmt = new Double(msrp.getValue());
        String desc = line.getDescription();
        Double totalAmt = new Double(line.getTotalLineAmount());

        //setting OrderLineItems' attributes/elements
        Element orderLine = doc.createElement(LINE);
        orderLine.setAttribute(ID, orderLineId.toString());

        Element orderLineQuantity = doc.createElement(QUANTITY);
        orderLineQuantity.appendChild(doc.createTextNode(quantity.toString()));
        orderLine.appendChild(orderLineQuantity);

        Element orderLineProductID = doc.createElement(PRODUCT_ID);
        orderLineProductID.appendChild(doc.createTextNode(line.getProductIdentifier()));
        orderLine.appendChild(orderLineProductID);

        Element orderLineTax = doc.createElement(TAX);
        Element orderLineTaxAmount = doc.createElement(AMOUNT);
        orderLineTaxAmount.appendChild(doc.createTextNode(taxAmt.toString()));
        orderLineTax.appendChild(orderLineTaxAmount);
        Element orderLineTaxCurr = doc.createElement(CURRENCY);
        orderLineTaxCurr.appendChild(doc.createTextNode(tax.getCurrency()));
        orderLineTax.appendChild(orderLineTaxCurr);
        orderLine.appendChild(orderLineTax);

        Element orderLineShipping = doc.createElement(SHIPPING);
        Element orderLineShippingAmount = doc.createElement(AMOUNT);
        orderLineShippingAmount.appendChild(doc.createTextNode(shippingAmt.toString()));
        Element orderLineShippingCurr = doc.createElement(CURRENCY);
        orderLineShippingCurr.appendChild(doc.createTextNode(shipping.getCurrency()));
        orderLineShipping.appendChild(orderLineShippingAmount);
        orderLineShipping.appendChild(orderLineShippingCurr);
        orderLine.appendChild(orderLineShipping);

        Element orderLineUnitPrice = doc.createElement(UNIT_PRICE);
        Element orderLineUnitPriceAmount = doc.createElement(AMOUNT);
        orderLineUnitPriceAmount.appendChild(
            doc.createTextNode(unitPriceAmt.toString()));
        orderLineUnitPrice.appendChild(orderLineUnitPriceAmount);
        Element orderLineUnitPriceCurr = doc.createElement(CURRENCY);
        orderLineUnitPriceCurr.appendChild(doc.createTextNode(up.getCurrency()));
        orderLineUnitPrice.appendChild(orderLineUnitPriceCurr);
        orderLine.appendChild(orderLineUnitPrice);

        Element orderLineMsrp = doc.createElement(MSRP);
        Element orderLineMsrpAmount = doc.createElement(AMOUNT);
        orderLineMsrpAmount.appendChild(doc.createTextNode(msrpAmt.toString()));
        Element orderLineMsrpCurr = doc.createElement(CURRENCY);
        orderLineMsrpCurr.appendChild(doc.createTextNode(msrp.getCurrency()));
        orderLineMsrp.appendChild(orderLineMsrpAmount);
        orderLineMsrp.appendChild(orderLineMsrpCurr);
        orderLine.appendChild(orderLineMsrp);

        Element orderLineDesc = doc.createElement(DESC);
        orderLineDesc.appendChild(doc.createTextNode(desc));
        orderLine.appendChild(orderLineDesc);

        Element orderLineTotalAmount = doc.createElement(TOTAL_AMOUNT);
        orderLineTotalAmount.appendChild(doc.createTextNode(totalAmt.toString()));
        orderLine.appendChild(orderLineTotalAmount);

        orderLines.appendChild(orderLine);
    }
    root.appendChild(orderLines);

    return transform(doc);

}

public static String generateXML(PurchaseQuote quote)
    throws
        ParserConfigurationException,
        TransformerConfigurationException,
        TransformerException,
        RemoteException {

    StringBuffer sb = new StringBuffer();
    Document doc = null;

    DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
    DocumentBuilder db = dbf.newDocumentBuilder();
    doc = db.newDocument();

    doc.appendChild(doc.createElement(PO_REQUEST));

    Element root = doc.getDocumentElement();

    Element reqId = doc.createElement(QPA_REQUEST_ID);
    reqId.appendChild(doc.createTextNode(quote.requestId()));
    root.appendChild(reqId);

    Element resId = doc.createElement(QPA_RESPONSE_ID);
    resId.appendChild(doc.createTextNode(quote.quoteId()));
    root.appendChild(resId);

    Element suppName = doc.createElement(SUPPLIER_NAME);
    suppName.appendChild(doc.createTextNode(quote.supplierName()));
    root.appendChild(suppName);

    Element poItems = doc.createElement(PO_ITEMS);

    Element partId = doc.createElement(PART_ID);
    partId.appendChild(doc.createTextNode(quote.productId()));
    poItems.appendChild(partId);

    Element desc = doc.createElement(PART_DESCRIPTION);
    desc.appendChild(doc.createTextNode(quote.description()));
    poItems.appendChild(desc);

    Element qty = doc.createElement(WLI_QUANTITY);
    qty.appendChild(doc.createTextNode(new Long(quote.quotedQty()).toString()));
    poItems.appendChild(qty);

    Element unitPrice = doc.createElement(WLI_UNIT_PRICE);

    unitPrice.appendChild(
        doc.createTextNode(new Double(quote.quotedUnitPrice().getValue()).toString()));

    poItems.appendChild(unitPrice);

    Element unitCurr = doc.createElement(UNIT_CURRENCY);

    unitCurr.appendChild(doc.createTextNode(quote.quotedUnitPrice().getCurrency()));

    poItems.appendChild(unitCurr);

    Element deliveryDate = doc.createElement(DELIVERY_DATE);

    deliveryDate.appendChild(
        doc.createTextNode(
            getFormattedDate(quote.promiseDate().getTime(), "yyyyMMdd")));

    poItems.appendChild(deliveryDate);

    root.appendChild(poItems);
    return transform(doc);

}

public static String generateXML(QPA qpa)
    throws
        ParserConfigurationException,
        TransformerConfigurationException,
        TransformerException,
        RemoteException {

    StringBuffer sb = new StringBuffer();
    Document doc = null;

    DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
    DocumentBuilder db = dbf.newDocumentBuilder();
    doc = db.newDocument();

    doc.appendChild(doc.createElement(QPA_REQUEST));

    Element root = doc.getDocumentElement();

    Element reqId = doc.createElement(QPA_REQUEST_ID);
    reqId.appendChild(doc.createTextNode(qpa.requestId()));
    root.appendChild(reqId);

    Element createDate = doc.createElement(CREATION_DATE);

    if (qpa.creationDate() == null) {

        createDate.appendChild(doc.createTextNode(getFormattedCurrentDate("yyyyMMdd")));

    } else {

        createDate.appendChild(
            doc.createTextNode(getFormattedDate(qpa.creationDate().getTime(), "yyyyMMdd")));

    }

    root.appendChild(createDate);

    Element avail = doc.createElement(AVAILABILITY);

    Element partId = doc.createElement(PART_ID);
    partId.appendChild(doc.createTextNode(qpa.productId()));
    avail.appendChild(partId);

    Element desc = doc.createElement(PART_DESCRIPTION);
    desc.appendChild(doc.createTextNode(qpa.description()));
    avail.appendChild(desc);

    Element qty = doc.createElement(WLI_QUANTITY);
    qty.appendChild(doc.createTextNode(new Long(qpa.desiredQty()).toString()));
    avail.appendChild(qty);

    Element unitPrice = doc.createElement(WLI_UNIT_PRICE);
    unitPrice.appendChild(
        doc.createTextNode(new Double(qpa.desiredUnitPrice().getValue()).toString()));
    avail.appendChild(unitPrice);

    Element unitCurr = doc.createElement(UNIT_CURRENCY);
    unitCurr.appendChild(doc.createTextNode(qpa.desiredUnitPrice().getCurrency()));
    avail.appendChild(unitCurr);

    Element reqDate = doc.createElement(REQUIRED_DATE);
    reqDate.appendChild(
        doc.createTextNode(getFormattedDate(qpa.desiredDate().getTime(), "yyyyMMdd")));

    avail.appendChild(reqDate);

    root.appendChild(avail);

    return transform(doc);

}

public static String getFormattedCurrentDate(String format) {

    java.text.SimpleDateFormat formatter = new java.text.SimpleDateFormat(format);

    java.util.Date currentTime = new java.util.Date();
    String dateString = formatter.format(currentTime);
    return dateString;

}

public static String getFormattedDate(java.util.Date date, String format) {

    java.text.SimpleDateFormat formatter = new java.text.SimpleDateFormat(format);

    String dateString = formatter.format(date);
    return dateString;

}
}
