package examples.e2e.common;

/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 */

import com.bea.commerce.ebusiness.price.quote.Money;
import java.text.SimpleDateFormat;
import java.util.Calendar;


public class QPAImpl implements QPA {

    private static final SimpleDateFormat format = 
            new SimpleDateFormat( "dd MMM yyyy" );

    private String productId;
    private String requestId;
    private String description;
    private Money desiredUnitPrice;
    private long desiredQty;
    private Calendar creationDate;
    private Calendar desiredDate;


    /**
     * QPAImpl constructor.
     */
    public QPAImpl(
            String productId,
            String requestId,
            String desciption,
            Money desiredUnitPrice,
            long desiredQty,
            Calendar creationDate,
            Calendar desiredDate) {

        //any other constructor required with defaults?

        this.productId = productId;
        this.requestId = requestId;
        this.description = desciption;
        this.desiredUnitPrice = desiredUnitPrice;
        this.desiredQty = desiredQty;
        this.creationDate = creationDate;
        this.desiredDate = desiredDate;
    }
/**
 * Returns the date this QPA was created.
 */
    public Calendar creationDate() {
        return creationDate;
    }
/**
 * Returns a description for the product to be quoted.
 */
    public String description() {
        return description;
    }
/**
 * Returns the date the purchaser desires to receive the product.
 */
    public Calendar desiredDate() {
        return desiredDate;
    }
/**
 * Returns the quantity of the product desired by the purchaser.
 */
    public long desiredQty() {
        return desiredQty;
    }
/**
 * Returns the unit price desired by the purchaser.
 */
    public Money desiredUnitPrice() {
        return desiredUnitPrice;
    }
/**
 * Compares two objects for equality. Returns a boolean that indicates
 * whether this object is equivalent to the specified object. This method
 * is used when an object is stored in a hashtable.
 * @param obj the Object to compare with
 * @return true if these Objects are equal; false otherwise.
 * @see java.util.Hashtable
 */
    public boolean equals(Object obj) {

        if ( obj == this ) {

            return true;
        }
        else if ( obj == null || getClass() != obj.getClass() ) {

            return false;
        }

        QPA other = (QPA) obj;
        return this.requestId().equals( other.requestId() ); 
    }

/**
 * Code to perform when this object is garbage collected.
 * 
 * Any exception thrown by a finalize method causes the finalization to
 * halt. But otherwise, it is ignored.
 */
    protected void finalize() throws Throwable {
        super.finalize();
    }
/**
 * Generates a hash code for the receiver.
 * This method is supported primarily for
 * hash tables, such as those provided in java.util.
 * @return an integer hash code for the receiver
 * @see java.util.Hashtable
 */
    public int hashCode() {
        /** 
         * override hashCode() to get consistent
         * results when instances of the class with overridden equals() are used as
         * keys in a java.util.Hashtable 
         */

        return this.toString().hashCode();
    }
/**
 * Returns the identifier for the product to be quoted.
 */
    public String productId() {
        return productId;
    }
/**
 * Returns the identifier for this QPA.
 */
    public String requestId() {
        return requestId;
    }
/**
 * Returns a String that represents the value of this object.
 * @return a string representation of the receiver
 */
    public String toString() {
        // Insert code to print the receiver here.
        StringBuffer sb = new StringBuffer( 64 );

        sb.append( this.getClass().getName() + "[ " );
        sb.append( "productID( " + productId() + " )" );
        sb.append( ", requestId( " + requestId() + " )" );
        sb.append( ", description( " + description() + " )" );
        sb.append( ", unit price( " + desiredUnitPrice() + " )" );
        sb.append( ", desiredQty( " + desiredQty() + " )" );
        sb.append( ", creation date( " +
                format.format( creationDate().getTime() ) + " )" );
        sb.append( ", deliveryDate( " + 
                format.format( desiredDate().getTime() ) + " )" );
        sb.append( " ]" );

        return sb.toString();

    }
}
