
/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 */

package examples.e2e.common;

import java.io.Serializable;
import java.util.Map;
import java.util.HashMap;


public final class QuoteElement implements Serializable {

    private int intValue;
    private String name;

    public static final int I_AGGREGATED_RESPONSE = 0;
    public static final QuoteElement AGGREGATED_RESPONSE =
            new QuoteElement( I_AGGREGATED_RESPONSE, "E2E_AggregatedQPAResponse" );

    public static final int I_REQUEST_ID = 1;
    public static final QuoteElement REQUEST_ID =
            new QuoteElement( I_REQUEST_ID, "QPARequestId" );

    public static final int I_CREATION_DATE = 2;
    public static final QuoteElement CREATION_DATE =
            new QuoteElement( I_CREATION_DATE, "CreationDate" );

    public static final int I_RESPONSE = 3;
    public static final QuoteElement RESPONSE =
            new QuoteElement( I_RESPONSE, "QPAResponse" );

    public static final int I_RESPONSE_ID = 4;
    public static final QuoteElement RESPONSE_ID =
            new QuoteElement( I_RESPONSE_ID, "QPAResponseId" );

    public static final int I_RESPONSE_DATE = 5;
    public static final QuoteElement RESPONSE_DATE =
            new QuoteElement( I_RESPONSE_DATE, "ResponseDate" );

    public static final int I_SUPPLIER_NAME = 6;
    public static final QuoteElement SUPPLIER_NAME =
            new QuoteElement( I_SUPPLIER_NAME, "SupplierName" );

    public static final int I_AVAILABILITY = 7;
    public static final QuoteElement AVAILABILITY =
            new QuoteElement( I_AVAILABILITY, "Availability" );

    public static final int I_PART_ID = 8;
    public static final QuoteElement PART_ID =
            new QuoteElement( I_PART_ID, "PartId" );

    public static final int I_PART_DESC = 9;
    public static final QuoteElement PART_DESC =
            new QuoteElement( I_PART_DESC, "PartDescription" );

    public static final int I_QUANTITY = 10;
    public static final QuoteElement QUANTITY =
            new QuoteElement( I_QUANTITY, "Quantity" );

    public static final int I_UNIT_PRICE = 11;
    public static final QuoteElement UNIT_PRICE =
            new QuoteElement( I_UNIT_PRICE, "UnitPrice" );

    public static final int I_UNIT_CURRENCY = 12;
    public static final QuoteElement UNIT_CURRENCY =
            new QuoteElement( I_UNIT_CURRENCY, "UnitCurrency" );

    public static final int I_AVAILABLE_DATE = 13;
    public static final QuoteElement AVAILABLE_DATE =
            new QuoteElement( I_AVAILABLE_DATE, "AvailableDate" );

    public static final int I_LOCATION = 14;
    public static final QuoteElement LOCATION =
            new QuoteElement( I_LOCATION, "Location" );

    public static final int I_NOTE = 15;
    public static final QuoteElement NOTE =
            new QuoteElement( I_NOTE, "Note" );

    // set this to the number of elements defined
    public static final int SIZE = 16;
    // set the size to prevent rehashing
    private static final Map stringToObject =
            new HashMap( (int)( SIZE / 0.75f ) + 1 );

    static {

        stringToObject.put( AGGREGATED_RESPONSE.getString(), AGGREGATED_RESPONSE );
        stringToObject.put( REQUEST_ID.getString(), REQUEST_ID );
        stringToObject.put( CREATION_DATE.getString(), CREATION_DATE );
        stringToObject.put( RESPONSE.getString(), RESPONSE );
        stringToObject.put( RESPONSE_ID.getString(), RESPONSE_ID );
        stringToObject.put( RESPONSE_DATE.getString(), RESPONSE_DATE );
        stringToObject.put( SUPPLIER_NAME.getString(), SUPPLIER_NAME );
        stringToObject.put( AVAILABILITY.getString(), AVAILABILITY );
        stringToObject.put( PART_ID.getString(), PART_ID );
        stringToObject.put( PART_DESC.getString(), PART_DESC );
        stringToObject.put( QUANTITY.getString(), QUANTITY );
        stringToObject.put( UNIT_PRICE.getString(), UNIT_PRICE );
        stringToObject.put( UNIT_CURRENCY.getString(), UNIT_CURRENCY );
        stringToObject.put( AVAILABLE_DATE.getString(), AVAILABLE_DATE );
        stringToObject.put( LOCATION.getString(), LOCATION );
        stringToObject.put( NOTE.getString(), NOTE );
    }


    /**
     */
    private QuoteElement( int intValue, String name ) {

        this.intValue = intValue;
        this.name = name;
    }

    public String toString() {

        return getString();
    }

    public String getString() {

        return name;
    }

    public int getInt() {

        return intValue;
    }

    public static QuoteElement getObject( String token ) {

        return(QuoteElement) stringToObject.get( token );
    }

    public boolean equals( Object object ) {
        if ( this == object ) {
            return true;
        }
        else if ( object == null || getClass() != object.getClass() ) {
            return false;
        }

        QuoteElement other = (QuoteElement) object;
        return this.intValue == other.intValue;
    }

    private int hashCode;
    private boolean hashCodeValid = false;
    private static final int HASHCODE_SEED = 1000003;

    public int hashCode() {

        if ( hashCodeValid == true ) {
            return hashCode;
        }

        computeHashCode();
        return hashCode;
    }
    private void computeHashCode() {

        hashCode = intValue * HASHCODE_SEED;
        hashCodeValid = true;
    }

}
