
/*                 
 * B E A   S Y S T E M S   
 *
 * C O M M E R C E   C O M P O N E N T S
 *                        
 * Copyright (c) 2000-2002  BEA Systems, Inc.  
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws 
 * of the United States. The software contained on this media is proprietary 
 * to and embodies the confidential technology of BEA Systems, Inc. The 
 * possession or receipt of this information does not convey any right to disclose 
 * its contents, reproduce it, or use,  or license the use, for manufacture or 
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/src/examples/e2e/common/TestBuyerFactory.java,v 1.2 2002/05/02 00:22:12 toneill Exp $
 */

package examples.e2e.common;

import com.bea.p13n.util.debug.Debug;
import com.bea.commerce.ebusiness.price.quote.Money;
import com.bea.commerce.ebusiness.price.quote.ImmutableMoney;

import org.xml.sax.InputSource;

import java.util.Calendar;
import java.util.List;
import java.util.ArrayList;


public class TestBuyerFactory {

    private static Debug debug =
            Debug.getInstance( TestBuyerFactory.class );
    
    private static final Calendar c0 = Calendar.getInstance();
    private static final Calendar c1 = Calendar.getInstance();
    private static final Calendar c2 = Calendar.getInstance();

    private static final List QUOTES = new ArrayList( 2 );
    static {

        c1.add( Calendar.MONTH, 1 );
        c2.add( Calendar.MONTH, 2 );
        
        QUOTES.add( new PurchaseQuoteImpl( "r1", "q1",
                c0, c0,
                "Ozzy's Cash Cow Inc.", "part1", "description of part1",
                1000L, (Money) new ImmutableMoney( 1.0d, "USD" ),
                c1 ) );

        QUOTES.add( new PurchaseQuoteImpl( "r2", "q2",
                c0, c0,
                "Ozzy's Cash Cow Inc.", "part2", "description of part2",
                2000L, (Money) new ImmutableMoney( 2.0d, "USD" ),
                c2 ) );
    }

    private static final List POS = new ArrayList( 4 );
    static {
        
        POS.add( new POImpl( "po1", c0,
                POStatus.PENDING, "supplier 1",
                "part1", "description of part1",
                1000L, 
                (Money) new ImmutableMoney( 2.0d, "USD" ),
                c1 ) );
        
        POS.add( new POImpl( "po2", c0,
                POStatus.ACKNOWLEDGED, "supplier 2",
                "part2", "description of part2",
                2000L, 
                (Money) new ImmutableMoney( 2.0d, "USD" ),
                c2 ) );
        
        POS.add( new POImpl( "po3", c0,
                POStatus.SHIPPED, "supplier 3",
                "part3", "description of part3",
                3000L, 
                (Money) new ImmutableMoney( 2.0d, "USD" ),
                c1 ) );
        
        POS.add( new POImpl( "po4", c0,
                POStatus.RECEIVED, "supplier 4",
                "part4", "description of part4",
                4000L, 
                (Money) new ImmutableMoney( 2.0d, "USD" ),
                c2 ) );
    }


    private TestBuyerFactory() {
    } 


    /**
     * Returns a <code>List</code> of <code>PurchaseQuote</code> objects
     * populated from the data supplied in the XML document.
     *
     * @param xml to use to create <code>PurchaseQuote</code>s.
     */
    public static List createQuote( InputSource xmlSource ) {

        return QUOTES;
    }

    /**
     * Returns a <code>List</code> of <code>PO</code> objects
     * populated from the data supplied in the XML document.
     *
     * @param xml <code>InputSource</code> to use to create <code>PurchaseQuote</code>s.
     */
    public static List createPO( InputSource xmlSource ) {

        return POS;
    }

}


