/**
 * GetProductInventoryAction.java
 *
 * @author Copyright (c) 2000-2002  BEA Systems, Inc. All Rights Reserved.
 */

package examples.e2e.common.inventory.ref;

import com.bea.wlai.client.*;
import com.bea.document.IDocument;
import javax.naming.InitialContext;
import com.bea.document.IDocumentDefinition;
import javax.naming.Context;
import java.util.*;
import com.bea.p13n.util.debug.Debug;
import examples.e2e.common.*;

public class GetProductInventoryAction {
	private String sku_ = null;
    private Context ctx_;
	private static final Debug DEBUG =
		Debug.getInstance(GetProductInventoryAction.class);
    

 public GetProductInventoryAction(String url, String sku) {
	sku_ = sku;
	try {
		Hashtable attrs = new Hashtable();
		attrs.put(Context.PROVIDER_URL, url);
		attrs.put(Context.INITIAL_CONTEXT_FACTORY, 	"weblogic.jndi.WLInitialContextFactory");
		ctx_ = new InitialContext(attrs);
	} catch (Exception e) {
        e.printStackTrace();
	}
 }
 public Object run() {
	Inventory inv = null;
	try {

    	if (DEBUG.ON) {
		    DEBUG.out("GetProductInventoryAction:in run()");
        }
		ApplicationView appView = new ApplicationView(ctx_, "E2EAppView.sav");
		// Create a request document using the definition stored for the Application View at deployment time.
		IDocumentDefinition rd = appView.getRequestDocumentDefinition("getProductInventory");
		IDocument request = rd.createDefaultDocument();
		// Fill in the request with some data.  
		request.setStringInFirst("SKU", sku_);
		// Invoke the service and retrieve the response document
		IDocument response = appView.invokeService("getProductInventory", request);
		appView.close();
    	if (DEBUG.ON) {
		    DEBUG.out("GetProductInventoryAction:run: resp:\n" + response.toXML() + "\n");
        }
        //String desc = response.getStringFrom("/Rows/Row/DESCRIPTION");
		String max = response.getStringFrom("/Rows/Row/MAX_QTY_STOCKED");
 		String min = response.getStringFrom("/Rows/Row/MIN_QTY_STOCKED");
		String qty = response.getStringFrom("/Rows/Row/QTY_IN_STOCK");
		String stdImgURL = response.getStringFrom("/Rows/Row/STANDARD_IMAGE_URL");
		String wideImgURL = response.getStringFrom("/Rows/Row/WIDE_IMAGE_URL");

        inv = new InventoryImpl(sku_, Long.parseLong(min),
                   Long.parseLong(max), Long.parseLong(qty), stdImgURL, wideImgURL);

    	if (DEBUG.ON) {
		    DEBUG.out("GetProductInventoryAction:run: sku:" + sku_ + ": inv:" + inv);
        }
    } catch (NumberFormatException e) {
        // one of the parameters in the response was incorrect; hence return a NULL (finally block)
	} catch (Exception e) {
        e.printStackTrace();
	} finally {
        return inv;
    } 
     
 }
}
