/**
 * GetProductPartInventory.java
 *
 * @author Copyright (c) 2000-2002  BEA Systems, Inc. All Rights Reserved.
 */

package examples.e2e.common.inventory.ref;

import com.bea.wlai.client.*;
import com.bea.document.IDocument;
import javax.naming.InitialContext;
import com.bea.document.IDocumentDefinition;
import javax.naming.Context;
import java.util.*;
import com.bea.p13n.util.debug.Debug;
import examples.e2e.common.*;

public class GetProductPartInventoryAction {
	private String sku_ = null;
    private Context ctx_;
	private static final Debug DEBUG =
		Debug.getInstance(GetProductPartInventoryAction.class);
    

 public GetProductPartInventoryAction(String url, String sku) {
	sku_ = sku;
	try {
		Hashtable attrs = new Hashtable();
		attrs.put(Context.PROVIDER_URL, url);
		attrs.put(Context.INITIAL_CONTEXT_FACTORY, 	"weblogic.jndi.WLInitialContextFactory");
		ctx_ = new InitialContext(attrs);
	} catch (Exception e) {
        e.printStackTrace();
	}
 }
 public Object run() {
    Inventory inv;
	List invs = new ArrayList();
	try {
    	if (DEBUG.ON) {
		    DEBUG.out("GetProductPartInventoryAction:in run()");
        }
		ApplicationView appView = new ApplicationView(ctx_, "E2EAppView.sav");
		// Create a request document using the definition stored for the Application View at deployment time.
		IDocumentDefinition rd = appView.getRequestDocumentDefinition("getProductPartInventory");
		IDocument request = rd.createDefaultDocument();
		// Fill in the request with some data.  
		request.setStringInFirst("PRODUCT_SKU", sku_);
		// Invoke the service and retrieve the response document
		IDocument response = appView.invokeService("getProductPartInventory", request);
		appView.close();
    	if (DEBUG.ON) {
		    DEBUG.out("GetProductPartInventoryAction:run: resp:\n" + response.toXML() + "\n");
        }

        //Figure out how many inventory objects are being returned
        int rows = response.getIntegerFrom("count(/Rows/Row)");
        //Iterate to create Inventory objects
        for (int i =1; i<=rows; i++) {

             String mySku =  response.getStringFrom("/Rows/Row["+ i + "]/SKU");
             String desc = response.getStringFrom("/Rows/Row["+ i + "]/DESCRIPTION");
		     String qty = response.getStringFrom("/Rows/Row[" + i  + "]/QTY_IN_STOCK");
		     String max = response.getStringFrom("/Rows/Row[" + i + "]/MAX_QTY_STOCKED");
 		     String min = response.getStringFrom("/Rows/Row[" + i + "]/MIN_QTY_STOCKED");

             inv = new InventoryImpl(mySku, desc, Long.parseLong(min), Long.parseLong(max), Long.parseLong(qty));
             invs.add(inv);
        }
    	if (DEBUG.ON) {
		    DEBUG.out("GetProductPartInventoryAction:run: inventory objects: " + invs);
        }
    } catch (NumberFormatException e) {
        // one of the parameters in the response was incorrect; hence return an empty arraylist (finally block)
	} catch (Exception e) {
        e.printStackTrace();
	} finally {
        return invs;
    } 
 }
}
