package examples.e2e.common.inventory.ref;

/*
 * B E A   S Y S T E M S
 *
 * C O M M E R C E  C O M P O N E N T S
 *
 * Copyright (c) 2000-2002  BEA Systems, Inc.
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws
 * of the United States. The software contained on this media is proprietary
 * to and embodies the confidential technology of BEA Systems, Inc. The
 * possession or receipt of this information does not convey any right to disclose
 * its contents, reproduce it, or use,  or license the use, for manufacture or
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/src/examples/e2e/common/inventory/ref/InventoryProviderNoOpImpl.java,v 1.2 2002/05/02 00:22:12 toneill Exp $
 */

// java imports
import java.util.*;
import java.rmi.RemoteException;
import javax.ejb.CreateException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import examples.e2e.common.TestInventoryFactory;

//internal imports
import com.bea.p13n.exceptions.ApplicationException;

import com.bea.p13n.util.debug.Debug;

/**
 * InventoryProviderNoOpImpl is a stateless Session Bean. This bean provides:
 * the following services:
 * <ul>
 * <li> No Operation (NoOp) implementation to clients.
 * </ul>
 *
 */

public class InventoryProviderNoOpImpl implements SessionBean {

    private static final int DUMMY_INVENTORY_COUNT = 30;
    /**
      * The debug instance for this class
      */
    private static final Debug DEBUG =
            Debug.getInstance(InventoryProviderNoOpImpl.class);

    /**
    * License check flag.
    */
    private static boolean checkedLicense = false;

    private SessionContext ctx;

/**
    * Create an instance of InventoryProviderNoOpImpl.
    */
    public InventoryProviderNoOpImpl() {
        super();

        if ( !checkedLicense ) {
            com.bea.p13n.license.LicenseHelper.getInstance().checkLicense(
                    com.bea.p13n.common.P13nProduct.CORE);
            checkedLicense = true;
        }
    }
/**
   * Checks the quantity in stock for a given product (sku)
   *
   * @param aProductSKU sku (unique identifier for a given product)
   * @throws IllegalArgumentException if <code>aProductSKU</code> is <code>null</code> 
   * @throws ApplicationException if an exception was returned in accessing the information.   
   * @throws RemoteException if a communication error occurs during the execution 
   *   of a remote method call.  
   */

    public int checkInventory(String aProductSKU)
    throws RemoteException, IllegalArgumentException, ApplicationException {

        return DUMMY_INVENTORY_COUNT;

    }
/**
    * This method is required by the EJB Specification,
    */
    public void ejbActivate() {
        if ( DEBUG.ON )
            DEBUG.out("ejbActivate called");
    }
/**
    * This method corresponds to the create method in the home interface
    * @exception               javax.ejb.CreateException if there is
    *                          a communications or systems failure
    */
    public void ejbCreate() throws CreateException {
        if ( DEBUG.ON ) {
            DEBUG.out("ejbCreate called");
        }

    }
/**
    * This method is required by the EJB Specification,
    */
    public void ejbPassivate() {
        if ( DEBUG.ON )
            DEBUG.out("ejbPassivate called");
    }
/**
    * This method is required by the EJB Specification,
    */
    public void ejbRemove() {
        if ( DEBUG.ON )
            DEBUG.out("ejbRemove called");
    }


/**
 * Sets the session context.
 *
 * @param ctx    SessionContext Context for session
 */
    public void setSessionContext(SessionContext ctx) {

        if ( DEBUG.ON ) {
            DEBUG.out("setSessionContext called");
        }
        this.ctx = ctx;
    }


/**
   * Returns a list of <code> Inventory </code> objects
   *
   * @throws ApplicationException if an exception was returned in accessing the information.   
   * @throws RemoteException if a communication error occurs during the execution 
   *   of a remote method call.
   * @throws IllegalArgumentException if <code>skus</code> is <code>null</code> or 
   * of zero-length.

   */
    public List getProductInventory(List skus)
    throws RemoteException, IllegalArgumentException, ApplicationException {

        // returns canned data regardless of input
        return TestInventoryFactory.createProduct( skus );
    }

/**
   * Returns a list of <code> Inventory </code> objects.  Each inventory object 
   * represents a part for a product (referenced by one of the product SKUs contained
   * in the List passed).
   *
   *
   * @param aProductSKU sku (unique identifier for a given product)
   * @throws IllegalArgumentException if <code>sku</code> is <code>null</code> 
   * @throws ApplicationException if an exception was returned in accessing the information.   
   * @throws RemoteException if a communication error occurs during the execution 
   *   of a remote method call.  
   */
    public List getProductPartInventory(String sku)
    throws RemoteException, IllegalArgumentException, ApplicationException {

        // returns canned data regardless of input
        return TestInventoryFactory.createPart( sku );
    }
}
