package examples.e2e.common.inventory.spi;

/*
 * B E A   S Y S T E M S
 *
 * C O M M E R C E  C O M P O N E N T S
 *
 * Copyright (c) 2000-2002  BEA Systems, Inc.
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws
 * of the United States. The software contained on this media is proprietary
 * to and embodies the confidential technology of BEA Systems, Inc. The
 * possession or receipt of this information does not convey any right to disclose
 * its contents, reproduce it, or use,  or license the use, for manufacture or
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/src/examples/e2e/common/inventory/spi/InventoryProvider.java,v 1.2 2002/05/02 00:22:12 toneill Exp $
 */

//java imports
import java.rmi.RemoteException;
import javax.ejb.EJBObject;
import java.util.List;

//internal imports
import com.bea.p13n.exceptions.ApplicationException;

/**
 * This interface defines a stateless API for providing inventory related
 * functionality.  This interface is exposed as a stateless session EJB 
 * that can be created using the <code>InventoryProviderHome</code>.
 *
 * 
 * @see examples.e2e.common.inventory.spi.InventoryProviderHome
 */

public interface InventoryProvider extends EJBObject {
/**
   * Checks the quantity in stock for a given product (sku)
   *
   * @param aProductSKU sku (unique identifier for a given product)
   * @throws IllegalArgumentException if <code>aProductSKU</code> is <code>null</code>
   * @throws ApplicationException if an exception was returned in accessing the information.   
   * @throws RemoteException if a communication error occurs during the execution 
   *   of a remote method call.  
   */

public int checkInventory(String aProductSKU)
    throws RemoteException, IllegalArgumentException, ApplicationException;



/**
   * Returns a list of <code> Inventory </code> objects for a given list of SKUs
   *
   * @throws ApplicationException if an exception was returned in accessing the information.   
   * @throws RemoteException if a communication error occurs during the execution 
   *   of a remote method call.
   * @throws IllegalArgumentException if <code>skus</code> is <code>null</code> or 
   * of zero-length.

   */
public List getProductInventory(List skus)
    throws RemoteException, IllegalArgumentException, ApplicationException;

/**
   * Returns a list of <code> Inventory </code> objects for a given SKU
   *
   * @throws ApplicationException if an exception was returned in accessing the information.   
   * @throws RemoteException if a communication error occurs during the execution 
   *   of a remote method call.
   * @throws IllegalArgumentException if <code>aSKU</code> is <code>null</code>
   */
public List getProductPartInventory(String aSKU)
    throws RemoteException, IllegalArgumentException, ApplicationException;
}
