package examples.e2e.common.purchase.ref;

/*
 * B E A   S Y S T E M S
 *
 * C O M M E R C E  C O M P O N E N T S
 *
 * Copyright (c) 2000-2002  BEA Systems, Inc.
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws
 * of the United States. The software contained on this media is proprietary
 * to and embodies the confidential technology of BEA Systems, Inc. The
 * possession or receipt of this information does not convey any right to disclose
 * its contents, reproduce it, or use,  or license the use, for manufacture or
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/src/examples/e2e/common/purchase/ref/PurchaseManagerRefImpl.java,v 1.2 2002/05/02 00:22:12 toneill Exp $
 */

// java imports
import java.util.*;
import java.rmi.RemoteException;
import javax.ejb.CreateException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

//internal imports
import com.bea.p13n.exceptions.ApplicationException;

import com.bea.p13n.util.debug.Debug;

import javax.naming.InitialContext;
import javax.naming.Context;
import com.bea.p13n.util.JndiHelper;

import com.beasys.commerce.ebusiness.order.*;
import examples.e2e.common.*;

/**
 * PurchaseManagerRefImpl is a stateless Session Bean. This bean provides:
 * the following services:
 * <ul>
 * <li> Reference implementation (BEA Weblogic Integration - WLI).
 * </ul>
 *
 */

public class PurchaseManagerRefImpl implements SessionBean {

	/**
	  * The debug instance for this class
	  */
	private static final Debug DEBUG =
		Debug.getInstance(PurchaseManagerRefImpl.class);

	/**
	* License check flag.
	*/
	private static boolean checkedLicense = false;

	private SessionContext ctx;
	private String connectString_;
	private static final String INITIAL_CONTEXT_FACTORY =
		"weblogic.jndi.WLInitialContextFactory";
	private static final String QUEUE_FACTORY_LOOKUP =
		"java:comp/env/MyQueueConnectionFactory";
	private static final String QUEUE_NAME_LOOKUP = "java:comp/env/MyQueue";
	private String queueFactory_;
	private String queueName_;

/**
	* Create an instance of PurchaseManagerNoOpImpl.
	*/
public PurchaseManagerRefImpl() {
	super();

	if (!checkedLicense) {
		com.bea.p13n.license.LicenseHelper.getInstance().checkLicense(
			com.bea.p13n.common.P13nProduct.CORE);
		checkedLicense = true;
	}
}

/**
	* This method is required by the EJB Specification,
	*/
public void ejbActivate() {
	if (DEBUG.ON)
		DEBUG.out("ejbActivate called");
}
/**
	* This method corresponds to the create method in the home interface
	* @exception               javax.ejb.CreateException if there is
	*                          a communications or systems failure
	*/
public void ejbCreate() throws CreateException {
	if (DEBUG.ON) {
		DEBUG.out("ejbCreate called");
	}

	// Using the property specified as env-entry in ejb-jar.xml file,

	try {
		InitialContext jndiContext = (InitialContext) JndiHelper.getContext();

		queueFactory_ = (String) jndiContext.lookup(QUEUE_FACTORY_LOOKUP);
		queueName_ = (String) jndiContext.lookup(QUEUE_NAME_LOOKUP);
		

		weblogic.management.configuration.ServerMBean serverMBean =
					com.bea.p13n.management.ServerConfiguration.getServerMBean();

				if (serverMBean == null) {
					throw new CreateException("serverMBean is NULL");
				}
			
				String serverName =
					(serverMBean.getListenAddress() == null)
						? "localhost"
						: serverMBean.getListenAddress();
				connectString_ = "t3://" + serverName + ":" + serverMBean.getListenPort();
	} catch (Exception e) {

		throw new CreateException(e.getMessage());
	}

}
/**
	* This method is required by the EJB Specification,
	*/
public void ejbPassivate() {
	if (DEBUG.ON)
		DEBUG.out("ejbPassivate called");
}
/**
	* This method is required by the EJB Specification,
	*/
public void ejbRemove() {
	if (DEBUG.ON)
		DEBUG.out("ejbRemove called");

}


/**
 * Sets the session context.
 *
 * @param ctx    SessionContext Context for session
 */
public void setSessionContext(SessionContext ctx) {

	if (DEBUG.ON) {
		DEBUG.out("setSessionContext called");
	}
	this.ctx = ctx;
	
}

/**
   *
   * @param aOrder A commerce order
   * @throws IllegalArgumentException if <code>aOrder</code> is <code>null</code> 
   * @throws ApplicationException if an exception was returned in accessing the information.   
   * @throws RemoteException if a communication error occurs during the execution 
   *   of a remote method call.  
   */

public void queueOrder(OrderValue aOrder)
	throws IllegalArgumentException, ApplicationException, RemoteException {

	if (aOrder == null) {

		throw new IllegalArgumentException("Purchase Manager: Order is null");
	}

	String xmlRep = null;

	try {
		xmlRep = PurchaseXMLHelper.generateXML(aOrder);
	} catch (Exception e) { //throw e2e exception
		throw new E2EException(e);
	}
	SendOrderAction so =
		new SendOrderAction(connectString_, INITIAL_CONTEXT_FACTORY, queueFactory_, queueName_, xmlRep);
	so.run();

}

/**
   *
   * @param aQuote A quote originally sent by the supplier
   * @throws IllegalArgumentException if <code>aQuote</code> is <code>null</code> 
   * @throws ApplicationException if an exception was returned in accessing the information.   
   * @throws RemoteException if a communication error occurs during the execution 
   *   of a remote method call.  
   */

public void queuePORequest(PurchaseQuote aQuote)
	throws IllegalArgumentException, ApplicationException, RemoteException {

	String xmlRep = null;

	try {
		xmlRep = PurchaseXMLHelper.generateXML(aQuote);
	} catch (Exception e) { //throw e2e exception
		throw new E2EException(e);
	}
	SendOrderAction so =
		new SendOrderAction(connectString_, INITIAL_CONTEXT_FACTORY, queueFactory_, queueName_, xmlRep);
	so.run();

}

/**
   * 
   *
   * @param aQPA Query for Price and Availability from the buyer to the supplier
   * @throws IllegalArgumentException if <code>aQPARequest</code> is <code>null</code> 
   * @throws ApplicationException if an exception was returned in accessing the information.   
   * @throws RemoteException if a communication error occurs during the execution 
   *   of a remote method call.  
   */

public void queueQPA(QPA aQPARequest)
	throws IllegalArgumentException, ApplicationException, RemoteException {

	String xmlRep = null;

	try {
		xmlRep = PurchaseXMLHelper.generateXML(aQPARequest);
	} catch (Exception e) { //throw e2e exception
		throw new E2EException(e);
	}
	SendOrderAction so =
		new SendOrderAction(connectString_, INITIAL_CONTEXT_FACTORY, queueFactory_, queueName_, xmlRep);
	so.run();

}

}
