/*
 * B E A   S Y S T E M S
 *
 * P O R T A L
 *
 * Copyright (c) 2000-2002  BEA Systems, Inc.
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws
 * of the United States. The software contained on this media is proprietary
 * to and embodies the confidential technology of BEA Systems, Inc. The
 * possession or receipt of this information does not convey any right to disclose
 * its contents, reproduce it, or use,  or license the use, for manufacture or
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/src/examples/e2e/common/tourguide/TourGuide.java,v 1.2 2002/05/02 00:22:12 toneill Exp $
 */

package examples.e2e.common.tourguide;

import java.io.Serializable;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;

import examples.e2e.common.tourguide.TourGuideData;

/**
 * Classes which implement this interface provide the logic
 * to decide what the Tour Guide should display based on 
 * the state of the portal application. The class should
 * maintain data about each instance of the Tour Guide that
 * is running in the application. It is important to make
 * sure that the implementation class is serializable since 
 * it will be part of the pipline session.
 */
public interface TourGuide extends Serializable
{
    /* Document retrieval conventions */

    /** The location of the Tour Guide content files */
    public static final String DOCUMENT_URI =
        "/portlets/tourguide/docs/";

    /** The location of the JSP code viewer */
    public static final String CODE_VIEWER_URI =
        "/portlets/tourguide/view_code.jsp";

    /** Prefix for "What's New" content files */
    public static final String WHATS_NEW_DOC_PREFIX =
        "wn-";

    /** Prefix for "Next Steps" content files */
    public static final String NEXT_STEPS_DOC_PREFIX =
        "ns-";

    /** Prefix for technical details content files */
    public static final String TECH_DETAILS_DOC_PREFIX =
        "td-";

    /** Prefix for edocs content files */
    public static final String EDOCS_DOC_PREFIX =
        "edocs-";

    /** Prefix for maximized left margin content files */
    public static final String MAX_LEFT_MARGIN_DOC_PREFIX =
        "lm-";


    /* Tour guide maximized section icons */

    /** Icon for "Technical Details" section */
    public static final String TECH_DETAILS_MAX_ICON =
        "tour_max_icon_technical.gif";

    /** Icon for "View Code" section */
    public static final String VIEW_CODE_MAX_ICON =
        "tour_max_icon_view.gif";

    /** Icon for "Read eDocs" section */
    public static final String EDOCS_MAX_ICON =
        "tour_max_icon_read.gif";


    /* HTTP request parameters */

    public static final String OPTION_PARAM =
        "e2e_tourguide_option";


    /* HTTP request parameter values */

    /** Display "Technical Details" for maximized tour guide */
    public static final String TECH_DETAILS_VALUE = "tech";

    /** Display "View Code" for maximized tour guide */
    public static final String VIEW_CODE_VALUE = "code";

    /** Display "Read eDocs" for maximized tour guide */
    public static final String EDOCS_VALUE = "edocs";


    /* Pipeline session attributes */

    /** 
     * The pipeline session attribute the Tour Guide object will be stored as
     */
    public static final String PIPELINE_ATTR = "e2e_tourguide";


    /* Public interface */

    /**
     * This method should gather information about the state of the portal
     * application and decide what content is appropriate for the Tour
     * Guide to display.
     */
    public TourGuideData determineState( HttpServletRequest request )
        throws JspException;

    /**
     * This method should return the Tour Guide instance data for the specified
     * portal page.
     */
    public TourGuideData getInstanceData( String pageName );
}
