package examples.wlcs.sampleapp.payment;

import java.io.*;
import java.util.*;

/**
 * This class is used to send a payment service request to a payment
 * fulfillment service. Its contents should be defined by the particular
 * payment vendor in use.
 * 
 * @author BEA Systems, Inc.
 */
public class PSRequest
{
        private String  authorizationMode;
        private String	parameters[];
 
	/**
	 * Default constructor
	 */
	public PSRequest()
	{
	}

        /**
         * A constructor that initializes the arglist
         */
        public PSRequest( String argList[] )
        {
            parameters = argList;
        }

	/**
	 * Input Parameters needed by the specific service for a specific payment vendor.
	 *
	 * @param parms Array of strings parameters
	 */
	public void setParameters( String parms[] )
	{
		parameters = parms;
	}

        /**
	 * Get the request parameters
         *
	 * @return the array of request parameters
	 */
	public String [] getParameters()
	{
		return( parameters );
	}

	/**
	 * Get the auth mode setting
	 *
	 * @return the mode
         */
        public String getAuthorizationMode()
        {
            return authorizationMode;
        }

        /**
         * This is an optional request parameter. It indicates whether authorization includes
         * a capture or not. Your specific payment vendor may not require it.
         *
         * @param aMode The mode setting
         */
        public void setAuthorizationMode( String aMode )
        {
            authorizationMode = aMode;
        }

 	/**
	 * toString is used to generate debugging type output.
	 *
	 * @return the String for display to the output of your choice
	 */
	public String toString()
	{
		StringBuffer	outputBuffer = new StringBuffer();
		String		lineSeparator = System.getProperty( "line.separator" );


		outputBuffer.append( "PSRequest.toString() Not implemented - Contents depend on payment vendor" + lineSeparator );
 		return( outputBuffer.toString() );
	}
}
