package examples.wlcs.sampleapp.payment;

import java.io.*;
import java.util.*;

/**
 * This class is an element in a collection of results returned by a payment
 * service. It was designed to be a flexible object for returning an arbitrary
 * number of data back to the payment service requester, identified by some
 * key.  
 *
 * @author BEA Systems, Inc.
 */

public class PSResponseElement
{
	private String	key = null;
	private String	value = null;
 
	/**
	 * Default constructor
	 */
	public PSResponseElement()
	{
	}

	/**
	 * Set the key which identifies the nature of this returned data
         *
	 * @param aKey used for storing the returned data
	 */
	public void setKey( String aKey )
	{
		key = aKey;
	}

	/**
	 * set the actual returned data paired with the above key
         *
	 * @param aValue the data to be stored
	 */
	public void setValue( String aValue )
	{
		value = aValue;
	}

	/**
	 * retrieve the key to this return data element
	 *
	 * @return the key
	 */
	public String getKey()
	{
		return( key );
	}

	/**
	 * retrieve the value of this returned data element
	 *
	 * @return the data value
	 */
	public String getValue()
	{
		return( value );
	}

        /**
	 * toString is used to generate debugging type output.
	 *
	 * @return the String for display to the output of your choice
	 */
	public String toString()
	{
		StringBuffer	outputBuffer = new StringBuffer();
		String		lineSeparator = System.getProperty( "line.separator" );


		outputBuffer.append( "PSResponse: Key=" + key + " Value=" + value + lineSeparator );
		
		return( outputBuffer.toString() );
	}
}
