/*
 * B E A   S Y S T E M S
 *
 * C O M M E R C E   C O M P O N E N T S
 *
 * Copyright (c) 2000-2002  BEA Systems, Inc.
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws
 * of the United States. The software contained on this media is proprietary
 * to and embodies the confidential technology of BEA Systems, Inc. The
 * possession or receipt of this information does not convey any right to disclose
 * its contents, reproduce it, or use,  or license the use, for manufacture or
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/src/examples/wlcs/sampleapp/payment/PaymentWebServiceHelper.java,v 1.2 2002/05/02 00:22:12 toneill Exp $
 */

package examples.wlcs.sampleapp.payment;

import java.util.Properties;
import java.rmi.RemoteException;
import java.sql.SQLException;
import java.sql.Connection;
import javax.sql.DataSource;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.ejb.FinderException;

import com.bea.p13n.util.jdbc.Sequencer;
import com.bea.p13n.util.jdbc.SequencerFactory;
import com.beasys.commerce.axiom.util.helper.CommercePropertiesHelper;
import com.beasys.commerce.foundation.exception.SystemException;
import com.beasys.commerce.foundation.exception.RemoteSystemException;
import com.beasys.commerce.foundation.IllegalWorkflowTransitionException;
import com.beasys.commerce.ebusiness.payment.PaymentTransaction;
import com.beasys.commerce.ebusiness.payment.PaymentTransactionPk;
import com.beasys.commerce.ebusiness.payment.PaymentTransactionHome;
import com.beasys.commerce.ebusiness.payment.PaymentException;

import com.bea.commerce.mbeans.PaymentServiceClientMBean;
import com.bea.commerce.mbeans.CommercePipelineComponentSupportMBean;

import com.bea.p13n.management.ApplicationHelper;
import com.bea.p13n.util.JndiHelper;
import com.bea.p13n.util.debug.Debug;

/**
 * This helper class contains Payment related utility methods for
 * invoking payment webservices, that cen be used by pipeline
 * components as well as admin JSPs.
 *
 */
public class PaymentWebServiceHelper
{
    private Debug debug;

    public PaymentWebServiceHelper()
    {
        debug = Debug.getInstance(PaymentWebServiceHelper.class);
    }

    /**
     * Returns an instance of the <code>CreditCardWebService</code>.
     *
     * @throws NamingException on webservice not found
     */
     public PaymentServicePort getPaymentWebService() throws PaymentException
     {
        PaymentServicePort paywsPort = null;

        if (debug.ON) {
         checkJAXRPCEnabled();
        }

        String wsdlURL = getPaymentServiceClientMBean().getPaymentWebServiceWSDL();

        if (wsdlURL == null)
           {
           throw new PaymentException("ebusiness-payment", "webservice.mbean.lookup");
           }

        try
           {
           paywsPort = new PaymentService(wsdlURL).getPaymentServicePort();
           }
        catch (Exception e)
           {
           if (debug.ON) { e.printStackTrace(); }
           throw new PaymentException("ebusiness-payment", "naming.exception");
           }

        return paywsPort;
     }

    /**
     * Returns an instance of the <code>PaymentTransactionHome</code>.
     */
    private PaymentTransactionHome getPaymentTransactionHome()
    {
        try
        {
	    String ptHome = ApplicationHelper.getApplicationName() + ".BEA_commerce.PaymentTransaction";
            return (PaymentTransactionHome) JndiHelper.lookupNarrow(ptHome, PaymentTransactionHome.class);
        }
        catch (NamingException e)
        {
            throw new SystemException("ebusiness-payment", "payment.transaction.home", e);
        }
    }


    // This method generates transacitonEntryIds for the transactionEntries of
    // a transaction. The logic used to accomplish this is to find the number of
    // transaction entries and then use the orderId of the transaction as the
    // prefix of the transactionEntryId which in turn is obtained by incrementing the
    // the transaction entry colleciton size by one.
    public String getTransactionEntryId(PaymentTransaction pt)
    {
        String transactionEntryId;
        try
        {
            transactionEntryId = pt.getTransactionId() + '-' + pt.transactionEntryCollectionSize() + 1;
        }
        catch(RemoteException re)
        {
            throw new RemoteSystemException("foundation", "remote.exception", re);
        }

        return transactionEntryId;

    }


    public PaymentTransaction getPaymentTransaction(String Id) throws FinderException, RemoteException
    {
        // Find the payment transaction using the supplied ID
        PaymentTransactionHome home = getPaymentTransactionHome();
        PaymentTransaction paymentTransaction = home.findByPrimaryKey(new PaymentTransactionPk(Id));
        return paymentTransaction;
    }

    // generate a new transactionId using JDBCSequencer
    private String generateTransactionId()
    {
        DataSource ds = null;

        // create a sequencer with name PAYMENTTRANSACTION_SEQUENCE
        // default sequence -- Sequence
        // default cache size -- 2000
        // default step size -- 1

        String persistenceURL= getCommercePipelineComponentSupportMBean().getJdbcPoolName();
        try
        {
            ds = (DataSource) JndiHelper.lookupNarrow(persistenceURL, DataSource.class);
        }
        catch (NamingException namingException)
        {
            String[] messageParameters = {"jdbc/commercePool"};
            throw new SystemException("ebusiness-payment",
                                      "payment.transaction.lookup",
                                      messageParameters, namingException);
        }

        Sequencer sequencer = SequencerFactory.createSequencer("TRANSACTION_ID_SEQUENCE", ds);
        Long longSequence;
        try
        {
            longSequence = new Long(sequencer.getNext());
        }
        catch(SQLException se)
        {
            throw new SystemException("ebusiness-payment", "sql.exception", se);
        }
        return ((new String(longSequence.toString())) + (new Long(System.currentTimeMillis()).toString()));
    }

    /**
     * Returns a database connection
     * <p>
     * Set the commerce.jdbc.pool.url property in the weblogiccommerce.properties
     * to the pool url. For example, for the commerce pool specified in the
     * weblogic.properties, this property should be set as:
     * <pre>
     *          commerce.jdbc.pool.url=weblogic.jdbc.jts.commercePool.
     * <pre>
     * </p>
     */
    public Connection getConnection() throws SystemException
    {
        Connection connection = null;

        String persistenceURL= getCommercePipelineComponentSupportMBean().getJdbcPoolName();

        try
        {
            DataSource ds = (DataSource) JndiHelper.lookupNarrow(persistenceURL, DataSource.class);
            connection = ds.getConnection();
        }
         catch(NamingException namingException)
        {
            String[] messageParameters = {"jdbc/commercePool"};
            // The following property should be moved into an appropriate place.
            throw new SystemException("orMap", "errorGetConnection", messageParameters);
        }
        catch(java.sql.SQLException sqlException)
        {
            String[] messageParameters = {"jdbc/commercePool"};
            // The following property should be moved into an appropriate place.
            throw new SystemException("orMap", "errorGetConnection", messageParameters, sqlException);
        }

        return connection;
    }

    public PaymentServiceClientMBean getPaymentServiceClientMBean()
    {
	PaymentServiceClientMBean paymentMBean = null;

	try {
	    paymentMBean = (PaymentServiceClientMBean) ApplicationHelper.getServiceConfigurationMBean( "PaymentServiceClient" );
	}
	catch ( Exception e ) {
	    if (debug.ON) {
               debug.out("Error while getting PaymentServiceClientMBean." );
            }
	    e.printStackTrace();
	}

	return( paymentMBean );
    }

    public CommercePipelineComponentSupportMBean getCommercePipelineComponentSupportMBean()
    {
        CommercePipelineComponentSupportMBean commercePipelineMBean = null;

        try {
            commercePipelineMBean = (CommercePipelineComponentSupportMBean) ApplicationHelper.getServiceConfigurationMBean( "CommercePipelineComponentSupport" );
        }
        catch ( Exception e ) {
            if (debug.ON) {
               debug.out("Error while getting CommercePipelineComponentSupportMBean.");
            }
            e.printStackTrace();
        }

        return( commercePipelineMBean );
     }


    /**
     * Check to see if use of JAXRPC client is enabled
     */
    private void checkJAXRPCEnabled()
    {
       	try
       	{
    	     Class.forName("weblogic.webservice.core.rpc.JAXRPCEnabler");
	     debug.out("JAXRPC is enabled.");
	} catch (ClassNotFoundException cnfe)
	{
	     debug.out("JAXRPC is disabled.");
	}

    }
 }
