package examples.wlcs.sampleapp.tax;

import java.io.*;
import java.util.*;

/**
 * This class is used to send an address verification request to an address
 * verification service.  An address verification service is one that verifies
 * that the parameters passed in are a valid combination.  Not all AVS's require
 * all of the parameters.  Please see the documentation for your address
 * verification service for more information.
 * 
 * @author BEA Systems, Inc.
 */
public class AVSRequest
{
	private String	streetAddress1 = null;
	private String	streetAddress2 = null;
	private String	cityName = null;
	private String	provinceName = null;
	private String	provinceCode = null;
	private String	postalCode = null;
	private String	countyName = null;
	private String	countyCode = null;
	private String	countryName = null;
	private String	countryCode = null;

	/**
	 * Default constructor
	 */
	public AVSRequest()
	{
	}

	/**
	 * streetAddress1 is the first line of an address.  This might be something
	 * like &quot;123 Main St.&quot;.  The second line may be something like
	 * &quot;Apt 123&quot;
	 *
	 * @param address the first line of the street address
	 */
	public void setStreetAddress1( String address )
	{
		streetAddress1 = address;
	}

	/**
	 * streetAddress2 is the second line of an address.  The first line might be
	 * something like &quot;123 Main St.&quot;.  The second line may be something
	 * like &quot;Apt 123&quot;
	 *
	 * @param address the second line of the street address
	 */
	public void setStreetAddress2( String address )
	{
		streetAddress2 = address;
	}

	/**
	 * cityName is the name of the city for the address.
	 *
	 * @param name the city name
	 */
	public void setCityName( String name )
	{
		cityName = name;
	}

	/**
	 * provinceName is the name of the province (or state) for the address
	 *
	 * @param name the province (or state) name
	 */
	public void setProvinceName( String name )
	{
		provinceName = name;
	}

	/**
	 * provinceCode is used by some address verification systems.  It's meaning
	 * is somewhat AVS specific.  Usually this is something like the abbreviation
	 * for the province or state.  For example, the state name may be
	 * &quot;Colorado&quot; and the province code is &quot;CO&quot;
	 *
	 * @param code the province (or state) code
	 */
	public void setProvinceCode( String code )
	{
		provinceCode = code;
	}

	/**
	 * postalCode is the postal (or zip) code for the address.  The
	 * format of this string is both AVS and country specific.
	 *
	 * @param code the postal (or zip) code
	 */
	public void setPostalCode( String code )
	{
		postalCode = code;
	}

	/**
	 * countyName is the name of the county for the address.
	 *
	 * @param county the name of the county
	 */
	public void setCountyName( String name )
	{
		countyName = name;
	}

	/**
	 * countyCode is used by some address verification systems.  It's meaning
	 * is somewhat AVS specific.  Usually this is something like the abbreviation
	 * for the county or a number that is assigned to the county.
	 *
	 * @param code the county code
	 */
	public void setCountyCode( String code )
	{
		countyCode = code;
	}

	/**
	 * countryName is the name of the country for the address.
	 *
	 * @param country the name of the country
	 */
	public void setCountryName( String name )
	{
		countryName = name;
	}

	/**
	 * countryCode is used by some address verification systems.  It's meaning
	 * is somewhat AVS specific.  Usually this is something like the abbreviation
	 * for the country or a number that is assigned to the county.  For example,
	 * the United States may have a code of &quot;US&quot; or &quot;840&quot;
	 * depending on the AVS.
	 *
	 * @param code the country code
	 */
	public void setCountryCode( String code )
	{
		countryCode = code;
	}

	/**
	 * streetAddress1 is the first line of an address.  This might be something
	 * like &quot;123 Main St.&quot;.  The second line may be something like
	 * &quot;Apt 123&quot;
	 *
	 * @return the first line of the street address
	 */
	public String getStreetAddress1()
	{
		return( streetAddress1 );
	}

	/**
	 * streetAddress2 is the second line of an address.  The first line might be
	 * something like &quot;123 Main St.&quot;.  The second line may be something
	 * like &quot;Apt 123&quot;
	 *
	 * @return the second line of the street address
	 */
	public String getStreetAddress2()
	{
		return( streetAddress2 );
	}

	/**
	 * cityName is the name of the city for the address.
	 *
	 * @return the city name
	 */
	public String getCityName()
	{
		return( cityName );
	}

	/**
	 * provinceName is the name of the province (or state) for the address
	 *
	 * @return the province (or state) name
	 */
	public String getProvinceName()
	{
		return( provinceName );
	}

	/**
	 * provinceCode is used by some address verification systems.  It's meaning
	 * is somewhat AVS specific.  Usually this is something like the abbreviation
	 * for the province or state.  For example, the state name may be
	 * &quot;Colorado&quot; and the province code is &quot;CO&quot;
	 *
	 * @return the province (or state) code
	 */
	public String getProvinceCode()
	{
		return( provinceCode );
	}

	/**
	 * postalCode is the postal (or zip) code for the address.  The
	 * format of this string is both AVS and country specific.
	 *
	 * @return the postal (or zip) code
	 */
	public String getPostalCode()
	{
		return( postalCode );
	}

	/**
	 * countyName is the name of the county for the address.
	 *
	 * @return the name of the county
	 */
	public String getCountyName()
	{
		return( countyName );
	}

	/**
	 * countyCode is used by some address verification systems.  It's meaning
	 * is somewhat AVS specific.  Usually this is something like the abbreviation
	 * for the county or a number that is assigned to the county.
	 *
	 * @return the county code
	 */
	public String getCountyCode()
	{
		return( countyCode );
	}

	/**
	 * countryName is the name of the country for the address.
	 *
	 * @return the name of the country
	 */
	public String getCountryName()
	{
		return( countryName );
	}

	/**
	 * countryCode is used by some address verification systems.  It's meaning
	 * is somewhat AVS specific.  Usually this is something like the abbreviation
	 * for the country or a number that is assigned to the county.  For example,
	 * the United States may have a code of &quot;US&quot; or &quot;840&quot;
	 * depending on the AVS.
	 *
	 * @return the country code
	 */
	public String getCountryCode()
	{
		return( countryCode );
	}

	/**
	 * toString is used to generate debugging type output.
	 *
	 * @return the String for display to the output of your choice
	 */
	public String toString()
	{
		StringBuffer	outputBuffer = new StringBuffer();
		String			lineSeparator = System.getProperty( "line.separator" );


		outputBuffer.append( "AVSRequest contents:" + lineSeparator );
		outputBuffer.append( "\tstreetAddress1: \"" + streetAddress1  + "\"" + lineSeparator );
		outputBuffer.append( "\tstreetAddress2: \"" + streetAddress2  + "\"" + lineSeparator );
		outputBuffer.append( "\tcityName: \"" + cityName  + "\"" + lineSeparator );
		outputBuffer.append( "\tprovinceName: \"" + provinceName + "\"" + lineSeparator );
		outputBuffer.append( "\tprovinceCode: \"" + provinceCode + "\"" + lineSeparator );
		outputBuffer.append( "\tpostalCode: \"" + postalCode + "\"" + lineSeparator );
		outputBuffer.append( "\tcountyName: \"" + countyName + "\"" + lineSeparator );
		outputBuffer.append( "\tcountyCode: \"" + countyCode + "\"" + lineSeparator );
		outputBuffer.append( "\tcountryName: \"" + countryName + "\"" + lineSeparator );
		outputBuffer.append( "\tcountryCode: \"" + countryCode + "\"" + lineSeparator );

		return( outputBuffer.toString() );
	}
}
