package examples.wlcs.sampleapp.tax;

import java.io.*;
import java.util.*;
import examples.wlcs.sampleapp.tax.AVSResponseElement;

/**
 * An AVSResponse is a wrapper around a collection of AVSResponseElements.
 * It holds the collection along with a response code and response message from
 * the AVS.  Not all AVS's will return all information.   Please see the
 * documentation for your address verification service for more information.
 *
 * @author BEA Systems, Inc.
 */
public class AVSResponse
{
	public static final String AVS_OK = "OK";
	public static final String AVS_INVALID_ADDRESS = "AVS_INVALID_ADDRESS";

	private String	responseCode = null;
	private String	responseMessage = null;
	private AVSResponseElement[] responseArray;


	/**
	 * The default constructor
	 */
	public AVSResponse()
	{
		makeResponseOK();
	}

	/**
	 * The response code is AVS specific.  It is usually used to indicate the
	 * success or failure of the request.
	 *
	 * @param code the code to return
	 */
	public void setResponseCode( String code )
	{
		responseCode = code;
	}

	/**
	 * The response message is AVS specific.  It is usually used in conjunction
	 * with the response code to provide a textual description of the response code.
	 *
	 * @param message the message to return
	 */
	public void setResponseMessage( String message )
	{
		responseMessage = message;
	}

	/**
	 * Adds a response element
	 *
	 * @param element the response element to add
	 */
	public void setResponseArray( AVSResponseElement[] results )
	{
        responseArray = results;
	}

	/**
	 * The results are stored in a collection (in this case a Vector) and can
	 * be retrieved from there.  Note that, depending on the AVS, this may
	 * return null if there where no results returned.  This can happen if an
	 * error occured in some AVS's.
	 *
	 * @return the results stored in a java.util.Vector
	 *
	 */
	public AVSResponseElement[] getResponseArray()
	{
		return( responseArray );
	}

    /*
	 * The response code is AVS specific.  It is usually used to indicate the
	 * success or failure of the request.
	 *
	 * @return the response code from the AVS
	 */
	public String getResponseCode()
	{
		return( responseCode );
	}

	/**
	 * The response message is AVS specific.  It is usually used in conjunction
	 * with the response code to provide a textual description of the response code.
	 *
	 * @return the message from the AVS
	 */
	public String getResponseMessage()
	{
		return( responseMessage );
	}


	public void makeResponseOK()
	{
		responseCode = AVS_OK;
	}

	public boolean checkResponseOK()
	{
		return( responseCode.equalsIgnoreCase( AVS_OK ) );
	}

	/**
	 * The toString() method is used to generate debugging output
	 *
	 * @return the debugging java.lang.String to be displayed on the output
	 * of your choice.
	 */
	public String toString()
	{
		StringBuffer	outputBuffer = new StringBuffer();
		String			lineSeparator = System.getProperty( "line.separator" );


		outputBuffer.append( "AVSResponse contents:" + lineSeparator );
		outputBuffer.append( "\tresponseCode: \"" + responseCode + "\"" + lineSeparator );
		outputBuffer.append( "\tresponseMessage: \"" + responseMessage + "\"" + lineSeparator );

		if( responseArray != null )
		{
			outputBuffer.append( "\tnumber of elements returned: \"" + responseArray.length + "\"" + lineSeparator );

			for( int i = 0; i < responseArray.length; i++ )
			{
				outputBuffer.append( "AVSResponseElement #" + i + lineSeparator +
								  	responseArray[i].toString() );
			}
		}
		else
		{
			outputBuffer.append( "\tnumber of element returned: \"0\"" + lineSeparator );
		}

		return( outputBuffer.toString() );
	}
}
