package examples.wlcs.sampleapp.tax;

import java.io.*;
import java.util.*;

import examples.wlcs.sampleapp.tax.TaxRequestElement;

/**
 * This class is used to send an tax calculation request to a tax calculation
 * service.  The vast majority of the parameters are in TaxRequestElement.
 * Not all tax calculation services require all of the parameters.  Please see
 * the documentation for your tax service for more information.
 *
 * @author BEA Systems, Inc.
 */
public class TaxRequest
{
	private TaxRequestElement[] requestArray = null;
	private	String	requestParameters = null;


	/**
	 * the default constructor
	 */
	public TaxRequest()
	{

	}

	/**
	 * Sets the request element array to the request.
	 *
	 * @param array the request element array
	 */
	public void setRequestArray( TaxRequestElement[] array )
	{
		requestArray = array;
	}

	/**
	 * Gets the elements for the request
	 *
	 * @return the Vector of elements
	 */
	public TaxRequestElement[]	getRequestArray()
	{
		return( requestArray );
	}


	/**
	 * This method is used to pass service specific parameters to the service.
	 * The format of the string is defined by the service provider.  Not all
	 * service providers require this.  Please see the documentation of your
	 * service provider for more information.
	 *
	 * @param params the vendor specific parameters that are passed to the service
	 */
	public void setRequestParameters( String params )
	{
		requestParameters = params;
	}


	/**
	 * Gets the service provider specific parameter string
	 *
	 * @return the String
	 */
	public String getRequestParameters()
	{
		return( requestParameters );
	}


	/**
	 * toString is used to generate debugging type output.
	 *
	 * @return the String for display to the output of your choice
	 */
	public String toString()
	{
		StringBuffer	outputBuffer = new StringBuffer();
		String			lineSeparator = System.getProperty( "line.separator" );


		outputBuffer.append( "TaxRequest contents:" + lineSeparator );
		outputBuffer.append( "\trequestParameters: \"" + requestParameters + "\"" + lineSeparator );

		if( requestArray != null )
		{
			outputBuffer.append( "\tnumber of elements: \"" + requestArray.length + "\"" + lineSeparator );
			for( int i = 0; i < requestArray.length; i++ )
			{
				outputBuffer.append( "TaxRequestElements number #" );
				outputBuffer.append( i + lineSeparator );
				outputBuffer.append( requestArray[i].toString() );
			}
		}

		return( outputBuffer.toString() );
	}
}
