package examples.wlcs.sampleapp.tax;

import java.io.*;
import java.util.*;
import java.text.*;


/**
 * The TaxRequestElement is the main class used for interaction with a
 * tax service.  It embodies all of the elements that are needed for a
 * request to the tax service.  Not all tax services require all of the
 * parameters.  Please see your tax service documentation for more details.
 *
 * Note: This value object is part of a web service signature and must conform
 *       to the JavaBean spec if WebLogic Server web service generation facilities
 *       are to be used.
 *
 * @author BEA Systems, Inc.
 */
public class TaxRequestElement
{
	private String	companyID = null;
	private String	divisionCode = null;

	private String	shipFromCountryCode = null;
	private String	shipFromProvinceCode = null;
	private String	shipFromCountyName = null;
	private String	shipFromCountyCode = null;
	private String	shipFromCityName = null;
	private String	shipFromPostalCode = null;
	private String	shipFromLocationCode = null;

	private String	shipToCountryCode = null;
	private String	shipToProvinceCode = null;
	private String	shipToCountyName = null;
	private String	shipToCountyCode = null;
	private String	shipToCityName = null;
	private String	shipToPostalCode = null;
	private String	shipToLocationCode = null;

	private String	originCountryCode = null;
	private String	originProvinceCode = null;
	private String	originCountyName = null;
	private String	originCountyCode = null;
	private String	originCityName = null;
	private String	originPostalCode = null;
	private String	originLocationCode = null;

	private String	poaCountryCode = null;
	private String	poaProvinceCode = null;
	private String	poaCountyName = null;
	private String	poaCountyCode = null;
	private String	poaCityName = null;
	private String	poaPostalCode = null;
	private String	poaLocationCode = null;

	private String	pointOfTitlePassage = null;
	private String	transactionType = null;
	private String	taxCode = null;
	private String	typeOfTax = null;
	private String	invoiceDate = null;
	private String	deliveryDate = null;
	private String	modeOfTransport = null;
	private String	commodityCode = null;
	private Boolean	isCredit = null;
	private String	sellerRegistrationNumber = null;
	private String	buyerRegistrationNumber = null;
	private Integer	numberOfItems = null;
	private Double	lineItemAmount = null;
        private Double  discountedLineItemAmount = null;
	private Double	freightAmount = null;
	private String	federalTaxType = null;
	private String	countyTaxType = null;
	private String	provinceTaxType = null;
	private String	cityTaxType = null;
	private String	primaryCurrencyCode = null;
	private String	documentNumber = null;


	/**
	 * Default constructor
	 */
	public TaxRequestElement()
	{
	}

	/**
	 * A company ID is sometimes used as a key into a database for a tax
	 * service provider.  Some providers may require this field when they
	 * act as an ASP and they need to have a unique company id/name for each
	 * of their customers.  Not all providers require this field.  See your
	 * provider documentation for more information.
	 *
	 * @param id the company id to set
	 **/
	public void setCompanyID( String id )
	{
		companyID = id;
	}

	/**
	 * Gets the company ID
	 *
	 * @return the company id
	 */
	public String getCompanyID()
	{
		return( companyID );
	}

	/**
	 * A division code is usually used to further qualify the company id.  For example,
	 * a particular company (identified by the id field above) may have more than one
	 * place where they do business.   A service provider can use this field to look
	 * up in some backing store the address information for the company and division
	 * that is needed to help them calculate tax correctly.  Not all providers require
	 * this field.  See your provider documentation for more information.
	 *
	 * @param code the company id to set
	 **/
	public void setDivisionCode( String code )
	{
		divisionCode = code;
	}

	/**
	 * Gets the division code
	 *
	 * @return the division code
	 */
	public String getDivisionCode()
	{
		return( divisionCode );
	}

	/**
	 * country code is used by some tax services.  It's meaning is somewhat
	 * tax service specific.  Usually this is something like the
	 * abbreviation for the country or a number that is assigned to the
	 * country.
	 *
	 * @param code the county code
	 */
	public void setShipFromCountryCode( String code )
	{
		shipFromCountryCode = code;
	}

	/**
	 * Gets the country code
	 *
	 * @return the country code
	 */
	public String getShipFromCountryCode()
	{
		return( shipFromCountryCode );
	}

	/**
	 * provinceCode is used by some address verification systems.  It's meaning
	 * is somewhat tax service specific.  Usually this is something like the abbreviation
	 * for the province or state.  For example, the state name may be
	 * &quot;Colorado&quot; and the province code is &quot;CO&quot;
	 *
	 * @param code the province (or state) code
	 */
	public void setShipFromProvinceCode( String code )
	{
		shipFromProvinceCode = code;
	}

	/**
	 * Gets the province code
	 *
	 * @return the province code
	 */
	public String getShipFromProvinceCode()
	{
		return( shipFromProvinceCode );
	}

	/**
	 * countyName is the name of the county for the address.
	 *
	 * @param name of the county
	 */
	public void setShipFromCountyName( String name )
	{
		shipFromCountyName = name;
	}

	/**
	 * Gets the county name
	 *
	 * @return the name of the county
	 */
	public String getShipFromCountyName()
	{
		return( shipFromCountyName );
	}

	/**
	 * countyCode is used by some tax service providers.  It's meaning
	 * is somewhat provider specific.  Usually this is something like the abbreviation
	 * for the county or a number that is assigned to the county.
	 *
	 * @param code the county code
	 */
	public void setShipFromCountyCode( String code )
	{
		shipFromCountyCode = code;
	}

	/**
	 * Gets the county code
	 *
	 * @return the code of the county
	 */
	public String getShipFromCountyCode()
	{
		return( shipFromCountyCode );
	}

	/**
	 * cityName is the name of the city for the ship from address.
	 *
	 * @param name the city name
	 */
	public void setShipFromCityName( String name )
	{
		shipFromCityName = name;
	}

	/**
	 * Gets the city name
	 *
	 * @return the name of the city
	 */
	public String getShipFromCityName()
	{
		return( shipFromCityName );
	}

	/**
	 * postalCode is the postal (or zip) code for the ship from address.  The
	 * format of this string is both tax service and country specific.
	 *
	 * @param code the postal (or zip) code
	 */
	public void setShipFromPostalCode( String code )
	{
		shipFromPostalCode = code;
	}

	/**
	 * Gets the postal code
	 *
	 * @return the postal code of the ship from address
	 */
	public String getShipFromPostalCode()
	{
		return( shipFromPostalCode );
	}

	/**
	 * locationCode is an extra code to pinpoint a location to a higher
	 * resolution than just the postal code.  Some services include
	 * this (taxware calls it the geo code) when there are multiple
	 * tax jusrisdictions in a single postal code.  The format of this
	 * string is tax service specific.
	 *
	 * @return the ship from location code
	 */
	public void setShipFromLocationCode( String code )
	{
		shipFromLocationCode = code;
	}

	/**
	 * Gets the ship from location code
	 *
	 * @return the location code of the ship from address
	 */
	public String getShipFromLocationCode()
	{
		return( shipFromLocationCode );
	}
//
// destination
//
	public void setShipToCountryCode( String code )
	{
		shipToCountryCode = code;
	}
	public String getShipToCountryCode()
	{
		return( shipToCountryCode );
	}

	public void setShipToProvinceCode( String code )
	{
		shipToProvinceCode = code;
	}
	public String getShipToProvinceCode()
	{
		return( shipToProvinceCode );
	}

	public void setShipToCountyName( String name )
	{
		shipToCountyName = name;
	}
	public String getShipToCountyName()
	{
		return( shipToCountyName );
	}

	public void setShipToCountyCode( String code )
	{
		shipToCountyCode = code;
	}
	public String getShipToCountyCode()
	{
		return( shipToCountyCode );
	}

	public void setShipToCityName( String name )
	{
		shipToCityName = name;
	}
	public String getShipToCityName()
	{
		return( shipToCityName );
	}

	public void setShipToPostalCode( String code )
	{
		shipToPostalCode = code;
	}
	public String getShipToPostalCode()
	{
		return( shipToPostalCode );
	}

	public void setShipToLocationCode( String code )
	{
		shipToLocationCode = code;
	}
	public String getShipToLocationCode()
	{
		return( shipToLocationCode );
	}

//
// origin
//
	public void setOriginCountryCode( String code )
	{
		originCountryCode = code;
	}
	public String getOriginCountryCode()
	{
		return( originCountryCode );
	}

	public void setOriginProvinceCode( String code )
	{
		originProvinceCode = code;
	}
	public String getOriginProvinceCode()
	{
		return( originProvinceCode );
	}

	public void setOriginCountyName( String name )
	{
		originCountyName = name;
	}
	public String getOriginCountyName()
	{
		return( originCountyName );
	}

	public void setOriginCountyCode( String code )
	{
		originCountyCode = code;
	}
	public String getOriginCountyCode()
	{
		return( originCountyCode );
	}

	public void setOriginCityName( String name )
	{
		originCityName = name;
	}
	public String getOriginCityName()
	{
		return( originCityName );
	}

	public void setOriginPostalCode( String code )
	{
		originPostalCode = code;
	}
	public String getOriginPostalCode()
	{
		return( originPostalCode );
	}

	public void setOriginLocationCode( String code )
	{
		originLocationCode = code;
	}
	public String getOriginLocationCode()
	{
		return( originLocationCode );
	}

//
// POA
//
	public void setPOACountryCode( String code )
	{
		poaCountryCode = code;
	}
	public String getPOACountryCode()
	{
		return( poaCountryCode );
	}

	public void setPOAProvinceCode( String code )
	{
		poaProvinceCode = code;
	}
	public String getPOAProvinceCode()
	{
		return( poaProvinceCode );
	}

	public void setPOACountyName( String name )
	{
		poaCountyName = name;
	}
	public String getPOACountyName()
	{
		return( poaCountyName );
	}

	public void setPOACountyCode( String code )
	{
		poaCountyCode = code;
	}
	public String getPOACountyCode()
	{
		return( poaCountyCode );
	}

	public void setPOACityName( String name )
	{
		poaCityName = name;
	}
	public String getPOACityName()
	{
		return( poaCityName );
	}

	public void setPOAPostalCode( String code )
	{
		poaPostalCode = code;
	}
	public String getPOAPostalCode()
	{
		return( poaPostalCode );
	}

	public void setPOALocationCode( String code )
	{
		poaLocationCode = code;
	}
	public String getPOALocationCode()
	{
		return( poaLocationCode );
	}


	/**
	 * Point of title passage is used to tell the tax service where legal
	 * title has transfered.  usually there are two values, one to indicate
	 * that the transfer occured at the ship from location and one to indicate
	 * that the transfer occured at the ship to location
	 *
	 * @param pot the point of title passage.  The format of this string is
	 * tax vendor specific
	 * 
	 */
	public void setPointOfTitlePassage( String pot )
	{
		pointOfTitlePassage = pot;
	}
	public String getPointOfTitlePassage()
	{
		return( pointOfTitlePassage );
	}

	/**
	 * Transaction type is used to indicate if this is a purchase or a sale
	 *
	 * @param type the type of transaction.  The format of this string is
	 * tax vendor specific
	 * 
	 */
	public void setTransactionType( String type )
	{
		transactionType = type;
	}
	public String getTransactionType()
	{
		return( transactionType );
	}

	/**
	 * Tax code is used by some tax providers.  The format is vendor specific
	 *
	 * @param code the tax code.  The format of this string is tax vendor specific
	 * 
	 */
	public void setTaxCode( String code )
	{
		taxCode = code;
	}
	public String getTaxCode()
	{
		return( taxCode );
	}

	/**
	 * The type of tax is meant to be one of acquisition, input, output, or export.
	 * It may not be used in for a particular tax service provider
	 *
	 * @param type the type of tax
	 *
	 */
	public void setTypeOfTax( String type )
	{
		typeOfTax = type;
	}
	public String getTypeOfTax()
	{
		return( typeOfTax );
	}

	/**
	 * Just to confuse you, we tossed in a "save" instead of a "set".  Why?
	 * Well wsgen is totally brain dead - you <b>can not</b> have convenience
	 * type methods to convert from one type to another.  And java.util.Date
	 * isn't supported.  The instance variable must have a single getter and
	 * setter that take the exact same type as the type of the variable.  In
	 * other words, your class is basically a C struct.<p>
	 *
	 * So for "convenience" we have a <code>saveInvoiceDate()</code> that takes
	 * a java.util.Date and a <code>setInvoiceDate()</code> that takes a string.
	 *
	 * Ranting aside, the invoice date is used to the date on which the transaction
	 * is invoiced.
	 *
	 * @param date the date of the invoice
	 *
	 */
	public void saveInvoiceDate( java.util.Date date )
	{
		SimpleDateFormat	formatter = new SimpleDateFormat( "yyyyMMdd" );


		invoiceDate = formatter.format( date );
	}

	/**
	 * For the same reason as above, we give you the "obtain" version of
	 * the getter for the invoice date.
	 *
	 */
	public java.util.Date obtainInvoiceDate()
	{
		SimpleDateFormat	formatter = new SimpleDateFormat( "yyyyMMdd" );
		ParsePosition		position = new ParsePosition( 0 );

		if (invoiceDate == null) 
			return null;
		else
			return( formatter.parse( invoiceDate, position ) );
	}


	/**
	 * As promised, the String version of manipulating the invoice date
	 *
	 * The String passed in <b>must</b> be in YYYYMMDD format.
	 *
	 * @param aDate the date
	 *
	 */
	public void setInvoiceDate( String aDate )
	{
		invoiceDate = aDate;
	}
	public String getInvoiceDate()
	{
		return invoiceDate;
	}

	/**
	 * The date that the goods where delivered, shipped, or moved.
	 *
	 * @param date the java.util.Date version of setting the delivery date
	 *
	 */
	public void saveDeliveryDate( java.util.Date date )
	{
		SimpleDateFormat	formatter = new SimpleDateFormat( "yyyyMMdd" );


		deliveryDate = formatter.format( date );
	}
	public java.util.Date obtainDeliveryDate()
	{
		SimpleDateFormat	formatter = new SimpleDateFormat( "yyyyMMdd" );
		ParsePosition		position = new ParsePosition( 0 );

		if (deliveryDate == null) 
			return null;
		else
			return( formatter.parse( deliveryDate, position ) );
	}

	public void setDeliveryDate( String aDate )
	{
		deliveryDate = aDate;
	}

	public String getDeliveryDate()
	{
		return deliveryDate;
	}

	/**
	 * The mode of transport is something like "sea", "rail", "road" and so
	 * on.  It may have different formats in different tax services.
	 *
	 * @param mot the mode of transport string
	 *
	 */
	public void setModeOfTransport( String mot )
	{
		modeOfTransport = mot;
	}
	public String getModeOfTransport()
	{
		return( modeOfTransport );
	}

	/**
	 * The "Harmonized Commodity Description And Coding System (HS)" field.
	 *
	 * @param code the hs code
	 *
	 */
	public void setCommodityCode( String code )
	{
		commodityCode = code;
	}
	public String getCommodityCode()
	{
		return( commodityCode );
	}

	/**
	 * If true, then this is a credit transaction.
	 *
	 * @param credit set to true if this is a credit transaction
	 *
	 */
	public void setIsCredit( Boolean credit )
	{
		isCredit = credit;
	}
	public Boolean getIsCredit()
	{
		return( isCredit );
	}

	/**
	 * Document number is used to send an invoice number, purchase order number,
	 * sales order number, etc to the tax service for reporting purposes.
	 *
	 * @param number the document number
	 *
	 */
	public void setDocumentNumber( String number )
	{
		documentNumber = number;
	}
	public String getDocumentNumber()
	{
		return( documentNumber );
	}

	/**
	 * The primary currency code of the transaction.  This is usually something
	 * like "840" for USD and so on (ISO 3166 codes).
	 *
	 * @param code the code for the currency
	 *
	 */
	public void setPrimaryCurrencyCode( String code )
	{
		primaryCurrencyCode = code;
	}
	public String getPrimaryCurrencyCode()
	{
		return( primaryCurrencyCode );
	}

	/**
	 * The tax types are meant to be something like "sales", "use", "rental",
	 * "consumer use", or "services".
	 *
	 */
	public void setFederalTaxType( String type )
	{
		federalTaxType = type;
	}
	public String getFederalTaxType()
	{
		return( federalTaxType );
	}
	public void setCountyTaxType( String type ) 
	{
		countyTaxType = type;
	}
	public String getCountyTaxType()
	{
		return( countyTaxType );
	}

	public void setProvinceTaxType( String type )
	{
		provinceTaxType = type;
	}
	public String getProvinceTaxType()
	{
		return( provinceTaxType );
	}

	public void setCityTaxType( String type )
	{
		cityTaxType = type;
	}
	public String getCityTaxType()
	{
		return( cityTaxType );
	}

	/**
	 * Seller registration numbers are used by some tax systems to help the system
	 * determine tye type of tax to calculation and the location at which to calculate
	 * the tax.  It's format is country specific.
	 *
	 */
	public void setSellerRegistrationNumber( String number )
	{
		sellerRegistrationNumber = number;
	}
	public String getSellerRegistrationNumber()
	{
		return( sellerRegistrationNumber );
	}

	public void setBuyerRegistrationNumber( String number )
	{
		buyerRegistrationNumber = number;
	}
	public String getBuyerRegistrationNumber()
	{
		return( buyerRegistrationNumber );
	}

	/**
	 * The number of items (quantity) in this line of of the transaction
	 *
	 */
	public void setNumberOfItems( Integer number )
	{
		numberOfItems = number;
	}
	public Integer getNumberOfItems()
	{
		return( numberOfItems );
	}

	/**
	 * The amount of the line item
	 *
	 */
	public void setLineItemAmount( Double amount )
	{
		lineItemAmount = new Double( amount.doubleValue() );
	}
	public Double getLineItemAmount()
	{
		return( lineItemAmount );
	}

        /**
	 * The discount amount of the line item, if any
	 *
	 */
	public void setDiscountedLineItemAmount( Double amount )
	{
		discountedLineItemAmount = new Double( amount.doubleValue() );
	}
	public Double getDiscountedLineItemAmount()
	{
		return( discountedLineItemAmount );
	}

        /**
	 * The freight amount for this line item
	 *
	 */
	public void setFreightAmount( Double amount )
	{
		freightAmount = new Double( amount.doubleValue() );
	}
	public Double getFreightAmount()
	{
		return( freightAmount );
	}

	public String toString()
	{
		StringBuffer	outputBuffer = new StringBuffer();
		String			lineSeparator = System.getProperty( "line.separator" );


		outputBuffer.append( "TaxRequestElement contents:" + lineSeparator );
		outputBuffer.append( "\tcompany id: \"" + getCompanyID() + "\"" + lineSeparator );
		outputBuffer.append( "\tdivision code: \"" + getDivisionCode() + "\"" + lineSeparator );
		outputBuffer.append( "\tship from country code: \"" + getShipFromCountryCode() + "\"" + lineSeparator );
		outputBuffer.append( "\tship from province: \"" + getShipFromProvinceCode() + "\"" + lineSeparator );
		outputBuffer.append( "\tship from county name: \"" + getShipFromCountyName() + "\"" + lineSeparator );
		outputBuffer.append( "\tship from county code: \"" + getShipFromCountyCode() + "\"" + lineSeparator );
		outputBuffer.append( "\tship from city name: \"" + getShipFromCityName() + "\"" + lineSeparator );
		outputBuffer.append( "\tship from postal code: \"" + getShipFromPostalCode() + "\"" + lineSeparator );
		outputBuffer.append( "\tship from location code: \"" + getShipFromLocationCode() + "\"" + lineSeparator );
		outputBuffer.append( "\tship to country code: \"" + getShipToCountryCode() + "\"" + lineSeparator );
		outputBuffer.append( "\tship to province: \"" + getShipToProvinceCode() + "\"" + lineSeparator );
		outputBuffer.append( "\tship to county name: \"" + getShipToCountyName() + "\"" + lineSeparator );
		outputBuffer.append( "\tship to county code: \"" + getShipToCountyCode() + "\"" + lineSeparator );
		outputBuffer.append( "\tship to city name: \"" + getShipToCityName() + "\"" + lineSeparator );
		outputBuffer.append( "\tship to postal code: \"" + getShipToPostalCode() + "\"" + lineSeparator );
		outputBuffer.append( "\tship to location code: \"" + getShipToLocationCode() + "\"" + lineSeparator );
		outputBuffer.append( "\torigin country code: \"" + getOriginCountryCode() + "\"" + lineSeparator );
		outputBuffer.append( "\torigin province: \"" + getOriginProvinceCode() + "\"" + lineSeparator );
		outputBuffer.append( "\torigin county name: \"" + getOriginCountyName() + "\"" + lineSeparator );
		outputBuffer.append( "\torigin city name: \"" + getOriginCityName() + "\"" + lineSeparator );
		outputBuffer.append( "\torigin postal code: \"" + getOriginPostalCode() + "\"" + lineSeparator );
		outputBuffer.append( "\torigin location code: \"" + getOriginLocationCode() + "\"" + lineSeparator );
		outputBuffer.append( "\tpoa country: \"" + getPOACountryCode() + "\"" + lineSeparator );
		outputBuffer.append( "\tpoa province: \"" + getPOAProvinceCode() + "\"" + lineSeparator );
		outputBuffer.append( "\tpoa county code: \"" + getPOACountyCode() + "\"" + lineSeparator );
		outputBuffer.append( "\tpoa city name: \"" + getPOACityName() + "\"" + lineSeparator );
		outputBuffer.append( "\tpoa postal code: \"" + getPOAPostalCode() + "\"" + lineSeparator );
		outputBuffer.append( "\tpoa location code: \"" + getPOALocationCode() + "\"" + lineSeparator );
		outputBuffer.append( "\tpoint of title passage: \"" + getPointOfTitlePassage() + "\"" + lineSeparator );
		outputBuffer.append( "\ttransaction type: \"" + getTransactionType() + "\"" + lineSeparator );
		outputBuffer.append( "\ttax code: \"" + getTaxCode() + "\"" + lineSeparator );
		outputBuffer.append( "\ttype of tax: \"" + getTypeOfTax() + "\"" + lineSeparator );
		outputBuffer.append( "\tinvoice date: \"" + getInvoiceDate() + "\"" + lineSeparator );
		outputBuffer.append( "\tdelivery date: \"" + getDeliveryDate() + "\"" + lineSeparator );
		outputBuffer.append( "\tmode of transport: \"" + getModeOfTransport() + "\"" + lineSeparator );
		outputBuffer.append( "\tcommodity code: \"" + getCommodityCode() + "\"" + lineSeparator );
		outputBuffer.append( "\tis credit: \"" + getIsCredit() + "\"" + lineSeparator );
		outputBuffer.append( "\tseller registration number: \"" + getSellerRegistrationNumber() + "\"" + lineSeparator );
		outputBuffer.append( "\tbuyer registration number: \"" + getBuyerRegistrationNumber() + "\"" + lineSeparator );
		outputBuffer.append( "\tnumber of items: \"" + getNumberOfItems() + "\"" + lineSeparator );
		outputBuffer.append( "\tline item amount: \"" + getLineItemAmount() + "\"" + lineSeparator );
		outputBuffer.append( "\tfreight amount: \"" + getFreightAmount() + "\"" + lineSeparator );
		outputBuffer.append( "\tfederal tax type: \"" + getFederalTaxType() + "\"" + lineSeparator );
		outputBuffer.append( "\tprovince tax type: \"" + getProvinceTaxType() + "\"" + lineSeparator );
		outputBuffer.append( "\tcounty tax type: \"" + getCountyTaxType() + "\"" + lineSeparator );
		outputBuffer.append( "\tcity tax type: \"" + getCityTaxType() + "\"" + lineSeparator );
		outputBuffer.append( "\tprimary currency code: \"" + getPrimaryCurrencyCode() + "\"" + lineSeparator );
		outputBuffer.append( "\tdocument number: \"" + getDocumentNumber() + "\"" + lineSeparator );

		return( outputBuffer.toString() );
	}
}
