package examples.wlcs.sampleapp.tax;

import java.io.*;
import java.util.*;

//import com.beasys.commerce.ebusiness.tax.*;

/**
 * A TaxRespons is a wrapper around a collection of TaxResponseElements.
 * It holds the collection along with a response code and response message from
 * the tax service.  Not all tax service's will return all information.   Please
 * see the documentation for your tax service for more information.
 *
 * @author BEA Systems, Inc.
 */
public class TaxResponse
{
	private String	responseCode = null;
	private String	responseMessage = null;
	private TaxResponseElement[]	responseArray = null;

	/**
	 * The default constructor
	 */
	public TaxResponse()
	{

	}

	/**
	 * The response code is tax service specific.  It is usually used to indicate the
	 * success or failure of the request.
	 *
	 * @param code the code to return
	 */
	public void setResponseCode( String code )
	{
		responseCode = code;
	}

	/**
	 * The response message is tax service specific.  It is usually used in conjunction
	 * with the response code to provide a textual description of the response code.
	 *
	 * @param message the message to return
	 */
	public void setResponseMessage( String message )
	{
		responseMessage = message;
	}

	/**
	 * Adds a response element
	 *
	 * @param element the response element to add
	 */
	public void setResponseArray( TaxResponseElement[] array )
	{
		responseArray = array;
	}

	/**
	 * The response code is tax service specific.  It is usually used to indicate the
	 * success or failure of the request.
	 *
	 * @return the response code from the tax service
	 */
	public String getResponseCode()
	{
		return( responseCode );
	}

	/**
	 * The response message is tax service specific.  It is usually used in conjunction
	 * with the response code to provide a textual description of the response code.
	 *
	 * @return the message from the tax service
	 */
	public String getResponseMessage()
	{
		return( responseMessage );
	}

	/**
	 * The results are stored in a collection (in this case a Vector) and can
	 * be retrieved from there.  Note that, depending on the tax service, this may
	 * return null if there where no results returned.  This can happen if an
	 * error occured in some tax service's.
	 *
	 * @return the results stored in a java.util.Vector
	 *
	 */
	public TaxResponseElement[] getResponseArray()
	{
		return( responseArray );
	}

	/**
	 * The toString() method is used to generate debugging output
	 *
	 * @return the debugging java.lang.String to be displayed on the output
	 * of your choice.
	 */
	public String toString()
	{
		StringBuffer	outputBuffer = new StringBuffer();
		String			lineSeparator = System.getProperty( "line.separator" );
		int				size = 0;


		outputBuffer.append( "TaxResponse contents:" + lineSeparator );
		outputBuffer.append( "\tresponseCode: \"" + responseCode + "\"" + lineSeparator );
		outputBuffer.append( "\tresponseMessage: \"" + responseMessage + "\"" + lineSeparator );

		if( responseArray != null )
		{

			outputBuffer.append( "\tnumber of elements returned: \"" + responseArray.length + "\"" + lineSeparator );

			for( int i = 0; i < responseArray.length; i++ )
			{
				outputBuffer.append( "TaxResponseElement #" + i + lineSeparator +
								  	 responseArray[i].toString() );
			}
		}
		else
		{
			outputBuffer.append( "\tnumber of element returned: \"0\"" + lineSeparator );
		}

		return( outputBuffer.toString() );
	}
}
