/*
 * B E A   S Y S T E M S
 *
 * C O M M E R C E   C O M P O N E N T S
 *
 * Copyright (c) 2000-2002  BEA Systems, Inc.
 *
 * All Rights Reserved. Unpublished rights reserved under the copyright laws
 * of the United States. The software contained on this media is proprietary
 * to and embodies the confidential technology of BEA Systems, Inc. The
 * possession or receipt of this information does not convey any right to disclose
 * its contents, reproduce it, or use,  or license the use, for manufacture or
 * sale, the information or anything described therein. Any use, disclosure, or
 * reproduction without BEA System's prior written permission is strictly prohibited.
 *
 * $Header: /cvsroot/site/bea-lsd/deploy/e2eDomain/beaApps/e2eApp/src/examples/wlcs/sampleapp/tax/TaxWebServiceHelper.java,v 1.2 2002/05/02 00:22:12 toneill Exp $
 */

package examples.wlcs.sampleapp.tax;

// java imports
import java.rmi.*;
import javax.ejb.*;
import java.sql.*;
import javax.sql.*;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.util.*;

// com.beasys imports
import com.beasys.commerce.axiom.util.helper.CommercePropertiesHelper;
import com.beasys.commerce.foundation.exception.SystemException;
import com.beasys.commerce.foundation.exception.RemoteSystemException;
import com.beasys.commerce.axiom.contact.Address;
import com.beasys.commerce.axiom.contact.AddressHome;
import com.beasys.commerce.ebusiness.tax.TaxException;
import com.bea.p13n.util.debug.Debug;

// weblogic imports

// other imports


//
// mbean imports
//
import com.bea.p13n.management.MBeanAccess;
import com.bea.p13n.management.ServiceConfigurationMBean;
import com.bea.commerce.mbeans.TaxServiceClientMBean;
import com.bea.p13n.management.ApplicationHelper;
import javax.management.InstanceNotFoundException;


/**
 * This helper class contains Payment related utility methods for invoking payment webservices, that cen be used by pipeline components as well as admin JSPs.
 *
 */
public class TaxWebServiceHelper
{
    private Debug debug;

    public TaxWebServiceHelper()
    {
       	debug = Debug.getInstance(TaxWebServiceHelper.class);
    }

    /**
     * Returns an instance of the <code>TaxServicePort</code> remote interface, representing the Tax Web Service.
     *
     * @throws TaxException on webservice not found, either:
     * <ul>
     * <li>There was no tax web service URL defined in the TaxServiceClientMBean </li>
     * <li>An invalid tax web service URL was defined in the TaxServiceClientMBean </li>
     * </ul>
     */
    public TaxServicePort getTaxWebService() throws TaxException
    {
        String wsdlURL = null;

        if (debug.ON) {
	   checkJAXRPCEnabled();
	}

        try
        {
             wsdlURL = TaxWebServiceHelper.getTaxServiceClientMBean().getTaxCalculatorWSDL();

             if (wsdlURL == null)
                {
                throw new TaxException("ebusiness-tax", "webservice.mbean.lookup");
                }

             TaxServicePort taxPort = new TaxService(wsdlURL).getTaxServicePort();

             return taxPort;
        }
        catch (Exception e)
        {
            if (debug.ON) { e.printStackTrace(); }
            throw new TaxException("ebusiness-tax", "tax.pipeline.lookup.exception", wsdlURL);
        }
    }

    public Address[] extractAddresses(AVSResponse avsResponse)
    {
        AVSResponseElement[] srcVector = avsResponse.getResponseArray();
        Address addrList[] = new Address[srcVector.length];
        AVSResponseElement elem = null;

        for (int i=0; i < srcVector.length; i++) {
            elem = srcVector[i];
            addrList[i] = toAddress(elem);
        }

        return addrList;
    }

    public AVSRequest convertToAVSRequest(Address addr)
    {
        AVSRequest avsRequest = new AVSRequest();

        avsRequest.setStreetAddress1( addr.getStreet1() );
        avsRequest.setStreetAddress2( addr.getStreet2() );
        avsRequest.setCityName( addr.getCity() );
        avsRequest.setPostalCode( addr.getPostalCode() );
        avsRequest.setCountyName( addr.getCounty() );
        avsRequest.setCountyCode( addr.getCountyCode() );
        avsRequest.setCountryName( addr.getCountry() );

        return avsRequest;
    }

    public void handleError(AVSResponse avsResponse) throws TaxException
    {
        throw new TaxException("ebusiness-tax", "tax.verazip.system.general.exception");
    }

    private Address toAddress(AVSResponseElement avsElement)
    {
        Address addr = AddressHome.create();

        addr.setStreet1( avsElement.getStreetAddress1() );
        addr.setStreet2( avsElement.getStreetAddress2() );
        addr.setCity( avsElement.getCityName() );
        addr.setPostalCode( avsElement.getPostalCode1() );
        addr.setCounty( avsElement.getCountyName() );
        addr.setCountyCode( avsElement.getCountyCode() );
        addr.setCountry( avsElement.getCountryName() );

        return addr;
    }

    public static TaxServiceClientMBean getTaxServiceClientMBean()
    {
       	TaxServiceClientMBean taxMBean = null;

       	if( taxMBean == null )
       	{
    	    try
	    {
	    	taxMBean = (TaxServiceClientMBean)ApplicationHelper.getServiceConfigurationMBean( "TaxServiceClient" );
	    }
	    catch( Exception e )
	    {
	    	System.err.println( "caught exception getting tax mbean - " );
	    	e.printStackTrace();
	    }
	}

	return( taxMBean );
    }

    /**
     * Check to see if use of JAXRPC client is enabled
     */
    private void checkJAXRPCEnabled()
    {
       	try
       	{
    	     Class.forName("weblogic.webservice.core.rpc.JAXRPCEnabler");
	     debug.out("JAXRPC is enabled.");
	} catch (ClassNotFoundException cnfe)
	{
	     debug.out("JAXRPC is disabled.");
	}

    }
}
