-- SQL Use Case 2 DROP, CREATE, INSERT scripts
-- Has to be run with an account that has admin priviledges for the Avitek schema
--
-- Author - Jason Lee

-- DROP all use case 2 tables

DROP TABLE AVITEK_MRP;
DROP TABLE AVITEK_ORDERS;
DROP TABLE AVITEK_PRODUCT_INVENTORY;
DROP TABLE AVITEK_SHIPPING_HISTORY;
DROP TABLE AVITEK_SHIPPING_LOCATION;
DROP TABLE AVITEK_APL_SHIPMENT;
DROP TABLE AVITEK_DHL_SHIPMENT;
DROP TABLE AVITEK_FEDEX_SHIPMENT;
DROP TABLE AVITEK_UPS_SHIPMENT;
DROP TABLE AVITEK_USPS_SHIPMENT;

COMMIT;

-- CREATE use case 2 tables

CREATE TABLE AVITEK_MRP(
       ID			NUMBER(8)	NOT NULL,
       LOT_NUMBER		VARCHAR(15)	NULL,
       STATUS			VARCHAR(255)	NULL,
       QUANTITY			NUMBER(5)	NULL
       );

CREATE TABLE AVITEK_ORDERS(
       ID			NUMBER(8)	NOT NULL,
       LOT_ID			NUMBER(8)	NOT NULL,
       SKU			VARCHAR(15)	NOT NULL,
       CARRIER_ID		VARCHAR(8)	NOT NULL,
       TRACKING_ID		VARCHAR(10)	NULL,
       CREATE_DATE		VARCHAR(15)	NULL,
       QUANTITY_ORDERED		NUMBER(5)	NULL,
       STATUS			VARCHAR(255)	NULL
       );

CREATE TABLE AVITEK_PRODUCT_INVENTORY(
       ID			NUMBER(8)	NOT NULL,
       SKU			VARCHAR(15)	NOT NULL,
       NUM_AVAIL		NUMBER(5)	NULL,
       DESCRIPTION		VARCHAR(255)	NULL,
       MIN_UNITS		NUMBER(5)	NULL,
       MAX_UNITS		NUMBER(5)	NULL
       );

CREATE TABLE AVITEK_SHIPPING_HISTORY(
       ID			NUMBER(8)	NOT NULL,
       CARRIER_ID		VARCHAR(8)	NOT NULL,
       SHIPMENT_ID		NUMBER(8)	NOT NULL,
       LOCATION_ID		NUMBER(8)	NOT NULL
       );

CREATE TABLE AVITEK_SHIPPING_LOCATION(
       ID			NUMBER(8)	NOT NULL,
       NAME			VARCHAR(50)	NULL,
       ADDRESS			VARCHAR(50)	NULL,
       CITY			VARCHAR(50)	NULL,
       STATE			VARCHAR(2)	NULL,
       COUNTRY			VARCHAR(25)	NULL,
       ZIP			VARCHAR(10)	NULL,
       LOC_DATE			VARCHAR(15)	NULL
       );

CREATE TABLE AVITEK_APL_SHIPMENT(
       ID			NUMBER(8)	NOT NULL,
       TRACKING_ID		VARCHAR(10)	NOT NULL,
       ARRIVAL_TIME		VARCHAR(16)	NULL,
       QUANTITY_SHIPPING	NUMBER(5)	NULL,
       SHIPPING_METHOD		VARCHAR(15)	NULL,
       SERVICE_REP		VARCHAR(25)	NULL,
       SERVICE_PHONE		VARCHAR(25)	NULL,
       SERVICE_MSG		VARCHAR(255)	NULL
       );

CREATE TABLE AVITEK_DHL_SHIPMENT(
       ID			NUMBER(8)	NOT NULL,
       TRACKING_ID		VARCHAR(10)	NOT NULL,
       ARRIVAL_TIME		VARCHAR(16)	NULL,
       QUANTITY_SHIPPING	NUMBER(5)	NULL,
       SHIPPING_METHOD		VARCHAR(15)	NULL,
       SERVICE_REP		VARCHAR(25)	NULL,
       SERVICE_PHONE		VARCHAR(25)	NULL,
       SERVICE_MSG		VARCHAR(255)	NULL
       );

CREATE TABLE AVITEK_FEDEX_SHIPMENT(
       ID			NUMBER(8)	NOT NULL,
       TRACKING_ID		VARCHAR(10)	NOT NULL,
       ARRIVAL_TIME		VARCHAR(16)	NULL,
       QUANTITY_SHIPPING	NUMBER(5)	NULL,
       SHIPPING_METHOD		VARCHAR(15)	NULL,
       SERVICE_REP		VARCHAR(25)	NULL,
       SERVICE_PHONE		VARCHAR(25)	NULL,
       SERVICE_MSG		VARCHAR(255)	NULL
       );

CREATE TABLE AVITEK_UPS_SHIPMENT(
       ID			NUMBER(8)	NOT NULL,
       TRACKING_ID		VARCHAR(10)	NOT NULL,
       ARRIVAL_TIME		VARCHAR(16)	NULL,
       QUANTITY_SHIPPING	NUMBER(5)	NULL,
       SHIPPING_METHOD		VARCHAR(15)	NULL,
       SERVICE_REP		VARCHAR(25)	NULL,
       SERVICE_PHONE		VARCHAR(25)	NULL,
       SERVICE_MSG		VARCHAR(255)	NULL
       );

CREATE TABLE AVITEK_USPS_SHIPMENT(
       ID			NUMBER(8)	NOT NULL,
       TRACKING_ID		VARCHAR(10)	NOT NULL,
       ARRIVAL_TIME		VARCHAR(16)	NULL,
       QUANTITY_SHIPPING	NUMBER(5)	NULL,
       SHIPPING_METHOD		VARCHAR(15)	NULL,
       SERVICE_REP		VARCHAR(25)	NULL,
       SERVICE_PHONE		VARCHAR(25)	NULL,
       SERVICE_MSG		VARCHAR(255)	NULL
       );

COMMIT;

-----------------------------------------
-- INSERT values into tables
--
-----------------------------------------

-- INSERT MRP items
-- This is a hack... the shipped and open status items should not be here.
-- Until this is fixed the "Shipped" and "Open" items must be kept in sync
-- with the _orders_ table
-- The solution will be to have the MRP lookup really pull just the
-- "confirmed/packaged/assembled" items and update the web service client with
-- that data only

INSERT INTO AVITEK_MRP (ID,LOT_NUMBER,STATUS,QUANTITY)
VALUES(1,'BA_123455','Shipped',2000);
INSERT INTO AVITEK_MRP (ID,LOT_NUMBER,STATUS,QUANTITY)
VALUES(2,'BA_123456','Shipped',1000);
INSERT INTO AVITEK_MRP (ID,LOT_NUMBER,STATUS,QUANTITY)
VALUES(3,'BA_123457','Packaged',1000);
INSERT INTO AVITEK_MRP (ID,LOT_NUMBER,STATUS,QUANTITY)
VALUES(4,'BA_123458','Assembled',1500);
INSERT INTO AVITEK_MRP (ID,LOT_NUMBER,STATUS,QUANTITY)
VALUES(5,'BA_123459','Confirmed',2500);
INSERT INTO AVITEK_MRP (ID,LOT_NUMBER,STATUS,QUANTITY)
VALUES(6,'BA_123460','Open',3000);

-- INSERT product inventory

INSERT INTO AVITEK_PRODUCT_INVENTORY (ID,SKU,NUM_AVAIL,DESCRIPTION,MIN_UNITS,MAX_UNITS)
VALUES (1,'AVI1000',4535,'AviPix 1000',6000,9000);

INSERT INTO AVITEK_PRODUCT_INVENTORY (ID,SKU,NUM_AVAIL,DESCRIPTION,MIN_UNITS,MAX_UNITS)
VALUES (2,'AVI3000',6405,'AviPix 3000',4000,2000);

INSERT INTO AVITEK_PRODUCT_INVENTORY (ID,SKU,NUM_AVAIL,DESCRIPTION,MIN_UNITS,MAX_UNITS)
VALUES (3,'AVI5000',3500,'AviPix 5000',7000,5000);

-- INSERT orders

INSERT INTO AVITEK_ORDERS (ID,LOT_ID,SKU,CARRIER_ID,TRACKING_ID,CREATE_DATE,QUANTITY_ORDERED,STATUS)
VALUES (1,1,'AVI1000','UPS','1234312q','12/10/01',2000,'Shipped');
INSERT INTO AVITEK_ORDERS (ID,LOT_ID,SKU,CARRIER_ID,TRACKING_ID,CREATE_DATE,QUANTITY_ORDERED,STATUS)
VALUES (2,2,'AVI5000','FEDEX','1234423r','12/15/01',2000,'Shipped');
INSERT INTO AVITEK_ORDERS (ID,LOT_ID,SKU,CARRIER_ID,TRACKING_ID,CREATE_DATE,QUANTITY_ORDERED,STATUS)
VALUES (3,3,'AVI1000','UPS','1235213z','12/21/01',1000,'Open');
INSERT INTO AVITEK_ORDERS (ID,LOT_ID,SKU,CARRIER_ID,TRACKING_ID,CREATE_DATE,QUANTITY_ORDERED,STATUS)
VALUES (4,4,'AVI3000','FEDEX','1236448r','12/24/01',1000,'Open');
INSERT INTO AVITEK_ORDERS (ID,LOT_ID,SKU,CARRIER_ID,TRACKING_ID,CREATE_DATE,QUANTITY_ORDERED,STATUS)
VALUES (5,5,'AVI5000','USPS','1236448r','01/02/02',1000,'Open');
INSERT INTO AVITEK_ORDERS (ID,LOT_ID,SKU,CARRIER_ID,TRACKING_ID,CREATE_DATE,QUANTITY_ORDERED,STATUS)
VALUES (6,6,'AVI1000','DHL','1237301q','01/04/02',1000,'Open');

-----------------------------------------
-- INSERT for shipments
--
-- NOTE: All the shipment tables are basically the same with pretty much the same data
--       This can be changed here for whatever need there is.
--
-- ID 1 is used for the "before" state
-- ID 2 represents the "after" state
-----------------------------------------

-- INSERT apl shipments

INSERT INTO AVITEK_APL_SHIPMENT(ID,TRACKING_ID,ARRIVAL_TIME,QUANTITY_SHIPPING,SERVICE_REP,SERVICE_PHONE,SHIPPING_METHOD,SERVICE_MSG)
VALUES (1,'1234312q','1/09/02 6:00 pm',1000,'Joe Andrews','1-800-555-1313','5-day Ground',' ');
INSERT INTO AVITEK_APL_SHIPMENT(ID,TRACKING_ID,ARRIVAL_TIME,QUANTITY_SHIPPING,SERVICE_REP,SERVICE_PHONE,SHIPPING_METHOD,SERVICE_MSG)
VALUES (2,'1237301q','1/10/02 6:00 pm',2000,'Jim Sanderson','1-800-555-1212','5-day Ground',' ');


-- INSERT dhl shipments

INSERT INTO AVITEK_DHL_SHIPMENT(ID,TRACKING_ID,ARRIVAL_TIME,QUANTITY_SHIPPING,SERVICE_REP,SERVICE_PHONE,SHIPPING_METHOD,SERVICE_MSG)
VALUES (1,'1237301q','1/09/02 6:00 pm',1000,'Joe Andrews','1-800-555-1313','5-day Ground',' ');
INSERT INTO AVITEK_DHL_SHIPMENT(ID,TRACKING_ID,ARRIVAL_TIME,QUANTITY_SHIPPING,SERVICE_REP,SERVICE_PHONE,SHIPPING_METHOD,SERVICE_MSG)
VALUES (2,'1235213z','1/10/02 6:00 pm',2000,'Jim Sanderson','1-800-555-1212','5-day Ground',' ');

-- INSERT fedex shipments

INSERT INTO AVITEK_FEDEX_SHIPMENT(ID,TRACKING_ID,ARRIVAL_TIME,QUANTITY_SHIPPING,SERVICE_REP,SERVICE_PHONE,SHIPPING_METHOD,SERVICE_MSG)
VALUES (1,'1234423r','1/09/02 6:00 pm',1000,'Joe Andrews','1-800-555-1313','5-day Ground',' ');
INSERT INTO AVITEK_FEDEX_SHIPMENT(ID,TRACKING_ID,ARRIVAL_TIME,QUANTITY_SHIPPING,SERVICE_REP,SERVICE_PHONE,SHIPPING_METHOD,SERVICE_MSG)
VALUES (2,'1236448r','1/10/02 6:00 pm',2000,'Jim Sanderson','1-800-555-1212','5-day Ground',' ');

-- INSERT ups shipments

INSERT INTO AVITEK_UPS_SHIPMENT(ID,TRACKING_ID,ARRIVAL_TIME,QUANTITY_SHIPPING,SERVICE_REP,SERVICE_PHONE,SHIPPING_METHOD,SERVICE_MSG)
VALUES (1,'1234312q','1/09/02 6:00 pm',1000,'Joe Andrews','1-800-555-1313','5-day Ground',' ');
INSERT INTO AVITEK_UPS_SHIPMENT(ID,TRACKING_ID,ARRIVAL_TIME,QUANTITY_SHIPPING,SERVICE_REP,SERVICE_PHONE,SHIPPING_METHOD,SERVICE_MSG)
VALUES (2,'1235213z','1/10/02 6:00 pm',2000,'Jim Sanderson','1-800-555-1212','5-day Ground',' ');

-- INSERT usps shipments

INSERT INTO AVITEK_USPS_SHIPMENT(ID,TRACKING_ID,ARRIVAL_TIME,QUANTITY_SHIPPING,SERVICE_REP,SERVICE_PHONE,SHIPPING_METHOD,SERVICE_MSG)
VALUES (1,'1234312q','1/09/02 6:00 pm',1000,'Joe Andrews','1-800-555-1313','5-day Ground',' ');
INSERT INTO AVITEK_USPS_SHIPMENT(ID,TRACKING_ID,ARRIVAL_TIME,QUANTITY_SHIPPING,SERVICE_REP,SERVICE_PHONE,SHIPPING_METHOD,SERVICE_MSG)
VALUES (2,'1235213z','1/10/02 6:00 pm',2000,'Jim Sanderson','1-800-555-1212','5-day Ground',' ');

-----------------------------------------
-- INSERT for shipping locations
-----------------------------------------

INSERT INTO AVITEK_SHIPPING_LOCATION(ID,NAME,ADDRESS,CITY,STATE,COUNTRY,ZIP,LOC_DATE)
VALUES (1,'Avitek Manufacturing Center','1234 Main Street','Palo Alto','CA','US','94108','01/07/02 4:00pm');
INSERT INTO AVITEK_SHIPPING_LOCATION(ID,NAME,ADDRESS,CITY,STATE,COUNTRY,ZIP,LOC_DATE)
VALUES (2,'Central Depot','1234 Main Street','Dallas','TX','US','43102','01/09/02 1:00pm');
INSERT INTO AVITEK_SHIPPING_LOCATION(ID,NAME,ADDRESS,CITY,STATE,COUNTRY,ZIP,LOC_DATE)
VALUES (3,'East Coast Distribution Hub','5521 Woodlawn','Baltimore','MD','US','21075','01/13/02 9:40am');

INSERT INTO AVITEK_SHIPPING_LOCATION(ID,NAME,ADDRESS,CITY,STATE,COUNTRY,ZIP,LOC_DATE)
VALUES (4,'Avitek Manufacturing South','100 West Street','Mesa','AZ','US','56874','02/12/02 3:15pm');
INSERT INTO AVITEK_SHIPPING_LOCATION(ID,NAME,ADDRESS,CITY,STATE,COUNTRY,ZIP,LOC_DATE)
VALUES (5,'Central Distribution Hub','1234 Main Street','Dallas','TX','US','43102','02/09/02 5:45am');
INSERT INTO AVITEK_SHIPPING_LOCATION(ID,NAME,ADDRESS,CITY,STATE,COUNTRY,ZIP,LOC_DATE)
VALUES (6,'Avitek Outlet','1234 Main Street','San Francisco','CA','US','94102','02/03/02 10:15am');

-----------------------------------------      
-- INSERT for shipping history

-- The following 'join' table is only provided as a means to easily change the 
-- shipments and their locations without breaking any of the business logic.
-- They should be used as follows:
-- 
-- Each has a set number of orders. In the case of APL (below) they have 2 orders.
-- Since we basically share the shipping location table (for now) between all the
-- shippers, we need to provide a mapping between the shipper, shipment and location.
-- So the first line in the APL example below provides a mapping between the APL
-- service partner, the first APL order shipment, and the first shipping location.
-- The second entry provides a mapping for the APL partner, the first APL shipment
-- and the second shipping location, etc... Each shipping partner will need
-- n number of shipments * n number of shipping locations in order for this to fully
-- work. Enjoy! 
-----------------------------------------

-- APL mapping

INSERT INTO AVITEK_SHIPPING_HISTORY(ID,CARRIER_ID,SHIPMENT_ID,LOCATION_ID)
VALUES (1,'APL',1,1);
INSERT INTO AVITEK_SHIPPING_HISTORY(ID,CARRIER_ID,SHIPMENT_ID,LOCATION_ID)
VALUES (2,'APL',1,2);
INSERT INTO AVITEK_SHIPPING_HISTORY(ID,CARRIER_ID,SHIPMENT_ID,LOCATION_ID)
VALUES (3,'APL',1,3);
INSERT INTO AVITEK_SHIPPING_HISTORY(ID,CARRIER_ID,SHIPMENT_ID,LOCATION_ID)
VALUES (4,'APL',2,4);
INSERT INTO AVITEK_SHIPPING_HISTORY(ID,CARRIER_ID,SHIPMENT_ID,LOCATION_ID)
VALUES (5,'APL',2,5);
INSERT INTO AVITEK_SHIPPING_HISTORY(ID,CARRIER_ID,SHIPMENT_ID,LOCATION_ID)
VALUES (6,'APL',2,6);

-- DHL mapping

INSERT INTO AVITEK_SHIPPING_HISTORY(ID,CARRIER_ID,SHIPMENT_ID,LOCATION_ID)
VALUES (1,'DHL',1,1);
INSERT INTO AVITEK_SHIPPING_HISTORY(ID,CARRIER_ID,SHIPMENT_ID,LOCATION_ID)
VALUES (2,'DHL',1,2);
INSERT INTO AVITEK_SHIPPING_HISTORY(ID,CARRIER_ID,SHIPMENT_ID,LOCATION_ID)
VALUES (3,'DHL',1,3);
INSERT INTO AVITEK_SHIPPING_HISTORY(ID,CARRIER_ID,SHIPMENT_ID,LOCATION_ID)
VALUES (4,'DHL',2,4);
INSERT INTO AVITEK_SHIPPING_HISTORY(ID,CARRIER_ID,SHIPMENT_ID,LOCATION_ID)
VALUES (5,'DHL',2,5);
INSERT INTO AVITEK_SHIPPING_HISTORY(ID,CARRIER_ID,SHIPMENT_ID,LOCATION_ID)
VALUES (6,'DHL',2,6);

-- FEDEX mapping

INSERT INTO AVITEK_SHIPPING_HISTORY(ID,CARRIER_ID,SHIPMENT_ID,LOCATION_ID)
VALUES (1,'FEDEX',1,1);
INSERT INTO AVITEK_SHIPPING_HISTORY(ID,CARRIER_ID,SHIPMENT_ID,LOCATION_ID)
VALUES (2,'FEDEX',1,2);
INSERT INTO AVITEK_SHIPPING_HISTORY(ID,CARRIER_ID,SHIPMENT_ID,LOCATION_ID)
VALUES (3,'FEDEX',1,3);
INSERT INTO AVITEK_SHIPPING_HISTORY(ID,CARRIER_ID,SHIPMENT_ID,LOCATION_ID)
VALUES (4,'FEDEX',2,4);
INSERT INTO AVITEK_SHIPPING_HISTORY(ID,CARRIER_ID,SHIPMENT_ID,LOCATION_ID)
VALUES (5,'FEDEX',2,5);
INSERT INTO AVITEK_SHIPPING_HISTORY(ID,CARRIER_ID,SHIPMENT_ID,LOCATION_ID)
VALUES (6,'FEDEX',2,6);

-- UPS mapping

INSERT INTO AVITEK_SHIPPING_HISTORY(ID,CARRIER_ID,SHIPMENT_ID,LOCATION_ID)
VALUES (1,'UPS',1,1);
INSERT INTO AVITEK_SHIPPING_HISTORY(ID,CARRIER_ID,SHIPMENT_ID,LOCATION_ID)
VALUES (2,'UPS',1,2);
INSERT INTO AVITEK_SHIPPING_HISTORY(ID,CARRIER_ID,SHIPMENT_ID,LOCATION_ID)
VALUES (3,'UPS',1,3);
INSERT INTO AVITEK_SHIPPING_HISTORY(ID,CARRIER_ID,SHIPMENT_ID,LOCATION_ID)
VALUES (4,'UPS',2,4);
INSERT INTO AVITEK_SHIPPING_HISTORY(ID,CARRIER_ID,SHIPMENT_ID,LOCATION_ID)
VALUES (5,'UPS',2,5);
INSERT INTO AVITEK_SHIPPING_HISTORY(ID,CARRIER_ID,SHIPMENT_ID,LOCATION_ID)
VALUES (6,'UPS',2,6);

-- USPS mapping

INSERT INTO AVITEK_SHIPPING_HISTORY(ID,CARRIER_ID,SHIPMENT_ID,LOCATION_ID)
VALUES (1,'USPS',1,1);
INSERT INTO AVITEK_SHIPPING_HISTORY(ID,CARRIER_ID,SHIPMENT_ID,LOCATION_ID)
VALUES (2,'USPS',1,2);
INSERT INTO AVITEK_SHIPPING_HISTORY(ID,CARRIER_ID,SHIPMENT_ID,LOCATION_ID)
VALUES (3,'USPS',1,3);
INSERT INTO AVITEK_SHIPPING_HISTORY(ID,CARRIER_ID,SHIPMENT_ID,LOCATION_ID)
VALUES (4,'USPS',2,4);
INSERT INTO AVITEK_SHIPPING_HISTORY(ID,CARRIER_ID,SHIPMENT_ID,LOCATION_ID)
VALUES (5,'USPS',2,5);
INSERT INTO AVITEK_SHIPPING_HISTORY(ID,CARRIER_ID,SHIPMENT_ID,LOCATION_ID)
VALUES (6,'USPS',2,6);

COMMIT;