@rem *************************************************************************
@rem This script is used to start WebLogic Server
@rem
@rem To create your own start script for your domain, all you need to set is 
@rem SERVER_NAME, then call this script from the domain 
@rem directory.
@rem If you set DOMAIN_NAME, then this will assume WLS6.1 compability -- it
@rem pass in -Dweblogic.domain and you should invoke it from the parent
@rem directory of config\%DOMAIN_NAME%
@rem
@rem Other variables that startWebLogic takes are:
@rem
@rem DB_SETTINGS  - the db_settings.properties to use pass to
@rem                startPBServer.bat, if DATABASE==POINTBASE
@rem WLS_USER     - admin username for server startup
@rem WLS_PW       - cleartext password for server startup
@rem ADMIN_URL    - if this variable is set, the server started will be a 
@rem                managed server, and will look to the url specified (i.e. 
@rem                http://localhost:7001) as the admin server.
@rem WLS_PROD_MODE- set to true for production mode servers, false for 
@rem                development mode
@rem WLS_MGMT_DISC- set to true for production mode servers, false for 
@rem                development mode
@rem JAVA_OPTIONS - Java command-line options for running the server. (These
@rem                will be tagged on to the end of the JAVA_VM and MEM_ARGS)
@rem JAVA_VM      - The java arg specifying the VM to run.  (i.e. -server, 
@rem                -client, etc.)
@rem MEM_ARGS     - The variable to override the standard memory arguments
@rem                passed to java
@rem JAVA_SEC_POLICY - The Java Security policy file to use (defaults to
@rem                   weblogic.policy)
@rem
@rem This assumes that your PATH and CLASSPATH are configured for
@rem the various native libraries and system classes required by the server.
@rem This also supports starting up a Pointbase server
@rem 
@rem *************************************************************************

@echo off
setlocal

@rem Invoke set-environment.bat if we haven't already
if "%WL_COMMERCE_HOME%"=="" call "..\..\..\portal\bin\win32\set-environment.bat"

@rem Now BEA_HOME, WEBLOGIC_HOME, WL_COMMERCE_HOME, and JDK_HOME should be set

@rem Check that the WebLogic classes are where we expect them to be
:checkWLS
if exist "%WEBLOGIC_HOME%\lib\weblogic.jar" goto checkJava
echo The WebLogic Server wasn't found in directory %WEBLOGIC_HOME%.
echo Please edit your set-environment script so that the WEBLOGIC_HOME variable
echo points to the WebLogic Server installation directory.
goto finish

@rem Check that java is where we expect it to be
:checkJava
if exist "%JDK_HOME%\bin\java.exe" goto runWebLogic
echo The JDK wasn't found in directory %JDK_HOME%.
echo Please edit your set-environment script so that the JDK_HOME variable 
echo points to the location of your JDK.
goto finish

:runWebLogic

@rem set defaults for these, if they weren't set in either the invoker or
@rem set-environment
if "%JAVA_VM%"=="" set JAVA_VM=-hotspot
if "%MEM_ARGS%"=="" set MEM_ARGS=-Xms200m -Xmx200m
if "%WLS_PROD_MODE%"=="" set WLS_PROD_MODE=true
if "%WLS_MGMT_DISC%"=="" set WLS_MGMT_DISC=false
if "%JAVA_SEC_POLICY%"=="" set JAVA_SEC_POLICY=%WEBLOGIC_HOME%\lib\weblogic.policy

if not "%DOMAIN_NAME%"=="" set JAVA_OPTIONS=%JAVA_OPTIONS% -Dweblogic.Domain=%DOMAIN_NAME%

set SAVED_CLASSPATH=%CLASSPATH%
set CLASSPATH=%CLASSPATH%;%WL_COMMERCE_HOME%\lib\p13n\ejb\p13n_util.jar

@rem Start PointBase, if needed
set PBOWNER=
if not "%DATABASE%"=="POINTBASE" goto NOT_PB1

set PB_HOST=localhost
set PB_PORT=9092
set PB_DB=wlportal

@rem Get the PointBase host, port, and database name from db_settings.properties
if "%DB_SETTINGS%"=="" goto _NO_DB_SETTINGS
FOR /F "eol=# tokens=1,2 delims==" %%i in (%DB_SETTINGS%) do  (
    if %%i == host SET PB_HOST=%%j
)
FOR /F "eol=# tokens=1,2 delims==" %%i in (%DB_SETTINGS%) do  (
    if %%i == port SET PB_PORT=%%j
)
FOR /F "eol=# tokens=1,2 delims==" %%i in (%DB_SETTINGS%) do  (
    if %%i == db_name SET PB_DB=%%j
)

:_NO_DB_SETTINGS
@rem Check if PointBase is up (sets errorlevel)
"%JDK_HOME%\bin\java" com.bea.p13n.db.internal.PointBasePing -host %PB_HOST% -port %PB_PORT% -database %PB_DB%

@rem PointBase already running
IF NOT ERRORLEVEL 1 GOTO NOT_PB1

IF "%DB_SETTINGS%"=="" START /I /B "PointBase Server" ".\startPBServer.bat"
IF NOT "%DB_SETTINGS%"=="" START /I /B "PointBase Server" ".\startPBServer.bat" %DB_SETTINGS%

@rem set PBOWNER so that we will shutdown down PB at the end
set PBOWNER=THIS_SCRIPT

@rem  if not using PointBase or already started, continue on...
:NOT_PB1

set CLASSPATH=%SAVED_CLASSPATH%

@rem Start Server

echo ***************************************************
echo *  To start WebLogic Server, use a username and   *
echo *  password assigned to an admin-level user.  By  *
echo *  default, this is user: weblogic                *
echo *  and password: weblogic                         *
echo *  These should both be changed using the         *
echo *  WebLogic Server console at                     *
echo *       http://[hostname]:[port]/console          *
echo ***************************************************

echo on
@if "%ADMIN_URL%" == "" goto runAdmin
"%JDK_HOME%\bin\java" %JAVA_VM% %MEM_ARGS% %JAVA_OPTIONS% -Dweblogic.Name=%SERVER_NAME% -Dbea.home="%BEA_HOME%" -Dweblogic.management.username=%WLS_USER% -Dweblogic.management.password=%WLS_PW% -Dweblogic.management.server=%ADMIN_URL% -Dweblogic.ProductionModeEnabled=%WLS_PROD_MODE% -Djava.security.policy=="%JAVA_SEC_POLICY%" weblogic.Server
@goto finish

@:runAdmin
"%JDK_HOME%\bin\java" %JAVA_VM% %MEM_ARGS% %JAVA_OPTIONS% -Dweblogic.Name=%SERVER_NAME% -Dbea.home="%BEA_HOME%" -Dweblogic.management.username=%WLS_USER% -Dweblogic.management.password=%WLS_PW% -Dweblogic.ProductionModeEnabled=%WLS_PROD_MODE% -Dweblogic.management.discover=%WLS_MGMT_DISC% -Djava.security.policy=="%JAVA_SEC_POLICY%" weblogic.Server
@goto finish

:finish

@echo off
@rem ###########################################################################
@rem if PointBase was launched by this script then shut it down
@rem ###########################################################################
if not "%PBOWNER%"=="THIS_SCRIPT" goto _the_end
    
if "%DB_SETTINGS%"=="" call "%WL_COMMERCE_HOME%\bin\win32\stopPBServer.bat"
if not "%DB_SETTINGS%"=="" call "%WL_COMMERCE_HOME%\bin\win32\stopPBServer.bat" %DB_SETTINGS%

:_the_end
endlocal
